@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2016-2017 Intel, Ltd.
  * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,12 +19,17 @@
  *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+
+#if defined(HAVE_SYS_AUXV_H)
+#include <sys/auxv.h>
+#endif
 
 #if defined(__linux__)
 
 typedef struct {
-	const char	*name;		/* cache type name */
-	const uint32_t	value;		/* cache type ID */
+	const char	*name;			/* cache type name */
+	const stress_cache_type_t value;	/* cache type ID */
 } stress_generic_map_t;
 
 typedef enum {
@@ -38,10 +44,10 @@ typedef enum {
 #endif
 
 static const stress_generic_map_t cache_type_map[] = {
-	{"data"        , CACHE_TYPE_DATA},
-	{"instruction" , CACHE_TYPE_INSTRUCTION},
-	{"unified"     , CACHE_TYPE_UNIFIED},
-	{ NULL         , CACHE_TYPE_UNKNOWN}
+	{ "data",		CACHE_TYPE_DATA },
+	{ "instruction",	CACHE_TYPE_INSTRUCTION },
+	{ "unified",		CACHE_TYPE_UNIFIED },
+	{  NULL,		CACHE_TYPE_UNKNOWN }
 };
 
 /*
@@ -90,23 +96,22 @@ static int stress_get_string_from_file(
  * - kibibytes ('K' - aka KiB).
  * - mebibytes ('M' - aka MiB).
  * - gibibytes ('G' - aka GiB).
+ * - tebibutes ('T' - aka TiB).
  *
  * Returns: size in bytes, or 0 on error.
  */
 static uint64_t stress_size_to_bytes(const char *str)
 {
-	uint64_t            bytes;
-	uint64_t            multiplier;
-	unsigned long int   value;
-	int                 ret;
-	char                sz;
+	uint64_t bytes;
+	int	 ret;
+	char	 sz;
 
 	if (!str) {
 		pr_dbg("%s: empty string specified\n", __func__);
 		return 0;
 	}
 
-	ret = sscanf(str, "%lu%c", &value, &sz);
+	ret = sscanf(str, "%" SCNu64 "%c", &bytes, &sz);
 	if (ret != 2) {
 		pr_dbg("%s: failed to parse suffix from \"%s\"\n",
 			__func__, str);
@@ -115,25 +120,25 @@ static uint64_t stress_size_to_bytes(const char *str)
 
 	switch (sz) {
 	case 'B':
-		multiplier = 1;
+		/* no-op */
 		break;
 	case 'K':
-		multiplier = KB;
+		bytes *= KB;
 		break;
 	case 'M':
-		multiplier = MB;
+		bytes *= MB;
 		break;
 	case 'G':
-		multiplier = GB;
+		bytes *= GB;
+		break;
+	case 'T':
+		bytes *= TB;
 		break;
 	default:
 		pr_err("unable to convert '%c' size to bytes\n", sz);
 		bytes = 0;
-		goto out;
+		break;
 	}
-
-	bytes = value * multiplier;
-out:
 	return bytes;
 }
 
@@ -307,6 +312,7 @@ stress_cpu_cache_t *stress_get_cpu_cache(const stress_cpus_t *cpus, const uint16
 	return stress_get_cache_by_cpu(cpu, cache_level);
 }
 
+#if defined(STRESS_ARCH_SPARC)
 static int stress_get_cpu_cache_value(
 	const char *cpu_path,
 	const char *file,
@@ -323,6 +329,7 @@ static int stress_get_cpu_cache_value(
 	}
 	return -1;
 }
+#endif
 
 /*
  *  stress_get_cpu_cache_auxval()
@@ -412,9 +419,102 @@ static int stress_get_cpu_cache_auxval(stress_cpu_t *cpu)
 #endif
 }
 
+#if defined(STRESS_ARCH_ALPHA)
+/*
+ *  stress_get_cpu_cache_alpha()
+ *	find cache information as provided by linux Alpha from
+ *	/proc/cpu. Assume cache layout for 1st CPU is same for
+ *	all CPUs.
+ */
+static int stress_get_cpu_cache_alpha(
+	stress_cpu_t *cpu,
+	const char *cpu_path)
+{
+	FILE *fp;
+	const size_t count = 4;
+	size_t idx = 0;
+
+	(void)cpu_path;
+
+	/*
+	 * parse /proc/cpu info in the form:
+	 * L1 Icache		: 64K, 2-way, 64b line
+	 * L1 Dcache		: 64K, 2-way, 64b line
+	 * L2 cache		: n/a
+	 * L3 cache		: n/a
+	 */
+	cpu->caches = calloc(count, sizeof(*(cpu->caches)));
+	if (!cpu->caches) {
+		pr_err("failed to allocate %zu bytes for cpu caches\n",
+			count * sizeof(*(cpu->caches)));
+		return 0;
+	}
+
+	fp = fopen("/proc/cpuinfo", "r");
+	if (fp) {
+		char buffer[4096];
+
+		while ((idx < count) && fgets(buffer, sizeof(buffer), fp)) {
+			stress_cache_type_t cache_type = CACHE_TYPE_UNKNOWN;
+			uint16_t cache_level = 0;
+			char *ptr;
+			int cache_size, cache_ways, cache_line_size, n;
+
+			if (!strncmp("L1 Icache", buffer, 9)) {
+				cache_type = CACHE_TYPE_INSTRUCTION;
+				cache_level = 1;
+			} else if (!strncmp("L1 Dcache", buffer, 9))  {
+				cache_type = CACHE_TYPE_DATA;
+				cache_level = 1;
+			}
+			else if (!strncmp("L2 cache", buffer, 8)) {
+				cache_type = CACHE_TYPE_DATA;
+				cache_level = 2;
+			}
+			else if (!strncmp("L3 cache", buffer, 8)) {
+				cache_type = CACHE_TYPE_DATA;
+				cache_level = 3;
+			}
+			else
+				continue;
+			ptr = strchr(buffer, ':');
+			if (!ptr)
+				continue;
+			ptr++;
+			cache_size = 0;
+			cache_ways = 0;
+			cache_line_size = 0;
+			n = sscanf(ptr, "%dK, %d-way, %db line",
+				&cache_size, &cache_ways, &cache_line_size);
+			if (n != 3)
+				continue;
+			cpu->caches[idx].type = cache_type;
+			cpu->caches[idx].level = cache_level;
+			cpu->caches[idx].size = cache_size * 1024;
+			cpu->caches[idx].ways = cache_ways;
+			cpu->caches[idx].line_size = cache_line_size;
+			idx++;
+		}
+		(void)fclose(fp);
+	}
+
+	if (idx == 0) {
+		free(cpu->caches);
+		cpu->caches = NULL;
+		cpu->cache_count = 0;
+
+		return 0;
+	}
+	cpu->cache_count = idx;
+
+	return idx;
+}
+#endif
+
+#if defined(STRESS_ARCH_SPARC)
 /*
  *  stress_get_cpu_cache_sparc64()
- *	find cache information as provided by linux SPARC64 
+ *	find cache information as provided by linux SPARC64
  *	/sys/devices/system/cpu/cpu0
  */
 static int stress_get_cpu_cache_sparc64(
@@ -450,25 +550,25 @@ static int stress_get_cpu_cache_sparc64(
 	}
 
 	for (i = 0; i < SIZEOF_ARRAY(cache_info); i++) {
-		const size_t index = cache_info[i].index;
+		const size_t idx = cache_info[i].index;
 		uint64_t value;
 
 		if (stress_get_cpu_cache_value(cpu_path, cache_info[i].filename, &value) < 0)
 			continue;
 
-		cpu->caches[index].type = cache_info[i].type;
-		cpu->caches[index].level = cache_info[i].level;
+		cpu->caches[idx].type = cache_info[i].type;
+		cpu->caches[idx].level = cache_info[i].level;
 		switch (cache_info[i].size_type) {
 		case CACHE_SIZE:
-			cpu->caches[index].size = value;
+			cpu->caches[idx].size = value;
 			valid = true;
 			break;
 		case CACHE_LINE_SIZE:
-			cpu->caches[index].line_size = (uint32_t)value;
+			cpu->caches[idx].line_size = (uint32_t)value;
 			valid = true;
 			break;
 		case CACHE_WAYS:
-			cpu->caches[index].size = (uint32_t)value;
+			cpu->caches[idx].size = (uint32_t)value;
 			valid = true;
 			break;
 		default:
@@ -487,6 +587,7 @@ static int stress_get_cpu_cache_sparc64(
 
 	return count;
 }
+#endif
 
 /*
  *  index_filter()
@@ -612,9 +713,16 @@ static void stress_get_cpu_cache_details(stress_cpu_t *cpu, const char *cpu_path
 	if (stress_get_cpu_cache_auxval(cpu) > 0)
 		return;
 
+#if defined(STRESS_ARCH_SPARC)
 	/* Try cache info for sparc CPUs */
 	if (stress_get_cpu_cache_sparc64(cpu, cpu_path) > 0)
 		return;
+#endif
+
+#if defined(STRESS_ARCH_ALPHA)
+	if (stress_get_cpu_cache_alpha(cpu, cpu_path) > 0)
+		return;
+#endif
 
 	return;
 }
