@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+#include "core-ftrace.h"
+
+#if defined(HAVE_SYS_TREE_H)
+#include <sys/tree.h>
+#endif
+
+#if defined(HAVE_SYS_STATFS_H)
+#include <sys/statfs.h>
+#endif
+
+#if defined(HAVE_BSD_SYS_TREE_H)
+#include <bsd/sys/tree.h>
+#endif
 
 #if defined(HAVE_LIB_BSD) &&	\
     defined(__linux__)
@@ -271,7 +280,7 @@ void stress_ftrace_add_pid(const pid_t pid)
 	if (pid == -1) {
 		strcpy(buffer, " ");
 	} else {
-		snprintf(buffer, sizeof(buffer), "%d", (int)pid);
+		snprintf(buffer, sizeof(buffer), "%" PRIdMAX , (intmax_t)pid);
 	}
 	ret = write(fd, buffer, strlen(buffer));
 	(void)ret;
