@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,55 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 #include "git-commit-id.h"
+#include "core-capabilities.h"
+#include "core-hash.h"
+
+#if defined(HAVE_LINUX_FIEMAP_H)
+#include <linux/fiemap.h>
+#endif
+
+#if defined(HAVE_SYS_AUXV_H)
+#include <sys/auxv.h>
+#endif
+
+#if defined(HAVE_SYS_CAPABILITY_H)
+#include <sys/capability.h>
+#endif
+
+#if defined(HAVE_SYS_LOADAVG_H)
+#include <sys/loadavg.h>
+#endif
+
+#if defined(HAVE_SYS_MOUNT_H)
+#include <sys/mount.h>
+#endif
+
+#if defined(HAVE_SYS_PRCTL_H)
+#include <sys/prctl.h>
+#endif
+
+#if defined(HAVE_SYS_STATVFS_H)
+#include <sys/statvfs.h>
+#endif
+
+#if defined(HAVE_SYS_UTSNAME_H)
+#include <sys/utsname.h>
+#endif
+
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#endif
+
+/* prctl(2) timer slack support */
+#if defined(HAVE_SYS_PRCTL_H) && \
+    defined(HAVE_PRCTL) && \
+    defined(PR_SET_TIMERSLACK) && \
+    defined(PR_GET_TIMERSLACK)
+#define HAVE_PRCTL_TIMER_SLACK
+#endif
 
 #if !defined(PR_SET_DISABLE)
 #define SUID_DUMP_DISABLE	(0)       /* No setuid dumping */
@@ -192,15 +233,10 @@ int stress_set_temp_path(const char *path)
 	stress_temp_path_free();
 
 	stress_temp_path = stress_const_optdup(path);
-	if (!stress_temp_path)
-		return -1;
-
-	if (access(path, R_OK | W_OK) < 0) {
-		(void)fprintf(stderr, "aborting: temp-path '%s' must be readable "
-			"and writeable\n", path);
+	if (!stress_temp_path) {
+		(void)fprintf(stderr, "aborting: cannot allocate memory for '%s'\n", path);
 		return -1;
 	}
-
 	return 0;
 }
 
@@ -215,6 +251,22 @@ const char *stress_get_temp_path(void)
 	return stress_temp_path;
 }
 
+/*
+ *  stress_check_temp_path()
+ *	check if temp path is accessible
+ */
+int stress_check_temp_path(void)
+{
+	const char *path = stress_get_temp_path();
+
+	if (access(path, R_OK | W_OK) < 0) {
+		(void)fprintf(stderr, "aborting: temp-path '%s' must be readable "
+			"and writeable\n", path);
+		return -1;
+	}
+	return 0;
+}
+
 /*
  *  stress_mk_filename()
  *	generate a full file name from a path and filename
@@ -256,6 +308,8 @@ size_t stress_get_pagesize(void)
 			return page_size;
 		}
 	}
+#else
+	UNEXPECTED
 #endif
 #if defined(HAVE_GETPAGESIZE)
 	{
@@ -289,6 +343,7 @@ int32_t stress_get_processors_online(void)
 		processors_online = 1;
 #else
 	processors_online = 1;
+	UNEXPECTED
 #endif
 	return processors_online;
 }
@@ -310,6 +365,7 @@ int32_t stress_get_processors_configured(void)
 		processors_configured = stress_get_processors_online();
 #else
 	processors_configured = 1;
+	UNEXPECTED
 #endif
 	return processors_configured;
 }
@@ -329,6 +385,7 @@ int32_t stress_get_ticks_per_second(void)
 	ticks_per_second = (int32_t)sysconf(_SC_CLK_TCK);
 	return ticks_per_second;
 #else
+	UNEXPECTED
 	return -1;
 #endif
 }
@@ -373,6 +430,8 @@ void stress_get_memlimits(
 		return;
 	}
 	(void)fclose(fp);
+#else
+	UNEXPECTED
 #endif
 }
 
@@ -399,6 +458,7 @@ uint64_t stress_get_phys_mem_size(void)
 		phys_pages = max_pages;
 	return phys_pages * page_size;
 #else
+	UNEXPECTED
 	return 0ULL;
 #endif
 }
@@ -433,6 +493,7 @@ uint64_t stress_get_filesystem_size(void)
 
 	return (uint64_t)buf.f_bsize * blocks;
 #else
+	UNEXPECTED
 	return 0ULL;
 #endif
 }
@@ -459,6 +520,7 @@ uint64_t stress_get_filesystem_available_inodes(void)
 
 	return (uint64_t)buf.f_favail;
 #else
+	UNEXPECTED
 	return 0ULL;
 #endif
 }
@@ -476,6 +538,7 @@ int stress_set_nonblock(const int fd)
 		flags = 0;
 	return fcntl(fd, F_SETFL, O_NONBLOCK | flags);
 #else
+	UNEXPECTED
 	flags = 1;
 	return ioctl(fd, FIOBIO, &flags);
 #endif
@@ -490,7 +553,7 @@ int stress_get_load_avg(
 	double *min5,
 	double *min15)
 {
-#if NEED_GLIBC(2, 2, 0) &&	\
+#if defined(HAVE_GETLOADAVG) &&	\
     !defined(__UCLIBC__)
 	int rc;
 	double loadavg[3];
@@ -538,6 +601,8 @@ void stress_parent_died_alarm(void)
     defined(HAVE_SYS_PRCTL_H) &&	\
     defined(PR_SET_PDEATHSIG)
 	(void)prctl(PR_SET_PDEATHSIG, SIGALRM);
+#else
+	UNEXPECTED
 #endif
 }
 
@@ -565,6 +630,8 @@ int stress_process_dumpable(const bool dumpable)
 		lim.rlim_max = 0;
 		(void)setrlimit(RLIMIT_CORE, &lim);
 	}
+#else
+	UNEXPECTED
 #endif
 
 	/*
@@ -603,6 +670,7 @@ int stress_set_timer_slack_ns(const char *opt)
 	timer_slack = stress_get_uint32(opt);
 	(void)stress_set_setting("timer-slack", TYPE_ID_UINT32, &timer_slack);
 #else
+	UNEXPECTED
 	(void)opt;
 #endif
 	return 0;
@@ -621,6 +689,8 @@ void stress_set_timer_slack(void)
 
 	if (stress_get_setting("timer-slack", &timer_slack))
 		(void)prctl(PR_SET_TIMERSLACK, timer_slack);
+#else
+	UNEXPECTED
 #endif
 }
 
@@ -637,6 +707,7 @@ void stress_set_proc_name_init(int argc, char *argv[], char *envp[])
 	(void)argc;
 	(void)argv;
 	(void)envp;
+	UNEXPECTED
 #endif
 }
 
@@ -664,6 +735,25 @@ void stress_set_proc_name(const char *name)
 #endif
 }
 
+/*
+ *  stress_set_proc_state_str
+ *	set process name based on run state string, see
+ *	macros STRESS_STATE_*
+ */
+void stress_set_proc_state_str(const char *name, const char *str)
+{
+	if (g_opt_flags & OPT_FLAGS_KEEP_NAME)
+		return;
+
+#if defined(HAVE_BSD_UNISTD_H) &&	\
+    defined(HAVE_SETPROCTITLE)
+	setproctitle("-%s [%s]", name, str);
+#else
+	(void)name;
+	(void)str;
+#endif
+}
+
 /*
  *  stress_set_proc_state
  *	set process name based on run state, see
@@ -681,18 +771,10 @@ void stress_set_proc_state(const char *name, const int state)
 		"wait"
 	};
 
-	(void)name;
-
-	if (g_opt_flags & OPT_FLAGS_KEEP_NAME)
-		return;
-
 	if ((state < 0) || (state >= (int)SIZEOF_ARRAY(stress_states)))
 		return;
 
-#if defined(HAVE_BSD_UNISTD_H) &&	\
-    defined(HAVE_SETPROCTITLE)
-	setproctitle("-%s [%s]", name, stress_states[state]);
-#endif
+	stress_set_proc_state_str(name, stress_states[state]);
 }
 
 /*
@@ -913,7 +995,7 @@ int stress_temp_dir_mk(
 		ret = -errno;
 		pr_fail("%s: mkdir '%s' failed, errno=%d (%s)\n",
 			name, tmp, errno, strerror(errno));
-		(void)unlink(tmp);
+		(void)shim_rmdir(tmp);
 	}
 
 	return ret;
@@ -941,7 +1023,7 @@ int stress_temp_dir_rm(
 	char tmp[PATH_MAX + 1];
 
 	stress_temp_dir(tmp, sizeof(tmp), name, pid, instance);
-	ret = rmdir(tmp);
+	ret = shim_rmdir(tmp);
 	if (ret < 0) {
 		ret = -errno;
 		pr_fail("%s: rmdir '%s' failed, errno=%d (%s)\n",
@@ -1025,6 +1107,29 @@ void stress_strnrnd(char *str, const size_t len)
 	*str = '\0';
 }
 
+/*
+ *  stress_uint8rnd4()
+ *	fill a uint8_t buffer full of random data
+ *	buffer *must* be multiple of 4 bytes in size
+ */
+void stress_uint8rnd4(uint8_t *data, const size_t len)
+{
+	register size_t i = len >> 2;
+	register uint8_t *ptr = data;
+
+	while (i--) {
+		register uint32_t v = stress_mwc32();
+
+		*ptr++ = (uint8_t)v;
+		v >>= 8;
+		*ptr++ = (uint8_t)v;
+		v >>= 8;
+		*ptr++ = (uint8_t)v;
+		v >>= 8;
+		*ptr++ = (uint8_t)v;
+	}
+}
+
 /*
  *  pr_run_info()
  *	short info about the system we are running stress-ng on
@@ -1319,14 +1424,18 @@ bool stress_is_prime64(const uint64_t n)
 /*
  *  stress_get_prime64()
  *	find a prime that is not a multiple of n,
- *	used for file name striding
+ *	used for file name striding. Minimum is 1009,
+ *	max is unbounded. Return a prime > n, each
+ *	call will return the next prime to keep the
+ *	primes different each call.
  */
 uint64_t stress_get_prime64(const uint64_t n)
 {
-	static uint p = 1009;
+	static uint64_t p = 1009;
+	uint64_t odd_n = (n & ~1) + 1;
 
-	if (n != p)
-		return p;
+	if (p < odd_n)
+		p = odd_n;
 
 	/* Search for next prime.. */
 	for (;;) {
@@ -1363,6 +1472,7 @@ size_t stress_get_max_file_limit(void)
 	}
 #else
 	max_sysconf = SIZE_MAX;
+	UNEXPECTED
 #endif
 	/* return the lowest of these two */
 	return STRESS_MINIMUM(max_rlim, max_sysconf);
@@ -1431,10 +1541,36 @@ int stress_get_bad_fd(void)
 		if (fcntl((int)i, F_GETFL) == -1)
 			return i;
 	}
+#else
+	UNEXPECTED
 #endif
 	return -1;
 }
 
+/*
+ *  stress_sigaltstack_no_check()
+ *	attempt to set up an alternative signal stack with no
+ *	minimum size check on stack
+ *	  stack - must be at least MINSIGSTKSZ
+ *	  size  - size of stack (- STACK_ALIGNMENT)
+ */
+int stress_sigaltstack_no_check(void *stack, const size_t size)
+{
+#if defined(HAVE_SIGALTSTACK)
+	stack_t ss;
+
+	ss.ss_sp = (void *)stack;
+	ss.ss_size = size;
+	ss.ss_flags = 0;
+	return sigaltstack(&ss, NULL);
+#else
+	UNEXPECTED
+	(void)stack;
+	(void)size;
+	return 0;
+#endif
+}
+
 /*
  *  stress_sigaltstack()
  *	attempt to set up an alternative signal stack
@@ -1444,22 +1580,19 @@ int stress_get_bad_fd(void)
 int stress_sigaltstack(void *stack, const size_t size)
 {
 #if defined(HAVE_SIGALTSTACK)
-	stack_t ss;
-
 	if (size < (size_t)STRESS_MINSIGSTKSZ) {
 		pr_err("sigaltstack stack size %zu must be more than %zuK\n",
 			size, (size_t)STRESS_MINSIGSTKSZ / 1024);
 		return -1;
 	}
-	ss.ss_sp = (void *)stack;
-	ss.ss_size = size;
-	ss.ss_flags = 0;
-	if (sigaltstack(&ss, NULL) < 0) {
+
+	if (stress_sigaltstack_no_check(stack, size) < 0) {
 		pr_fail("sigaltstack failed: errno=%d (%s)\n",
 			errno, strerror(errno));
 		return -1;
 	}
 #else
+	UNEXPECTED
 	(void)stack;
 	(void)size;
 #endif
@@ -1593,10 +1726,15 @@ unsigned int stress_get_cpu(void)
  */
 const char *stress_get_compiler(void)
 {
-#if defined(__clang_major__) &&	\
-    defined(__clang_minor__)
+#if defined(__TINYC__)
+	static const char cc[] = "tcc" XSTRINGIFY(__TINYC__) "";
+#elif defined(__PCC__) &&			\
+       defined(__PCC_MINOR__)
+	static const char cc[] = "pcc" XSTRINGIFY(__PCC__) "." XSTRINGIFY(__PCC_MINOR__) "." XSTRINGIFY(__PCC_MINORMINOR__) "";
+#elif defined(__clang_major__) &&	\
+      defined(__clang_minor__)
 	static const char cc[] = "clang " XSTRINGIFY(__clang_major__) "." XSTRINGIFY(__clang_minor__) "";
-#elif defined(__GNUC__) &&	\
+#elif defined(__GNUC__) &&		\
       defined(__GNUC_MINOR__)
 	static const char cc[] = "gcc " XSTRINGIFY(__GNUC__) "." XSTRINGIFY(__GNUC_MINOR__) "";
 #else
@@ -1623,6 +1761,8 @@ const char *stress_get_uname_info(void)
 		(void)snprintf(str, sizeof(str), "%s %s %s", buf.machine, buf.sysname, buf.release);
 		return str;
 	}
+#else
+	UNEXPECTED
 #endif
 	return "unknown";
 }
@@ -1805,7 +1945,7 @@ char *stress_uint64_to_str(char *str, size_t len, const uint64_t val)
  *  stress_check_root()
  *	returns true if root
  */
-static bool stress_check_root(void)
+static inline bool stress_check_root(void)
 {
 	return (geteuid() == 0);
 }
@@ -2000,6 +2140,7 @@ bool stress_is_dev_tty(const int fd)
 		return true;
 	return !strncmp("/dev/tty", name, 8);
 #else
+	UNEXPECTED
 	(void)fd;
 
 	/* Assume it is */
@@ -2242,8 +2383,6 @@ ignore_eintr:
 			rbytes += ret;
 	} while (ret > 0 && (rbytes != size));
 
-	pr_dbg_v("stress_read_buffer: size=%ld read=%ld sz2=%ld\n", size, rbytes, ret);
-
 	return (ret <= 0)? ret : rbytes;
 }
 
@@ -2268,8 +2407,6 @@ ignore_eintr:
 			wbytes += ret;
 	} while (ret > 0 && (wbytes != size));
 
-	pr_dbg_v("stress_write_buffer: size=%ld written=%ld sz2=%ld\n", size, wbytes, ret);
-
 	return (ret <= 0)? ret : wbytes;
 }
 
@@ -2301,6 +2438,7 @@ int stress_get_kernel_release(void)
 
 	return stress_kernel_release(major, minor, patchlevel);
 #else
+	UNEXPECTED
 	return -1;
 #endif
 }
@@ -2419,12 +2557,15 @@ size_t stress_hostname_length(void)
  */
 static inline long stress_min_aux_sig_stack_size(void)
 {
-#if defined(HAVE_GETAUXVAL) &&	\
+#if defined(HAVE_SYS_AUXV_H) && \
+    defined(HAVE_GETAUXVAL) &&	\
     defined(AT_MINSIGSTKSZ)
 	long sz = getauxval(AT_MINSIGSTKSZ);
 
 	if (sz > 0)
 		return sz;
+#else
+	UNEXPECTED
 #endif
 	return -1;
 }
@@ -2556,8 +2697,8 @@ extern void __stack_chk_fail(void);
 NORETURN WEAK void __stack_chk_fail(void)
 {
 	if (stress_stack_check_flag) {
-		fprintf(stderr, "Stack overflow detected! Aborting stress-ng.\n");
-		fflush(stderr);
+		(void)fprintf(stderr, "Stack overflow detected! Aborting stress-ng.\n");
+		(void)fflush(stderr);
 		abort();
 	}
 	/* silently exit */
@@ -2590,6 +2731,7 @@ int stress_tty_width(void)
 		return max_width;
 	return ret;
 #else
+	UNEXPECTED
 	return max_width;
 #endif
 }
@@ -2613,6 +2755,7 @@ size_t stress_get_extents(const int fd)
 
 	return fiemap.fm_mapped_extents;
 #else
+	UNEXPECTED
 	(void)fd;
 
 	return 0;
@@ -2635,3 +2778,25 @@ bool stress_redo_fork(const int err)
 	}
 	return false;
 }
+
+/*
+ *  stress_sighandler_nop()
+ *	no-operation signal handler
+ */
+void stress_sighandler_nop(int sig)
+{
+	(void)sig;
+}
+
+/*
+ *  stress_clear_warn_once()
+ *	clear the linux warn once warnings flag, kernel warn once
+ *	messages can be re-issued
+ */
+void stress_clear_warn_once(void)
+{
+#if defined(__linux__)
+	if (stress_check_capability(SHIM_CAP_IS_ROOT))
+		(void)system_write("/sys/kernel/debug/clear_warn_once", "1", 1);
+#endif
+}
