@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
 
 typedef struct {
 	const char *path;		/* Path of /sys control */
@@ -44,13 +40,14 @@ static stress_cpu_setting_t *cpu_settings; /* Array of cpu settings */
 
 static pid_t pid;			/* PID of ignite process */
 static bool enabled;			/* true if ignite process running */
-static uint32_t max_cpus;		/* max cpus configured */
+static int32_t max_cpus;		/* max cpus configured */
 
 #define SETTING(path, default_setting)	\
 	{ path, default_setting, 0, NULL, 0, false }
 
 static stress_settings_t settings[] = {
-#if defined(__linux__) && defined(STRESS_ARCH_X86)
+#if defined(__linux__) &&	\
+    defined(STRESS_ARCH_X86)
 	/* x86 Intel P-State maximizing settings */
 	SETTING("/sys/devices/system/cpu/intel_pstate/max_perf_pct", "100"),
 	SETTING("/sys/devices/system/cpu/intel_pstate/no_turbo", "0"),
@@ -59,18 +56,18 @@ static stress_settings_t settings[] = {
 };
 
 static int stress_ignite_cpu_set(
-	const uint32_t cpu,
+	const int32_t cpu,
 	const uint64_t freq,
 	const char *governor)
 {
 	char path[PATH_MAX];
-	int ret1 = 0, ret2 = 0;
+	ssize_t ret1 = 0, ret2 = 0;
 
 	if (freq > 0) {
 		char buffer[128];
 
 		(void)snprintf(path, sizeof(path),
-			"/sys/devices/system/cpu/cpu%" PRIu32
+			"/sys/devices/system/cpu/cpu%" PRId32
 			"/cpufreq/scaling_setspeed", cpu);
 		(void)snprintf(buffer, sizeof(buffer), "%" PRIu64 "\n", freq);
 		ret1 = system_write(path, buffer, strlen(buffer));
@@ -105,7 +102,7 @@ void stress_ignite_cpu_start(void)
 	if (!cpu_settings) {
 		pr_dbg("ignite-cpu: no cpu settings allocated\n");
 	} else {
-		uint32_t cpu;
+		int32_t cpu;
 		char buffer[128];
 		char path[PATH_MAX];
 
@@ -113,7 +110,7 @@ void stress_ignite_cpu_start(void)
 		 *  Gather per-cpu max scaling frequencies and governors
 		 */
 		for (cpu = 0; cpu < max_cpus; cpu++) {
-			int ret;
+			ssize_t ret;
 
 			/* Assume failed */
 			cpu_settings[cpu].max_freq = 0;
@@ -157,7 +154,7 @@ void stress_ignite_cpu_start(void)
 	pid = -1;
 	for (i = 0; settings[i].path; i++) {
 		char buf[4096];
-		int ret;
+		ssize_t ret;
 		size_t len;
 
 		settings[i].ignore = true;
@@ -177,9 +174,9 @@ void stress_ignite_cpu_start(void)
 			settings[i].default_setting_len);
 		if (ret < 0) {
 			pr_dbg("ignite-cpu: cannot set %s to %s, "
-				"errno=%d (%s)\n",
+				"errno=%zd (%s)\n",
 				settings[i].path, settings[i].default_setting,
-				-ret, strerror(-ret));
+				-ret, strerror((int)-ret));
 			continue;
 		}
 
@@ -220,7 +217,7 @@ void stress_ignite_cpu_start(void)
 			}
 
 			if (cpu_settings) {
-				uint32_t cpu;
+				int32_t cpu;
 
 				/*
 				 *  Attempt to crank CPUs up to max freq
@@ -263,7 +260,7 @@ void stress_ignite_cpu_stop(void)
 	}
 
 	if (cpu_settings) {
-		uint32_t cpu;
+		int32_t cpu;
 
 		for (cpu = 0; cpu < max_cpus; cpu++) {
 			stress_ignite_cpu_set(cpu,
