@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +16,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYSLOG_H)
+#include <syslog.h>
+#endif
+
 static uint16_t	abort_fails;	/* count of failures */
 static bool	abort_msg_emitted;
 static FILE	*log_file = NULL;
 
+static inline FILE *pr_file(void)
+{
+	return (g_opt_flags & OPT_FLAGS_STDOUT) ? stdout : stderr;
+}
+
 /*
  *  pr_lock()
  *	attempt to get a lock, it's just too bad
@@ -43,15 +47,13 @@ void pr_lock(bool *lock)
 
 	*lock = false;
 
-	fd = open("/dev/stdout", O_RDONLY);
+	fd = fileno(pr_file());
 	if (fd < 0)
 		return;
 
 	ret = flock(fd, LOCK_EX);
 	if (ret == 0)
 		*lock = true;
-
-	(void)close(fd);
 #else
 	(void)lock;
 #endif
@@ -71,15 +73,13 @@ void pr_unlock(bool *lock)
 	if (!*lock)
 		return;
 
-	fd = open("/dev/stdout", O_RDONLY);
+	fd = fileno(pr_file());
 	if (fd < 0)
 		return;
 
 	ret = flock(fd, LOCK_UN);
 	if (ret == 0)
 		*lock = false;
-
-	(void)close(fd);
 #else
 	(void)lock;
 #endif
@@ -247,10 +247,9 @@ static int pr_msg_lockable(
 		    (!(flag & PR_DEBUG))) {
 			syslog(LOG_INFO, "%s", buf);
 		}
-
+#endif
 		if (!locked)
 			pr_unlock(&lock);
-#endif
 	}
 	return ret;
 }
@@ -264,7 +263,7 @@ void pr_dbg(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg_lockable(stderr, PR_DEBUG, false, fmt, ap);
+	(void)pr_msg_lockable(pr_file(), PR_DEBUG, false, fmt, ap);
 	va_end(ap);
 }
 
@@ -278,7 +277,7 @@ void pr_dbg_skip(const char *fmt, ...)
 
 	va_start(ap, fmt);
 	if (!(g_opt_flags & OPT_FLAGS_SKIP_SILENT))
-		(void)pr_msg_lockable(stderr, PR_DEBUG, false, fmt, ap);
+		(void)pr_msg_lockable(pr_file(), PR_DEBUG, false, fmt, ap);
 	va_end(ap);
 }
 
@@ -294,7 +293,7 @@ void pr_dbg_lock(bool *lock, const char *fmt, ...)
 	(void)lock;
 
 	va_start(ap, fmt);
-	(void)pr_msg_lockable(stderr, PR_DEBUG, true, fmt, ap);
+	(void)pr_msg_lockable(pr_file(), PR_DEBUG, true, fmt, ap);
 	va_end(ap);
 }
 
@@ -307,7 +306,7 @@ void pr_inf(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg_lockable(stderr, PR_INFO, false, fmt, ap);
+	(void)pr_msg_lockable(pr_file(), PR_INFO, false, fmt, ap);
 	va_end(ap);
 }
 
@@ -321,7 +320,7 @@ void pr_inf_skip(const char *fmt, ...)
 
 	va_start(ap, fmt);
 	if (!(g_opt_flags & OPT_FLAGS_SKIP_SILENT))
-		(void)pr_msg_lockable(stderr, PR_INFO, false, fmt, ap);
+		(void)pr_msg_lockable(pr_file(), PR_INFO, false, fmt, ap);
 	va_end(ap);
 }
 
@@ -337,7 +336,7 @@ void pr_inf_lock(bool *lock, const char *fmt, ...)
 	(void)lock;
 
 	va_start(ap, fmt);
-	(void)pr_msg_lockable(stderr, PR_INFO, true, fmt, ap);
+	(void)pr_msg_lockable(pr_file(), PR_INFO, true, fmt, ap);
 	va_end(ap);
 }
 
@@ -350,7 +349,7 @@ void pr_err(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg_lockable(stderr, PR_ERROR, false, fmt, ap);
+	(void)pr_msg_lockable(pr_file(), PR_ERROR, false, fmt, ap);
 	va_end(ap);
 }
 
@@ -364,7 +363,7 @@ void pr_err_skip(const char *fmt, ...)
 
 	va_start(ap, fmt);
 	if (!(g_opt_flags & OPT_FLAGS_SKIP_SILENT))
-		(void)pr_msg_lockable(stderr, PR_ERROR, false, fmt, ap);
+		(void)pr_msg_lockable(pr_file(), PR_ERROR, false, fmt, ap);
 	va_end(ap);
 }
 
@@ -377,7 +376,7 @@ void pr_fail(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg_lockable(stderr, PR_FAIL, false, fmt, ap);
+	(void)pr_msg_lockable(pr_file(), PR_FAIL, false, fmt, ap);
 	va_end(ap);
 }
 
@@ -390,6 +389,6 @@ void pr_tidy(const char *fmt, ...)
 	va_list ap;
 
 	va_start(ap, fmt);
-	(void)pr_msg_lockable(stderr, g_caught_sigint ? PR_INFO : PR_DEBUG, true, fmt, ap);
+	(void)pr_msg_lockable(pr_file(), g_caught_sigint ? PR_INFO : PR_DEBUG, true, fmt, ap);
 	va_end(ap);
 }
