@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -88,6 +83,14 @@ static const int madvise_options[] = {
 	MADV_FREE
 #endif
 */
+/* Linux 5.14 */
+#if defined(MADV_POPULATE_READ)
+	MADV_POPULATE_READ,
+#endif
+/* Linux 5.14 */
+#if defined(MADV_POPULATE_WRITE)
+	MADV_POPULATE_WRITE,
+#endif
 };
 #endif
 
@@ -95,7 +98,7 @@ static const int madvise_options[] = {
  *  stress_madvise_random()
  *	apply random madvise setting to a memory region
  */
-int  stress_madvise_random(void *addr, const size_t length)
+int stress_madvise_random(void *addr, const size_t length)
 {
 #if defined(HAVE_MADVISE)
 	if (g_opt_flags & OPT_FLAGS_MMAP_MADVISE) {
@@ -104,8 +107,65 @@ int  stress_madvise_random(void *addr, const size_t length)
 		return madvise(addr, length, madvise_options[i]);
 	}
 #else
+	UNEXPECTED
 	(void)addr;
 	(void)length;
 #endif
 	return 0;
 }
+
+/*
+ *  stress_madvise_pid_all_pages()
+ *	apply madvise advise to all pages in a progress
+ */
+void stress_madvise_pid_all_pages(const pid_t pid, const int advise)
+{
+#if defined(HAVE_MADVISE) &&	\
+    defined(__linux__)
+	FILE *fp;
+	char path[PATH_MAX];
+	char buf[4096];
+
+	(void)snprintf(path, sizeof(path), "/proc/%" PRIdMAX "/maps", (intmax_t)pid);
+
+	fp = fopen(path, "r");
+	if (!fp)
+		return;
+	while (fgets(buf, sizeof(buf), fp)) {
+		void *start, *end, *offset;
+		int major, minor, n, ret;
+		uint64_t inode;
+		char prot[5];
+
+		n = sscanf(buf, "%p-%p %4s %p %x:%x %" PRIu64 " %s\n",
+			&start, &end, prot, &offset, &major, &minor,
+			&inode, path);
+		if (n < 7)
+			continue;	/* bad sscanf data */
+		if (start >= end)
+			continue;	/* invalid address range */
+
+		ret = madvise(start, (size_t)((uint8_t *)end - (uint8_t *)start), advise);
+		(void)ret;
+
+		/*
+		 *  Readable protection? read pages
+		 */
+		if ((prot[0] == 'r') && (path[0] != '[')) {
+			const size_t page_size = stress_get_pagesize();
+
+			volatile uint8_t *ptr = (volatile uint8_t *)start;
+
+			while (ptr < (uint8_t *)end) {
+				(*ptr);
+				ptr += page_size;
+			}
+		}
+	}
+
+	(void)fclose(fp);
+#else
+	(void)pid;
+	(void)advise;
+#endif
+}
