@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,27 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_AUXV_H)
+#include <sys/auxv.h>
+#endif
+
+#if defined(HAVE_UTIME_H)
+#include <utime.h>
+#endif
+
+/* MWC random number initial seed */
+#define STRESS_MWC_SEED_Z	(362436069UL)
+#define STRESS_MWC_SEED_W	(521288629UL)
+
+/* Fast random number generator state */
+typedef struct {
+	uint32_t w;
+	uint32_t z;
+} stress_mwc_t;
+
 static stress_mwc_t mwc = {
 	STRESS_MWC_SEED_W,
 	STRESS_MWC_SEED_Z
@@ -126,10 +139,10 @@ void stress_mwc_reseed(void)
 }
 
 /*
- *  stress_mwc_seed()
+ *  stress_mwc_set_seed()
  *      set mwc seeds
  */
-void stress_mwc_seed(const uint32_t w, const uint32_t z)
+void stress_mwc_set_seed(const uint32_t w, const uint32_t z)
 {
 	mwc.w = w;
 	mwc.z = z;
@@ -137,6 +150,25 @@ void stress_mwc_seed(const uint32_t w, const uint32_t z)
 	mwc_flush();
 }
 
+/*
+ *  stress_mwc_get_seed()
+ *      get mwc seeds
+ */
+void stress_mwc_get_seed(uint32_t *w, uint32_t *z)
+{
+	*w = mwc.w;
+	*z = mwc.z;
+}
+
+/*
+ *  stress_mwc_seed()
+ *      set default mwc seed
+ */
+void stress_mwc_seed(void)
+{
+	stress_mwc_set_seed(STRESS_MWC_SEED_W, STRESS_MWC_SEED_Z);
+}
+
 /*
  *  stress_mwc32()
  *      Multiply-with-carry random numbers
