@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
+
+#if defined(HAVE_SYS_UN_H)
+#include <sys/un.h>
+#else
+UNEXPECTED
+#endif
+
+#include <netinet/in.h>
 
 typedef struct {
 	const char *name;
@@ -47,9 +51,13 @@ void stress_set_net_port(
 	const int max_port,
 	int *port)
 {
-	*port = stress_get_uint64(opt);
-	stress_check_range(optname, *port,
-		min_port, max_port - STRESS_PROCS_MAX);
+	const uint64_t val = stress_get_uint64(opt);
+
+	stress_check_range(optname, val,
+		(uint64_t)min_port,
+		(uint64_t)(max_port - STRESS_PROCS_MAX));
+
+	*port = (int)val;
 }
 
 /*
@@ -95,13 +103,19 @@ void stress_set_sockaddr(
 {
 	(void)ppid;
 
+	uint16_t sin_port = (uint16_t)port + (uint16_t)instance;
+
+	/* Handle overflow to omit ports 0..1023 */
+	if ((port > 1024) && (sin_port < 1024))
+		sin_port += 1024;
+
 	switch (domain) {
 #if defined(AF_INET)
 	case AF_INET: {
 		static struct sockaddr_in addr;
 
 		(void)memset(&addr, 0, sizeof(addr));
-		addr.sin_family = domain;
+		addr.sin_family = (sa_family_t)domain;
 		switch (net_addr) {
 		case NET_ADDR_LOOPBACK:
 			addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
@@ -111,7 +125,7 @@ void stress_set_sockaddr(
 			addr.sin_addr.s_addr = htonl(INADDR_ANY);
 			break;
 		}
-		addr.sin_port = htons(port + instance);
+		addr.sin_port = htons(sin_port);
 		*sockaddr = (struct sockaddr *)&addr;
 		*len = sizeof(addr);
 		break;
@@ -125,7 +139,7 @@ void stress_set_sockaddr(
 		static const struct in6_addr in6addr_loopback = IN6ADDR_LOOPBACK_INIT;
 #endif
 		(void)memset(&addr, 0, sizeof(addr));
-		addr.sin6_family = domain;
+		addr.sin6_family = (sa_family_t)domain;
 		switch (net_addr) {
 		case NET_ADDR_LOOPBACK:
 			addr.sin6_addr = in6addr_loopback;
@@ -135,7 +149,7 @@ void stress_set_sockaddr(
 			addr.sin6_addr = in6addr_any;
 			break;
 		}
-		addr.sin6_port = htons(port + instance);
+		addr.sin6_port = htons(sin_port);
 		*sockaddr = (struct sockaddr *)&addr;
 		*len = sizeof(addr);
 		break;
@@ -183,6 +197,7 @@ void HOT stress_set_sockaddr_port(
 #if defined(AF_INET6)
 	case AF_INET6: {
 		struct sockaddr_in6 *addr = (struct sockaddr_in6 *)sockaddr;
+
 		addr->sin6_port = htons(port);
 		break;
 	}
