@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -87,6 +82,8 @@ static int stress_set_adjustment(const char *procname, const char *name, const c
 	const size_t len = strlen(str);
 	int i, saved_errno = 0;
 
+	/* pr_dbg("%s: %s -> %s\n", name, procname, str); */
+
 	for (i = 0; i < 32; i++) {
 		ssize_t n;
 		int fd;
@@ -128,13 +125,19 @@ void stress_set_oom_adjustment(const char *name, const bool killable)
 	char *str;
 	int ret;
 
+	/*
+	 *  --no-oom-adjust option ignores any oom adjustments
+	 */
+	if (g_opt_flags & OPT_FLAGS_NO_OOM_ADJUST)
+		return;
+
 	high_priv = (getuid() == 0) && (geteuid() == 0);
 
 	/*
 	 *  main cannot be killable; if OPT_FLAGS_OOMABLE set make
 	 *  all child procs easily OOMable
 	 */
-	if (!strcmp(name, "main") && (g_opt_flags & OPT_FLAGS_OOMABLE))
+	if (strcmp(name, "main") && (g_opt_flags & OPT_FLAGS_OOMABLE))
 		make_killable = true;
 
 	/*
@@ -154,9 +157,9 @@ void stress_set_oom_adjustment(const char *name, const bool killable)
 	 *  Fall back to old oom interface if we got -ENOENT
 	 */
 	if (make_killable)
-		str = high_priv ? OOM_ADJ_NO_OOM : OOM_ADJ_MIN;
-	else
 		str = OOM_ADJ_MAX;
+	else
+		str = high_priv ? OOM_ADJ_NO_OOM : OOM_ADJ_MIN;
 
 	(void)stress_set_adjustment("/proc/self/oom_adj", name, str);
 }
