@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,25 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
 /*
- *  stress_check_value()
- *	sanity check number of workers
+ *  stress_check_max_stressors()
+ *	sanity check number of stressors
  */
-void stress_check_value(
+void stress_check_max_stressors(
 	const char *const msg,
 	const int val)
 {
 	if ((val < 0) || (val > STRESS_PROCS_MAX)) {
-		(void)fprintf(stderr, "Number of %s workers must be between "
+		(void)fprintf(stderr, "Number of %s stressors must be between "
 			"0 and %d\n", msg, STRESS_PROCS_MAX);
 		longjmp(g_error_env, 1);
 	}
@@ -268,7 +263,7 @@ uint64_t stress_get_uint64_percent(
 
 	/* Convert to % over N instances */
 	if ((len > 1) && (str[len - 1] == '%')) {
-		double val;
+		double val, percent;
 
 		/* Avoid division by zero */
 		if (max == 0) {
@@ -289,7 +284,8 @@ uint64_t stress_get_uint64_percent(
 			(void)fprintf(stderr, "Invalid percentage %s\n", str);
 			longjmp(g_error_env, 1);
 		}
-		return (uint64_t)((double)(max * val) / (100.0 * instances));
+		percent = ((double)max * (double)val) / (100.0 * (double)instances);
+		return (uint64_t)percent;
 	}
 	return stress_get_uint64_byte(str);
 }
@@ -297,7 +293,7 @@ uint64_t stress_get_uint64_percent(
 /*
  *  stress_get_uint64_byte_memory()
  *	get memory size from string. If it contains %
- *	at the end, then covert it into the available
+ *	at the end, then convert it into the available
  *	physical memory scaled by that percentage divided
  *	by the number of stressor instances
  */
@@ -314,7 +310,7 @@ uint64_t stress_get_uint64_byte_memory(
 /*
  *  stress_get_uint64_byte_filesystem()
  *	get file size from string. If it contains %
- *	at the end, then covert it into the available
+ *	at the end, then convert it into the available
  *	file system space scaled by that percentage divided
  *	by the number of stressor instances
  */
