@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-perf.h"
 #include "core-perf-event.h"
 
+#if defined(HAVE_LINUX_PERF_EVENT_H)
+#include <linux/perf_event.h>
+#endif
+
+#if defined(HAVE_LOCALE_H)
+#include <locale.h>
+#endif
+
 #if defined(STRESS_PERF_STATS) &&	\
     defined(HAVE_LINUX_PERF_EVENT_H)
 /* perf enabled systems */
@@ -224,8 +228,11 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 #if STRESS_PERF_DEFINED(SW_CONTEXT_SWITCHES)
 	PERF_INFO_SW(SW_CONTEXT_SWITCHES,	"Context Switches"),
 #endif
+#if STRESS_PERF_DEFINED(SW_CGROUP_SWITCHES)
+	PERF_INFO_SW(SW_CGROUP_SWITCHES,	"Cgroup Switches"),
+#endif
 #if STRESS_PERF_DEFINED(SW_CPU_MIGRATIONS)
-	PERF_INFO_SW(SW_CPU_MIGRATIONS,	"CPU Migrations"),
+	PERF_INFO_SW(SW_CPU_MIGRATIONS,		"CPU Migrations"),
 #endif
 #if STRESS_PERF_DEFINED(SW_ALIGNMENT_FAULTS)
 	PERF_INFO_SW(SW_ALIGNMENT_FAULTS,	"Alignment Faults"),
@@ -273,6 +280,7 @@ static stress_perf_info_t perf_info[STRESS_PERF_MAX] = {
 	PERF_INFO_TP("irq/irq_handler_exit",		"IRQ Exit"),
 	PERF_INFO_TP("irq/softirq_entry",		"Soft IRQ Entry"),
 	PERF_INFO_TP("irq/softirq_exit",		"Soft IRQ Exit"),
+	PERF_INFO_TP("nmi/nmi_handler",			"NMI handler"),
 
 	PERF_INFO_TP("writeback/writeback_dirty_inode",	"Writeback Dirty Inode"),
 	PERF_INFO_TP("writeback/writeback_dirty_page",	"Writeback Dirty Page"),
