@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#ifndef SCHED_FLAG_RESET_ON_FORK
+#define SCHED_FLAG_RESET_ON_FORK        (0x01)
+#endif
+
+#ifndef SCHED_FLAG_RECLAIM
+#define SCHED_FLAG_RECLAIM              (0x02)
+#endif
+
+#ifndef SCHED_FLAG_DL_OVERRUN
+#define SCHED_FLAG_DL_OVERRUN           (0x04)
+#endif
+
 typedef struct {
 	const int sched;
 	const char *const sched_name;
@@ -70,7 +77,7 @@ const char *stress_get_sched_name(const int sched)
     !defined(__minix__) &&						\
     !defined(__APPLE__)
 
-static const char *prefix = "sched";
+static const char prefix[] = "sched";
 
 #if defined(SCHED_DEADLINE) &&	\
     defined(__linux__)
