@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,65 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
+#define STRESS_CORE_SHIM
+
 #include "stress-ng.h"
+#include "core-pragma.h"
+#include "core-arch.h"
+
+#if defined(__NR_pkey_get)
+#define HAVE_PKEY_GET
+#endif
+
+#if defined(__NR_pkey_set)
+#define HAVE_PKEY_SET
+#endif
+
+#if defined(HAVE_USTAT_H)
+#if defined(__sun__)
+/* ustat and long file support on sun does not build */
+#undef HAVE_USTAT_H
+#else
+#include <ustat.h>
+#endif
+#endif
+
+#if defined(HAVE_LINUX_FUTEX_H)
+#include <linux/futex.h>
+#endif
+
+#if defined(HAVE_ASM_CACHECTL_H)
+#include <asm/cachectl.h>
+#endif
+
+#if defined(HAVE_MODIFY_LDT)
+#include <asm/ldt.h>
+#endif
+
+#if defined(HAVE_SYS_MOUNT_H)
+#include <sys/mount.h>
+#endif
+
+#if defined(HAVE_SYS_PRCTL_H)
+#include <sys/prctl.h>
+#endif
+
+#if defined(HAVE_SYS_RANDOM_H)
+#include <sys/random.h>
+#endif
+
+#if defined(HAVE_SYS_XATTR_H)
+#include <sys/xattr.h>
+#undef HAVE_ATTR_XATTR_H
+#elif defined(HAVE_ATTR_XATTR_H)
+#include <attr/xattr.h>
+#endif
+/*  Sanity check */
+#if defined(HAVE_SYS_XATTR_H) &&        \
+    defined(HAVE_ATTR_XATTR_H)
+#error cannot have both HAVE_SYS_XATTR_H and HAVE_ATTR_XATTR_H
+#endif
 
 /*
  *  Various shim abstraction wrappers around systems calls and
@@ -54,6 +106,7 @@ int shim_sched_yield(void)
 #elif defined(__NR_sched_yield)
 	return syscall(__NR_sched_yield);
 #else
+	UNEXPECTED
 	return sleep(0);
 #endif
 }
@@ -65,7 +118,20 @@ int shim_sched_yield(void)
  */
 int shim_cacheflush(char *addr, int nbytes, int cache)
 {
-#if defined(__NR_cacheflush)
+#if defined(HAVE_ASM_CACHECTL_H) &&	\
+    defined(HAVE_CACHEFLUSH) && 	\
+    defined(STRESS_ARCH_MIPS)
+	extern int cacheflush(void *addr, int nbytes, int cache);
+
+	return cacheflush((void *)addr, nbytes, cache);
+#elif defined(HAVE_BUILTIN___CLEAR_CACHE)
+	/* More portable builtin */
+	(void)cache;
+
+	__builtin___clear_cache((void *)addr, (void *)(addr + nbytes));
+	return 0;
+#elif defined(__NR_cacheflush)
+	/* potentially incorrect args, needs per-arch fixing */
 	return (int)syscall(__NR_cacheflush, addr, nbytes, cache);
 #else
 	return (int)shim_enosys(0, addr, nbytes, cache);
@@ -194,9 +260,12 @@ int shim_fallocate(int fd, int mode, off_t offset, off_t len)
  */
 int shim_gettid(void)
 {
-#if defined(__NR_gettid)
+#if defined(HAVE_GETTID)
+	return gettid();
+#elif defined(__NR_gettid)
 	return (int)syscall(__NR_gettid);
 #else
+	UNEXPECTED
 	return (int)shim_enosys(0);
 #endif
 }
@@ -211,9 +280,13 @@ long shim_getcpu(
 	unsigned *node,
 	void *tcache)
 {
-#if defined(__NR_getcpu)
+#if defined(HAVE_GETCPU) && !defined(STRESS_ARCH_S390)
+	(void)tcache;
+	return (long)getcpu(cpu, node);
+#elif defined(__NR_getcpu)
 	return (long)syscall(__NR_getcpu, cpu, node, tcache);
 #else
+	UNEXPECTED
 	return (long)shim_enosys(0, cpu, node, tcache);
 #endif
 }
@@ -256,13 +329,17 @@ int shim_getdents64(
  */
 int shim_getrandom(void *buff, size_t buflen, unsigned int flags)
 {
-#if defined(__NR_getrandom)
+#if defined(HAVE_SYS_RANDOM_H) &&	\
+    defined(HAVE_GETRANDOM)
+	return (int)getrandom(buff, buflen, flags);
+#elif defined(__NR_getrandom)
 	return (int)syscall(__NR_getrandom, buff, buflen, flags);
 #elif defined(__OpenBSD__) || defined(__APPLE__)
 	(void)flags;
 
 	return getentropy(buff, buflen);
 #else
+	UNEXPECTED
 	return (int)shim_enosys(0, buff, buflen, flags);
 #endif
 }
@@ -276,7 +353,7 @@ void shim_flush_icache(void *begin, void *end)
 #if defined(__GNUC__) &&	\
     defined(STRESS_ARCH_ARM)
 	__clear_cache(begin, end);
-#elif defined(STRESS_ARCH_RISC_V) &&	\
+#elif defined(STRESS_ARCH_RISCV) &&	\
       defined(__NR_riscv_flush_icache)
 	(void)syscall(__NR_riscv_flush_icache, begin, end, 0);
 #else
@@ -331,7 +408,9 @@ int shim_membarrier(int cmd, int flags, int cpu_id)
  */
 int shim_memfd_create(const char *name, unsigned int flags)
 {
-#if defined(__NR_memfd_create)
+#if defined(HAVE_MEMFD_CREATE)
+	return memfd_create(name, flags);
+#elif defined(__NR_memfd_create)
 	return (int)syscall(__NR_memfd_create, name, flags);
 #else
 	return (int)shim_enosys(0, name, flags);
@@ -464,12 +543,10 @@ int shim_seccomp(unsigned int operation, unsigned int flags, void *args)
  */
 int shim_unshare(int flags)
 {
-#if defined(__NR_unshare)
-#if NEED_GLIBC(2,14,0)
+#if defined(HAVE_UNSHARE)
 	return unshare(flags);
-#else
+#elif defined(__NR_unshare)
 	return (int)syscall(__NR_unshare, flags);
-#endif
 #else
 	return (int)shim_enosys(0, flags);
 #endif
@@ -540,12 +617,16 @@ int shim_munlock(const void *addr, size_t len)
 
 /*
  *  shim_mlock2()
- *	wrapper for mlock2(2) - lock memory
+ *	wrapper for mlock2(2) - lock memory; force call
+ *	via syscall if possible as zero flags call goes
+ *	via mlock() in libc or mlock2() for non-zero flags.
  */
 int shim_mlock2(const void *addr, size_t len, int flags)
 {
 #if defined(__NR_mlock2)
 	return (int)syscall(__NR_mlock2, addr, len, flags);
+#elif defined(HAVE_MLOCK2)
+	return mlock2(addr, len, flags);
 #else
 	return (int)shim_enosys(0, addr, len, flags);
 #endif
@@ -609,7 +690,7 @@ int shim_nanosleep_uint64(uint64_t nsec)
 	}
 #else
 	useconds_t usec = nsec / 1000;
-	const double t_end = stress_time_now() + ((double)usec) / 1000000.0;
+	const double t_end = stress_time_now() + (((double)usec) * ONE_MILLIONTH);
 
 	for (;;) {
 		errno = 0;
@@ -653,7 +734,7 @@ int shim_usleep_interruptible(uint64_t usec)
 #if defined(HAVE_NANOSLEEP)
 	struct timespec t, trem;
 
-	t.tv_sec = usec / 1000000;
+	t.tv_sec = usec * ONE_MILLIONTH;
 	t.tv_nsec = ((long int)usec - (t.tv_sec * 1000000)) * 1000;
 
 	errno = 0;
@@ -828,20 +909,30 @@ int shim_mincore(void *addr, size_t length, unsigned char *vec)
 #endif
 }
 
-ssize_t shim_statx(
+/*
+ *  shim_statx()
+ *	extended stat, Linux only variant
+ */
+int shim_statx(
 	int dfd,
 	const char *filename,
 	unsigned int flags,
 	unsigned int mask,
-	struct shim_statx *buffer)
+	shim_statx_t *buffer)
 {
-#if defined(__NR_statx)
-	return (ssize_t)syscall(__NR_statx, dfd, filename, flags, mask, buffer);
+	(void)memset(buffer, 0, sizeof(*buffer));
+#if defined(HAVE_STATX)
+	return statx(dfd, filename, flags, mask, (struct statx *)buffer);
+#elif defined(__NR_statx)
+	return syscall(__NR_statx, dfd, filename, flags, mask, buffer);
 #else
-	return (ssize_t)shim_enosys(0, dfd, filename, flags, mask, buffer);
+	return shim_enosys(0, dfd, filename, flags, mask, buffer);
 #endif
 }
 
+#undef STATX_COPY
+#undef STATX_COPY_TS
+
 /*
  *  futex wake()
  *	wake n waiters on futex
@@ -874,7 +965,7 @@ int shim_futex_wait(
 }
 
 /*
- *  dup3()
+ *  shim_dup3()
  *	linux special dup3
  */
 int shim_dup3(int oldfd, int newfd, int flags)
@@ -888,7 +979,10 @@ int shim_dup3(int oldfd, int newfd, int flags)
 #endif
 }
 
-
+/*
+ *  shim_sync_file_range()
+ *	sync data on a specified data range
+ */
 int shim_sync_file_range(
 	int fd,
 	shim_off64_t offset,
@@ -997,6 +1091,7 @@ void *shim_sbrk(intptr_t increment)
 #elif defined(__NR_sbrk)
 	return (void *)syscall(__NR_sbrk, increment);
 #else
+	UNEXPECTED
 	return (void *)(intptr_t)shim_enosys(0, increment);
 #endif
 }
@@ -1010,7 +1105,7 @@ STRESS_PRAGMA_POP
  */
 size_t shim_strlcpy(char *dst, const char *src, size_t len)
 {
-#if defined(HAVE_STRLCPY) &&	\
+#if defined(HAVE_BSD_STRLCPY) &&	\
     !defined(BUILD_STATIC)
 	return strlcpy(dst, src, len);
 #else
@@ -1045,7 +1140,7 @@ size_t shim_strlcpy(char *dst, const char *src, size_t len)
  */
 size_t shim_strlcat(char *dst, const char *src, size_t len)
 {
-#if defined(HAVE_STRLCAT) &&	\
+#if defined(HAVE_BSD_STRLCAT) &&	\
     !defined(BUILD_STATIC)
 	return strlcat(dst, src, len);
 #else
@@ -1353,8 +1448,8 @@ int shim_pidfd_getfd(int pidfd, int targetfd, unsigned int flags)
 }
 
 /*
- * If we have the fancy new Linux 5.2 mount system
- * calls then provide shim wrappers for these
+ *  If we have the fancy new Linux 5.2 mount system
+ *  calls then provide shim wrappers for these
  */
 int shim_fsopen(const char *fsname, unsigned int flags)
 {
@@ -1404,6 +1499,10 @@ int shim_move_mount(
 #endif
 }
 
+/*
+ *  sys_clone3()
+ *	Linux clone3 system call wrapper
+ */
 int sys_clone3(struct shim_clone_args *cl_args, size_t size)
 {
 #if defined(__NR_clone3)
@@ -1413,6 +1512,10 @@ int sys_clone3(struct shim_clone_args *cl_args, size_t size)
 #endif
 }
 
+/*
+ *  shim_ustat
+ *	ustat system call wrapper
+ */
 int shim_ustat(dev_t dev, struct shim_ustat *ubuf)
 {
 #if defined(HAVE_USTAT) &&	\
@@ -1677,21 +1780,6 @@ ssize_t shim_process_madvise(
 #endif
 }
 
-/*
- *   shim_clock_adjtime
- *	wrapper for linux clock_adjtime system call
- */
-int shim_clock_adjtime(clockid_t clk_id, struct shim_timex *tx)
-{
-#if defined(HAVE_SYS_TIMEX_H) &&	\
-    defined(CLOCK_THREAD_CPUTIME_ID) &&	\
-    defined(__NR_clock_adjtime)
-	return (int)syscall(__NR_clock_adjtime, clk_id, tx);
-#else
-	return (int)shim_enosys(0, clk_id, tx);
-#endif
-}
-
 /*
  *   shim_clock_getres
  *	wrapper for linux clock_getres system call,
@@ -1771,6 +1859,7 @@ int shim_nice(int inc)
 #else
 	(void)inc;
 
+	UNEXPECTED
 	errno = -ENOSYS;
 	return -1;
 #endif
@@ -2033,3 +2122,77 @@ int shim_process_mrelease(int pidfd, unsigned int flags)
 #endif
 }
 
+/*
+ *  shim_futex_waitv()
+ *      system call wrapper for Linux 5.16 futex_waitv
+ */
+int shim_futex_waitv(
+	struct shim_futex_waitv *waiters,
+	unsigned int nr_futexes,
+	unsigned int flags,
+	struct timespec *timeout,
+	clockid_t clockid)
+{
+#if defined(__NR_futex_waitv)
+	return (int)syscall(__NR_futex_waitv, waiters, nr_futexes, flags, timeout, clockid);
+#else
+	return (int)shim_enosys(0, waiters, nr_futexes, flags, timeout, clockid);
+#endif
+}
+
+/*
+ *  shim_unlink()
+ *	unlink, skip operation if --keep-files is enabled
+ */
+int shim_unlink(const char *pathname)
+{
+	if (g_opt_flags & OPT_FLAGS_KEEP_FILES)
+		return 0;
+	return unlink(pathname);
+}
+
+/*
+ *  shim_force_unlink(const char *pathname)
+ *	unlink always
+ */
+int shim_force_unlink(const char *pathname)
+{
+	return unlink(pathname);
+}
+
+/*
+ *  shim_unlinkat()
+ *	unlinkat, skip operation if --keep-files is enabled
+ */
+int shim_unlinkat(int dirfd, const char *pathname, int flags)
+{
+	if (g_opt_flags & OPT_FLAGS_KEEP_FILES)
+		return 0;
+#if defined(HAVE_UNLINKAT)
+	return unlinkat(dirfd, pathname, flags);
+#elif defined(__NR_unlinkat)
+	return (int)syscall(__NR_unlinkat, dirfd, pathname, flags);
+#else
+	return (int)shim_enosys(0, dirfd, pathname, flags);
+#endif
+}
+
+/*
+ *  shim_rmdir()
+ *	rmdir, skip operation if --keep-files is enabled
+ */
+int shim_rmdir(const char *pathname)
+{
+	if (g_opt_flags & OPT_FLAGS_KEEP_FILES)
+		return 0;
+	return rmdir(pathname);
+}
+
+/*
+ *  shim_force_rmdir()
+ *	rmdir always
+ */
+int shim_force_rmdir(const char *pathname)
+{
+	return rmdir(pathname);
+}
