@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2021 Canonical, Ltd.
+ * Copyright (C) 2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,7 +18,16 @@
  *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+#include "core-smart.h"
 
+#if defined(HAVE_SCSI_SG_H)
+#include <scsi/sg.h>
+#endif
+
+#if defined(HAVE_SCSI_SCSI_IOCTL_H)
+#include <scsi/scsi_ioctl.h>
+#endif
 
 #define DEVS_MAX      		(256)
 
@@ -278,9 +288,9 @@ static size_t stress_smart_data_diff_count(stress_smart_dev_t *dev)
 	begin = dev->data_begin;
 	end = dev->data_end;
 
-	if (!begin || !end)
+	if (!begin || !begin->count)
 		return 0;
-	if (!begin->count || !end->count)
+	if (!end || !end->count)
 		return 0;
 
 	for (n = 0, i = 0; i < begin->count; i++) {
