@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-thermal-zone.h"
 
 #if defined(STRESS_THERMAL_ZONES)
 
@@ -48,6 +44,21 @@ static uint32_t stress_tz_type_instance(
 	return type_instance;
 }
 
+/*
+ *  stress_tz_type_fix()
+ *	fix up type name, replace non-alpha/digits with _
+ */
+static void stress_tz_type_fix(char *type)
+{
+	char *ptr;
+
+	for (ptr = type; *ptr; ptr++) {
+		if (isalnum((int)*ptr))
+			continue;
+		*ptr = '_';
+	}
+}
+
 /*
  *  stress_tz_init()
  *	gather all thermal zones
@@ -96,8 +107,9 @@ int stress_tz_init(stress_tz_info_t **tz_info_list)
 
 			if (fgets(type, sizeof(type), fp) != NULL) {
 				type[strcspn(type, "\n")] = '\0';
+				stress_tz_type_fix(type);
 				tz_info->type = strdup(type);
-				tz_info->type_instance = stress_tz_type_instance(*tz_info_list, tz_info->type);
+				tz_info->type_instance = stress_tz_type_instance(*tz_info_list, type);
 			}
 			(void)fclose(fp);
 		}
