@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-thrash.h"
 
 #if defined(__linux__) &&	\
     defined(HAVE_PTRACE)
@@ -150,6 +146,42 @@ static inline void stress_zone_reclaim(void)
 #endif
 }
 
+/*
+ *  stress_slab_shrink()
+ *	shrink slabs to help release some memory
+ */
+static inline void stress_slab_shrink(void)
+{
+	DIR *dir;
+	struct dirent *d;
+	static const char slabpath[] = "/sys/kernel/slab";
+	int ret;
+
+	/*
+	 *  older shrink interface, may fail
+	 */
+	ret = system_write("/sys/kernel/slab/cache/shrink", "1", 1);
+	(void)ret;
+
+	dir = opendir(slabpath);
+	if (!dir)
+		return;
+
+	/*
+	 *  shrink all slabs
+	 */
+	while ((d = readdir(dir)) != NULL) {
+		if (isalpha((int)d->d_name[0]))  {
+			char path[PATH_MAX];
+
+			(void)snprintf(path, sizeof(path), "%s/%s", slabpath, d->d_name);
+			ret = system_write(path, "1", 1);
+			(void)ret;
+		}
+	}
+	(void)closedir(dir);
+}
+
 /*
  *  stress_drop_caches()
  *	drop caches
@@ -259,8 +291,10 @@ int stress_thrash_start(void)
 			_exit(0);
 
 		while (thrash_run) {
-			if ((stress_mwc8() & 0x3) == 0)
+			if ((stress_mwc8() & 0x3) == 0) {
+				stress_slab_shrink();
 				stress_pagein_all_procs();
+			}
 			if ((stress_mwc8() & 0x7) == 0)
 				stress_drop_caches();
 			stress_compact_memory();
