@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2021 Canonical, Ltd.
+ * Copyright (C)      2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +16,63 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-thermal-zone.h"
+
+#if defined(HAVE_SYS_SYSMACROS_H)
+#include <sys/sysmacros.h>
+#endif
+
+#if defined(HAVE_SYS_SYSCTL_H) &&       \
+    !defined(__linux__)
+#include <sys/sysctl.h>
+#endif
+
+#if defined(HAVE_SYS_VMMETER_H)
+#include <sys/vmmeter.h>
+#endif
+
+#if defined(HAVE_MNTENT_H)
+#include <mntent.h>
+#endif
 
 static int32_t vmstat_delay = 0;
 static int32_t thermalstat_delay = 0;
 static int32_t iostat_delay = 0;
 
+#if defined(__FreeBSD__)
+static int freebsd_getsysctl(const char *name, void *ptr, size_t size)
+{
+	int ret;
+	size_t nsize = size;
+	if (!ptr)
+		return -1;
+
+	(void)memset(ptr, 0, size);
+
+	ret = sysctlbyname(name, ptr, &nsize, NULL, 0);
+	if ((ret < 0) || (nsize != size)) {
+		(void)memset(ptr, 0, size);
+		return -1;
+	}
+	return 0;
+}
+
+/*
+ *  freebsd_getsysctl_uint()
+ *	get an unsigned int sysctl value by name
+ */
+static unsigned int freebsd_getsysctl_uint(const char *name)
+{
+	unsigned int val;
+
+	freebsd_getsysctl(name, &val, sizeof(val));
+
+	return val;
+}
+#endif
+
 static int stress_set_generic_stat(
 	const char *const opt,
 	const char *name,
@@ -56,123 +101,108 @@ int stress_set_iostat(const char *const opt)
 	return stress_set_generic_stat(opt, "iostat", &iostat_delay);
 }
 
+/*
+ *  stress_find_mount_dev()
+ *	find the path of the device that the file is located on
+ */
+char *stress_find_mount_dev(const char *name)
+{
 #if defined(__linux__)
+	static char dev_path[PATH_MAX];
+	struct stat statbuf;
+	dev_t dev;
+	FILE *mtab_fp;
+	struct mntent *mnt;
+
+	if (stat(name, &statbuf) < 0)
+		return NULL;
+
+	/* Cater for UBI char mounts */
+	if (S_ISBLK(statbuf.st_mode) || S_ISCHR(statbuf.st_mode))
+		dev = statbuf.st_rdev;
+	else
+		dev = statbuf.st_dev;
+
+	/* Try /proc/mounts then /etc/mtab */
+	mtab_fp = setmntent("/proc/mounts", "r");
+	if (!mtab_fp) {
+		mtab_fp = setmntent("/etc/mtab", "r");
+		if (!mtab_fp)
+			return NULL;
+	}
+
+	while ((mnt = getmntent(mtab_fp))) {
+		if ((!strcmp(name, mnt->mnt_dir)) ||
+		    (!strcmp(name, mnt->mnt_fsname)))
+			break;
+
+		if ((mnt->mnt_fsname[0] == '/') &&
+		    (stat(mnt->mnt_fsname, &statbuf) == 0) &&
+		    (statbuf.st_rdev == dev))
+			break;
+
+		if ((stat(mnt->mnt_dir, &statbuf) == 0) &&
+		    (statbuf.st_dev == dev))
+			break;
+	}
+	(void)endmntent(mtab_fp);
+
+	if (!mnt)
+		return NULL;
+	if (!mnt->mnt_fsname)
+		return NULL;
+
+	return realpath(mnt->mnt_fsname, dev_path);
+#elif defined(HAVE_SYS_SYSMACROS_H)
+	static char dev_path[PATH_MAX];
+	struct stat statbuf;
+	dev_t dev;
+	DIR *dir;
+	struct dirent *d;
+	dev_t majdev;
+
+	if (stat(name, &statbuf) < 0)
+		return NULL;
+
+	/* Cater for UBI char mounts */
+	if (S_ISBLK(statbuf.st_mode) || S_ISCHR(statbuf.st_mode))
+		dev = statbuf.st_rdev;
+	else
+		dev = statbuf.st_dev;
+
+	majdev = makedev(major(dev), 0);
+
+	dir = opendir("/dev");
+	if (!dir)
+		return NULL;
+
+	while ((d = readdir(dir)) != NULL) {
+		int ret;
+		struct stat stat_buf;
+
+		stress_mk_filename(dev_path, sizeof(dev_path), "/dev", d->d_name);
+		ret = stat(dev_path, &stat_buf);
+		if ((ret == 0) &&
+		    (S_ISBLK(stat_buf.st_mode)) &&
+		    (stat_buf.st_rdev == majdev)) {
+			(void)closedir(dir);
+			return dev_path;
+		}
+	}
+	(void)closedir(dir);
+
+	return NULL;
+#else
+	(void)name;
+
+	return NULL;
+#endif
+}
 
 static pid_t vmstat_pid;
 
-/*
- *  stress_iostat_follow_link
- *	recursively follow link until no more links
- */
-static void stress_iostat_follow_link(
-	const char *path,
-	char *buf,
-	const size_t buflen)
-{
-	ssize_t ret;
-	char tmp[buflen], prev[buflen];
-
-	(void)memset(prev, 0, sizeof(prev));
-
-	shim_strlcpy(tmp, path, buflen);
-	for (;;) {
-		ret = shim_readlink(tmp, buf, buflen - 1);
-		if (ret < 0) {
-			(void)shim_strlcpy(buf, path, buflen);
-			return;
-		}
-		buf[ret] = '\0';
-		if (!strncmp(prev, buf, buflen))
-			return;
-		(void)shim_strlcpy(prev, buf, buflen);
-	}
-}
-
-/*
- *  stress_iostat_dev_trim()
- *	trim device numbers off end, /dev/sda2 -> /dev/sda
- */
-static void stress_iostat_dev_trim(char *devname)
-{
-	char *ptr;
-
-	for (ptr = devname; *ptr; ptr++)
-		;
-
-	for (--ptr; (ptr > devname) && isdigit((int)*ptr); ptr--)
-		*ptr = '\0';
-}
-
-/*
- *  stress_iostat_dev_trim_nvme()
- *	trim 'p' character off end for an already trimmed nvme devname,
- *	/dev/nvme0n1p -> /dev/nvme0n1
- */
-static void stress_iostat_dev_trim_nvme(char *devname)
-{
-	char *ptr;
-
-	for (ptr = devname; *ptr; ptr++)
-		;
-
-	if ((--ptr > devname) && (*ptr == 'p'))
-		*ptr = '\0';
-}
-
-/*
- *  stress_iostat_get_iostat_name
- *	try to find the /sys/block/$dev/stat name from a given
- *	device number. Returns null if not found.
- */
-static char *stress_iostat_get_iostat_name(
-	const dev_t rdev,
-	char *name,
-	const size_t namelen)
-{
-	DIR *dp;
-	struct dirent *d;
-	struct stat statbuf;
-
-	(void)memset(name, 0, namelen);
-
-	dp = opendir("/dev/");
-	if (!dp)
-		return NULL;
-	while ((d = readdir(dp)) != NULL) {
-		if ((d->d_type & DT_BLK) == 0)
-			continue;
-
-		(void)snprintf(name, namelen, "/dev/%s", d->d_name);
-		if (stat(name, &statbuf) < 0)
-			continue;
-		if (statbuf.st_rdev == rdev) {
-			(void)closedir(dp);
-
-			(void)snprintf(name, namelen, "/sys/block/%s/stat", d->d_name);
-			if (stat(name, &statbuf) == 0)
-				return name;
-
-			/* strip off digits from end of dev, retry */
-			stress_iostat_dev_trim(d->d_name);
-			(void)snprintf(name, namelen, "/sys/block/%s/stat", d->d_name);
-			if (stat(name, &statbuf) == 0)
-				return name;
-
-			/* strip off 'p' from end of dev in case of a nvme
-			 * device, retry */
-			stress_iostat_dev_trim_nvme(d->d_name);
-			(void)snprintf(name, namelen, "/sys/block/%s/stat", d->d_name);
-			if (stat(name, &statbuf) == 0)
-				return name;
-
-			(void)memset(name, 0, namelen);
-			return NULL;
-		}
-	}
-	(void)memset(name, 0, namelen);
-	(void)closedir(dp);
-	return NULL;
-}
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 
 /*
  *  stress_iostat_iostat_name()
@@ -183,102 +213,35 @@ static char *stress_iostat_iostat_name(
 	char *iostat_name,
 	const size_t iostat_name_len)
 {
-	const char *temp_path = stress_get_temp_path();
-	struct stat statbuf;
-	int ret;
-	FILE *fp;
-	char buf[4096];
-	int f_major, f_minor;
+	char *temp_path, *dev;
 
 	/* Resolve links */
-	stress_iostat_follow_link(temp_path, iostat_name, iostat_name_len);
-
-	/* Get dev info */
-	ret = stat(iostat_name, &statbuf);
-	if (ret < 0) {
-		*iostat_name = '\0';
+	temp_path = realpath(stress_get_temp_path(), NULL);
+	if (!temp_path)
 		return NULL;
-	}
 
-	/*
-	 *  Scan mountinfo for mount points that match the device
-	 */
-	fp = fopen("/proc/self/mountinfo", "r");
-	if (!fp) {
-		*iostat_name = '\0';
+	/* Find device */
+	dev = stress_find_mount_dev(temp_path);
+	if (!dev)
 		return NULL;
-	}
 
-	f_major = major(statbuf.st_dev);
-	f_minor = major(statbuf.st_dev);
+	/* Skip over leading /dev */
+	if (!strncmp(dev, "/dev", 4))
+		dev += 4;
 
-	(void)memset(buf, 0, sizeof(buf));
-	while (fgets(buf, sizeof(buf), fp)) {
-		int mnt_major, mnt_minor;
+	(void)snprintf(iostat_name, iostat_name_len, "/sys/block/%s/stat", dev);
 
-		ret = sscanf(buf, "%*d %*d %d:%d", &mnt_major, &mnt_minor);
-		if (ret != 2)
-			continue;
-		if ((f_major == mnt_major) && (f_minor == mnt_minor)) {
-			/*
-			 * parse something like:
-			 *  31 1 253:1 / / rw,relatime shared:1 - ext4 /dev/mapper/vgubuntu-root rw,errors=remount-ro
-			 *  ..and look for - field, e.g. ext4 /dev/mapper/vgubuntu-root
-			 */
-			char *start = strstr(buf, " - "), *end, *tmp;
-
-			if (!start)
-				continue;
-
-			/* Skip over ' - ' and following non-spaces */
-			for (start += 3; *start && *start != ' '; start++)
-				start++;
-			if (!*start)
-				continue;
-
-			/* skip over spaces to get to dev name */
-			while (*start && *start == ' ')
-				start++;
-			if (!*start)
-				continue;
-
-			/* now look for the end of dev name */
-			for (end = start; *end && *end != ' '; end++)
-				;
-			if (!*end)
-				continue;
-			*end = '\0';
-
-			/* get rdev block info of this device */
-			ret = stat(start, &statbuf);
-			if (ret < 0)
-				continue;
-
-			/* ..and find any matching devices in /dev and resolve the iostate_name */
-			tmp = stress_iostat_get_iostat_name(statbuf.st_rdev, iostat_name, iostat_name_len);
-			if (tmp) {
-				(void)fclose(fp);
-				return tmp;
-			}
-		}
-	}
-	/* nothing found */
-	(void)fclose(fp);
-	*iostat_name = '\0';
-
-	return NULL;
+	return iostat_name;
 }
 
 /*
  *  stress_read_iostat()
- *	read the stats from an iostat stat file
+ *	read the stats from an iostat stat file, linux variant
  */
 static void stress_read_iostat(const char *iostat_name, stress_iostat_t *iostat)
 {
 	FILE *fp;
 
-	(void)memset(iostat, 0, sizeof(*iostat));
-
 	fp = fopen(iostat_name, "r");
 	if (fp) {
 		int ret;
@@ -307,7 +270,6 @@ static void stress_read_iostat(const char *iostat_name, stress_iostat_t *iostat)
 	}
 }
 
-
 #define STRESS_IOSTAT_DELTA(field)					\
 	iostat->field = ((iostat_current.field > iostat_prev.field) ?	\
 	(iostat_current.field - iostat_prev.field) : 0)
@@ -321,6 +283,7 @@ static void stress_get_iostat(const char *iostat_name, stress_iostat_t *iostat)
 	static stress_iostat_t iostat_prev;
 	stress_iostat_t iostat_current;
 
+	(void)memset(&iostat_current, 0, sizeof(iostat_current));
 	stress_read_iostat(iostat_name, &iostat_current);
 	STRESS_IOSTAT_DELTA(read_io);
 	STRESS_IOSTAT_DELTA(read_merges);
@@ -339,7 +302,9 @@ static void stress_get_iostat(const char *iostat_name, stress_iostat_t *iostat)
 	STRESS_IOSTAT_DELTA(discard_ticks);
 	(void)memcpy(&iostat_prev, &iostat_current, sizeof(iostat_prev));
 }
+#endif
 
+#if defined(__linux__)
 /*
  *  stress_next_field()
  *	skip to next field, returns false if end of
@@ -374,8 +339,6 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 	FILE *fp;
 	char buffer[1024];
 
-	(void)memset(vmstat, 0, sizeof(*vmstat));
-
 	fp = fopen("/proc/stat", "r");
 	if (fp) {
 		while (fgets(buffer, sizeof(buffer), fp)) {
@@ -452,13 +415,13 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 			if (!strncmp(buffer, "procs_running", 13)) {
 				if (!stress_next_field(&ptr))
 					continue;
-				/* context switches */
+				/* processes running */
 				vmstat->procs_running = (uint64_t)atoll(ptr);
 			}
 			if (!strncmp(buffer, "procs_blocked", 13)) {
 				if (!stress_next_field(&ptr))
 					continue;
-				/* context switches */
+				/* procesess blocked */
 				vmstat->procs_blocked = (uint64_t)atoll(ptr);
 			}
 			if (!strncmp(buffer, "swap", 4)) {
@@ -539,6 +502,37 @@ static void stress_read_vmstat(stress_vmstat_t *vmstat)
 		(void)fclose(fp);
 	}
 }
+#elif defined(__FreeBSD__)
+/*
+ *  stress_read_vmstat()
+ *	read vmstat statistics, FreeBSD variant, partially implemented
+ */
+static void stress_read_vmstat(stress_vmstat_t *vmstat)
+{
+	struct vmtotal t;
+
+	vmstat->interrupt = freebsd_getsysctl_uint("vm.stats.sys.v_intr");
+	vmstat->context_switch = freebsd_getsysctl_uint("vm.stats.sys.v_swtch");
+	vmstat->swap_in = freebsd_getsysctl_uint("vm.stats.vm.v_swapin");
+	vmstat->swap_out = freebsd_getsysctl_uint("vm.stats.vm.v_swapout");
+	vmstat->block_in = freebsd_getsysctl_uint("vm.stats.vm.v_vnodepgsin");
+	vmstat->block_out = freebsd_getsysctl_uint("vm.stats.vm.v_vnodepgsin");
+	vmstat->memory_free = freebsd_getsysctl_uint("vm.stats.vm.v_free_count");
+
+	freebsd_getsysctl("vm.vmtotal", &t, sizeof(t));
+	vmstat->procs_running = t.t_rq - 1;
+	vmstat->procs_blocked = t.t_dw + t.t_pw;
+}
+#else
+/*
+ *  stress_read_vmstat()
+ *	read vmstat statistics, no-op
+ */
+static void stress_read_vmstat(stress_vmstat_t *vmstat)
+{
+	(void)vmstat;
+}
+#endif
 
 #define STRESS_VMSTAT_COPY(field)	vmstat->field = (vmstat_current.field)
 #define STRESS_VMSTAT_DELTA(field)					\
@@ -554,6 +548,8 @@ static void stress_get_vmstat(stress_vmstat_t *vmstat)
 	static stress_vmstat_t vmstat_prev;
 	stress_vmstat_t vmstat_current;
 
+	(void)memset(&vmstat_current, 0, sizeof(vmstat_current));
+	(void)memset(vmstat, 0, sizeof(*vmstat));
 	stress_read_vmstat(&vmstat_current);
 	STRESS_VMSTAT_COPY(procs_running);
 	STRESS_VMSTAT_COPY(procs_blocked);
@@ -576,6 +572,7 @@ static void stress_get_vmstat(stress_vmstat_t *vmstat)
 	(void)memcpy(&vmstat_prev, &vmstat_current, sizeof(vmstat_prev));
 }
 
+#if defined(__linux__)
 /*
  *  stress_get_tz_info()
  *	get temperature in degrees C from a thermal zone
@@ -597,7 +594,9 @@ static double stress_get_tz_info(stress_tz_info_t *tz_info)
 	}
 	return temp;
 }
+#endif
 
+#if defined(__linux__)
 /*
  *  stress_get_cpu_ghz_average()
  *	compute average CPU frequencies in GHz
@@ -633,8 +632,33 @@ static double stress_get_cpu_ghz_average(void)
 	if (n_cpus > -1)
 		free(cpu_list);
 
-	return (n == 0) ? 0.0 : (total_freq / n) / 1000000.0;
+	return (n == 0) ? 0.0 : (total_freq / n) * ONE_MILLIONTH;
 }
+#elif defined(__FreeBSD__)
+static double stress_get_cpu_ghz_average(void)
+{
+	const int32_t ncpus = stress_get_processors_configured();
+	int32_t i;
+	double total = 0.0;
+
+	for (i = 0; i < ncpus; i++) {
+		char name[32];
+
+		(void)snprintf(name, sizeof(name), "dev.cpu.%" PRIi32 ".freq", i);
+		total += (double)freebsd_getsysctl_uint(name);
+	}
+	total /= 1000.0;
+	if (ncpus > 0)
+		return total / (double)ncpus;
+
+	return 0.0;
+}
+#else
+static double stress_get_cpu_ghz_average(void)
+{
+	return 0.0;
+}
+#endif
 
 /*
  *  stress_vmstat_start()
@@ -643,11 +667,14 @@ static double stress_get_cpu_ghz_average(void)
 void stress_vmstat_start(void)
 {
 	stress_vmstat_t vmstat;
-	stress_iostat_t iostat;
 	size_t tz_num = 0;
 	stress_tz_info_t *tz_info, *tz_info_list;
 	int32_t vmstat_sleep, thermalstat_sleep, iostat_sleep;
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 	char iostat_name[PATH_MAX];
+	stress_iostat_t iostat;
+#endif
 
 	if ((vmstat_delay == 0) &&
 	    (thermalstat_delay == 0) &&
@@ -673,11 +700,13 @@ void stress_vmstat_start(void)
 			tz_num++;
 	}
 
-
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 	if (stress_iostat_iostat_name(iostat_name, sizeof(iostat_name)) == NULL)
 		iostat_sleep = 0;
 	if (iostat_delay)
 		stress_get_iostat(iostat_name, &iostat);
+#endif
 
 	while (keep_stressing_flag()) {
 		int32_t sleep_delay = INT_MAX;
@@ -687,8 +716,11 @@ void stress_vmstat_start(void)
 			sleep_delay = STRESS_MINIMUM(vmstat_delay, sleep_delay);
 		if (thermalstat_delay > 0)
 			sleep_delay = STRESS_MINIMUM(thermalstat_delay, sleep_delay);
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 		if (iostat_delay > 0)
 			sleep_delay = STRESS_MINIMUM(iostat_delay, sleep_delay);
+#endif
 
 		(void)sleep((unsigned int)sleep_delay);
 
@@ -750,25 +782,30 @@ void stress_vmstat_start(void)
 
 		if (thermalstat_delay == thermalstat_sleep) {
 			double min1, min5, min15, ghz;
-			char therms[1 + (tz_num * 6)];
+			char therms[1 + (tz_num * 7)];
 			char cpuspeed[6];
+#if defined(__linux__)
 			char *ptr;
+#endif
 			static uint32_t thermalstat_count = 0;
 
 			(void)memset(therms, 0, sizeof(therms));
 
+#if defined(__linux__)
 			for (ptr = therms, tz_info = tz_info_list; tz_info; tz_info = tz_info->next) {
 				(void)snprintf(ptr, 8, " %6.6s", tz_info->type);
 				ptr += 7;
 			}
-
+#endif
 			if ((thermalstat_count++ % 25) == 0)
 				pr_inf("therm:   GHz  LdA1  LdA5 LdA15 %s\n", therms);
 
+#if defined(__linux__)
 			for (ptr = therms, tz_info = tz_info_list; tz_info; tz_info = tz_info->next) {
 				(void)snprintf(ptr, 8, " %6.2f", stress_get_tz_info(tz_info));
 				ptr += 7;
 			}
+#endif
 			ghz = stress_get_cpu_ghz_average();
 			if (ghz > 0.0)
 				(void)snprintf(cpuspeed, sizeof(cpuspeed), "%5.2f", ghz);
@@ -784,6 +821,8 @@ void stress_vmstat_start(void)
 			}
 		}
 
+#if defined(HAVE_SYS_SYSMACROS_H) &&	\
+    defined(__linux__)
 		if (iostat_delay == iostat_sleep) {
 			double clk_scale = (iostat_delay > 0) ? 1.0 / iostat_delay : 0.0;
 			static uint32_t iostat_count = 0;
@@ -802,6 +841,7 @@ void stress_vmstat_start(void)
 				(double)iostat.write_io * clk_scale,
 				(double)iostat.discard_io * clk_scale);
 		}
+#endif
 	}
 }
 
@@ -818,18 +858,3 @@ void stress_vmstat_stop(void)
 		(void)waitpid(vmstat_pid, &status, 0);
 	}
 }
-
-#else
-
-/*
- *  no-ops for non-linux cases
- */
-void stress_vmstat_start(void)
-{
-}
-
-void stress_vmstat_stop(void)
-{
-}
-
-#endif
