@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 typedef struct {
 	const mode_t	chmod_mode;
@@ -124,15 +120,21 @@ static const int access_flags[] = {
  *	this calls faccessat and/or fstatat. If we don't have the system
  *	call number than revert to the libc implementation
  */
-#if defined(HAVE_FACCESSAT) 
-static int shim_faccessat(int dirfd, const char *pathname, int mode, int flags)
+#if defined(HAVE_FACCESSAT)
+static int shim_faccessat(int dir_fd, const char *pathname, int mode, int flags)
 {
 #if defined(HAVE_FACCESSAT2)
-	return faccessat2(dirfd, pathname, mode, flags);
+	return faccessat2(dir_fd, pathname, mode, flags);
 #elif defined(__NR_faccessat2)
-	return (int)syscall(__NR_faccessat2, dirfd, pathname, mode, flags);
+	int ret;
+
+	ret = (int)syscall(__NR_faccessat2, dir_fd, pathname, mode, flags);
+	if ((ret < 0) && (errno != ENOSYS))
+		return ret;
+	else
+		return faccessat(dir_fd, pathname, mode, flags);
 #else
-	return faccessat(dirfd, pathname, mode, flags);
+	return faccessat(dir_fd, pathname, mode, flags);
 #endif
 }
 #endif
@@ -150,7 +152,7 @@ static int stress_access(const stress_args_t *args)
 #if defined(HAVE_FACCESSAT)
 	const int bad_fd = stress_get_bad_fd();
 #endif
-	const bool is_root = (geteuid() == 0);
+	const bool is_root = stress_check_capability(SHIM_CAP_IS_ROOT);
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
@@ -177,7 +179,7 @@ static int stress_access(const stress_args_t *args)
 
 			ret = fchmod(fd, modes[i].chmod_mode);
 			if (CHMOD_ERR(ret)) {
-				pr_err("%s: fchmod %3.3o failed: %d (%s)\n",
+				pr_fail("%s: fchmod %3.3o failed: %d (%s)\n",
 					args->name, (unsigned int)modes[i].chmod_mode,
 					errno, strerror(errno));
 				goto tidy;
@@ -210,10 +212,12 @@ static int stress_access(const stress_args_t *args)
 			}
 
 			/*
-			 *  Exercise bad dirfd
+			 *  Exercise bad dir_fd
 			 */
 			ret = shim_faccessat(bad_fd, filename, modes[i].access_mode, 0);
 			(void)ret;
+#else
+	UNEXPECTED
 #endif
 #if defined(HAVE_FACCESSAT2) &&	\
     defined(AT_SYMLINK_NOFOLLOW)
@@ -227,11 +231,13 @@ static int stress_access(const stress_args_t *args)
 					errno, strerror(errno));
 			}
 			/*
-			 *  Exercise bad dirfd
+			 *  Exercise bad dir_fd
 			 */
 			ret = faccessat2(bad_fd, filename, modes[i].access_mode,
 				AT_SYMLINK_NOFOLLOW);
 			(void)ret;
+#else
+			/* UNEXPECTED */
 #endif
 			if (modes[i].access_mode != 0) {
 				const mode_t chmod_mode = modes[i].chmod_mode ^ all_mask;
@@ -240,7 +246,7 @@ static int stress_access(const stress_args_t *args)
 
 				ret = fchmod(fd, chmod_mode);
 				if (CHMOD_ERR(ret)) {
-					pr_err("%s: fchmod %3.3o failed: %d (%s)\n",
+					pr_fail("%s: fchmod %3.3o failed: %d (%s)\n",
 						args->name, (unsigned int)chmod_mode,
 						errno, strerror(errno));
 					goto tidy;
@@ -263,6 +269,8 @@ static int stress_access(const stress_args_t *args)
 						(unsigned int)chmod_mode,
 						errno, strerror(errno));
 				}
+#else
+	UNEXPECTED
 #endif
 			}
 		}
@@ -276,7 +284,7 @@ tidy:
 		(void)fchmod(fd, S_IRUSR | S_IWUSR);
 		(void)close(fd);
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 
 	return rc;
@@ -291,5 +299,6 @@ static const stress_help_t help[] = {
 stressor_info_t stress_access_info = {
 	.stressor = stress_access,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
