@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_IF_ALG_H)
+#include <linux/if_alg.h>
+#endif
+
+#if defined(HAVE_LINUX_SOCKET_H)
+#include <linux/socket.h>
+#endif
+
 #define ALLOC_SLOP	(64)
 
 static const stress_help_t help[] = {
@@ -70,6 +73,11 @@ typedef enum {
 	CRYPTO_UNKNOWN,
 } stress_crypto_type_t;
 
+typedef enum {
+	SOURCE_DEFCONFIG,
+	SOURCE_PROC_CRYPTO,
+} stress_crypto_source_t;
+
 typedef struct {
 	const stress_crypto_type_t type;
 	const char		*type_string;
@@ -81,7 +89,7 @@ static const stress_crypto_type_info_t crypto_type_info[] = {
 	{ CRYPTO_SHASH,		"CRYPTO_SHASH",		"shash" },
 	{ CRYPTO_CIPHER,	"CRYPTO_CIPHER",	"cipher" },
 	{ CRYPTO_AKCIPHER,	"CRYPTO_AKCIPHER",	"akcipher" },
-	{ CRYPTO_SKCIPHER,	"CRYPTO_SKCIPHER",	"skciper" },
+	{ CRYPTO_SKCIPHER,	"CRYPTO_SKCIPHER",	"skcipher" },
 	{ CRYPTO_RNG,		"CRYPTO_RNG",		"rng" },
 	{ CRYPTO_AEAD,		"CRYPTO_AEAD",		"aead" },
 	{ CRYPTO_UNKNOWN,	"CRYPTO_UNKNOWN",	"unknown" },
@@ -89,6 +97,7 @@ static const stress_crypto_type_info_t crypto_type_info[] = {
 
 typedef struct stress_crypto_info {
 	stress_crypto_type_t	crypto_type;
+	stress_crypto_source_t	source;
 	char 	*type;
 	char 	*name;
 	int 	block_size;
@@ -96,7 +105,7 @@ typedef struct stress_crypto_info {
 	int	max_auth_size;
 	int	iv_size;
 	int	digest_size;
-	bool	internal;
+	bool	internal;		/* true if accessible to userspace */
 	bool	ignore;
 	bool	selftest;		/* true if passed */
 	struct stress_crypto_info *next;
@@ -123,8 +132,8 @@ static void stress_af_alg_add_crypto_defconfigs(void);
  */
 static stress_crypto_type_t name_to_type(const char *buffer)
 {
-	char *ptr = strchr(buffer, ':');
-        const char *end = ptr + strlen(buffer);
+	const char *ptr = strchr(buffer, ':');
+	const char *end = ptr + strlen(buffer);
 	size_t i;
 
 	if (!ptr)
@@ -166,7 +175,7 @@ static void stress_af_alg_ignore(
 	stress_crypto_info_t *info)
 {
 	if ((args->instance == 0) && (!info->ignore)) {
-		pr_dbg("%s: sendmsg using %s failed with EINVAL, skipping crypto engine\n",
+		pr_dbg_skip("%s: sendmsg using %s failed with EINVAL, skipping crypto engine\n",
 			args->name, info->name);
 		info->ignore = true;
 	}
@@ -336,6 +345,13 @@ retry_bind:
 				goto err;
 			}
 		}
+
+		/* Internal unavailable crypto engines need to be ignored */
+		if ((errno == ENOENT) && (info->internal)) {
+			stress_af_alg_ignore(args, info);
+			rc = EXIT_SUCCESS;
+			goto err;
+		}
 		if ((errno == 0) || (errno == ENOKEY) ||
 		    (errno == ENOENT) || (errno == EBUSY)) {
 			rc = EXIT_SUCCESS;
@@ -534,9 +550,10 @@ retry_bind:
 			goto err_close;
 		} else {
 			if (memcmp(input, output, DATA_LEN)) {
-				pr_err("%s: decrypted data "
+				pr_fail("%s: decrypted data "
 					"different from original data "
-					"using %s\n",
+					"using kernel crypto engine %s "
+					"(possible kernel bug)\n",
 					args->name, info->name);
 			}
 		}
@@ -577,6 +594,12 @@ static int stress_af_alg_rng(
 
 retry_bind:
 	if (bind(sockfd, (struct sockaddr *)&sa, sizeof(sa)) < 0) {
+		/* Internal unavailable crypto engines need to be ignored */
+		if ((errno == ENOENT) && (info->internal)) {
+			stress_af_alg_ignore(args, info);
+			rc = EXIT_SUCCESS;
+			goto err;
+		}
 		/* Perhaps the rng does not exist with this kernel */
 		if ((errno == ENOENT) || (errno == EBUSY)) {
 			rc = EXIT_SUCCESS;
@@ -624,16 +647,75 @@ err:
 	return rc;
 }
 
-static int stress_af_alg_count_crypto(void)
+static void stress_af_alg_count_crypto(int *count, int *internal)
 {
-	int count = 0;
 	stress_crypto_info_t *ci;
 
-	/* Scan for duplications */
-	for (ci = crypto_info_list; ci; ci = ci->next)
-		count++;
+	*count = 0;
+	*internal = 0;
 
-	return count;
+	/* Scan for duplications */
+	for (ci = crypto_info_list; ci; ci = ci->next) {
+		if (ci->internal)
+			(*internal)++;
+		(*count)++;
+	}
+}
+
+/*
+ *  stress_af_alg_cmp_crypto()
+ *	qsort comparison on type then name
+ */
+static int stress_af_alg_cmp_crypto(const void *p1, const void *p2)
+{
+	int n;
+	const stress_crypto_info_t * const *ci1 = (const stress_crypto_info_t * const *)p1;
+	const stress_crypto_info_t * const *ci2 = (const stress_crypto_info_t * const *)p2;
+
+	n = strcmp((*ci1)->type, (*ci2)->type);
+	if (n < 0)
+		return -1;
+	if (n > 0)
+		return 1;
+
+	n = strcmp((*ci1)->name, (*ci2)->name);
+	if (n < 0)
+		return -1;
+	if (n > 0)
+		return 1;
+	return 0;
+}
+
+/*
+ *  stress_af_alg_sort_crypto()
+ *	sort crypto_info_list keyed on type and name
+ */
+static void stress_af_alg_sort_crypto(void)
+{
+	stress_crypto_info_t **array, *ci;
+
+	int i, n, internal;
+
+	stress_af_alg_count_crypto(&n, &internal);
+
+	/* Attempt to sort, if we can't silently don't sort */
+	array = calloc((size_t)n, sizeof(*array));
+	if (!array)
+		return;
+
+	for (i = 0, ci = crypto_info_list; ci; ci = ci->next, i++) {
+		array[i] = ci;
+	}
+
+	qsort(array, n, sizeof(*array), stress_af_alg_cmp_crypto);
+
+	for (i = 0; i < n - 1; i++) {
+		array[i]->next = array[i + 1];
+	}
+	array[i]->next = NULL;
+	crypto_info_list = array[0];
+
+	free(array);
 }
 
 /*
@@ -647,26 +729,26 @@ static void stress_af_alg_dump_crypto_list(void)
 	for (ci = crypto_info_list; ci; ci = ci->next) {
 		if (ci->internal)
 			continue;
-		fprintf(stdout, "{ .crypto_type = %s, .type = \"%s\", .name = \"%s\"",
+		(void)fprintf(stdout, "{ .crypto_type = %s, .type = \"%s\", .name = \"%s\"",
 			type_to_type_string(ci->crypto_type), ci->type, ci->name);
 		if (ci->block_size)
-			fprintf(stdout, ",\t.block_size = %d",
+			(void)fprintf(stdout, ",\t.block_size = %d",
 				ci->block_size);
 		if (ci->max_key_size)
-			fprintf(stdout, ",\t.max_key_size = %d",
+			(void)fprintf(stdout, ",\t.max_key_size = %d",
 				ci->max_key_size);
 		if (ci->max_auth_size)
-			fprintf(stdout, ",\t.max_auth_size = %d",
+			(void)fprintf(stdout, ",\t.max_auth_size = %d",
 				ci->max_auth_size);
 		if (ci->iv_size)
-			fprintf(stdout, ",\t.iv_size = %d",
+			(void)fprintf(stdout, ",\t.iv_size = %d",
 				ci->iv_size);
 		if (ci->digest_size)
-			fprintf(stdout, ",\t.digest_size = %d",
+			(void)fprintf(stdout, ",\t.digest_size = %d",
 				ci->digest_size);
-		fprintf(stdout, " },\n");
+		(void)fprintf(stdout, " },\n");
 	}
-	fflush(stdout);
+	(void)fflush(stdout);
 }
 
 /*
@@ -677,28 +759,36 @@ static int stress_af_alg(const stress_args_t *args)
 {
 	int sockfd = -1, rc = EXIT_FAILURE;
 	int retries = MAX_AF_ALG_RETRIES;
-	int count = stress_af_alg_count_crypto();
+	int proc_count, count, internal;
 	bool af_alg_dump = false;
 
+	stress_af_alg_count_crypto(&proc_count, &internal);
+
 	(void)stress_get_setting("af-alg-dump", &af_alg_dump);
 
 	if (af_alg_dump && args->instance == 0) {
 		pr_inf("%s: dumping cryptographic algorithms found in /proc/crypto to stdout\n",
 			args->name);
+		stress_af_alg_sort_crypto();
 		stress_af_alg_dump_crypto_list();
 	}
 
-	if (args->instance == 0) {
-		pr_inf("%s: %d cryptographic algorithms found in /proc/crypto\n",
-			args->name, count);
-	}
-
 	stress_af_alg_add_crypto_defconfigs();
-	count = stress_af_alg_count_crypto();
+	stress_af_alg_sort_crypto();
+	stress_af_alg_count_crypto(&count, &internal);
 
 	if (args->instance == 0) {
-		pr_inf("%s: %d cryptographic algorithms max (with defconfigs)\n",
+		bool lock;
+
+		pr_lock(&lock);
+		pr_inf("%s: %d cryptographic algorithms found in /proc/crypto\n",
+			args->name, proc_count);
+		pr_inf("%s: %d cryptographic algorithms in total (with defconfigs)\n",
 			args->name, count);
+		if (internal)
+			pr_inf("%s: %d cryptographic algorithms are internal and may be unused\n",
+				args->name, internal);
+		pr_unlock(&lock);
 	}
 
 	for (;;) {
@@ -779,7 +869,7 @@ static int stress_af_alg(const stress_args_t *args)
  */
 static char *dup_field(const char *buffer)
 {
-	char *ptr = strchr(buffer, ':');
+	const char *ptr = strchr(buffer, ':');
 	char *eol = strchr(buffer, '\n');
 
 	if (!ptr)
@@ -836,14 +926,14 @@ static bool stress_af_alg_add_crypto(stress_crypto_info_t *info)
 
 	/* Scan for duplications */
 	for (ci = crypto_info_list; ci; ci = ci->next) {
-		if (strcmp(ci->name, info->name) == 0 &&
-		    strcmp(ci->type, info->type) == 0 &&
-		    ci->block_size == info->block_size &&
-		    ci->max_key_size == info->max_key_size &&
-		    ci->max_auth_size == info->max_auth_size &&
-		    ci->iv_size == info->iv_size &&
-		    ci->digest_size == info->digest_size &&
-		    ci->internal == info->internal)
+		if ((strcmp(ci->name, info->name) == 0) &&
+		    (strcmp(ci->type, info->type) == 0) &&
+		    (ci->block_size == info->block_size) &&
+		    (ci->max_key_size == info->max_key_size) &&
+		    (ci->max_auth_size == info->max_auth_size) &&
+		    (ci->iv_size == info->iv_size) &&
+		    (ci->digest_size == info->digest_size) &&
+		    (ci->internal == info->internal))
 			return false;
 	}
 	/*
@@ -868,8 +958,10 @@ static void stress_af_alg_add_crypto_defconfigs(void)
 {
 	size_t i;
 
-	for (i = 0; i < SIZEOF_ARRAY(crypto_info_defconfigs); i++)
+	for (i = 0; i < SIZEOF_ARRAY(crypto_info_defconfigs); i++) {
+		crypto_info_defconfigs[i].source = SOURCE_DEFCONFIG;
 		stress_af_alg_add_crypto(&crypto_info_defconfigs[i]);
+	}
 }
 
 /*
@@ -930,6 +1022,7 @@ static void stress_af_alg_init(void)
 			info.selftest = bool_field(buffer);
 		else if (buffer[0] == '\n') {
 			if (info.crypto_type != CRYPTO_UNKNOWN) {
+				info.source = SOURCE_PROC_CRYPTO;
 				if (!stress_af_alg_add_crypto(&info)) {
 					free(info.name);
 					free(info.type);
@@ -970,6 +1063,7 @@ stressor_info_t stress_af_alg_info = {
 	.deinit = stress_af_alg_deinit,
 	.class = CLASS_CPU | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 
