@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -152,7 +147,7 @@ static void stress_affinity_reap(const pid_t *pids)
  *	if we lock the args->counter, so sum per-process
  *	counters in a racy way.
  */
-static uint64_t stress_affinity_racy_count(uint64_t *counters)
+static uint64_t stress_affinity_racy_count(const uint64_t *counters)
 {
 	register uint64_t count = 0;
 	register size_t i;
@@ -288,8 +283,8 @@ static int stress_affinity(const stress_args_t *args)
 	pid_t pids[STRESS_AFFINITY_PROCS];
 	size_t i;
 	stress_affinity_info_t *info;
-	size_t counters_sz = sizeof(info->counters[0]) * STRESS_AFFINITY_PROCS;
-	size_t info_sz = ((sizeof(*info) + counters_sz) + args->page_size) & ~(args->page_size - 1);
+	const size_t counters_sz = sizeof(info->counters[0]) * STRESS_AFFINITY_PROCS;
+	const size_t info_sz = ((sizeof(*info) + counters_sz) + args->page_size) & ~(args->page_size - 1);
 
 	info = (stress_affinity_info_t *)mmap(NULL, info_sz, PROT_READ | PROT_WRITE,
 			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
@@ -352,7 +347,8 @@ stressor_info_t stress_affinity_info = {
 	.class = CLASS_SCHEDULER,
 	.supported = stress_affinity_supported,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.verify = VERIFY_OPTIONAL,
+	.help = help,
 };
 #else
 stressor_info_t stress_affinity_info = {
