@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,17 +16,23 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
-#define BUFFER_SZ		(4096)
-#define DEFAULT_AIO_MAX_NR	(65536)
+#if defined(HAVE_LIBAIO_H)
+#include <libaio.h>
+#endif
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#endif
+
+#define MIN_AIO_LINUX_REQUESTS		(1)
+#define MAX_AIO_LINUX_REQUESTS		(4096)
+#define DEFAULT_AIO_LINUX_REQUESTS	(64)
+
+#define BUFFER_SZ			(4096)
+#define DEFAULT_AIO_MAX_NR		(65536)
 
 static const stress_help_t help[] = {
 	{ NULL,	"aiol N",	   "start N workers that exercise Linux async I/O" },
@@ -117,11 +124,11 @@ static int shim_io_getevents(
  *	try it again.
  */
 static int shim_io_getevents_random(
-	io_context_t ctx_id,
-	long min_nr,
-	long nr,
-	struct io_event *events,
-	struct timespec *timeout)
+	const io_context_t ctx_id,
+	const long min_nr,
+	const long nr,
+	struct io_event *const events,
+	struct timespec *const timeout)
 {
 #if defined(__NR_io_pgetevents)
 	static bool try_io_pgetevents = true;
@@ -139,6 +146,8 @@ static int shim_io_getevents_random(
 			return ret;
 	}
 	/* ..fall through and use vanilla io_getevents */
+#else
+	UNEXPECTED
 #endif
 	return shim_io_getevents(ctx_id, min_nr, nr, events, timeout);
 }
@@ -148,14 +157,15 @@ static int shim_io_getevents_random(
  *	fill buffer with some known pattern
  */
 static inline void aio_linux_fill_buffer(
-	uint8_t pattern,
+	const uint8_t pattern,
 	uint8_t *const buffer,
 	const size_t size)
 {
 	register size_t i;
+	register uint8_t pat = pattern;
 
-	for (i = 0; i < size; i++, pattern++)
-		buffer[i] = (uint8_t)pattern;
+	for (i = 0; i < size; i++, pat++)
+		buffer[i] = (uint8_t)pat;
 }
 
 /*
@@ -217,7 +227,7 @@ static ssize_t stress_aiol_wait(
 	const stress_args_t *args,
 	const io_context_t ctx,
 	struct io_event events[],
-	size_t n)
+	const size_t n)
 {
 	size_t i = 0;
 
@@ -349,16 +359,19 @@ static int stress_aiol(const stress_args_t *args)
 	int j = 0;
 	size_t i;
 	int warnings = 0;
+#if defined(__NR_io_cancel)
+	int bad_fd;
+#endif
 
 	if (!stress_get_setting("aiol-requests", &aio_linux_requests)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			aio_linux_requests = MAX_AIO_REQUESTS;
+			aio_linux_requests = MAX_AIO_LINUX_REQUESTS;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			aio_linux_requests = MIN_AIO_REQUESTS;
+			aio_linux_requests = MIN_AIO_LINUX_REQUESTS;
 	}
-	if ((aio_linux_requests < MIN_AIO_REQUESTS) ||
-	    (aio_linux_requests > MAX_AIO_REQUESTS)) {
-		pr_err("%s: iol_requests out of range", args->name);
+	if ((aio_linux_requests < MIN_AIO_LINUX_REQUESTS) ||
+	    (aio_linux_requests > MAX_AIO_LINUX_REQUESTS)) {
+		pr_fail("%s: iol_requests out of range", args->name);
 		return EXIT_FAILURE;
 	}
 
@@ -403,20 +416,20 @@ static int stress_aiol(const stress_args_t *args)
 		 *  return value, so set errno accordingly
 		 */
 		if ((errno == EAGAIN) || (errno == EACCES)) {
-			pr_err("%s: io_setup failed, ran out of "
+			pr_fail("%s: io_setup failed, ran out of "
 				"available events, consider increasing "
 				"/proc/sys/fs/aio-max-nr, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			rc = EXIT_NO_RESOURCE;
 			goto free_memory;
 		} else if (errno == ENOMEM) {
-			pr_err("%s: io_setup failed, ran out of "
+			pr_fail("%s: io_setup failed, ran out of "
 				"memory, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			rc = EXIT_NO_RESOURCE;
 			goto free_memory;
 		} else if (errno == ENOSYS) {
-			pr_err("%s: io_setup failed, no io_setup "
+			pr_fail("%s: io_setup failed, no io_setup "
 				"system call with this kernel, "
 				"errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -448,10 +461,14 @@ retry_open:
 		rc = exit_status(errno);
 		pr_fail("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		goto finish;
 	}
 
+#if defined(__NR_io_cancel)
+	bad_fd = stress_get_bad_fd();
+#endif
+
 	/*
 	 *  Make aio work harder by using lots of different fds on the
 	 *  same file. If we can't open a file (e.g. out of file descriptors)
@@ -462,7 +479,7 @@ retry_open:
 		if (fds[i] < 0)
 			fds[i] = fds[0];
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -539,20 +556,103 @@ retry_open:
 #if defined(__NR_io_cancel)
 		{
 			static int cancel;
-			struct io_event event;
 
 			cancel++;
 			if (cancel >= 127) {
+				struct io_event event;
+				io_context_t bad_ctx;
+				struct iocb bad_iocb;
+				struct iocb *bad_iocbs[1];
+				struct timespec timeout;
+
+				cancel = 0;
+
 				ret = shim_io_cancel(ctx, &cb[0], &event);
 				(void)ret;
-				cancel = 0;
+
+				/* Exercise with io_cancel invalid context */
+				(void)memset(&bad_ctx, stress_mwc8() | 0x1, sizeof(bad_ctx));
+				ret = shim_io_cancel(bad_ctx, &cb[0], &event);
+				(void)ret;
+
+				/* Exercise with io_invalid iocb */
+				bad_iocb.aio_fildes = bad_fd;
+				bad_iocb.aio_lio_opcode = ~0;
+				bad_iocb.u.c.buf = NULL;
+				bad_iocb.u.c.offset = 0;
+				bad_iocb.u.c.nbytes = 0;
+				ret = shim_io_cancel(ctx, &bad_iocb, &event);
+				(void)ret;
+
+				/* Exercise io_destroy with illegal context, EINVAL */
+				ret = shim_io_destroy(bad_ctx);
+				(void)ret;
+				ret = shim_io_destroy(NULL);
+				(void)ret;
+
+				/* Exercise io_getevents with illegal context, EINVAL */
+				timeout.tv_sec = 0;
+				timeout.tv_nsec = 100000;
+				ret = shim_io_getevents(bad_ctx, 1, 1, events, &timeout);
+				(void)ret;
+
+				/* Exercise io_getevents with illegal min */
+				timeout.tv_sec = 0;
+				timeout.tv_nsec = 100000;
+				ret = shim_io_getevents(ctx, 1, 0, events, &timeout);
+				(void)ret;
+				ret = shim_io_getevents(ctx, -1, 0, events, &timeout);
+				(void)ret;
+
+				/* Exercise io_getevents with illegal nr */
+				timeout.tv_sec = 0;
+				timeout.tv_nsec = 100000;
+				ret = shim_io_getevents(ctx, 0, -1, events, &timeout);
+				(void)ret;
+
+				/* Exercise io_getevents with illegal timeout */
+				timeout.tv_sec = 0;
+				timeout.tv_nsec = ~0L;
+				ret = shim_io_getevents(ctx, 0, 1, events, &timeout);
+				(void)ret;
+
+				/* Exercise io_setup with illegal nr_events */
+				ret = shim_io_setup(0, &bad_ctx);
+				if (ret == 0)
+					shim_io_destroy(bad_ctx);
+				ret = shim_io_setup(INT_MAX, &bad_ctx);
+				if (ret == 0)
+					shim_io_destroy(bad_ctx);
+
+				/* Exercise io_submit with illegal context */
+				bad_iocb.aio_fildes = bad_fd;
+				bad_iocb.aio_lio_opcode = ~0;
+				bad_iocb.u.c.buf = NULL;
+				bad_iocb.u.c.offset = 0;
+				bad_iocb.u.c.nbytes = 0;
+				bad_iocbs[0] = &bad_iocb;
+				ret = shim_io_submit(bad_ctx, 1, bad_iocbs);
+				(void)ret;
+
+				/* Exercise io_submit with useless or illegal nr ios */
+				ret = shim_io_submit(ctx, 0, bad_iocbs);
+				(void)ret;
+				ret = shim_io_submit(ctx, -1, bad_iocbs);
+				(void)ret;
+
+				/* Exercise io_submit with illegal iocb */
+				ret = shim_io_submit(ctx, 1, bad_iocbs);
+				(void)ret;
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 		inc_counter(args);
 		if (!keep_stressing(args))
 			break;
 
+#if defined(HAVE_POLL_H)
 		/*
 		 *  Exercise aio_poll with illegal settings
 		 */
@@ -573,6 +673,9 @@ retry_open:
 		inc_counter(args);
 		if (!keep_stressing(args))
 			break;
+#else
+		UNEXPECTED
+#endif
 
 		/*
 		 *  Async fdsync and fsync every 256 iterations, older kernels don't
@@ -628,6 +731,7 @@ stressor_info_t stress_aiol_info = {
 	.stressor = stress_aiol,
 	.class = CLASS_IO | CLASS_INTERRUPT | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
