@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_SYS_APPARMOR_H)
+#include <sys/apparmor.h>
+#endif
+
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"apparmor",	  "start N workers exercising AppArmor interfaces" },
@@ -747,6 +751,7 @@ stressor_info_t stress_apparmor_info = {
 	.stressor = stress_not_implemented,
 	.supported = stress_apparmor_supported,
 	.class = CLASS_OS | CLASS_SECURITY,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #endif
