@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-put.h"
+#include "core-pragma.h"
+
+#if defined(HAVE_SYS_AUXV_H)
+#include <sys/auxv.h>
+#endif
 
 static const stress_help_t help[] =
 {
@@ -46,18 +47,18 @@ static void *bus_stack;
 #endif
 static sigjmp_buf jmpbuf;
 
+STRESS_PRAGMA_PUSH
+STRESS_PRAGMA_WARN_OFF
 static inline void stress_bad_altstack_force_fault(uint8_t *stack_start)
 {
 	volatile uint8_t *vol_stack = (volatile uint8_t *)stack_start;
 	/* trigger segfault on stack */
 
-STRESS_PRAGMA_PUSH
-STRESS_PRAGMA_WARN_OFF
 	stress_uint8_put(*vol_stack);	/* cppcheck-suppress nullPointer */
 	*vol_stack = 0;			/* cppcheck-suppress nullPointer */
 	(void)*vol_stack;		/* cppcheck-suppress nullPointer */
-STRESS_PRAGMA_POP
 }
+STRESS_PRAGMA_POP
 
 #if defined(SIGXCPU) &&	\
     defined(RLIMIT_CPU)
@@ -95,6 +96,8 @@ static void NORETURN MLOCKED_TEXT stress_signal_handler(int signum)
 #if defined(O_TMPFILE)
 	if (bus_stack != MAP_FAILED)
 		(void)munmap(bus_stack, STRESS_MINSIGSTKSZ);
+#else
+	UNEXPECTED
 #endif
 	/*
 	 *  If we've not got this far we've not
@@ -115,6 +118,8 @@ static int stress_bad_altstack(const stress_args_t *args)
 	stress_set_oom_adjustment(args->name, true);
 #if defined(HAVE_VDSO_VIA_GETAUXVAL)
 	unsigned long vdso = getauxval(AT_SYSINFO_EHDR);
+#else
+	UNEXPECTED
 #endif
 	int fd;
 #if defined(O_TMPFILE)
@@ -139,6 +144,8 @@ static int stress_bad_altstack(const stress_args_t *args)
 				MAP_PRIVATE, tmp_fd, 0);
 		(void)close(tmp_fd);
 	}
+#else
+	UNEXPECTED
 #endif
 
 	fd = open("/dev/zero", O_RDONLY);
@@ -208,6 +215,7 @@ again:
 			uint32_t rnd;
 			int ret;
 			stack_t ss, old_ss;
+			size_t sz;
 #if defined(SIGXCPU) &&	\
     defined(RLIMIT_CPU)
 			struct rlimit rlim;
@@ -258,6 +266,8 @@ again:
 			rlim.rlim_cur = 1;
 			rlim.rlim_max = 1;
 			(void)setrlimit(RLIMIT_CPU, &rlim);
+#else
+			UNEXPECTED
 #endif
 
 			/* Set alternative stack for testing */
@@ -266,7 +276,7 @@ again:
 
 			/* Child */
 			stress_mwc_reseed();
-			rnd = stress_mwc32() % 10;
+			rnd = stress_mwc32() % 11;
 
 			stress_set_oom_adjustment(args->name, true);
 			stress_process_dumpable(false);
@@ -298,8 +308,16 @@ again:
 				if (!keep_stressing(args))
 					break;
 				CASE_FALLTHROUGH;
-#endif
 			case 4:
+				/* Illegal write-only stack */
+				ret = mprotect(stack, STRESS_MINSIGSTKSZ, PROT_WRITE);
+				if (ret == 0)
+					stress_bad_altstack_force_fault(stack);
+				if (!keep_stressing(args))
+					break;
+				CASE_FALLTHROUGH;
+#endif
+			case 5:
 				/* Illegal NULL stack */
 				ret = stress_sigaltstack(NULL, STRESS_SIGSTKSZ);
 				if (ret == 0)
@@ -307,7 +325,7 @@ again:
 				if (!keep_stressing(args))
 					break;
 				CASE_FALLTHROUGH;
-			case 5:
+			case 6:
 				/* Illegal text segment stack */
 				ret = stress_sigaltstack(stress_signal_handler, STRESS_SIGSTKSZ);
 				if (ret == 0)
@@ -315,11 +333,20 @@ again:
 				if (!keep_stressing(args))
 					break;
 				CASE_FALLTHROUGH;
-			case 6:
-				/* Small stack */
-				stress_bad_altstack_force_fault(NULL);
-				CASE_FALLTHROUGH;
 			case 7:
+				/* Small stack */
+				for (sz = 0; sz <= STRESS_SIGSTKSZ; sz += 256) {
+					ret = stress_sigaltstack_no_check(stack, sz);
+					if (ret == 0)
+						break;
+				}
+				if (ret == 0)
+					stress_bad_altstack_force_fault(stack);
+				if (!keep_stressing(args))
+					break;
+				stress_bad_altstack_force_fault(g_shared->nullptr);
+				CASE_FALLTHROUGH;
+			case 8:
 #if defined(HAVE_VDSO_VIA_GETAUXVAL)
 				/* Illegal stack on VDSO, otherwises NULL stack */
 				if (vdso) {
@@ -331,7 +358,7 @@ again:
 				}
 #endif
 				CASE_FALLTHROUGH;
-			case 8:
+			case 9:
 				/* Illegal /dev/zero mapped stack */
 				if (zero_stack != MAP_FAILED) {
 					ret = stress_sigaltstack(zero_stack, STRESS_MINSIGSTKSZ);
@@ -342,7 +369,7 @@ again:
 				}
 				CASE_FALLTHROUGH;
 #if defined(O_TMPFILE)
-			case 9:
+			case 10:
 				/* Illegal mapped stack to empty file, causes BUS error */
 				if (bus_stack != MAP_FAILED) {
 					ret = stress_sigaltstack(bus_stack, STRESS_MINSIGSTKSZ);
@@ -358,7 +385,7 @@ again:
 			case 0:
 				/* Illegal unmapped stack */
 				(void)munmap(stack, STRESS_MINSIGSTKSZ);
-				stress_bad_altstack_force_fault(NULL);
+				stress_bad_altstack_force_fault(g_shared->nullptr);
 				break;
 			}
 
@@ -372,6 +399,8 @@ again:
 #if defined(O_TMPFILE)
 	if (bus_stack != MAP_FAILED)
 		(void)munmap(bus_stack, STRESS_MINSIGSTKSZ);
+#else
+	UNEXPECTED
 #endif
 	if (zero_stack != MAP_FAILED)
 		(void)munmap(zero_stack, STRESS_MINSIGSTKSZ);
