@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"bad-ioctl N",		"start N stressors that perform illegal read ioctls on devices" },
@@ -36,10 +32,19 @@ static const stress_help_t help[] = {
 
 #define MAX_DEV_THREADS		(4)
 
+typedef struct dev_ioctl_info {
+	char	*dev_path;
+	struct dev_ioctl_info *left;
+	struct dev_ioctl_info *right;
+	bool	ignore;
+	uint16_t	ioctl_state;
+} dev_ioctl_info_t;
+
 static sigset_t set;
 static shim_pthread_spinlock_t lock;
-static char *dev_path;
 static uint32_t mixup;
+static dev_ioctl_info_t *dev_ioctl_info_head;
+static volatile dev_ioctl_info_t *dev_ioctl_node;
 
 typedef struct stress_bad_ioctl_func {
 	const char *devpath;
@@ -47,9 +52,131 @@ typedef struct stress_bad_ioctl_func {
 	void (*func)(const char *name, const int fd, const char *devpath);
 } stress_bad_ioctl_func_t;
 
-static stress_hash_table_t *dev_hash_table;
 static sigjmp_buf jmp_env;
 
+/*
+ *  stress_bad_ioctl_dev_new()
+ *	add a new ioctl device path to tree
+ */
+static dev_ioctl_info_t *stress_bad_ioctl_dev_new(
+	dev_ioctl_info_t **head,
+	const char *dev_path)
+{
+	dev_ioctl_info_t *node;
+
+	while (*head) {
+		const int cmp = strcmp((*head)->dev_path, dev_path);
+
+		if (cmp == 0)
+			return *head;
+		head = (cmp > 0) ? &(*head)->left : &(*head)->right;
+	}
+	node = calloc(1, sizeof(*node));
+	node->dev_path = strdup(dev_path);
+	if (!node->dev_path) {
+		free(node);
+		return NULL;
+	}
+	node->ignore = false;
+	node->ioctl_state = ~0;
+
+	*head = node;
+	return node;
+}
+
+/*
+ *  stress_bad_ioctl_dev_free()
+ *	free tree
+ */
+static void stress_bad_ioctl_dev_free(dev_ioctl_info_t *node)
+{
+	if (node) {
+		stress_bad_ioctl_dev_free(node->left);
+		stress_bad_ioctl_dev_free(node->right);
+		free(node->dev_path);
+		free(node);
+	}
+}
+
+/*
+ *  stress_bad_ioctl_dev_dir()
+ *	read dev directory, add device information to tree
+ */
+static void stress_bad_ioctl_dev_dir(
+	const stress_args_t *args,
+	const char *path,
+	const int depth)
+{
+	struct dirent **dlist;
+	const mode_t flags = S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
+	int i, n;
+
+	if (!keep_stressing_flag())
+		return;
+
+	/* Don't want to go too deep */
+	if (depth > 20)
+		return;
+
+	dlist = NULL;
+	n = scandir(path, &dlist, NULL, alphasort);
+	if (n <= 0)
+		goto done;
+
+	for (i = 0; i < n; i++) {
+		int ret;
+		struct stat buf;
+		char tmp[PATH_MAX];
+		struct dirent *d = dlist[i];
+		size_t len;
+
+		if (!keep_stressing(args))
+			break;
+		if (stress_is_dot_filename(d->d_name))
+			continue;
+
+		len = strlen(d->d_name);
+
+		/*
+		 *  Exercise no more than 1 of the same device
+		 *  driver, e.g. ttyS0..ttyS1
+		 */
+		if (len > 1) {
+			int dev_n;
+			char *ptr = d->d_name + len - 1;
+
+			while (ptr > d->d_name && isdigit((int)*ptr))
+				ptr--;
+			ptr++;
+			dev_n = atoi(ptr);
+			if (dev_n > 1)
+				continue;
+		}
+
+		(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
+		switch (d->d_type) {
+		case DT_DIR:
+			ret = stat(tmp, &buf);
+			if (ret < 0)
+				continue;
+			if ((buf.st_mode & flags) == 0)
+				continue;
+			stress_bad_ioctl_dev_dir(args, tmp, depth + 1);
+			break;
+		case DT_BLK:
+		case DT_CHR:
+			if (strstr(tmp, "watchdog"))
+				continue;
+			stress_bad_ioctl_dev_new(&dev_ioctl_info_head, tmp);
+			break;
+		default:
+			break;
+		}
+	}
+done:
+	stress_dirent_list_free(dlist, n);
+}
+
 static int stress_bad_ioctl_supported(const char *name)
 {
         if (stress_check_capability(SHIM_CAP_IS_ROOT) ||
@@ -75,13 +202,11 @@ static void NORETURN MLOCKED_TEXT stress_segv_handler(int signum)
  */
 static inline void stress_bad_ioctl_rw(
 	const stress_args_t *args,
-	const bool is_main_process)
+	const bool is_pthread)
 {
 	int ret;
-	char path[PATH_MAX];
 	const double threshold = 0.25;
 	const size_t page_size = args->page_size;
-	NOCLOBBER uint16_t i = 0;
 	uint64_t *buf, *buf_page1;
 	uint8_t *buf8;
 	uint16_t *buf16;
@@ -124,18 +249,20 @@ static inline void stress_bad_ioctl_rw(
 	do {
 		int fd;
 		double t_start;
-		uint8_t type = (i >> 8) & 0xff;
-		uint8_t nr = i & 0xff;
+		uint8_t type, nr;
 		uint64_t rnd = stress_mwc32();
+		volatile dev_ioctl_info_t *node;
 
 		ret = shim_pthread_spin_lock(&lock);
 		if (ret)
 			return;
-		(void)shim_strlcpy(path, dev_path, sizeof(path));
+		node = dev_ioctl_node;
 		(void)shim_pthread_spin_unlock(&lock);
 
-		if (!*path || !keep_stressing_flag())
+		if (!node || !keep_stressing_flag())
 			break;
+		type = (node->ioctl_state >> 8) & 0xff;
+		nr = (node->ioctl_state) & 0xff;
 
 		t_start = stress_time_now();
 
@@ -143,7 +270,7 @@ static inline void stress_bad_ioctl_rw(
 			*ptr ^= rnd;
 		}
 
-		if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
+		if ((fd = open(node->dev_path, O_RDONLY | O_NONBLOCK)) < 0)
 			break;
 
 		ret = sigsetjmp(jmp_env, 1);
@@ -209,8 +336,7 @@ static inline void stress_bad_ioctl_rw(
 		}
 next:
 		(void)close(fd);
-		i++;
-	} while (is_main_process);
+	} while (is_pthread);
 
 	(void)munmap((void *)buf, page_size);
 }
@@ -242,108 +368,38 @@ static void *stress_bad_ioctl_thread(void *arg)
  *  stress_bad_ioctl_dir()
  *	read directory
  */
-static void stress_bad_ioctl_dir(
-	const stress_args_t *args,
-	const char *path,
-	const bool recurse,
-	const int depth)
+static void stress_bad_ioctl_dir(const stress_args_t *args, dev_ioctl_info_t *node, uint32_t offset)
 {
-	struct dirent **dlist;
-	const mode_t flags = S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
-	int i, n;
+	int ret;
 
 	if (!keep_stressing_flag())
 		return;
-
-	/* Don't want to go too deep */
-	if (depth > 20)
+	if (!node)
 		return;
 
-	dlist = NULL;
-	n = scandir(path, &dlist, NULL, alphasort);
-	if (n <= 0)
-		goto done;
-
-	for (i = 0; i < n; i++) {
-		int ret;
-		struct stat buf;
-		char filename[PATH_MAX];
-		char tmp[PATH_MAX];
-		struct dirent *d = dlist[i];
-		size_t len;
-
-		if (!keep_stressing(args))
-			break;
-		if (stress_is_dot_filename(d->d_name))
-			continue;
-
-		len = strlen(d->d_name);
-
-		/*
-		 *  Exercise no more than 3 of the same device
-		 *  driver, e.g. ttyS0..ttyS2
-		 */
-		if (len > 1) {
-			int dev_n;
-			char *ptr = d->d_name + len - 1;
-
-			while (ptr > d->d_name && isdigit((int)*ptr))
-				ptr--;
-			ptr++;
-			dev_n = atoi(ptr);
-			if (dev_n > 2)
-				continue;
-		}
-
-		(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
-		switch (d->d_type) {
-		case DT_DIR:
-			if (!recurse)
-				continue;
-			if (stress_hash_get(dev_hash_table, tmp))
-				continue;
-			ret = stat(tmp, &buf);
-			if (ret < 0) {
-				stress_hash_add(dev_hash_table, tmp);
-				continue;
-			}
-			if ((buf.st_mode & flags) == 0) {
-				stress_hash_add(dev_hash_table, tmp);
-				continue;
-			}
-			inc_counter(args);
-			stress_bad_ioctl_dir(args, tmp, recurse, depth + 1);
-			break;
-		case DT_BLK:
-		case DT_CHR:
-			if (stress_hash_get(dev_hash_table, tmp))
-				continue;
-			if (strstr(tmp, "watchdog")) {
-				stress_hash_add(dev_hash_table, tmp);
-				continue;
-			}
-			ret = stress_try_open(args, tmp, O_RDONLY | O_NONBLOCK, 1500000000);
-			if (ret == STRESS_TRY_OPEN_FAIL) {
-				stress_hash_add(dev_hash_table, tmp);
-				continue;
-			}
+	stress_bad_ioctl_dir(args, node->left, offset);
+	ret = stress_try_open(args, node->dev_path, O_RDONLY | O_NONBLOCK, 15000000);
+	if (ret == STRESS_TRY_OPEN_FAIL) {
+		node->ignore = true;
+	} else if (!node->ignore) {
+		if (offset > 1) {
+			offset--;
+		} else {
 			ret = shim_pthread_spin_lock(&lock);
 			if (!ret) {
-				(void)shim_strlcpy(filename, tmp, sizeof(filename));
-				dev_path = filename;
+				node->ioctl_state += stress_mwc8();
+				dev_ioctl_node = node;
 				(void)shim_pthread_spin_unlock(&lock);
 				stress_bad_ioctl_rw(args, false);
-				inc_counter(args);
 			}
-			break;
-		default:
-			break;
 		}
+		inc_counter(args);
 	}
-done:
-	stress_dirent_list_free(dlist, n);
+	stress_bad_ioctl_dir(args, node->right, offset);
 }
 
+
+
 /*
  *  stress_bad_ioctl
  *	stress read-only ioctls on all of /dev
@@ -354,12 +410,14 @@ static int stress_bad_ioctl(const stress_args_t *args)
 	int ret[MAX_DEV_THREADS], rc = EXIT_SUCCESS;
 	stress_pthread_args_t pa;
 
-	dev_path = "/dev/null";
 	pa.args = args;
 	pa.data = NULL;
 
 	(void)memset(ret, 0, sizeof(ret));
 
+	stress_bad_ioctl_dev_dir(args, "/dev", 0);
+	dev_ioctl_node = dev_ioctl_info_head;
+
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -393,23 +451,17 @@ again:
 		} else if (pid == 0) {
 			size_t i;
 			int r, ssjret;
+			uint32_t offset;
 
 			ssjret = sigsetjmp(jmp_env, 1);
 			if (ssjret != 0) {
-				pr_err("%s: caught an unexpected segmentation fault\n", args->name);
+				pr_fail("%s: caught an unexpected segmentation fault\n", args->name);
 				_exit(EXIT_FAILURE);
 			}
 
 			if (stress_sighandler(args->name, SIGSEGV, stress_segv_handler, NULL) < 0)
 				_exit(EXIT_NO_RESOURCE);
 
-			dev_hash_table = stress_hash_create(251);
-			if (!dev_hash_table) {
-				pr_err("%s: cannot create device hash table: %d (%s))\n",
-					args->name, errno, strerror(errno));
-				_exit(EXIT_NO_RESOURCE);
-			}
-
 			(void)setpgid(0, g_pgrp);
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
@@ -429,15 +481,17 @@ again:
 						stress_bad_ioctl_thread, (void *)&pa);
 			}
 
+			offset = args->instance;
 			do {
-				stress_bad_ioctl_dir(args, "/dev", true, 0);
+				stress_bad_ioctl_dir(args, dev_ioctl_info_head, offset);
+				offset = 0;
 			} while (keep_stressing(args));
 
 			r = shim_pthread_spin_lock(&lock);
 			if (r) {
 				pr_dbg("%s: failed to lock spin lock for dev_path\n", args->name);
 			} else {
-				dev_path = "";
+				dev_ioctl_node = NULL;
 				r = shim_pthread_spin_unlock(&lock);
 				(void)r;
 			}
@@ -446,7 +500,6 @@ again:
 				if (ret[i] == 0)
 					(void)pthread_join(pthreads[i], NULL);
 			}
-			stress_hash_delete(dev_hash_table);
 			_exit(EXIT_SUCCESS);
 		}
 	} while (keep_stressing(args));
@@ -454,6 +507,7 @@ again:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	(void)shim_pthread_spin_destroy(&lock);
+	stress_bad_ioctl_dev_free(dev_ioctl_info_head);
 
 	return rc;
 }
