@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_BIGHEAP_GROWTH	(4 * KB)
+#define MAX_BIGHEAP_GROWTH	(64 * MB)
+#define DEFAULT_BIGHEAP_GROWTH	(64 * KB)
+
 static const stress_help_t help[] = {
 	{ "B N","bigheap N",		"start N workers that grow the heap using calloc()" },
 	{ NULL,	"bigheap-ops N",	"stop after N bogo bigheap operations" },
@@ -161,5 +160,6 @@ stressor_info_t stress_bigheap_info = {
 	.stressor = stress_bigheap,
 	.class = CLASS_OS | CLASS_VM,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
