@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_MOUNT_H)
+#include <sys/mount.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"bind-mount N",	    "start N workers exercising bind mounts" },
 	{ NULL,	"bind-mount-ops N", "stop after N bogo bind mount operations" },
@@ -40,7 +39,7 @@ static const stress_help_t help[] = {
 
 #define CLONE_STACK_SIZE	(128*1024)
 
-static void stress_bind_mount_child_handler(int signum)
+static void NORETURN MLOCKED_TEXT stress_bind_mount_child_handler(int signum)
 {
 	(void)signum;
 
@@ -55,6 +54,8 @@ static void stress_bind_mount_child_handler(int signum)
 static int stress_bind_mount_child(void *parg)
 {
 	const stress_args_t *args = ((stress_pthread_args_t *)parg)->args;
+	char path[PATH_MAX];
+	int ret;
 
 	if (stress_sighandler(args->name, SIGALRM,
 	    stress_bind_mount_child_handler, NULL) < 0) {
@@ -71,25 +72,73 @@ static int stress_bind_mount_child(void *parg)
 	(void)setpgid(0, g_pgrp);
 	stress_parent_died_alarm();
 
-	do {
-		int rc;
+	(void)stress_temp_dir(path, sizeof(path), args->name, getpid(), args->instance);
+	ret = mkdir(path, S_IRUSR | S_IRWXU);
+	if (ret < 0) {
+		pr_err("%s: mkdir %s failed, errno=%d (%s)\n",
+			args->name, path, errno, strerror(errno));
+		return -1;
+	}
 
-		rc = mount("/", "/", "", MS_BIND | MS_REC, 0);
+	do {
+		int rc, retries;
+		DIR *dir;
+		struct dirent *d;
+
+		rc = mount("/", path, "", MS_BIND | MS_REC | MS_RDONLY, 0);
 		if (rc < 0) {
 			if (errno != ENOSPC)
 				pr_fail("%s: bind mount failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			break;
 		}
+
 		/*
-		 * The following fails with -EBUSY, but try it anyhow
-		 *  just to make the kernel work harder
+		 *  Check if we can stat the files in the bound mount path
 		 */
-		(void)umount("/");
+		dir = opendir("/");
+		if (!dir)
+			goto bind_umount;
+		while ((d = readdir(dir)) != NULL) {
+			char bindpath[PATH_MAX + sizeof(d->d_name) + 1];
+			const char *name = d->d_name;
+			struct stat statbuf;
+
+			if (*name == '.')
+				continue;
+
+			(void)snprintf(bindpath, sizeof(bindpath), "%s/%s", path, name);
+			rc = stat(bindpath, &statbuf);
+			if (rc < 0) {
+				pr_fail("%s: failed to stat bind mounted file %s, errno=%d (%s)\n",
+					args->name, bindpath, errno, strerror(errno));
+			}
+		}
+		(void)closedir(dir);
+
+bind_umount:
+		for (retries = 0; retries < 15; retries++) {
+#if defined(HAVE_UMOUNT2) &&	\
+    defined(MNT_DETACH)
+			rc = umount2(path, MNT_DETACH);
+			if (rc == 0)
+				break;
+			(void)shim_usleep(50000);
+#else
+			/*
+			 * The following fails with -EBUSY, but try it anyhow
+			 *  just to make the kernel work harder
+			 */
+			rc = umount(path);
+			if (rc == 0)
+				break;
+#endif
+		}
 		inc_counter(args);
 	} while (keep_stressing_flag() &&
-		 (!args->max_ops || get_counter(args) < args->max_ops));
+		 (!args->max_ops || (get_counter(args) < args->max_ops)));
 
+	(void)rmdir(path);
 	return 0;
 }
 
@@ -101,16 +150,13 @@ static int stress_bind_mount(const stress_args_t *args)
 {
 	int pid = 0, status;
 	stress_pthread_args_t pargs = { args, NULL, 0 };
-	const ssize_t stack_offset =
-		stress_get_stack_direction() *
-		(CLONE_STACK_SIZE - 64);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		int ret;
 		static char stack[CLONE_STACK_SIZE];
-		char *stack_top = stack + stack_offset;
+		char *stack_top = (char *)stress_get_stack_top((void *)stack, CLONE_STACK_SIZE);
 
 		(void)memset(stack, 0, sizeof stack);
 
@@ -137,6 +183,7 @@ static int stress_bind_mount(const stress_args_t *args)
 stressor_info_t stress_bind_mount_info = {
 	.stressor = stress_bind_mount,
 	.class = CLASS_FILESYSTEM | CLASS_OS | CLASS_PATHOLOGICAL,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
