@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_ANDROID_BINDER_H)
+#include <linux/android/binder.h>
+#endif
+
+#if defined(HAVE_LINUX_ANDROID_BINDERFS_H)
+#include <linux/android/binderfs.h>
+#endif
+
+#if defined(HAVE_SYS_MOUNT_H)
+#include <sys/mount.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"binderfs N",		"start N workers exercising binderfs" },
@@ -37,7 +45,7 @@ static const stress_help_t help[] = {
 static int stress_binderfs_supported(const char *name)
 {
 	if (!stress_check_capability(SHIM_CAP_SYS_ADMIN)) {
-		pr_inf("%s stressor will be skipped, "
+		pr_inf_skip("%s stressor will be skipped, "
 			"need to be running with CAP_SYS_ADMIN "
 			"rights for this stressor\n", name);
 		return -1;
@@ -65,13 +73,13 @@ static int stress_binderfs_umount(const stress_args_t *args, const char *pathnam
 			break;
 
 		if (errno != EBUSY) {
-			pr_err("%s: umount failed on binderfs, errno=%d (%s)\n",
+			pr_fail("%s: umount failed on binderfs, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		}
 		t2 = stress_time_now();
 		if (t2 - t1 > UNMOUNT_TIMEOUT) {
-			pr_err("%s: umount failed, timed out trying after %.3f seconds\n",
+			pr_fail("%s: umount failed, timed out trying after %.3f seconds\n",
 				args->name, t2 - t1);
 			return EXIT_FAILURE;
 		}
@@ -116,12 +124,12 @@ static int stress_binderfs(const stress_args_t *args)
 		ret = mount("binder", pathname, "binder", 0, 0);
 		if (ret < 0) {
 			if ((errno == ENODEV) || (errno == ENOSPC) || (errno == ENOMEM)) {
-				pr_inf("%s: mount failed on binderfs at %s, errno=%d (%s), skipping stress test\n",
+				pr_inf_skip("%s: mount failed on binderfs at %s, errno=%d (%s), skipping stress test\n",
 					args->name, pathname, errno, strerror(errno));
 				rc = EXIT_NO_RESOURCE;
 				goto clean;
 			} else {
-				pr_err("%s: mount failed on binderfs at %s, errno=%d (%s)\n",
+				pr_fail("%s: mount failed on binderfs at %s, errno=%d (%s)\n",
 					args->name, pathname, errno, strerror(errno));
 				rc = EXIT_FAILURE;
 				goto clean;
@@ -132,7 +140,7 @@ static int stress_binderfs(const stress_args_t *args)
 			pathname, "binder-control");
 		fd = open(filename, O_RDONLY | O_CLOEXEC);
 		if (fd < 0) {
-			pr_err("%s: cannot open binder control file, errno=%d (%s)\n",
+			pr_fail("%s: cannot open binder control file, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)stress_binderfs_umount(args, pathname);
 			rc = EXIT_FAILURE;
@@ -147,6 +155,8 @@ static int stress_binderfs(const stress_args_t *args)
 				goto close_control;
 		}
 close_control:
+#else
+		UNEXPECTED
 #endif
 		(void)close(fd);
 
