@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -103,6 +98,8 @@ static int stress_brk_child(const stress_args_t *args, void *context)
 #if defined(MCL_FUTURE)
 	if (brk_context->brk_mlock)
 		(void)shim_mlockall(MCL_FUTURE);
+#else
+	UNEXPECTED
 #endif
 
 	do {
@@ -132,8 +129,9 @@ static int stress_brk_child(const stress_args_t *args, void *context)
 
 				ignore = shim_brk(start_ptr);
 				(void)ignore;
+
 			} else {
-				pr_err("%s: sbrk(%d) failed: errno=%d (%s)\n",
+				pr_fail("%s: sbrk(%d) failed: errno=%d (%s)\n",
 					args->name, (int)page_size, errno,
 					strerror(errno));
 				return EXIT_FAILURE;
@@ -142,6 +140,13 @@ static int stress_brk_child(const stress_args_t *args, void *context)
 			/* Touch page, force it to be resident */
 			if (!brk_context->brk_notouch)
 				*(ptr - 1) = 0;
+
+#if defined(HAVE_MADVISE) &&	\
+    defined(MADV_MERGEABLE)
+			(void)madvise(ptr, page_size, MADV_MERGEABLE);
+#else
+			UNEXPECTED
+#endif
 		}
 		inc_counter(args);
 	} while (keep_stressing(args));
