@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_CAPABILITY_H)
+#include <sys/capability.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"cap N",	"start N workers exercising capget" },
 	{ NULL,	"cap-ops N",	"stop cap workers after N bogo capget operations" },
@@ -68,7 +67,44 @@ static int stress_capgetset_pid(
 					args->name, (intmax_t)pid, errno, strerror(errno));
 			}
 		}
+
+		/*
+		 *  Exercise invalid pid -> EPERM
+		 */
+		uch.pid = INT_MIN;
+		ret = capset(&uch, ucd);
+		(void)ret;
+
+		/*
+		 *  Exercise invalid version -> EINVAL
+		 */
+		uch.version = 0x1234dead;
+		uch.pid = pid;
+		ret = capset(&uch, ucd);
+		(void)ret;
 	}
+#else
+	UNEXPECTED
+#endif
+
+	/*
+	 *  Exercise invalid version -> EINVAL
+	 */
+	uch.version = 0x1234dead;
+	uch.pid = pid;
+	ret = capget(&uch, ucd);
+	(void)ret;
+
+#if defined(_LINUX_CAPABILITY_VERSION_3)
+	/*
+	 *  Exercise invalid PID -> EINVAL
+	 */
+	uch.version = _LINUX_CAPABILITY_VERSION_3;
+	uch.pid = -pid;
+	ret = capget(&uch, ucd);
+	(void)ret;
+#else
+	UNEXPECTED
 #endif
 
 #if defined(_LINUX_CAPABILITY_VERSION_3)
@@ -79,6 +115,8 @@ static int stress_capgetset_pid(
 	uch.pid = stress_get_unused_pid_racy(false);
 	ret = capget(&uch, ucd);
 	(void)ret;
+#else
+	UNEXPECTED
 #endif
 
 	/*
@@ -154,6 +192,7 @@ static int stress_cap(const stress_args_t *args)
 stressor_info_t stress_cap_info = {
 	.stressor = stress_cap,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
