@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -90,6 +85,8 @@ static int do_chattr(
 		if (fd < 0)
 			continue;
 
+		if (!keep_stressing(args))
+			goto tidy_fd;
 		ret = ioctl(fd, SHIM_EXT2_IOC_GETFLAGS, &orig);
 		if (ret < 0) {
 			if ((errno != EOPNOTSUPP) &&
@@ -97,9 +94,11 @@ static int do_chattr(
 			    (errno != EINVAL))
 				pr_inf("%s: ioctl SHIM_EXT2_IOC_GETFLAGS failed: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			goto tidy;
+			goto tidy_fd;
 		}
 
+		if (!keep_stressing(args))
+			goto tidy_fd;
 		ret = ioctl(fd, SHIM_EXT2_IOC_SETFLAGS, &zero);
 		if (ret < 0) {
 			rc = -1;
@@ -108,31 +107,41 @@ static int do_chattr(
 			    (errno != EINVAL))
 				pr_inf("%s: ioctl SHIM_EXT2_IOC_SETFLAGS failed: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-			goto tidy;
+			goto tidy_fd;
 		}
 
+		if (!keep_stressing(args))
+			goto tidy_fd;
 		fdw = open(filename, O_RDWR);
 		if (fdw < 0)
-			goto tidy;
+			goto tidy_fd;
 		n = write(fdw, &zero, sizeof(zero));
 		(void)n;
 
+		if (!keep_stressing(args))
+			goto tidy_fdw;
 		ret = ioctl(fd, SHIM_EXT2_IOC_SETFLAGS, &flag);
-		if ((ret < 0) && 
+		if ((ret < 0) &&
                     ((errno == EOPNOTSUPP) ||
                      (errno == ENOTTY) ||
 		     (errno == EINTR)))
 			rc = -1;
 
+		if (!keep_stressing(args))
+			goto tidy_fdw;
 		n = write(fdw, &zero, sizeof(zero));
 		(void)n;
 
+		if (!keep_stressing(args))
+			goto tidy_fdw;
 		ret = ioctl(fd, SHIM_EXT2_IOC_SETFLAGS, &zero);
 		(void)ret;
 
 		/*
 		 *  Try some random flag, exercises any illegal flags
 		 */
+		if (!keep_stressing(args))
+			goto tidy_fdw;
 		rnd = 1ULL << (stress_mwc8() & 0x1f);
 		ret = ioctl(fd, SHIM_EXT2_IOC_SETFLAGS, &rnd);
 		(void)ret;
@@ -140,13 +149,16 @@ static int do_chattr(
 		/*
 		 *  Restore original setting
 		 */
+		if (!keep_stressing(args))
+			goto tidy_fdw;
 		ret = ioctl(fd, SHIM_EXT2_IOC_SETFLAGS, &orig);
 		(void)ret;
+tidy_fdw:
 
 		(void)close(fdw);
-tidy:
+tidy_fd:
 		(void)close(fd);
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		return rc;
 	}
 	return rc;
@@ -189,7 +201,7 @@ static int stress_chattr(const stress_args_t *args)
 
 		if (fail == i) {
 			if (args->instance == 0)
-				pr_inf("%s: chattr not supported on filesystem, skipping stressor\n",
+				pr_inf_skip("%s: chattr not supported on filesystem, skipping stressor\n",
 					args->name);
 			rc = EXIT_NOT_IMPLEMENTED;
 			break;
@@ -199,8 +211,8 @@ static int stress_chattr(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)unlink(filename);
-	(void)rmdir(pathname);
+	(void)shim_unlink(filename);
+	(void)shim_rmdir(pathname);
 
 	return rc;
 }
