@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#define MIN_CHDIR_DIRS		(64)
+#define MAX_CHDIR_DIRS		(65536)
+#define DEFAULT_CHDIR_DIRS	(8192)
 
 static const stress_help_t help[] = {
 	{ NULL, "chdir N",	"start N workers thrashing chdir on many paths" },
@@ -112,6 +112,8 @@ static int stress_chdir(const stress_args_t *args)
 
 #if defined(O_DIRECTORY)
 		flags |= O_DIRECTORY;
+#else
+		UNEXPECTED
 #endif
 		(void)stress_temp_filename_args(args,
 			path, sizeof(path), rnd | gray_code);
@@ -256,7 +258,7 @@ tidy:
 		if (fds[i] >= 0)
 			(void)close(fds[i]);
 		if (paths[i]) {
-			(void)rmdir(paths[i]);
+			(void)shim_rmdir(paths[i]);
 			free(paths[i]);
 		}
 	}
@@ -279,5 +281,6 @@ stressor_info_t stress_chdir_info = {
 	.stressor = stress_chdir,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
