@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LIBGEN_H)
+#include <libgen.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"chmod N",	"start N workers thrashing chmod file mode bits " },
 	{ NULL,	"chmod-ops N",	"stop chmod workers after N bogo operations" },
@@ -207,6 +206,8 @@ static int stress_chmod(const stress_args_t *args)
 	}
 #if defined(O_DIRECTORY)
 	dfd = open(pathname, O_DIRECTORY | O_RDONLY);
+#else
+	UNEXPECTED
 #endif
 
 	stress_strnrnd(longpath, sizeof(longpath));
@@ -289,13 +290,15 @@ tidy:
 #if defined(O_DIRECTORY)
 	if (dfd >= 0)
 		(void)close(dfd);
+#else
+	UNEXPECTED
 #endif
 	if (fd >= 0) {
 		(void)fchmod(fd, 0666);
 		(void)close(fd);
 	}
-	(void)unlink(filename);
-	(void)rmdir(pathname);
+	(void)shim_unlink(filename);
+	(void)shim_rmdir(pathname);
 
 	return rc;
 }
@@ -303,5 +306,6 @@ tidy:
 stressor_info_t stress_chmod_info = {
 	.stressor = stress_chmod,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
