@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -45,24 +40,24 @@ static int do_fchown(
 
 	if (fchown(fd, uid, gid) < 0)
 		return -errno;
-	if (fchown(fd, -1, gid) < 0)
+	if (fchown(fd, (uid_t)-1, gid) < 0)
 		return -errno;
-	if (fchown(fd, uid, -1) < 0)
+	if (fchown(fd, uid, (gid_t)-1) < 0)
 		return -errno;
-	if (fchown(fd, -1, -1) < 0)
+	if (fchown(fd, (uid_t)-1, (gid_t)-1) < 0)
 		return -errno;
 
 	if (cap_chown)
 		return 0;
-	if (fchown(fd, 0, 0) == 0)
+	if (fchown(fd, (uid_t)0, (gid_t)0) == 0)
 		goto restore;
 	if (errno != EPERM)
 		goto restore;
-	if ((fchown(fd, -1, 0) == 0) && (errno != EPERM))
+	if ((fchown(fd, (uid_t)-1, (gid_t)0) == 0) && (errno != EPERM))
 		goto restore;
 	if (errno != EPERM)
 		goto restore;
-	if (fchown(fd, 0, -1) == 0)
+	if (fchown(fd, (uid_t)0, (gid_t)-1) == 0)
 		goto restore;
 	if (errno != EPERM)
 		goto restore;
@@ -98,24 +93,24 @@ static int do_chown(
 
 	if (chown_func(filename, uid, gid) < 0)
 		return -errno;
-	if (chown_func(filename, -1, gid) < 0)
+	if (chown_func(filename, (uid_t)-1, gid) < 0)
 		return -errno;
-	if (chown_func(filename, uid, -1) < 0)
+	if (chown_func(filename, uid, (gid_t)-1) < 0)
 		return -errno;
-	if (chown_func(filename, -1, -1) < 0)
+	if (chown_func(filename, (uid_t)-1, (gid_t)-1) < 0)
 		return -errno;
 
 	if (cap_chown)
 		return 0;
-	if (chown_func(filename, 0, 0) == 0)
+	if (chown_func(filename, (uid_t)0, (gid_t)0) == 0)
 		goto restore;
 	if (errno != EPERM)
 		goto restore;
-	if ((chown_func(filename, -1, 0) == 0) && (errno != EPERM))
+	if ((chown_func(filename, (uid_t)-1, (gid_t)0) == 0) && (errno != EPERM))
 		goto restore;
 	if (errno != EPERM)
 		goto restore;
-	if (chown_func(filename, 0, -1) == 0)
+	if (chown_func(filename, (uid_t)0, (gid_t)-1) == 0)
 		goto restore;
 	if (errno != EPERM)
 		goto restore;
@@ -209,7 +204,7 @@ static int stress_chown(const stress_args_t *args)
 
 		ret = do_chown(chown, filename, cap_chown, uid, gid);
 		if (ret < 0) {
-			if (ret == -ENOENT || ret == -ENOTDIR) {
+			if ((ret == -ENOENT) || (ret == -ENOTDIR)) {
 				/*
 				 * File was removed during test by
 				 * another worker
@@ -223,7 +218,7 @@ static int stress_chown(const stress_args_t *args)
 		}
 		ret = do_chown(lchown, filename, cap_chown, uid, gid);
 		if (ret < 0) {
-			if (ret == -ENOENT || ret == -ENOTDIR) {
+			if ((ret == -ENOENT) || (ret == -ENOTDIR)) {
 				/*
 				 * File was removed during test by
 				 * another worker
@@ -244,8 +239,8 @@ tidy:
 
 	if (fd >= 0)
 		(void)close(fd);
-	(void)unlink(filename);
-	(void)rmdir(pathname);
+	(void)shim_unlink(filename);
+	(void)shim_rmdir(pathname);
 
 	return rc;
 }
@@ -253,5 +248,6 @@ tidy:
 stressor_info_t stress_chown_info = {
 	.stressor = stress_chown,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
