@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#endif
+
+#if defined(HAVE_SYS_TIMEX_H)
+#include <sys/timex.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"clock N",	"start N workers thrashing clocks and POSIX timers" },
@@ -30,6 +34,14 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
+#if defined(HAVE_SYS_TIMEX_H)
+#define shim_timex      timex
+#else
+struct shim_timex {
+	char data[128];
+};
+#endif
+
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_CLOCK_GETTIME) &&	\
     defined(HAVE_CLOCK_SETTIME)
@@ -151,6 +163,20 @@ static inline bool check_invalid_clock_id(const clockid_t id) {
 
 #define FD_TO_CLOCKID(fd)	((~(clockid_t)(fd) << 3) | 3)
 
+#if defined(__NR_clock_adjtime) &&	\
+    defined(HAVE_SYS_TIMEX_H) &&	\
+    defined(CLOCK_THREAD_CPUTIME_ID) &&	\
+    defined(ADJ_SETOFFSET)
+/*
+ *   shim_clock_adjtime
+ *	wrapper for linux clock_adjtime system call
+ */
+static int shim_clock_adjtime(clockid_t clk_id, struct shim_timex *tx)
+{
+	return (int)syscall(__NR_clock_adjtime, clk_id, tx);
+}
+#endif
+
 /*
  *  stress_clock()
  *	stress system by rapid clocking system calls
@@ -162,7 +188,7 @@ static int stress_clock(const stress_args_t *args)
 	 * test run to ensure predictable repeatable
 	 * 'random' sleep duration timings
 	 */
-	stress_mwc_seed(0xf238, 0x1872);
+	stress_mwc_set_seed(0xf238, 0x1872);
 	bool test_invalid_timespec = true;
 	const bool is_root = stress_check_capability(SHIM_CAP_IS_ROOT);
 
@@ -202,6 +228,8 @@ static int stress_clock(const stress_args_t *args)
 			ret = clock_settime(CLOCK_THREAD_CPUTIME_ID, &t);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_CLOCK_GETRES)
@@ -233,6 +261,8 @@ static int stress_clock(const stress_args_t *args)
 						args->name, clocks[i].name, errno, strerror(errno));
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_CLOCK_GETTIME) &&	\
@@ -302,6 +332,8 @@ static int stress_clock(const stress_args_t *args)
 				}
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_CLOCK_NANOSLEEP) &&	\
@@ -322,7 +354,6 @@ static int stress_clock(const stress_args_t *args)
 					(void)ret_st;
 				}
 
-				/* Exercise clock_adjtime on non-permitted timespec object values */
 				(void)memset(&t, 0, sizeof(t));
 				t.tv_sec = -1;
 				ret_st = clock_nanosleep(clocks_nanosleep[0], TIMER_ABSTIME, &t, NULL);
@@ -354,10 +385,13 @@ static int stress_clock(const stress_args_t *args)
 						errno, strerror(errno));
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(__NR_clock_adjtime) &&	\
     defined(HAVE_SYS_TIMEX_H) &&	\
+    defined(CLOCK_THREAD_CPUTIME_ID) &&	\
     defined(ADJ_SETOFFSET)
 		{
 			size_t i;
@@ -393,6 +427,8 @@ static int stress_clock(const stress_args_t *args)
 				}
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 
@@ -478,6 +514,8 @@ static int stress_clock(const stress_args_t *args)
 				}
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(__linux__)
@@ -497,6 +535,8 @@ static int stress_clock(const stress_args_t *args)
 
 				ret = poll(pollfds, 1, 0);
 				(void)ret;
+#else
+				UNEXPECTED
 #endif
 				ret = clock_gettime(clkid, &t);
 				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
@@ -511,6 +551,8 @@ static int stress_clock(const stress_args_t *args)
 					pr_fail("%s: clock_getres failed for /dev/ptp0, errno=%d (%s)",
 					args->name, errno, strerror(errno));
 				}
+#else
+				UNEXPECTED
 #endif
 				(void)close(fd);
 			}
@@ -527,6 +569,7 @@ static int stress_clock(const stress_args_t *args)
 stressor_info_t stress_clock_info = {
 	.stressor = stress_clock,
 	.class = CLASS_INTERRUPT | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
