@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_MODIFY_LDT)
+#include <asm/ldt.h>
+#endif
+
+#define MIN_CLONES		(1)
+#define MAX_CLONES		(1000000)
+#define DEFAULT_CLONES		(8192)
+
 #define CLONE_STACK_SIZE	(8*1024)
 
 #if defined(HAVE_CLONE)
@@ -63,30 +66,6 @@ static stress_clone_list_t clones;
  */
 static const uint64_t flags[] = {
 	0,
-#if defined(CLONE_FILES)
-	CLONE_FILES,
-#endif
-#if defined(CLONE_FS)
-	CLONE_FS,
-#endif
-#if defined(CLONE_IO)
-	CLONE_IO,
-#endif
-#if defined(CLONE_NEWIPC)
-	CLONE_NEWIPC,
-#endif
-#if defined(CLONE_NEWNET)
-	CLONE_NEWNET,
-#endif
-#if defined(CLONE_NEWNS)
-	CLONE_NEWNS,
-#endif
-#if defined(CLONE_NEWUSER)
-	CLONE_NEWUSER,
-#endif
-#if defined(CLONE_NEWUTS)
-	CLONE_NEWUTS,
-#endif
 /*
  * Avoid CLONE_VM for now as child may memory clobber parent
 #if defined(CLONE_SIGHAND) && 	\
@@ -94,21 +73,87 @@ static const uint64_t flags[] = {
 	CLONE_SIGHAND | CLONE_VM,
 #endif
  */
-#if defined(CLONE_SYSVSEM)
-	CLONE_SYSVSEM,
+#if defined(CLONE_FS)
+	CLONE_FS,
 #endif
-#if defined(CLONE_UNTRACED)
-	CLONE_UNTRACED,
+#if defined(CLONE_FILES)
+	CLONE_FILES,
+#endif
+#if defined(CLONE_SIGHAND)
+	CLONE_SIGHAND,
 #endif
 #if defined(CLONE_PIDFD)
 	CLONE_PIDFD,
 #endif
+/*
+#if defined(CLONE_PTRACE)
+	CLONE_PTRACE,
+#endif
+*/
+/*
+#if defined(CLONE_VFORK)
+	CLONE_VFORK,
+#endif
+*/
+#if defined(CLONE_PARENT)
+	CLONE_PARENT,
+#endif
+#if defined(CLONE_THREAD)
+	CLONE_THREAD,
+#endif
+#if defined(CLONE_NEWNS)
+	CLONE_NEWNS,
+#endif
+#if defined(CLONE_SYSVSEM)
+	CLONE_SYSVSEM,
+#endif
+/*
+#if defined(CLONE_SETTLS)
+	CLONE_SETTLS,
+#endif
+*/
+#if defined(CLONE_PARENT_SETTID)
+	CLONE_PARENT_SETTID,
+#endif
+#if defined(CLONE_CHILD_CLEARTID)
+	CLONE_CHILD_CLEARTID,
+#endif
+#if defined(CLONE_DETACHED)
+	CLONE_DETACHED,
+#endif
+#if defined(CLONE_UNTRACED)
+	CLONE_UNTRACED,
+#endif
+#if defined(CLONE_CHILD_SETTID)
+	CLONE_CHILD_SETTID,
+#endif
 #if defined(CLONE_NEWCGROUP)
 	CLONE_NEWCGROUP,
 #endif
+#if defined(CLONE_NEWUTS)
+	CLONE_NEWUTS,
+#endif
+#if defined(CLONE_NEWIPC)
+	CLONE_NEWIPC,
+#endif
+#if defined(CLONE_NEWUSER)
+	CLONE_NEWUSER,
+#endif
+#if defined(CLONE_NEWPID)
+	CLONE_NEWPID,
+#endif
+#if defined(CLONE_NEWNET)
+	CLONE_NEWNET,
+#endif
+#if defined(CLONE_IO)
+	CLONE_IO,
+#endif
 #if defined(CLONE_CLEAR_SIGHAND)
 	CLONE_CLEAR_SIGHAND,
 #endif
+#if defined(CLONE_INTO_CGROUP)
+	CLONE_INTO_CGROUP,
+#endif
 #if defined(CLONE_NEWTIME)
 	CLONE_NEWTIME,
 #endif
@@ -152,7 +197,7 @@ static int stress_set_clone_max(const char *opt)
 
 	clone_max = stress_get_uint32(opt);
 	stress_check_range("clone-max", clone_max,
-		MIN_ZOMBIES, MAX_ZOMBIES);
+		MIN_CLONES, MAX_CLONES);
 	return stress_set_setting("clone-max", TYPE_ID_UINT32, &clone_max);
 }
 
@@ -280,6 +325,8 @@ static int clone_func(void *arg)
 			(void)close(fd);
 		}
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_MODIFY_LDT) &&	\
@@ -288,8 +335,14 @@ static int clone_func(void *arg)
 		struct user_desc ud;
 
 		(void)memset(&ud, 0, sizeof(ud));
-		if (shim_modify_ldt(0, &ud, sizeof(ud)) == 0)
+		if (shim_modify_ldt(0, &ud, sizeof(ud)) == 0) {
 			(void)shim_modify_ldt(1, &ud, sizeof(ud));
+			/* Exercise invalid size */
+			(void)shim_modify_ldt(1, &ud, 1);
+			/* Exercise invalid entries */
+			ud.entry_number = ~0;
+			(void)shim_modify_ldt(1, &ud, sizeof(ud));
+		}
 
 		(void)memset(&ud, 0, sizeof(ud));
 		if (shim_modify_ldt(0, &ud, sizeof(ud)) == 0) {
@@ -301,6 +354,10 @@ static int clone_func(void *arg)
 
 		/* Exercise invalid command */
 		(void)shim_modify_ldt(0xff, &ud, sizeof(ud));
+
+		/* Exercise invalid ldt size */
+		(void)memset(&ud, 0, sizeof(ud));
+		(void)shim_modify_ldt(0, &ud, 0);
 	}
 #endif
 	for (i = 0; i < SIZEOF_ARRAY(unshare_flags); i++) {
@@ -314,7 +371,7 @@ static int stress_clone_child(const stress_args_t *args, void *context)
 {
 	/* Child */
 	uint32_t max_clones = 0;
-	uint32_t clone_max = DEFAULT_ZOMBIES;
+	uint32_t clone_max = DEFAULT_CLONES;
 	bool use_clone3 = true;
 	const size_t mmap_size = args->page_size * 32768;
 	void *ptr;
@@ -328,9 +385,9 @@ static int stress_clone_child(const stress_args_t *args, void *context)
 
 	if (!stress_get_setting("clone-max", &clone_max)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			clone_max = MAX_ZOMBIES;
+			clone_max = MAX_CLONES;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			clone_max = MIN_ZOMBIES;
+			clone_max = MIN_CLONES;
 	}
 
 	/*
