@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,37 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(__NR_userfaultfd)
+#define HAVE_USERFAULTFD
+#endif
+
+#if defined(HAVE_SYS_EPOLL_H)
+#include <sys/epoll.h>
+#endif
+
+#if defined(HAVE_SYS_EVENTFD_H)
+#include <sys/eventfd.h>
+#endif
+
+#if defined(HAVE_SYS_FANOTIFY_H)
+#include <sys/fanotify.h>
+#endif
+
+#if defined(HAVE_SYS_INOTIFY_H)
+#include <sys/inotify.h>
+#endif
+
+#if defined(HAVE_LINUX_NETLINK_H)
+#include <linux/netlink.h>
+#endif
+
+#if defined(HAVE_SYS_SIGNALFD_H)
+#include <sys/signalfd.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"close N",	"start N workers that exercise races on close" },
@@ -143,6 +167,8 @@ static void *stress_close_func(void *arg)
 		 */
 		if (fcntl((int)fd_rnd, F_GETFL) == -1)
 			(void)close(fd_rnd);
+#else
+		UNEXPECTED
 #endif
 		/*
 		 *  close a range of fds
@@ -228,8 +254,10 @@ static int stress_close(const stress_args_t *args)
 			pr_err("%s: cannot create %s\n", args->name, filename);
 			return exit_status(errno);
 		}
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 	}
+#else
+	UNEXPECTED
 #endif
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -340,6 +368,9 @@ static int stress_close(const stress_args_t *args)
 #if defined(HAVE_FCHOWNAT)
 				ret = fchownat(fd, "", uid, gid, 0);
 				(void)ret;
+
+				ret = fchownat(fd, "", uid, gid, ~0);
+				(void)ret;
 #endif
 				ret = fchown(fd, uid, gid);
 				(void)ret;
@@ -371,6 +402,8 @@ static int stress_close(const stress_args_t *args)
 				(void)close(ret);
 				goto tidy;
 			}
+#else
+			UNEXPECTED
 #endif
 			ret = fstat(fd, &statbuf);
 			(void)ret;
@@ -411,6 +444,8 @@ tidy:
 	if (file_fd >= 0)
 		(void)close(file_fd);
 	(void)stress_temp_dir_rm_args(args);
+#else
+	UNEXPECTED
 #endif
 
 	return rc;
