@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_UCONTEXT_H)
+#include <ucontext.h>
+#endif
+
 static stress_help_t help[] = {
 	{ NULL,	"context N",	 "start N workers exercising user context" },
 	{ NULL,	"context-ops N", "stop context workers after N bogo operations" },
@@ -54,33 +53,33 @@ typedef struct {
 
 static context_info_t context[3];
 static ucontext_t uctx_main;
-static uint64_t __counter, __max_ops;
+static uint64_t context_counter;
+static uint64_t stress_max_ops;
 
 static void thread1(void)
 {
 	do {
-		__counter++;
+		context_counter++;
 		(void)swapcontext(&context[0].cu.uctx, &context[1].cu.uctx);
-	} while (keep_stressing_flag() && (!__max_ops || (__counter < __max_ops)));
-
+	} while (keep_stressing_flag() && (!stress_max_ops || (context_counter < stress_max_ops)));
 	(void)swapcontext(&context[0].cu.uctx, &uctx_main);
 }
 
 static void thread2(void)
 {
 	do {
-		__counter++;
+		context_counter++;
 		(void)swapcontext(&context[1].cu.uctx, &context[2].cu.uctx);
-	} while (keep_stressing_flag() && (!__max_ops || (__counter < __max_ops)));
+	} while (keep_stressing_flag() && (!stress_max_ops || (context_counter < stress_max_ops)));
 	(void)swapcontext(&context[1].cu.uctx, &uctx_main);
 }
 
 static void thread3(void)
 {
 	do {
-		__counter++;
+		context_counter++;
 		(void)swapcontext(&context[2].cu.uctx, &context[0].cu.uctx);
-	} while (keep_stressing_flag() && (!__max_ops || (__counter < __max_ops)));
+	} while (keep_stressing_flag() && (!stress_max_ops || (context_counter < stress_max_ops)));
 	(void)swapcontext(&context[2].cu.uctx, &uctx_main);
 }
 
@@ -90,8 +89,10 @@ static int stress_context_init(
 	ucontext_t *uctx_link,
 	context_info_t *context_info)
 {
+	(void)memset(context_info, 0, sizeof(*context_info));
+
 	if (getcontext(&context_info->cu.uctx) < 0) {
-		pr_err("%s: getcontext failed: %d (%s)\n",
+		pr_fail("%s: getcontext failed: %d (%s)\n",
 			args->name, errno, strerror(errno));
 		return -1;
 	}
@@ -116,22 +117,13 @@ static int stress_context_init(
  */
 static int stress_context(const stress_args_t *args)
 {
-	uint8_t *stack_sig;
 	size_t i;
 
-	stack_sig = (uint8_t *)mmap(NULL, STRESS_SIGSTKSZ, PROT_READ | PROT_WRITE,
-			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
-	if (stack_sig == MAP_FAILED) {
-		pr_inf("%s: cannot allocate signal stack, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
-		return EXIT_NO_RESOURCE;
-	}
+	(void)memset(&uctx_main, 0, sizeof(uctx_main));
+	(void)memset(context, 0, sizeof(context));
 
-	if (stress_sigaltstack(stack_sig, STRESS_SIGSTKSZ) < 0)
-		return EXIT_FAILURE;
-
-	__counter = 0;
-	__max_ops = args->max_ops * 1000;
+	context_counter = 0;
+	stress_max_ops = args->max_ops * 1000;
 
 	/* Create 3 micro threads */
 	if (stress_context_init(args, thread1, &uctx_main, &context[0]) < 0)
@@ -143,12 +135,12 @@ static int stress_context(const stress_args_t *args)
 
 	/* And start.. */
 	if (swapcontext(&uctx_main, &context[0].cu.uctx) < 0) {
-		pr_err("%s: swapcontext failed: %d (%s)\n",
+		pr_fail("%s: swapcontext failed: %d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
-	set_counter(args, __counter / 1000);
+	set_counter(args, context_counter / 1000);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -163,7 +155,6 @@ static int stress_context(const stress_args_t *args)
 		}
 	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)munmap((void *)stack_sig, STRESS_SIGSTKSZ);
 
 	return EXIT_SUCCESS;
 }
@@ -171,6 +162,7 @@ static int stress_context(const stress_args_t *args)
 stressor_info_t stress_context_info = {
 	.stressor = stress_context,
 	.class = CLASS_MEMORY | CLASS_CPU,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
