@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_COPY_FILE_BYTES	(128 * MB)
+#define MAX_COPY_FILE_BYTES	(MAX_FILE_LIMIT)
+#define DEFAULT_COPY_FILE_BYTES	(256 * MB)
+#define DEFAULT_COPY_FILE_SIZE	(2 * MB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"copy-file N",		"start N workers that copy file data" },
 	{ NULL,	"copy-file-ops N",	"stop after N copy bogo operations" },
@@ -49,6 +49,52 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 
 #if defined(HAVE_COPY_FILE_RANGE)
 
+/*
+ *  stress_copy_file_range_verify()
+ *	verify copy file from fd_in to fd_out worked correctly for
+ *	--verify option
+ */
+static int stress_copy_file_range_verify(
+	const int fd_in,
+	off_t *off_in,
+	const int fd_out,
+	off_t *off_out,
+	const ssize_t bytes)
+{
+	ssize_t n, bytes_in, bytes_out, bytes_left = bytes;
+	ssize_t buf_sz = STRESS_MINIMUM(bytes, 1024);
+	off_t off_ret;
+
+
+	while (bytes_left >= 0) {
+		char buf_in[buf_sz], buf_out[buf_sz];
+
+		off_ret = lseek(fd_in, *off_in, SEEK_SET);
+		if (off_ret != *off_in)
+			return -1;
+		off_ret = lseek(fd_out, *off_out, SEEK_SET);
+		if (off_ret != *off_out)
+			return -1;
+
+		bytes_in = read(fd_in, buf_in, sizeof(buf_in));
+		if (bytes_in <= 0)
+			break;
+		bytes_out = read(fd_out, buf_out, sizeof(buf_out));
+		if (bytes_out <= 0)
+			break;
+
+		n = STRESS_MINIMUM(bytes_in, bytes_out);
+		if (memcmp(buf_in, buf_out, n) != 0)
+			return -1;
+		bytes_left -= n;
+		*off_in += n;
+		*off_out += n;
+	}
+	if (bytes_left > 0)
+		return -1;
+	return 0;
+}
+
 /*
  *  stress_copy_file
  *	stress reading chunks of file using copy_file_range()
@@ -56,14 +102,15 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 static int stress_copy_file(const stress_args_t *args)
 {
 	int fd_in, fd_out, rc = EXIT_FAILURE, ret;
+	const int fd_bad = stress_get_bad_fd();
 	char filename[PATH_MAX - 5], tmp[PATH_MAX];
 	uint64_t copy_file_bytes = DEFAULT_COPY_FILE_BYTES;
 
 	if (!stress_get_setting("copy-file-bytes", &copy_file_bytes)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			copy_file_bytes = MAX_HDD_BYTES;
+			copy_file_bytes = MAX_COPY_FILE_BYTES;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			copy_file_bytes = MIN_HDD_BYTES;
+			copy_file_bytes = MIN_COPY_FILE_BYTES;
 	}
 
 	copy_file_bytes /= args->num_instances;
@@ -85,7 +132,7 @@ static int stress_copy_file(const stress_args_t *args)
 			args->name, tmp, errno, strerror(errno));
 		goto tidy_dir;
 	}
-	(void)unlink(tmp);
+	(void)shim_unlink(tmp);
 	if (ftruncate(fd_in, (off_t)copy_file_bytes) < 0) {
 		rc = exit_status(errno);
 		pr_fail("%s: ftruncated failed, errno=%d (%s)\n",
@@ -99,22 +146,24 @@ static int stress_copy_file(const stress_args_t *args)
 	}
 
 	(void)snprintf(tmp, sizeof(tmp), "%s-copy", filename);
-	if ((fd_out = open(tmp, O_CREAT | O_WRONLY,  S_IRUSR | S_IWUSR)) < 0) {
+	if ((fd_out = open(tmp, O_CREAT | O_RDWR,  S_IRUSR | S_IWUSR)) < 0) {
 		rc = exit_status(errno);
 		pr_fail("%s: open %s failed, errno=%d (%s)\n",
 			args->name, tmp, errno, strerror(errno));
 		goto tidy_in;
 	}
-	(void)unlink(tmp);
+	(void)shim_unlink(tmp);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		ssize_t copy_ret;
-		shim_loff_t off_in, off_out;
+		shim_loff_t off_in, off_out, off_in_orig, off_out_orig;
 
-		off_in = (shim_loff_t)(stress_mwc64() % (copy_file_bytes - DEFAULT_COPY_FILE_SIZE));
-		off_out = (shim_loff_t)(stress_mwc64() % (copy_file_bytes - DEFAULT_COPY_FILE_SIZE));
+		off_in_orig = (shim_loff_t)(stress_mwc64() % (copy_file_bytes - DEFAULT_COPY_FILE_SIZE));
+		off_in = off_in_orig;
+		off_out_orig = (shim_loff_t)(stress_mwc64() % (copy_file_bytes - DEFAULT_COPY_FILE_SIZE));
+		off_out = off_out_orig;
 
 		copy_ret = shim_copy_file_range(fd_in, &off_in, fd_out,
 						&off_out, DEFAULT_COPY_FILE_SIZE, 0);
@@ -135,6 +184,34 @@ static int stress_copy_file(const stress_args_t *args)
 				args->name, errno, strerror(errno));
 			goto tidy_out;
 		}
+		if (g_opt_flags & OPT_FLAGS_VERIFY) {
+			copy_ret = stress_copy_file_range_verify(fd_in, &off_in_orig,
+					fd_out, &off_out_orig, DEFAULT_COPY_FILE_SIZE);
+			if (copy_ret < 0) {
+				pr_fail("%s: copy_file_range verify failed, input offset=%jd, output offset=%jd\n",
+					args->name, (intmax_t)off_in_orig, off_out_orig);
+			}
+		}
+
+		/*
+		 *  Exercise with bad fds
+		 */
+		copy_ret = shim_copy_file_range(fd_bad, &off_in, fd_out,
+						&off_out, DEFAULT_COPY_FILE_SIZE, 0);
+		(void)copy_ret;
+		copy_ret = shim_copy_file_range(fd_in, &off_in, fd_bad,
+						&off_out, DEFAULT_COPY_FILE_SIZE, 0);
+		(void)copy_ret;
+		copy_ret = shim_copy_file_range(fd_out, &off_in, fd_in,
+						&off_out, DEFAULT_COPY_FILE_SIZE, 0);
+		(void)copy_ret;
+		/*
+		 *  Exercise with bad flags
+		 */
+		copy_ret = shim_copy_file_range(fd_in, &off_in, fd_out,
+						&off_out, DEFAULT_COPY_FILE_SIZE, ~0);
+		(void)copy_ret;
+
 		(void)shim_fsync(fd_out);
 		inc_counter(args);
 	} while (keep_stressing(args));
@@ -157,6 +234,7 @@ stressor_info_t stress_copy_file_info = {
 	.stressor = stress_copy_file,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
