@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -30,6 +25,8 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
+#define STRESS_CPU_ONLINE_MAX_CPUS	(65536)
+
 #if defined(__linux__)
 
 /*
@@ -89,7 +86,7 @@ static int stress_cpu_online_supported(const char *name)
  */
 static int stress_cpu_online(const stress_args_t *args)
 {
-	const int32_t cpus = stress_get_processors_configured();
+	int32_t cpus = stress_get_processors_configured();
 	int32_t i, cpu_online_count = 0;
 	bool *cpu_online;
 	int rc = EXIT_SUCCESS;
@@ -102,11 +99,17 @@ static int stress_cpu_online(const stress_args_t *args)
 		return EXIT_SUCCESS;
 	}
 
-	if ((cpus < 1) || (cpus > 65536)) {
-		pr_inf("%s: too few or too many CPUs (found %" PRId32 ")\n",
+	if (cpus < 1) {
+		pr_fail("%s: too few CPUs (detected %" PRId32 ")\n",
 			args->name, cpus);
 		return EXIT_FAILURE;
 	}
+	if (cpus > STRESS_CPU_ONLINE_MAX_CPUS) {
+		pr_inf("%s: more than %" PRId32 " CPUs detected, "
+			"limiting to %d\n",
+			args->name, cpus, STRESS_CPU_ONLINE_MAX_CPUS);
+		cpus = STRESS_CPU_ONLINE_MAX_CPUS;
+	}
 
 	cpu_online = calloc((size_t)cpus, sizeof(*cpu_online));
 	if (!cpu_online) {
@@ -194,6 +197,7 @@ stressor_info_t stress_cpu_online_info = {
 	.stressor = stress_cpu_online,
 	.supported = stress_cpu_online_supported,
 	.class = CLASS_CPU | CLASS_OS | CLASS_PATHOLOGICAL,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
