@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,30 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-cpu.h"
+#include "core-put.h"
+#include "core-target-clones.h"
+
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
+
+#if defined(HAVE_COMPLEX_H)
+#include <complex.h>
+#endif
+
+/*
+ *  S390x on QEMU (and maybe H/W) trips SIGILL for decimal
+ *  math with some compilers, so disable this for now
+ */
+#if defined(STRESS_ARCH_S390)
+#undef HAVE_FLOAT_DECIMAL32
+#undef HAVE_FLOAT_DECIMAL64
+#undef HAVE_FLOAT_DECIMAL128
+#endif
 
 #define GAMMA 		(0.57721566490153286060651209008240243104215933593992L)
 #define OMEGA		(0.56714329040978387299996866221035554975381578718651L)
@@ -38,6 +55,15 @@
 #define CORRELATE_LEN		(CORRELATE_DATA_LEN / 16)
 #define SIEVE_SIZE              (104730)
 
+/* do nothing */
+#if defined(HAVE_ASM_NOP)
+#define FORCE_DO_NOTHING() __asm__ __volatile__("nop;")
+#elif defined(HAVE_ASM_NOTHING)
+#define FORCE_DO_NOTHING() __asm__ __volatile__("")
+#else
+#define FORCE_DO_NOTHING() while (0)
+#endif
+
 /*
  * Some math workarounds for functions that some
  * math libraries don't have implemented (yet)
@@ -602,7 +628,7 @@ static void HOT OPTIMIZE3 stress_cpu_rand(const char *name)
 	uint32_t i_sum = 0;
 	const uint32_t sum = 0xc253698c;
 
-	STRESS_MWC_SEED();
+	stress_mwc_seed();
 	for (i = 0; i < 16384; i++)
 		i_sum += stress_mwc32();
 
@@ -611,6 +637,37 @@ static void HOT OPTIMIZE3 stress_cpu_rand(const char *name)
 			"pseudo-random values\n", name);
 }
 
+/*
+ *  stress_cpu_logmap()
+ *	Compute logistic map of x = r * x * (x - 1.0) where r is the
+ * 	accumulation point based on A098587. Data is scaled in the
+ *	range 0..255
+ */
+static void HOT OPTIMIZE3 stress_cpu_logmap(const char *name)
+{
+	static double x = 0.4;
+	/*
+	 * Use an accumulation point that is slightly larger
+	 * than the point where chaotic behaviour starts
+	 */
+	const double r = 3.569945671870944901842 * 1.0999999;
+	register int i;
+
+	(void)name;
+
+	for (i = 0; i < 16384; i++) {
+		/*
+		 *  Scale up a fractional part of x
+		 *  by an arbitrary value and take
+		 *  the bottom 8 bits of the result
+		 *  to make a quasi-chaotic random-ish
+		 *  value
+		 */
+		x = x * r * (1.0 - x);
+	}
+	stress_double_put(x);
+}
+
 /*
  *  stress_cpu_rand48()
  *	generate random values using rand48 family of functions
@@ -799,6 +856,8 @@ static void HOT TARGET_CLONES stress_cpu_fft(const char *name)
 	(void)memcpy(tmp, buf, sizeof(*tmp) * FFT_SIZE);
 	fft_partial(buf, tmp, FFT_SIZE, 1);
 }
+#else
+	UNEXPECTED
 #endif
 
 /*
@@ -863,146 +922,6 @@ static void HOT OPTIMIZE3 TARGET_CLONES stress_cpu_collatz(const char *name)
 			name);
 }
 
-/*
- *  stress_cpu_hash_generic()
- *	stress test generic string hash function
- */
-static void stress_cpu_hash_generic(
-	const char *name,
-	const char *hash_name,
-	uint32_t (*hash_func)(const char *str),
-	const uint32_t result)
-{
-	char buffer[128];
-	size_t i;
-	uint32_t i_sum = 0;
-
-	STRESS_MWC_SEED();
-	random_buffer((uint8_t *)buffer, sizeof(buffer));
-	/* Make it ASCII range ' '..'_' */
-	for (i = 0; i < sizeof(buffer); i++)
-		buffer[i] = (buffer[i] & 0x3f) + ' ';
-
-	for (i = sizeof(buffer) - 1; i; i--) {
-		buffer[i] = '\0';
-		i_sum += hash_func(buffer);
-	}
-	if ((g_opt_flags & OPT_FLAGS_VERIFY) && (i_sum != result))
-		pr_fail("%s: %s error detected, failed hash %s sum\n",
-			name, hash_name, hash_name);
-}
-
-/*
- *  stress_cpu_jenkin()
- *	multiple iterations on jenkin hash
- */
-static void stress_cpu_jenkin(const char *name)
-{
-	uint8_t buffer[128];
-	size_t i;
-	uint32_t i_sum = 0;
-	const uint32_t sum = 0xc53302a5;
-
-	STRESS_MWC_SEED();
-	random_buffer(buffer, sizeof(buffer));
-
-	for (i = sizeof(buffer) - 1; i; i--) {
-		buffer[i] = '\0';
-		i_sum += stress_hash_jenkin(buffer, sizeof(buffer));
-	}
-
-	if ((g_opt_flags & OPT_FLAGS_VERIFY) && (i_sum != sum))
-		pr_fail("%s: jenkin error detected, failed hash jenkin sum\n",
-			name);
-}
-
-/*
- *  stress_cpu_little_endian()
- *	returns true if CPU is little endian
- */
-static inline bool stress_cpu_little_endian(void)
-{
-	const uint32_t x = 0x12345678;
-	const uint8_t *y = (const uint8_t *)&x;
-
-	return *y == 0x78;
-}
-
-/*
- *  stress_cpu_murmur3_32
- *	 multiple iterations on murmur3_32 hash, based on
- *	 Austin Appleby's Murmur3 hash, code derived from
- *	 https://en.wikipedia.org/wiki/MurmurHash
- */
-static void stress_cpu_murmur3_32(const char *name)
-{
-	uint8_t buffer[128];
-	size_t i;
-	uint32_t sum, i_sum = 0;
-	const uint32_t seed = 0xf12b35e1; /* arbitrary value */
-
-	STRESS_MWC_SEED();
-	random_buffer(buffer, sizeof(buffer));
-	for (i = sizeof(buffer) - 1; i; i--) {
-		buffer[i] = '\0';
-		i_sum += stress_hash_murmur3_32((uint8_t *)buffer, sizeof(buffer), seed);
-	}
-
-	/*
-	 *  Murmur produces different results depending on the Endianness
-	 */
-	sum = stress_cpu_little_endian() ? 0xa53a4bb1 : 0x71eb83cc;
-
-	if ((g_opt_flags & OPT_FLAGS_VERIFY) && (i_sum != sum))
-		pr_fail("%s: murmur3_32 error detected, failed hash murmur3_32 sum\n",
-			name);
-}
-
-/*
- *  stress_cpu_pjw()
- *	stress test hash pjw
- */
-static void stress_cpu_pjw(const char *name)
-{
-	stress_cpu_hash_generic(name, "pjw", stress_hash_pjw, 0xa89a91c0);
-}
-
-/*
- *  stress_cpu_djb2a()
- *	stress test hash djb2a
- */
-static void stress_cpu_djb2a(const char *name)
-{
-	stress_cpu_hash_generic(name, "djb2a", stress_hash_djb2a, 0x6a60cb5a);
-}
-
-/*
- *  stress_cpu_fnv1a()
- *	stress test hash fnv1a
- */
-static void HOT stress_cpu_fnv1a(const char *name)
-{
-	stress_cpu_hash_generic(name, "fnv1a", stress_hash_fnv1a, 0x8ef17e80);
-}
-
-/*
- *  stress_cpu_sdbm()
- *	stress test hash sdbm
- */
-static void stress_cpu_sdbm(const char *name)
-{
-	stress_cpu_hash_generic(name, "sdbm", stress_hash_sdbm, 0x46357819);
-}
-
-/*
- *  stress_cpu_nhash()
- *	stress test hash nhash
- */
-static void stress_cpu_nhash(const char *name)
-{
-	stress_cpu_hash_generic(name, "nhash", stress_hash_nhash, 0x1cc86e3);
-}
-
 /*
  *  stress_cpu_idct()
  *	compute 8x8 Inverse Discrete Cosine Transform
@@ -1111,7 +1030,7 @@ static void HOT OPTIMIZE3 TARGET_CLONES stress_cpu_int ## _sz(const char *name)\
 	register _type a, b;					\
 	int i;							\
 								\
-	STRESS_MWC_SEED();					\
+	stress_mwc_seed();					\
 	a = (_type)stress_mwc32();				\
 	b = (_type)stress_mwc32();				\
 								\
@@ -1153,24 +1072,32 @@ stress_cpu_int(uint8_t, 8, \
 	0x12, 0x1a,
 	C1, C2, C3)
 
+#define float_thresh(x, _type)	x = (_type)		\
+	((fabs((double)x) > 1.0) ?	\
+	((_type)(0.1 + (double)x - (long)x)) :	\
+	((_type)(x)))
+
 #define float_ops(_type, a, b, c, d, _sin, _cos)	\
 	do {						\
 		a = a + b;				\
 		b = a * c;				\
 		c = a - b;				\
-		d = a / b;				\
-		a = c / (_type)0.1923L;			\
+		d = a / (_type)8.1;			\
+		float_thresh(d, _type);			\
+		a = c / (_type)5.1923;			\
+		float_thresh(a, _type);			\
+		float_thresh(c, _type);			\
 		b = c + a;				\
-		c = b * (_type)3.12L;			\
+		c = b * (_type)_sin(b);			\
 		d = d + b + (_type)_sin(a);		\
-		a = (b + c) / c;			\
+		a = (_type)_cos(b + c);			\
 		b = b * c;				\
-		c = c + (_type)1.0L;			\
+		c = c + (_type)1.5;			\
 		d = d - (_type)_sin(c);			\
 		a = a * (_type)_cos(b);			\
 		b = b + (_type)_cos(c);			\
-		c = (_type)_sin(a + b) / (_type)2.344L;	\
-		b = d - (_type)1.0L;			\
+		c = (_type)_sin(a + b) / (_type)2.344;	\
+		b = d - (_type)0.5;			\
 	} while (0)
 
 /*
@@ -1183,8 +1110,8 @@ static void HOT OPTIMIZE3 TARGET_CLONES stress_cpu_ ## _name(const char *name)\
 	const uint32_t r1 = stress_mwc32(),		\
 		       r2 = stress_mwc32();		\
 	_type a = (_type)0.18728L, 			\
-	      b = (_type)r1,				\
-	      c = (_type)r2,				\
+	      b = (_type)((double)r1 / 65536.0),	\
+	      c = (_type)((double)r2 / 65536.0),	\
 	      d = (_type)0.0,				\
 	      r;					\
 							\
@@ -1253,9 +1180,9 @@ static void HOT OPTIMIZE3 TARGET_CLONES stress_cpu_ ## _name(const char *name)\
 	_type cI = (_type)I;					\
 	_type a = FP(0.18728, _ltype) + 			\
 		cI * FP(0.2762, _ltype),			\
-		b = (_type)r1 - cI * FP(0.11121, _ltype),	\
-		c = (_type)r2 + cI * stress_mwc32(), 		\
-		d = (_type)0.0,					\
+		b = (_type)((double)r1/(double)(1UL<<31)) - cI * FP(0.11121, _ltype),	\
+		c = (_type)((double)r2/(double)(1UL<<31)) + cI * stress_mwc32(),	\
+		d = (_type)0.5,					\
 		r;						\
 								\
 	(void)name;						\
@@ -1343,7 +1270,7 @@ static void HOT OPTIMIZE3 TARGET_CLONES stress_cpu_int ## _sz ## _ ## _name(cons
 	       flt_d = (_ftype)0.0,				\
 	       flt_r;						\
 								\
-	STRESS_MWC_SEED();					\
+	stress_mwc_seed();					\
 	int_a = stress_mwc32();					\
 	int_b = stress_mwc32();					\
 								\
@@ -1815,6 +1742,8 @@ static void stress_cpu_zeta(const char *name)
 		stress_long_double_put((long double)z);
 	}
 }
+#else
+	UNEXPECTED
 #endif
 
 /*
@@ -2648,11 +2577,9 @@ static void TARGET_CLONES stress_cpu_div64(const char *name)
 /*
  *  stress_cpu_cpuid()
  *	get CPU id info, x86 only
+ *	see https://en.wikipedia.org/wiki/CPUID
  */
-#if defined(HAVE_CPUID_H) &&	\
-    defined(STRESS_ARCH_X86) &&	\
-    defined(HAVE_CPUID) &&	\
-    NEED_GNUC(4,6,0)
+#if defined(STRESS_ARCH_X86)
 static void TARGET_CLONES stress_cpu_cpuid(const char *name)
 {
 	register int i;
@@ -2663,54 +2590,156 @@ static void TARGET_CLONES stress_cpu_cpuid(const char *name)
 		uint32_t eax, ebx, ecx, edx;
 
 		/*  Highest Function Parameter and Manufacturer ID */
-		__cpuid(0x00, eax, ebx, ecx, edx);
+		eax = 0;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
 
 		/* Processor Info and Feature Bits */
-		__cpuid(0x01, eax, ebx, ecx, edx);
+		eax = 1;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
 
 		/*  Cache and TLB Descriptor information */
-		__cpuid(0x02, eax, ebx, ecx, edx);
+		eax = 2;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
 
 		/* Processor Serial Number */
-		__cpuid(0x03, eax, ebx, ecx, edx);
+		eax = 3;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
 
 		/* Intel thread/core and cache topology */
-		__cpuid(0x04, eax, ebx, ecx, edx);
+		eax = 4;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
-		__cpuid(0x0b, eax, ebx, ecx, edx);
+
+		/* Thermal and power management */
+		eax = 6;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
+		stress_uint32_put(eax);
+
+		/* Extended Features */
+		eax = 6;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Must be 0 */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
+		stress_uint32_put(eax);
+
+		/* Extended Features */
+		eax = 7;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Must be 0 */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
+		stress_uint32_put(eax);
+
+		/* Extended Features */
+		eax = 7;
+		ebx = 0; /* Not required */
+		ecx = 1; /* Must be 1 */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
+		stress_uint32_put(eax);
+
+		/* Intel thread/core and cache topology */
+		eax = 0xb;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
 
 		/* Get highest extended function index */
-		__cpuid(0x80000000, eax, ebx, ecx, edx);
+		eax = 0x80000000;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
 
 		/* Extended processor info */
-		__cpuid(0x80000001, eax, ebx, ecx, edx);
+		eax = 0x80000001;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
 
 		/* Processor brand string */
-		__cpuid(0x80000002, eax, ebx, ecx, edx);
+		eax = 0x80000002;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
-		__cpuid(0x80000003, eax, ebx, ecx, edx);
+
+		/* Processor brand string */
+		eax = 0x80000003;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
-		__cpuid(0x80000004, eax, ebx, ecx, edx);
+
+		/* Processor brand string */
+		eax = 0x80000004;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
 		stress_uint32_put(eax);
 
 		/* L1 Cache and TLB Identifiers */
-		__cpuid(0x80000005, eax, ebx, ecx, edx);
+		eax = 0x80000005;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
+		stress_uint32_put(eax);
 
 		/* Extended L2 Cache Features */
-		__cpuid(0x80000006, eax, ebx, ecx, edx);
+		eax = 0x80000006;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
+		stress_uint32_put(eax);
 
 		/* Advanced Power Management information */
-		__cpuid(0x80000007, eax, ebx, ecx, edx);
+		eax = 0x80000007;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
+		stress_uint32_put(eax);
 
 		/* Virtual and Physical address size */
-		__cpuid(0x80000008, eax, ebx, ecx, edx);
+		eax = 0x80000008;
+		ebx = 0; /* Not required */
+		ecx = 0; /* Not required */
+		edx = 0; /* Not required */
+		stress_x86_cpuid(&eax, &ebx, &ecx, &edx);
+		stress_uint32_put(eax);
 	}
 }
 #endif
@@ -2973,13 +3002,12 @@ static const stress_cpu_method_info_t cpu_methods[] = {
 	{ "cdouble",		stress_cpu_complex_double },
 	{ "cfloat",		stress_cpu_complex_float },
 	{ "clongdouble",	stress_cpu_complex_long_double },
+#else
+	UNEXPECTED
 #endif
 	{ "collatz",		stress_cpu_collatz },
 	{ "correlate",		stress_cpu_correlate },
-#if defined(HAVE_CPUID_H) &&	\
-    defined(STRESS_ARCH_X86) &&	\
-    defined(HAVE_CPUID) &&	\
-    NEED_GNUC(4,6,0)
+#if defined(STRESS_ARCH_X86)
 	{ "cpuid",		stress_cpu_cpuid },
 #endif
 	{ "crc16",		stress_cpu_crc16 },
@@ -2999,7 +3027,6 @@ static const stress_cpu_method_info_t cpu_methods[] = {
 	{ "div16",		stress_cpu_div16 },
 	{ "div32",		stress_cpu_div32 },
 	{ "div64",		stress_cpu_div64 },
-	{ "djb2a",		stress_cpu_djb2a },
 	{ "double",		stress_cpu_double },
 	{ "euler",		stress_cpu_euler },
 	{ "explog",		stress_cpu_explog },
@@ -3034,7 +3061,6 @@ static const stress_cpu_method_info_t cpu_methods[] = {
 	{ "float128",		stress_cpu_float128 },
 #endif
 	{ "floatconversion",	stress_cpu_floatconversion },
-	{ "fnv1a",		stress_cpu_fnv1a },
 	{ "gamma",		stress_cpu_gamma },
 	{ "gcd",		stress_cpu_gcd },
 	{ "gray",		stress_cpu_gray },
@@ -3074,28 +3100,24 @@ static const stress_cpu_method_info_t cpu_methods[] = {
 	{ "int32longdouble",	stress_cpu_int32_longdouble },
 	{ "intconversion",	stress_cpu_intconversion },
 	{ "ipv4checksum",	stress_cpu_ipv4checksum },
-	{ "jenkin",		stress_cpu_jenkin },
 	{ "jmp",		stress_cpu_jmp },
 	{ "lfsr32",		stress_cpu_lfsr32 },
 	{ "ln2",		stress_cpu_ln2 },
+	{ "logmap",		stress_cpu_logmap },
 	{ "longdouble",		stress_cpu_longdouble },
 	{ "loop",		stress_cpu_loop },
 	{ "matrixprod",		stress_cpu_matrix_prod },
-	{ "murmur3_32",		stress_cpu_murmur3_32 },
-	{ "nhash",		stress_cpu_nhash },
 	{ "nsqrt",		stress_cpu_nsqrt },
 	{ "omega",		stress_cpu_omega },
 	{ "parity",		stress_cpu_parity },
 	{ "phi",		stress_cpu_phi },
 	{ "pi",			stress_cpu_pi },
-	{ "pjw",		stress_cpu_pjw },
 	{ "prime",		stress_cpu_prime },
 	{ "psi",		stress_cpu_psi },
 	{ "queens",		stress_cpu_queens },
 	{ "rand",		stress_cpu_rand },
 	{ "rand48",		stress_cpu_rand48 },
 	{ "rgb",		stress_cpu_rgb },
-	{ "sdbm",		stress_cpu_sdbm },
 	{ "sieve",		stress_cpu_sieve },
 	{ "stats",		stress_cpu_stats },
 	{ "sqrt", 		stress_cpu_sqrt },
@@ -3305,5 +3327,6 @@ stressor_info_t stress_cpu_info = {
 	.set_default = stress_cpu_set_default,
 	.class = CLASS_CPU,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
