@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_CRYPT_H)
+#include <crypt.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"crypt N",	"start N workers performing password encryption" },
 	{ NULL,	"crypt-ops N",	"stop after N bogo crypt operations" },
@@ -33,6 +32,20 @@ static const stress_help_t help[] = {
 #if defined(HAVE_LIB_CRYPT) &&	\
     defined(HAVE_CRYPT_H)
 
+typedef struct {
+	const char id;
+	const char *method;
+} crypt_method_t;
+
+static const crypt_method_t crypt_methods[] = {
+	{ '1', "MD5" },
+	{ '5', "SHA-256" },
+	{ '6', "SHA-512" },
+	{ '7', "scrypt" },
+	{ '3', "NT" },
+	{ 'y', "yescrypt" },
+};
+
 /*
  *  stress_crypt_id()
  *	crypt a password with given seed and id
@@ -50,13 +63,30 @@ static int stress_crypt_id(
 	static struct crypt_data data;
 
 	(void)memset(&data, 0, sizeof(data));
+	errno = 0;
 	encrypted = crypt_r(passwd, salt, &data);
 #else
 	encrypted = crypt(passwd, salt);
 #endif
 	if (!encrypted) {
-		pr_fail("%s: cannot encrypt with %s\n", args->name, method);
-		return -1;
+		switch (errno) {
+		case 0:
+			break;
+		case EINVAL:
+			break;
+#if defined(ENOSYS)
+		case ENOSYS:
+			break;
+#endif
+#if defined(EOPNOTSUPP)
+		case EOPNOTSUPP:
+#endif
+			break;
+		default:
+			pr_fail("%s: cannot encrypt with %s, errno=%d (%s)\n",
+				args->name, method, errno, strerror(errno));
+			return -1;
+		}
 	}
 	return 0;
 }
@@ -76,7 +106,7 @@ static int stress_crypt(const stress_args_t *args)
 		char passwd[16];
 		char salt[] = "$x$........";
 		uint64_t seed[2];
-		size_t i;
+		size_t i, failed = 0;
 
 		seed[0] = stress_mwc64();
 		seed[1] = stress_mwc64();
@@ -87,15 +117,20 @@ static int stress_crypt(const stress_args_t *args)
 			passwd[i] = seedchars[stress_mwc32() % sizeof(seedchars)];
 		passwd[i] = '\0';
 
-		if (stress_crypt_id(args, '1', "MD5", passwd, salt) < 0)
+		for (i = 0; keep_stressing(args) && (i < SIZEOF_ARRAY(crypt_methods)); i++) {
+			int ret;
+
+			ret = stress_crypt_id(args,
+					      crypt_methods[i].id,
+					      crypt_methods[i].method,
+					      passwd, salt);
+			if (ret < 0)
+				failed++;
+			else
+				inc_counter(args);
+		}
+		if (failed)
 			break;
-#if NEED_GLIBC(2,7,0)
-		if (stress_crypt_id(args, '5', "SHA-256", passwd, salt) < 0)
-			break;
-		if (stress_crypt_id(args, '6', "SHA-512", passwd, salt) < 0)
-			break;
-#endif
-		inc_counter(args);
 	} while (keep_stressing(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -106,6 +141,7 @@ static int stress_crypt(const stress_args_t *args)
 stressor_info_t stress_crypt_info = {
 	.stressor = stress_crypt,
 	.class = CLASS_CPU,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
