@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 #define DEFAULT_DELAY_NS	(100000)
 #define MAX_SAMPLES		(10000)
@@ -146,6 +142,8 @@ static void stress_cyclic_stats(
 
 	rt_stats->ns += (double)delta_ns;
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_CLOCK_GETTIME) &&	\
@@ -173,6 +171,8 @@ static int stress_cyclic_clock_nanosleep(
 		stress_cyclic_stats(rt_stats, cyclic_sleep, &t1, &t2);
 	return 0;
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_CLOCK_GETTIME)	&&	\
@@ -200,6 +200,8 @@ static int stress_cyclic_posix_nanosleep(
 		stress_cyclic_stats(rt_stats, cyclic_sleep, &t1, &t2);
 	return 0;
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_CLOCK_GETTIME)
@@ -244,6 +246,8 @@ static int stress_cyclic_poll(
 	}
 	return 0;
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_PSELECT) &&		\
@@ -271,6 +275,8 @@ static int stress_cyclic_pselect(
 		stress_cyclic_stats(rt_stats, cyclic_sleep, &t1, &t2);
 	return 0;
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_CLOCK_GETTIME) &&	\
@@ -346,6 +352,8 @@ restore:
 	stress_sigrestore(args->name, SIGRTMIN, &old_action);
 	return ret;
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_CLOCK_GETTIME)
@@ -371,6 +379,8 @@ static int stress_cyclic_usleep(
 		stress_cyclic_stats(rt_stats, cyclic_sleep, &t1, &t2);
 	return 0;
 }
+#else
+	UNEXPECTED
 #endif
 
 static sigjmp_buf jmp_env;
@@ -675,8 +685,13 @@ static int stress_cyclic(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+again:
 	pid = fork();
 	if (pid < 0) {
+		if (stress_redo_fork(errno))
+			goto again;
+		if (!keep_stressing(args))
+			goto finish;
 		pr_inf("%s: cannot fork, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
@@ -852,6 +867,7 @@ tidy:
 		}
 	}
 
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	(void)munmap((void *)rt_stats, size);
