@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -29,9 +24,11 @@
 #elif defined(_NSIG)
 #define MAX_SIGNUM      _NSIG
 #else
-#define MAX_SIGNUM      256
+#define MAX_SIGNUM      (256)
 #endif
 
+#define MAX_BACKOFF	(10000)
+
 static const stress_help_t help[] = {
 	{ NULL,	"daemon N",	"start N workers creating multiple daemons" },
 	{ NULL,	"daemon-ops N",	"stop when N daemons have been created" },
@@ -84,8 +81,8 @@ static void daemons(const stress_args_t *args, const int fd)
 				/* Minor backoff before retrying */
 				(void)shim_usleep_interruptible(backoff);
 				backoff += 100;
-				if (backoff > 10000)
-					backoff = 10000;
+				if (backoff > MAX_BACKOFF)
+					backoff = MAX_BACKOFF;
 				continue;
 			}
 			goto tidy;
@@ -142,15 +139,12 @@ static int stress_daemon(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
-	if (!keep_stressing(args))
-		return EXIT_SUCCESS;
-
 	pid = fork();
 	if (pid < 0) {
-		if ((errno == ENOMEM) || (errno == EAGAIN)) {
-			(void)shim_usleep_interruptible(100);
+		if (stress_redo_fork(errno))
 			goto again;
-		}
+		if (!keep_stressing(args))
+			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fds[0]);
@@ -184,6 +178,7 @@ again:
 		} while (keep_stressing(args));
 	}
 
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return EXIT_SUCCESS;
