@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,18 +16,29 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
 
-#define DCCP_OPT_SEND		0x01
-#define DCCP_OPT_SENDMSG	0x02
-#define DCCP_OPT_SENDMMSG	0x03
+#if defined(HAVE_SYS_UN_H)
+#include <sys/un.h>
+#endif
+
+#if defined(HAVE_LINUX_SOCKIOS_H)
+#include <linux/sockios.h>
+#endif
+
+#include <netinet/in.h>
+
+#define MIN_DCCP_PORT		(1024)
+#define MAX_DCCP_PORT		(65535)
+#define DEFAULT_DCCP_PORT	(10000)
+
+#define DCCP_OPT_SEND		(0x01)
+#define DCCP_OPT_SENDMSG	(0x02)
+#define DCCP_OPT_SENDMMSG	(0x03)
+
+#define DCCP_BUF		(1024)	/* DCCP I/O buffer size */
 
 #define MSGVEC_SIZE		(4)
 
@@ -196,7 +208,7 @@ retry:
 	if (dccp_domain == AF_UNIX) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
 	/* Inform parent we're all done */
@@ -287,7 +299,6 @@ static int stress_dccp_server(
 			struct iovec vec[sizeof(buf)/16];
 #if defined(HAVE_SENDMMSG)
 			struct mmsghdr msgvec[MSGVEC_SIZE];
-			unsigned int msg_len = 0;
 #endif
 			len = sizeof(saddr);
 			if (getsockname(fd, &saddr, &len) < 0) {
@@ -343,7 +354,6 @@ again:
 				for (j = 0, i = 16; i < sizeof(buf); i += 16, j++) {
 					vec[j].iov_base = buf;
 					vec[j].iov_len = i;
-					msg_len += i;
 				}
 				for (i = 0; i < MSGVEC_SIZE; i++) {
 					msgvec[i].msg_hdr.msg_iov = vec;
@@ -388,7 +398,7 @@ die:
 	if (addr && (dccp_domain == AF_UNIX)) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
 
@@ -452,6 +462,7 @@ stressor_info_t stress_dccp_info = {
 	.stressor = stress_dccp,
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
