@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,20 +16,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
-#define ORDER_FORWARD	0x00
-#define ORDER_REVERSE	0x01
-#define ORDER_STRIDE	0x02
-#define ORDER_RANDOM	0x03
-#define ORDER_NONE	0x04
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
+
+#if defined(HAVE_UTIME_H)
+#include <utime.h>
+#endif
+
+#define MIN_DENTRIES		(1)
+#define MAX_DENTRIES		(1000000)
+#define DEFAULT_DENTRIES	(2048)
+
+#define ORDER_FORWARD		(0x00)
+#define ORDER_REVERSE		(0x01)
+#define ORDER_STRIDE		(0x02)
+#define ORDER_RANDOM		(0x03)
+#define ORDER_NONE		(0x04)
 
 typedef struct {
 	const char *name;
@@ -113,7 +120,7 @@ static void stress_dentry_unlink(
 
 			stress_temp_filename_args(args,
 				path, sizeof(path), gray_code * 2);
-			(void)unlink(path);
+			(void)shim_unlink(path);
 		}
 		break;
 	case ORDER_STRIDE:
@@ -125,7 +132,7 @@ static void stress_dentry_unlink(
 
 			stress_temp_filename_args(args,
 				path, sizeof(path), gray_code * 2);
-			(void)unlink(path);
+			(void)shim_unlink(path);
 		}
 		break;
 	case ORDER_FORWARD:
@@ -136,12 +143,37 @@ static void stress_dentry_unlink(
 
 			stress_temp_filename_args(args,
 				path, sizeof(path), gray_code * 2);
-			(void)unlink(path);
+			(void)shim_unlink(path);
 		}
 		break;
 	}
 }
 
+/*
+ *  stress_dentry_state()
+ *	determined the number of cached dentries
+ */
+static void stress_dentry_state(int64_t *nr_dentry)
+{
+#if defined(__linux__)
+	FILE *fp;
+	int n;
+
+	fp = fopen("/proc/sys/fs/dentry-state", "r");
+	if (!fp)
+		goto err;
+	n = fscanf(fp, "%" SCNd64, nr_dentry);
+	(void)fclose(fp);
+
+	if (n != 1)
+		goto err;
+	return;
+err:
+#endif
+	*nr_dentry = 0ULL;
+	return;
+}
+
 /*
  *  stress_dentry_misc()
  *	misc ways to exercise a directory file
@@ -152,11 +184,9 @@ static void stress_dentry_misc(const char *path)
 	ssize_t sret;
 	off_t offset;
 	struct stat statbuf;
-	struct timeval timeout;
 #if defined(HAVE_UTIME_H)
 	struct utimbuf utim;
 #endif
-	fd_set rdfds;
 	char buf[1024];
 	void *ptr;
 
@@ -215,12 +245,19 @@ static void stress_dentry_misc(const char *path)
 	}
 #endif
 
-	FD_ZERO(&rdfds);
-	FD_SET(fd, &rdfds);
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 0;
-	ret = select(fd + 1, &rdfds, NULL, NULL, &timeout);
-	(void)ret;
+#if defined(HAVE_SYS_SELECT_H)
+	{
+		struct timeval timeout;
+		fd_set rdfds;
+
+		FD_ZERO(&rdfds);
+		FD_SET(fd, &rdfds);
+		timeout.tv_sec = 0;
+		timeout.tv_usec = 0;
+		ret = select(fd + 1, &rdfds, NULL, NULL, &timeout);
+		(void)ret;
+	}
+#endif
 
 #if defined(HAVE_FLOCK) &&	\
     defined(LOCK_EX) &&		\
@@ -283,6 +320,7 @@ static int stress_dentry(const stress_args_t *args)
 	uint64_t dentry_offset = dentries;
 	uint8_t dentry_order = ORDER_RANDOM;
 	char dir_path[PATH_MAX];
+	int64_t nr_dentry1, nr_dentry2, nr_dentries;
 
 	if (!stress_get_setting("dentries", &dentries)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -300,6 +338,7 @@ static int stress_dentry(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	stress_dentry_state(&nr_dentry1);
 	do {
 		uint64_t i, n = dentries;
 		char path[PATH_MAX];
@@ -367,6 +406,12 @@ static int stress_dentry(const stress_args_t *args)
 	} while (keep_stressing(args));
 
 abort:
+	stress_dentry_state(&nr_dentry2);
+	nr_dentries = nr_dentry2 - nr_dentry1;
+	if ((args->instance == 0) && (nr_dentries > 0)) {
+		pr_inf("%s: %" PRId64 " dentries allocated\n",
+			args->name, nr_dentries);
+	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	/* force unlink of all files */
