@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_DIR_DIRS		(64)
+#define MAX_DIR_DIRS		(65536)
+#define DEFAULT_DIR_DIRS	(8192)
+
+#if defined(HAVE_LIBGEN_H)
+#include <libgen.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"dir N",	"start N directory thrashing stressors" },
 	{ NULL,	"dir-ops N",	"stop after N directory bogo operations" },
@@ -178,7 +181,7 @@ static void stress_dir_tidy(
 
 		(void)stress_temp_filename_args(args,
 			path, sizeof(path), gray_code);
-		(void)rmdir(path);
+		(void)shim_rmdir(path);
 	}
 }
 
@@ -228,9 +231,26 @@ static void stress_invalid_mkdir(const char *path)
 	(void)stress_strnrnd(filename + len, sizeof(filename) - len);
 	ret = mkdir(filename,  S_IRUSR | S_IWUSR);
 	if (ret == 0)
-		(void)rmdir(filename);
+		(void)shim_rmdir(filename);
 }
 
+/*
+ *  stress_invalid_mkdirat()
+ *	exercise invalid mkdirat fd
+ */
+static void stress_invalid_mkdirat(const int bad_fd)
+{
+#if defined(HAVE_MKDIRAT)
+	int ret;
+
+	ret = mkdirat(bad_fd, "bad", S_IRUSR | S_IWUSR);
+	(void)ret;
+#else
+	(void)bad_fd;
+#endif
+}
+
+
 /*
  *  stress_invalid_rmdir()
  *	exercise invalid rmdir paths
@@ -243,16 +263,16 @@ static void stress_invalid_rmdir(const char *path)
 	(void)shim_strlcpy(filename, path, sizeof(filename));
 	/* remove . - exercise EINVAL error */
 	(void)shim_strlcat(filename, "/.", sizeof(filename));
-	ret = rmdir(filename);
+	ret = shim_rmdir(filename);
 	(void)ret;
 
 	/* remove /.. - exercise ENOTEMPTY error */
 	(void)shim_strlcat(filename, ".", sizeof(filename));
-	ret = rmdir(filename);
+	ret = shim_rmdir(filename);
 	(void)ret;
 
 	/* remove / - exercise EBUSY error */
-	ret = rmdir("/");
+	ret = shim_rmdir("/");
 	(void)ret;
 }
 
@@ -266,6 +286,7 @@ static int stress_dir(const stress_args_t *args)
 	uint64_t dir_dirs = DEFAULT_DIR_DIRS;
 	char pathname[PATH_MAX];
 	int dir_fd = -1;
+	const int bad_fd = stress_get_bad_fd();
 
 	stress_temp_dir(pathname, sizeof(pathname), args->name, args->pid, args->instance);
 	(void)stress_get_setting("dir-dirs", &dir_dirs);
@@ -292,6 +313,7 @@ static int stress_dir(const stress_args_t *args)
 
 			(void)stress_temp_filename_args(args,
 				path, sizeof(path), gray_code);
+			(void)shim_force_rmdir(path);
 			if (stress_mkdir(dir_fd, path, S_IRUSR | S_IWUSR) < 0) {
 				if ((errno != ENOSPC) &&
 				    (errno != ENOMEM) &&
@@ -305,6 +327,7 @@ static int stress_dir(const stress_args_t *args)
 		}
 		stress_invalid_mkdir(pathname);
 		stress_invalid_rmdir(pathname);
+		stress_invalid_mkdirat(bad_fd);
 
 		if (!keep_stressing(args)) {
 			stress_dir_tidy(args, i);
@@ -325,7 +348,7 @@ static int stress_dir(const stress_args_t *args)
 	{
 		int rmret;
 
-		rmret = rmdir("");
+		rmret = shim_rmdir("");
 		(void)rmret;
 	}
 
@@ -349,5 +372,6 @@ stressor_info_t stress_dir_info = {
 	.stressor = stress_dir,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
