@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -32,6 +27,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
+/* digits and uppercase for very short directory names */
+static const char stress_dir_names[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+
 /*
  *  stress_set_dirdeep_dirs()
  *      set number of dirdeep directories from given option string
@@ -42,7 +40,7 @@ static int stress_set_dirdeep_dirs(const char *opt)
 
 	dirdeep_dirs = stress_get_uint32(opt);
 
-	stress_check_range("dirdeep-dirs", dirdeep_dirs, 1, 10);
+	stress_check_range("dirdeep-dirs", dirdeep_dirs, 1, strlen(stress_dir_names));
 	return stress_set_setting("dirdeep-dirs", TYPE_ID_UINT32, &dirdeep_dirs);
 }
 
@@ -65,7 +63,7 @@ static int stress_set_dirdeep_inodes(const char *opt)
  *	depth-first tree creation, create lots of sub-trees with
  *	dirdeep_dir number of subtress per level.
  */
-static void stress_dirdeep_make(
+static bool stress_dirdeep_make(
 	const stress_args_t *args,
 	const char *linkpath,
 	char *const path,
@@ -73,8 +71,9 @@ static void stress_dirdeep_make(
 	const size_t path_len,
 	const uint32_t dirdeep_dirs,
 	const uint64_t dirdeep_inodes,
-	const uint64_t inodes_target_free,
-	uint64_t *min_inodes_free,
+	const uint64_t inodes_start,
+	uint64_t *inodes_estimate,
+	uint64_t *inodes_min,
 	uint32_t depth)
 {
 	uint32_t i;
@@ -85,28 +84,33 @@ static void stress_dirdeep_make(
 #endif
 	const uint64_t inodes_avail = stress_get_filesystem_available_inodes();
 
-	if (*min_inodes_free > inodes_avail)
-		*min_inodes_free = inodes_avail;
-
-	if (inodes_avail <= inodes_target_free)
-		return;
+	if ((inodes_avail == 0) || (inodes_start == 0)) {
+		if (*inodes_estimate > dirdeep_inodes)
+			return true;
+	} else {
+		if (inodes_avail < *inodes_min)
+			*inodes_min = inodes_avail;
+		if (inodes_start - inodes_avail > dirdeep_inodes)
+			return true;
+	}
 	if (len + 2 >= path_len)
-		return;
+		return true;
 	if (!keep_stressing(args))
-		return;
+		return true;
 
 	errno = 0;
 	if (mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR) < 0) {
 		if ((errno == ENOSPC) || (errno == ENOMEM) ||
 		    (errno == ENAMETOOLONG) || (errno == EDQUOT) ||
 		    (errno == EMLINK) || (errno == EPERM)) {
-			return;
+			return true;
 		}
-		pr_fail("%s: mkdir %s failed, errno=%d (%s)\n",
-			args->name, path, errno, strerror(errno));
-		return;
+		pr_fail("%s: mkdir failed, errno=%d (%s)\n",
+			args->name, errno, strerror(errno));
+		return true;
 	}
 	inc_counter(args);
+	(*inodes_estimate)++;
 
 	/*
 	 *  Top level, create file to symlink and link to
@@ -118,7 +122,7 @@ static void stress_dirdeep_make(
 		if (fd < 0) {
 			pr_fail("%s: create %s failed, errno=%d (%s)\n",
 				args->name, linkpath, errno, strerror(errno));
-			return;
+			return true;
 		}
 		(void)close(fd);
 	}
@@ -128,21 +132,22 @@ static void stress_dirdeep_make(
 	path[len + 1] = 's';	/* symlink */
 	path[len + 2] = '\0';
 	ret = symlink(linkpath, path);
-	(void)ret;
+	if (ret == 0)
+		(*inodes_estimate)++;
 
 	path[len + 1] = 'h';	/* hardlink */
 	ret = link(linkpath, path);
 	(void)ret;
 
 	for (i = 0; keep_stressing(args) && (i < dirdeep_dirs); i++) {
-		path[len + 1] = (char)('0' + i);
-		stress_dirdeep_make(args, linkpath, path, len + 2,
+		path[len + 1] = stress_dir_names[i];
+		if (stress_dirdeep_make(args, linkpath, path, len + 2,
 				path_len, dirdeep_dirs, dirdeep_inodes,
-				inodes_target_free, min_inodes_free,
-				depth + 1);
+				inodes_start, inodes_estimate, inodes_min, depth + 1))
+			break;
 	}
 	if (!keep_stressing(args))
-		return;
+		return true;
 	path[len] = '\0';
 
 #if defined(HAVE_LINKAT) &&	\
@@ -166,7 +171,7 @@ static void stress_dirdeep_make(
 		 */
 		ret = linkat(dir_fd, "h", dir_fd, "i", ~0);
 		if (ret == 0) {
-			ret = unlinkat(dir_fd, "i", 0);
+			ret = shim_unlinkat(dir_fd, "i", 0);
 			(void)ret;
 		}
 #if defined(AT_SYMLINK_FOLLOW)
@@ -200,7 +205,7 @@ static void stress_dirdeep_make(
 #if defined(HAVE_UNLINKAT)
 		ret = linkat(dir_fd, "h", dir_fd, "u", 0);
 		if (ret == 0) {
-			ret = unlinkat(dir_fd, "u", 0);
+			ret = shim_unlinkat(dir_fd, "u", 0);
 			(void)ret;
 		}
 #endif
@@ -214,6 +219,7 @@ static void stress_dirdeep_make(
 		(void)close(dir_fd);
 	}
 #endif
+	return false;
 }
 
 /*
@@ -248,14 +254,21 @@ static int stress_dir_exercise(
 		return -1;
 
 	for (i = 0; (i < n) && keep_stressing(args); i++) {
-		if (namelist[i]->d_name[0] == '.')
+		register int ch;
+
+		/* Sanity check */
+		if (!namelist[n])
+			continue;
+
+		ch = (int)namelist[n]->d_name[0];
+		if (ch == '.')
 			continue;
 
 		path[len] = '/';
-		path[len + 1] = namelist[i]->d_name[0];
+		path[len + 1] = ch;
 		path[len + 2] = '\0';
 
-		if (isdigit((int)namelist[i]->d_name[0])) {
+		if (isdigit(ch) || isupper(ch)) {
 			stress_dir_exercise(args, path, len + 2, path_len);
 		} else {
 			int fd;
@@ -311,24 +324,29 @@ static void stress_dir_tidy(
 		return;
 
 	while (n--) {
-		if (namelist[n]->d_name[0] == '.')
+		register const int ch = (int)namelist[n]->d_name[0];
+
+		if (ch == '.') {
+			free(namelist[n]);
 			continue;
+		}
 
 		path[len] = '/';
-		path[len + 1] = namelist[n]->d_name[0];
+		path[len + 1] = ch;
 		path[len + 2] = '\0';
 
-		if (isdigit((int)namelist[n]->d_name[0]))
+		if (isdigit(ch) || isupper(ch)) {
+			free(namelist[n]);
 			stress_dir_tidy(args, path, len + 2, path_len);
-		else
-			(void)unlink(path);
-
-		free(namelist[n]);
+		} else {
+			free(namelist[n]);
+			(void)shim_unlink(path);
+		}
 	}
 	path[len] = '\0';
-	stress_dirent_list_free(namelist, n);
+	free(namelist);
 
-	(void)rmdir(path);
+	(void)shim_rmdir(path);
 }
 
 
@@ -345,27 +363,38 @@ static int stress_dirdeep(const stress_args_t *args)
 	size_t path_len;
 	uint32_t dirdeep_dirs = 1;
 	uint64_t dirdeep_inodes = ~0ULL;
-	const uint64_t inodes_avail = stress_get_filesystem_available_inodes();
-	uint64_t inodes_target_free;
-	uint64_t min_inodes_free = ~0ULL;
+	uint64_t inodes_start;
+	uint64_t inodes_estimate;
+	uint64_t inodes_min;
+	uint64_t inodes_exercised;
 
 	(void)stress_get_setting("dirdeep-dirs", &dirdeep_dirs);
 	(void)stress_get_setting("dirdeep-inodes", &dirdeep_inodes);
 
-	inodes_target_free = (inodes_avail > dirdeep_inodes) ?
-		inodes_avail - dirdeep_inodes : 0;
+	inodes_start = stress_get_filesystem_available_inodes();
 
 	(void)stress_temp_dir_args(args, rootpath, sizeof(rootpath));
 	path_len = strlen(rootpath);
 
 	(void)stress_mk_filename(linkpath, sizeof(linkpath), rootpath, "/f");
 
-	pr_dbg("%s: %" PRIu64 " inodes available, exercising up to %" PRIu64 " inodes\n",
-		args->name, inodes_avail, inodes_avail - inodes_target_free);
+	if (args->instance == 0) {
+		if (inodes_start) {
+			if (dirdeep_inodes > inodes_start)
+				dirdeep_inodes = inodes_start;
+			pr_dbg("%s: %" PRIu64 " inodes available, exercising up to %" PRIu64 " inodes\n",
+				args->name, inodes_start, dirdeep_inodes);
+		} else {
+			pr_dbg("%s: unknown inodes available, exercising up to %" PRIu64 " inodes\n",
+				args->name, dirdeep_inodes);
+		}
+	}
 
 	(void)shim_strlcpy(path, rootpath, sizeof(path));
+	inodes_estimate = 1;		/* created one for root */
+	inodes_min = inodes_start;
 	stress_dirdeep_make(args, linkpath, path, path_len, sizeof(path),
-		dirdeep_dirs, dirdeep_inodes, inodes_target_free, &min_inodes_free, 0);
+		dirdeep_dirs, dirdeep_inodes, inodes_start, &inodes_estimate, &inodes_min, 0);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -381,9 +410,13 @@ static int stress_dirdeep(const stress_args_t *args)
 	pr_tidy("%s: removing directories\n", args->name);
 	stress_dir_tidy(args, path, path_len, sizeof(path));
 
-	pr_dbg("%s: %" PRIu64 " inodes exercised\n", args->name, inodes_avail - min_inodes_free);
-	if ((args->instance == 0) && (inodes_target_free < min_inodes_free))
-		pr_inf("%s: note: specifying a larger --dirdeep setting or "
+	inodes_exercised = (inodes_start == 0) ? inodes_estimate : inodes_start - inodes_min;
+	pr_dbg("%s: %" PRIu64 " inodes exercised%s\n",
+		args->name, inodes_exercised,
+		(inodes_start == 0) ? " (estimated)" : "");
+
+	if ((args->instance == 0) && (inodes_exercised < dirdeep_inodes))
+		pr_inf("%s: note: specifying a larger --dirdeep or --dirdeep-dirs settings or "
 			"running the stressor for longer will use more "
 			"inodes\n", args->name);
 
@@ -400,5 +433,6 @@ stressor_info_t stress_dirdeep_info = {
 	.stressor = stress_dirdeep,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
