@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2012-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"dnotify N",	 "start N workers exercising dnotify events" },
 	{ NULL,	"dnotify-ops N", "stop dnotify workers after N bogo operations" },
@@ -142,7 +142,7 @@ cleanup:
  */
 static int rm_file(const stress_args_t *args, const char *path)
 {
-	if ((unlink(path) < 0) && errno != ENOENT) {
+	if ((shim_force_unlink(path) < 0) && errno != ENOENT) {
 		pr_err("%s: cannot remove file %s: errno=%d (%s)\n",
 			args->name, path, errno, strerror(errno));
 		return -1;
@@ -434,6 +434,7 @@ stressor_info_t stress_dnotify_info = {
 	.stressor = stress_dnotify,
 	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
 	.supported = stress_dnotify_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
