@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -193,7 +188,7 @@ static int stress_dup2_race(info_t *info)
 
 		(void)waitpid(pid, &status, 0);
 	}
-	(void)unlink(info->fifoname);
+	(void)shim_unlink(info->fifoname);
 
 	return 0;
 }
@@ -371,7 +366,7 @@ tidy_fds:
 #if defined(STRESS_DUP2_RACE)
 	if (info != MAP_FAILED) {
 		if (info->fifoname[0])
-			(void)unlink(info->fifoname);
+			(void)shim_unlink(info->fifoname);
 		(void)stress_temp_dir_rm_args(args);
 	}
 
@@ -392,5 +387,6 @@ tidy_mmap:
 stressor_info_t stress_dup_info = {
 	.stressor = stress_dup,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
