@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"efivar N",	"start N workers that read EFI variables" },
@@ -154,7 +154,7 @@ static int efi_get_data(
 		return rc;
 
 	if (fstat(fd, &statbuf) < 0) {
-		pr_err("%s: failed to stat %s, errno=%d (%s)\n",
+		pr_fail("%s: failed to stat %s, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto err_vars;
 	}
@@ -163,7 +163,7 @@ static int efi_get_data(
 
 	n = read(fd, buf, buf_len);
 	if ((n < 0) && (errno != EIO) && (errno != EAGAIN) && (errno != EINTR)) {
-		pr_err("%s: failed to read %s, errno=%d (%s)\n",
+		pr_fail("%s: failed to read %s, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto err_vars;
 	}
@@ -255,7 +255,7 @@ static int efi_get_variable(const stress_args_t *args, const char *varname, stre
 
 	ret = fstat(fd, &statbuf);
 	if (ret < 0) {
-		pr_err("%s: failed to stat %s, errno=%d (%s)\n",
+		pr_fail("%s: failed to stat %s, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		rc = -1;
 		goto err_efi_vars;
@@ -263,7 +263,7 @@ static int efi_get_variable(const stress_args_t *args, const char *varname, stre
 
 	n = read(fd, data, sizeof(data));
 	if ((n < 0) && (errno != EIO) && (errno != EAGAIN) && (errno != EINTR)) {
-		pr_err("%s: failed to read %s, errno=%d (%s)\n",
+		pr_fail("%s: failed to read %s, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		rc = -1;
 		goto err_efi_vars;
@@ -273,7 +273,7 @@ static int efi_get_variable(const stress_args_t *args, const char *varname, stre
     defined(FS_IOC_SETFLAGS)
 	ret = ioctl(fd, FS_IOC_GETFLAGS, &flags);
 	if (ret < 0) {
-		pr_err("%s: ioctl FS_IOC_GETFLAGS on %s failed, errno=%d (%s)\n",
+		pr_fail("%s: ioctl FS_IOC_GETFLAGS on %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		rc = -1;
 		goto err_efi_vars;
@@ -281,7 +281,7 @@ static int efi_get_variable(const stress_args_t *args, const char *varname, stre
 
 	ret = ioctl(fd, FS_IOC_SETFLAGS, &flags);
 	if (ret < 0) {
-		pr_err("%s: ioctl FS_IOC_SETFLAGS on %s failed, errno=%d (%s)\n",
+		pr_fail("%s: ioctl FS_IOC_SETFLAGS on %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		rc = -1;
 	}
@@ -384,19 +384,19 @@ static int stress_efivar(const stress_args_t *args)
 	efi_ignore = mmap(NULL, sz, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (efi_ignore == MAP_FAILED) {
-		pr_err("%s: cannot mmap shared memory: %d (%s))\n",
+		pr_fail("%s: cannot mmap shared memory: %d (%s))\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
-	if (!keep_stressing_flag())
-		return EXIT_SUCCESS;
 	pid = fork();
 	if (pid < 0) {
-		if ((errno == EAGAIN) || (errno == ENOMEM))
+		if (stress_redo_fork(errno))
 			goto again;
+		if (!keep_stressing(args))
+			goto finish;
 		pr_err("%s: fork failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 	} else if (pid > 0) {
@@ -430,6 +430,7 @@ again:
 		_exit(0);
 	}
 
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	(void)munmap(efi_ignore, sz);
@@ -456,6 +457,7 @@ stressor_info_t stress_efivar_info = {
 	.stressor = stress_not_implemented,
 	.supported = stress_efivar_supported,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #endif
