@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-cpu.h"
+#include "core-pragma.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"enosys N",	"start N workers that call non-existent system calls" },
@@ -34,6 +32,11 @@ static const stress_help_t help[] = {
 
 #define HASH_SYSCALL_SIZE	(1987)
 
+#define CALL_UNDEFINED		(0)
+#define CALL_BY_SYSCALL		(1)	/* syscall() call */
+#define CALL_BY_X86_SYSCALL	(2)	/* x86 syscall opcode */
+#define CALL_BY_X86_INT80	(3)	/* x86 int $80 */
+
 #if defined(__NR_syscalls)
 #define MAX_SYSCALL	__NR_syscalls
 #else
@@ -41,10 +44,9 @@ static const stress_help_t help[] = {
 #endif
 
 #if defined(__linux__) &&       \
-    (defined(__x86_64__) || defined(__x86_64)) && \
-    defined(HAVE_CPUID_H) &&    \
-    defined(HAVE_CPUID) &&      \
-    NEED_GNUC(4,6,0)
+    !defined(__TINYC__) &&	\
+    !defined(__PCC__) &&	\
+   (defined(__x86_64__) || defined(__x86_64))
 #define STRESS_EXERCISE_X86_SYSCALL
 #endif
 
@@ -56,28 +58,17 @@ static const stress_help_t help[] = {
 
 typedef struct hash_syscall {
 	struct hash_syscall *next;
-	long	number;
+	unsigned long	number;
 } stress_hash_syscall_t;
 
 static stress_hash_syscall_t *hash_syscall_table[HASH_SYSCALL_SIZE];
 
+static int stress_call_type = CALL_UNDEFINED;
+static sigjmp_buf jmp_env;
+
 #if defined(STRESS_EXERCISE_X86_SYSCALL)
 static bool stress_x86syscall_available;
 
-static bool stress_check_x86syscall(void)
-{
-	uint32_t eax, ebx, ecx, edx;
-
-	/* Intel CPU? */
-	if (!stress_cpu_is_x86())
-		return false;
-	/* ..and supports syscall? */
-	__cpuid(0x80000001, eax, ebx, ecx, edx);
-	if (!(edx & (1ULL << 11)))
-		return false;
-	return true;
-}
-
 /*
  *  x86_64_syscall6()
  *      syscall 6 arg wrapper
@@ -87,28 +78,34 @@ static inline long x86_64_syscall6(
 	long arg3, long arg4, long arg5, long arg6)
 {
 	long ret;
-	unsigned long _arg1 = arg1;
-	unsigned long _arg2 = arg2;
-	unsigned long _arg3 = arg3;
-	unsigned long _arg4 = arg4;
-	unsigned long _arg5 = arg5;
-	unsigned long _arg6 = arg6;
 
-	register long __arg1 asm ("rdi") = _arg1;
-	register long __arg2 asm ("rsi") = _arg2;
-	register long __arg3 asm ("rdx") = _arg3;
-	register long __arg4 asm ("r10") = _arg4;
-	register long __arg5 asm ("r8") = _arg5;
-	register long __arg6 asm ("r9") = _arg6;
+	stress_call_type = CALL_BY_X86_SYSCALL;
 
-	asm volatile ("syscall\n\t"
+	{
+		unsigned long _arg1 = arg1;
+		unsigned long _arg2 = arg2;
+		unsigned long _arg3 = arg3;
+		unsigned long _arg4 = arg4;
+		unsigned long _arg5 = arg5;
+		unsigned long _arg6 = arg6;
+
+		register long __arg1 asm ("rdi") = _arg1;
+		register long __arg2 asm ("rsi") = _arg2;
+		register long __arg3 asm ("rdx") = _arg3;
+		register long __arg4 asm ("r10") = _arg4;
+		register long __arg5 asm ("r8") = _arg5;
+		register long __arg6 asm ("r9") = _arg6;
+
+
+		asm volatile ("syscall\n\t"
 			: "=a" (ret)
 			: "0" (number), "r" (__arg1), "r" (__arg2), "r" (__arg3),
 			  "r" (__arg4), "r" (__arg5), "r" (__arg6)
 			: "memory", "cc", "r11", "cx");
-	if (ret < 0) {
-		errno = -ret;
-		ret = -1;
+		if (ret < 0) {
+			errno = -ret;
+			ret = -1;
+		}
 	}
 	return ret;
 }
@@ -122,6 +119,8 @@ static inline int x86_0x80_syscall6(
 {
 	int ret;
 
+	stress_call_type = CALL_BY_X86_INT80;
+
 	asm (
 	     "movl %6, %%eax\n"
 	     "movl %%eax, %%ebp\n"
@@ -157,6 +156,8 @@ static inline long syscall7(long number, long arg1, long arg2,
 			    long arg3, long arg4, long arg5,
 			    long arg6, long arg7)
 {
+	stress_call_type = CALL_BY_SYSCALL;
+
 	return syscall(number, arg1, arg2, arg3, arg4, arg5, arg6, arg7);
 }
 STRESS_PRAGMA_POP
@@ -194,13 +195,16 @@ static void exercise_syscall(
 	long arg6, long arg7)
 {
 	int ret;
-	bool enosys = false;
+	NOCLOBBER bool enosys = false;
 	const pid_t pid = getpid();
 
 	if (!keep_stressing(args))
 		_exit(EXIT_SUCCESS);
 
 	itimer_set(args);
+	ret = sigsetjmp(jmp_env, 1);
+	if (ret)
+		goto try_x86_syscall;
 	ret = (int)syscall7(number, arg1, arg2, arg3, arg4, arg5, arg6, arg7);
 	exit_if_child(pid);
 	if ((ret < 0) && (errno != ENOSYS))
@@ -211,21 +215,45 @@ static void exercise_syscall(
 		_exit(0);
 	}
 
+try_x86_syscall:
 #if defined(STRESS_EXERCISE_X86_SYSCALL)
-	itimer_set(args);
-	if (stress_x86syscall_available) {
-		ret = x86_64_syscall6(number, arg1, arg2, arg3, arg4, arg5, arg6);
-		exit_if_child(pid);
-		if ((ret < 0) && (errno != ENOSYS))
-			enosys = true;
+	{
+		static bool x86_syscall_ok = true;
+
+		itimer_set(args);
+		ret = sigsetjmp(jmp_env, 1);
+		if (ret) {
+			x86_syscall_ok = false;
+			goto try_x86_0x80;
+		}
+		if (stress_x86syscall_available && x86_syscall_ok) {
+			ret = x86_64_syscall6(number, arg1, arg2, arg3, arg4, arg5, arg6);
+			exit_if_child(pid);
+			if ((ret < 0) && (errno != ENOSYS))
+				enosys = true;
+		}
 	}
+try_x86_0x80:
 #endif
+
 #if defined(STRESS_EXERCISE_X86_0X80)
-	itimer_set(args);
-	ret = x86_0x80_syscall6(number, arg1, arg2, arg3, arg4, arg5, arg6);
-	exit_if_child(pid);
-	if ((ret < 0) && (errno != ENOSYS))
-		enosys = true;
+	{
+		static bool x86_0x80_ok = true;
+
+		itimer_set(args);
+		ret = sigsetjmp(jmp_env, 1);
+		if (ret) {
+			x86_0x80_ok = false;
+			goto try_no_more;
+		}
+		if (x86_0x80_ok) {
+			ret = x86_0x80_syscall6(number, arg1, arg2, arg3, arg4, arg5, arg6);
+			exit_if_child(pid);
+			if ((ret < 0) && (errno != ENOSYS))
+				enosys = true;
+		}
+	}
+try_no_more:
 #endif
 	if (enosys)
 		_exit(errno);
@@ -288,7 +316,7 @@ static const int syscall_ignore[] = {
 #endif
 };
 
-static inline bool HOT OPTIMIZE3 syscall_find(long number)
+static inline bool HOT OPTIMIZE3 syscall_find(unsigned long number)
 {
 	register stress_hash_syscall_t *h;
 	register int i;
@@ -309,7 +337,7 @@ static inline bool HOT OPTIMIZE3 syscall_find(long number)
 	return false;
 }
 
-static inline void HOT OPTIMIZE3 syscall_add(long number)
+static inline void HOT OPTIMIZE3 syscall_add(const unsigned long number)
 {
 	const long hash = number % HASH_SYSCALL_SIZE;
 	stress_hash_syscall_t *newh, *h = hash_syscall_table[hash];
@@ -786,9 +814,6 @@ static const long skip_syscalls[] = {
 #if defined(SYS_io_cancel)
 	SYS_io_cancel,
 #endif
-#if defined(SYS_ioctl)
-	SYS_ioctl,
-#endif
 #if defined(SYS_io_destroy)
 	SYS_io_destroy,
 #endif
@@ -825,6 +850,9 @@ static const long skip_syscalls[] = {
 #if defined(SYS_io_uring_setup)
 	SYS_io_uring_setup,
 #endif
+#if defined(SYS_ioctl)
+	SYS_ioctl,
+#endif
 #if defined(SYS_ipc)
 	SYS_ipc,
 #endif
@@ -897,6 +925,9 @@ static const long skip_syscalls[] = {
 #if defined(SYS_mbind)
 	SYS_mbind,
 #endif
+#if defined(SYS_memory_ordering)
+	SYS_memory_ordering,
+#endif
 #if defined(SYS_membarrier)
 	SYS_membarrier,
 #endif
@@ -1640,6 +1671,15 @@ static const long skip_syscalls[] = {
 #endif
 
 /* Linux syscall numbers */
+#if defined(__NR__llseek)
+	__NR__llseek,
+#endif
+#if defined(__NR__newselect)
+	__NR__newselect,
+#endif
+#if defined(__NR__sysctl)
+	__NR__sysctl,
+#endif
 #if defined(__NR_accept)
 	__NR_accept,
 #endif
@@ -1748,18 +1788,33 @@ static const long skip_syscalls[] = {
 #if defined(__NR_clock_adjtime)
 	__NR_clock_adjtime,
 #endif
+#if defined(__NR_clock_adjtime64)
+	__NR_clock_adjtime64,
+#endif
 #if defined(__NR_clock_getres)
 	__NR_clock_getres,
 #endif
+#if defined(__NR_clock_getres_time64)
+	__NR_clock_getres_time64,
+#endif
 #if defined(__NR_clock_gettime)
 	__NR_clock_gettime,
 #endif
+#if defined(__NR_clock_gettime64)
+	__NR_clock_gettime64,
+#endif
 #if defined(__NR_clock_nanosleep)
 	__NR_clock_nanosleep,
 #endif
+#if defined(__NR_clock_nanosleep64)
+	__NR_clock_nanosleep64,
+#endif
 #if defined(__NR_clock_settime)
 	__NR_clock_settime,
 #endif
+#if defined(__NR_clock_settime64)
+	__NR_clock_settime64,
+#endif
 #if defined(__NR_clone)
 	__NR_clone,
 #endif
@@ -1772,6 +1827,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_close)
 	__NR_close,
 #endif
+#if defined(__NR_close_range)
+	__NR_close_range,
+#endif
 #if defined(__NR_cmpxchg_badaddr)
 	__NR_cmpxchg_badaddr,
 #endif
@@ -1835,18 +1893,30 @@ static const long skip_syscalls[] = {
 #if defined(__NR_epoll_ctl)
 	__NR_epoll_ctl,
 #endif
+#if defined(__NR_epoll_ctl_old)
+	__NR_epoll_ctl_old,
+#endif
 #if defined(__NR_epoll_pwait)
 	__NR_epoll_pwait,
 #endif
+#if defined(__NR_epoll_pwait2)
+	__NR_epoll_pwait2,
+#endif
 #if defined(__NR_epoll_wait)
 	__NR_epoll_wait,
 #endif
+#if defined(__NR_epoll_wait_old)
+	__NR_epoll_wait_old,
+#endif
 #if defined(__NR_eventfd)
 	__NR_eventfd,
 #endif
 #if defined(__NR_eventfd2)
 	__NR_eventfd2,
 #endif
+#if defined(__NR_exec_with_loader)
+	__NR_exec_with_loader,
+#endif
 #if defined(__NR_execv)
 	__NR_execv,
 #endif
@@ -1856,9 +1926,6 @@ static const long skip_syscalls[] = {
 #if defined(__NR_execveat)
 	__NR_execveat,
 #endif
-#if defined(__NR_exec_with_loader)
-	__NR_exec_with_loader,
-#endif
 #if defined(__NR_exit)
 	__NR_exit,
 #endif
@@ -1937,6 +2004,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_fork)
 	__NR_fork,
 #endif
+#if defined(__NR_fp_udfiex_crtl)
+	__NR_fp_udfiex_crtl,
+#endif
 #if defined(__NR_free_hugepages)
 	__NR_free_hugepages,
 #endif
@@ -1988,9 +2058,30 @@ static const long skip_syscalls[] = {
 #if defined(__NR_futex)
 	__NR_futex,
 #endif
+#if defined(__NR_futex_time64)
+	__NR_futex_time64,
+#endif
+#if defined(__NR_futimens)
+	__NR_futimens,
+#endif
 #if defined(__NR_futimesat)
 	__NR_futimesat,
 #endif
+#if defined(__NR_get_kernel_syms)
+	__NR_get_kernel_syms,
+#endif
+#if defined(__NR_get_mempolicy)
+	__NR_get_mempolicy,
+#endif
+#if defined(__NR_get_robust_list)
+	__NR_get_robust_list,
+#endif
+#if defined(__NR_get_thread_area)
+	__NR_get_thread_area,
+#endif
+#if defined(__NR_gettls)
+	__NR_gettls,
+#endif
 #if defined(__NR_getcpu)
 	__NR_getcpu,
 #endif
@@ -2039,12 +2130,6 @@ static const long skip_syscalls[] = {
 #if defined(__NR_getitimer)
 	__NR_getitimer,
 #endif
-#if defined(__NR_get_kernel_syms)
-	__NR_get_kernel_syms,
-#endif
-#if defined(__NR_get_mempolicy)
-	__NR_get_mempolicy,
-#endif
 #if defined(__NR_getpagesize)
 	__NR_getpagesize,
 #endif
@@ -2087,9 +2172,6 @@ static const long skip_syscalls[] = {
 #if defined(__NR_getrlimit)
 	__NR_getrlimit,
 #endif
-#if defined(__NR_get_robust_list)
-	__NR_get_robust_list,
-#endif
 #if defined(__NR_getrusage)
 	__NR_getrusage,
 #endif
@@ -2102,9 +2184,6 @@ static const long skip_syscalls[] = {
 #if defined(__NR_getsockopt)
 	__NR_getsockopt,
 #endif
-#if defined(__NR_get_thread_area)
-	__NR_get_thread_area,
-#endif
 #if defined(__NR_gettid)
 	__NR_gettid,
 #endif
@@ -2156,9 +2235,6 @@ static const long skip_syscalls[] = {
 #if defined(__NR_io_cancel)
 	__NR_io_cancel,
 #endif
-#if defined(__NR_ioctl)
-	__NR_ioctl,
-#endif
 #if defined(__NR_io_destroy)
 	__NR_io_destroy,
 #endif
@@ -2168,17 +2244,8 @@ static const long skip_syscalls[] = {
 #if defined(__NR_io_pgetevents)
 	__NR_io_pgetevents,
 #endif
-#if defined(__NR_ioperm)
-	__NR_ioperm,
-#endif
-#if defined(__NR_iopl)
-	__NR_iopl,
-#endif
-#if defined(__NR_ioprio_get)
-	__NR_ioprio_get,
-#endif
-#if defined(__NR_ioprio_set)
-	__NR_ioprio_set,
+#if defined(__NR_io_pgetevents_time64)
+	__NR_io_pgetevents_time64,
 #endif
 #if defined(__NR_io_setup)
 	__NR_io_setup,
@@ -2195,6 +2262,21 @@ static const long skip_syscalls[] = {
 #if defined(__NR_io_uring_setup)
 	__NR_io_uring_setup,
 #endif
+#if defined(__NR_ioctl)
+	__NR_ioctl,
+#endif
+#if defined(__NR_ioperm)
+	__NR_ioperm,
+#endif
+#if defined(__NR_iopl)
+	__NR_iopl,
+#endif
+#if defined(__NR_ioprio_set)
+	__NR_ioprio_set,
+#endif
+#if defined(__NR_ioprio_get)
+	__NR_ioprio_get,
+#endif
 #if defined(__NR_ipc)
 	__NR_ipc,
 #endif
@@ -2300,6 +2382,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_memfd_create)
 	__NR_memfd_create,
 #endif
+#if defined(__NR_memfd_secret)
+	__NR_memfd_secret,
+#endif
 #if defined(__NR_memory_ordering)
 	__NR_memory_ordering,
 #endif
@@ -2348,12 +2433,18 @@ static const long skip_syscalls[] = {
 #if defined(__NR_mmap2)
 	__NR_mmap2,
 #endif
+#if defined(__NR_mmap_pgoff)
+	__NR_mmap_pgoff,
+#endif
 #if defined(__NR_modify_ldt)
 	__NR_modify_ldt,
 #endif
 #if defined(__NR_mount)
 	__NR_mount,
 #endif
+#if defined(__NR_mount_setattr)
+	__NR_mount_setattr,
+#endif
 #if defined(__NR_move_mount)
 	__NR_move_mount,
 #endif
@@ -2366,6 +2457,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_mpx)
 	__NR_mpx,
 #endif
+#if defined(__NR_mq_close)
+	__NR_mq_close,
+#endif
 #if defined(__NR_mq_getsetattr)
 	__NR_mq_getsetattr,
 #endif
@@ -2375,12 +2469,24 @@ static const long skip_syscalls[] = {
 #if defined(__NR_mq_open)
 	__NR_mq_open,
 #endif
+#if defined(__NR_mq_receive)
+	__NR_mq_receive,
+#endif
+#if defined(__NR_mq_send)
+	__NR_mq_send,
+#endif
 #if defined(__NR_mq_timedreceive)
 	__NR_mq_timedreceive,
 #endif
+#if defined(__NR_mq_timedreceive_time64)
+	__NR_mq_timedreceive_time64,
+#endif
 #if defined(__NR_mq_timedsend)
 	__NR_mq_timedsend,
 #endif
+#if defined(__NR_mq_timedsend_time64)
+	__NR_mq_timedsend_time64,
+#endif
 #if defined(__NR_mq_unlink)
 	__NR_mq_unlink,
 #endif
@@ -2462,6 +2568,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_olddebug_setcontext)
 	__NR_olddebug_setcontext,
 #endif
+#if defined(__NR_old_getpagesize)
+	__NR_old_getpagesize,
+#endif
 #if defined(__NR_oldfstat)
 	__NR_oldfstat,
 #endif
@@ -2489,18 +2598,22 @@ static const long skip_syscalls[] = {
 #if defined(__NR_open)
 	__NR_open,
 #endif
-#if defined(__NR_openat)
-	__NR_openat,
-#endif
 #if defined(__NR_open_by_handle_at)
 	__NR_open_by_handle_at,
 #endif
 #if defined(__NR_open_tree)
 	__NR_open_tree,
 #endif
+#if defined(__NR_openat)
+	__NR_openat,
+#endif
 #if defined(__NR_or1k_atomic)
 	__NR_or1k_atomic,
 #endif
+/* TODO: all NR_osf calls */
+#if defined(__NR_osf_adjtime)
+	__NR_osf_adjtime,
+#endif
 #if defined(__NR_pause)
 	__NR_pause,
 #endif
@@ -2513,18 +2626,21 @@ static const long skip_syscalls[] = {
 #if defined(__NR_pciconfig_write)
 	__NR_pciconfig_write,
 #endif
-#if defined(__NR_perfctr)
-	__NR_perfctr,
-#endif
 #if defined(__NR_perf_event_open)
 	__NR_perf_event_open,
 #endif
+#if defined(__NR_perfctr)
+	__NR_perfctr,
+#endif
 #if defined(__NR_perfmonctl)
 	__NR_perfmonctl,
 #endif
 #if defined(__NR_personality)
 	__NR_personality,
 #endif
+#if defined(__NR_pidfd_getfd)
+	__NR_pidfd_getfd,
+#endif
 #if defined(__NR_pidfd_open)
 	__NR_pidfd_open,
 #endif
@@ -2561,6 +2677,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_ppoll)
 	__NR_ppoll,
 #endif
+#if defined(__NR_ppoll_time64)
+	__NR_ppoll_time64,
+#endif
 #if defined(__NR_prctl)
 	__NR_prctl,
 #endif
@@ -2576,9 +2695,18 @@ static const long skip_syscalls[] = {
 #if defined(__NR_preadv2)
 	__NR_preadv2,
 #endif
+#if defined(__NR_prlimit)
+	__NR_prlimit,
+#endif
 #if defined(__NR_prlimit64)
 	__NR_prlimit64,
 #endif
+#if defined(__NR_process_madvise)
+	__NR_process_madvise,
+#endif
+#if defined(__NR_process_mrelease)
+	__NR_process_mrelease,
+#endif
 #if defined(__NR_process_vm_readv)
 	__NR_process_vm_readv,
 #endif
@@ -2594,6 +2722,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_pselect6)
 	__NR_pselect6,
 #endif
+#if defined(__NR_pselect6_time64)
+	__NR_pselect6_time64,
+#endif
 #if defined(__NR_ptrace)
 	__NR_ptrace,
 #endif
@@ -2618,6 +2749,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_quotactl)
 	__NR_quotactl,
 #endif
+#if defined(__NR_quotactl_fd)
+	__NR_quotactl_fd,
+#endif
 #if defined(__NR_read)
 	__NR_read,
 #endif
@@ -2648,6 +2782,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_recvmmsg)
 	__NR_recvmmsg,
 #endif
+#if defined(__NR_recvmmsg_time64)
+	__NR_recvmmsg_time64,
+#endif
 #if defined(__NR_recvmsg)
 	__NR_recvmsg,
 #endif
@@ -2702,9 +2839,6 @@ static const long skip_syscalls[] = {
 #if defined(__NR_rseq)
 	__NR_rseq,
 #endif
-#if defined(__NR_rtas)
-	__NR_rtas,
-#endif
 #if defined(__NR_rt_sigaction)
 	__NR_rt_sigaction,
 #endif
@@ -2726,9 +2860,15 @@ static const long skip_syscalls[] = {
 #if defined(__NR_rt_sigtimedwait)
 	__NR_rt_sigtimedwait,
 #endif
+#if defined(__NR_rt_sigtimedwait_time64)
+	__NR_rt_sigtimedwait_time64,
+#endif
 #if defined(__NR_rt_tgsigqueueinfo)
 	__NR_rt_tgsigqueueinfo,
 #endif
+#if defined(__NR_rtas)
+	__NR_rtas,
+#endif
 #if defined(__NR_s390_guarded_storage)
 	__NR_s390_guarded_storage,
 #endif
@@ -2744,24 +2884,24 @@ static const long skip_syscalls[] = {
 #if defined(__NR_s390_sthyi)
 	__NR_s390_sthyi,
 #endif
-#if defined(__NR_sched_getaffinity)
-	__NR_sched_getaffinity,
-#endif
 #if defined(__NR_sched_get_affinity)
 	__NR_sched_get_affinity,
 #endif
-#if defined(__NR_sched_getattr)
-	__NR_sched_getattr,
-#endif
-#if defined(__NR_sched_getparam)
-	__NR_sched_getparam,
-#endif
 #if defined(__NR_sched_get_priority_max)
 	__NR_sched_get_priority_max,
 #endif
 #if defined(__NR_sched_get_priority_min)
 	__NR_sched_get_priority_min,
 #endif
+#if defined(__NR_sched_getaffinity)
+	__NR_sched_getaffinity,
+#endif
+#if defined(__NR_sched_getattr)
+	__NR_sched_getattr,
+#endif
+#if defined(__NR_sched_getparam)
+	__NR_sched_getparam,
+#endif
 #if defined(__NR_sched_getscheduler)
 	__NR_sched_getscheduler,
 #endif
@@ -2771,12 +2911,12 @@ static const long skip_syscalls[] = {
 #if defined(__NR_sched_rr_stress_get_interval)
 	__NR_sched_rr_stress_get_interval,
 #endif
-#if defined(__NR_sched_setaffinity)
-	__NR_sched_setaffinity,
-#endif
 #if defined(__NR_sched_set_affinity)
 	__NR_sched_set_affinity,
 #endif
+#if defined(__NR_sched_setaffinity)
+	__NR_sched_setaffinity,
+#endif
 #if defined(__NR_sched_setattr)
 	__NR_sched_setattr,
 #endif
@@ -2834,6 +2974,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_semtimedop)
 	__NR_semtimedop,
 #endif
+#if defined(__NR_semtimedop_time64)
+	__NR_semtimedop_time64,
+#endif
 #if defined(__NR_send)
 	__NR_send,
 #endif
@@ -2852,6 +2995,21 @@ static const long skip_syscalls[] = {
 #if defined(__NR_sendto)
 	__NR_sendto,
 #endif
+#if defined(__NR_set_mempolicy)
+	__NR_set_mempolicy,
+#endif
+#if defined(__NR_set_mempolicy_home_node)
+	__NR_set_mempolicy_home_node,
+#endif
+#if defined(__NR_set_robust_list)
+	__NR_set_robust_list,
+#endif
+#if defined(__NR_set_thread_area)
+	__NR_set_thread_area,
+#endif
+#if defined(__NR_set_tid_address)
+	__NR_set_tid_address,
+#endif
 #if defined(__NR_setdomainname)
 	__NR_setdomainname,
 #endif
@@ -2888,8 +3046,8 @@ static const long skip_syscalls[] = {
 #if defined(__NR_setitimer)
 	__NR_setitimer,
 #endif
-#if defined(__NR_set_mempolicy)
-	__NR_set_mempolicy,
+#if defined(__NR_setmntent)
+	__NR_setmntent,
 #endif
 #if defined(__NR_setns)
 	__NR_setns,
@@ -2930,21 +3088,12 @@ static const long skip_syscalls[] = {
 #if defined(__NR_setrlimit)
 	__NR_setrlimit,
 #endif
-#if defined(__NR_set_robust_list)
-	__NR_set_robust_list,
-#endif
 #if defined(__NR_setsid)
 	__NR_setsid,
 #endif
 #if defined(__NR_setsockopt)
 	__NR_setsockopt,
 #endif
-#if defined(__NR_set_thread_area)
-	__NR_set_thread_area,
-#endif
-#if defined(__NR_set_tid_address)
-	__NR_set_tid_address,
-#endif
 #if defined(__NR_settimeofday)
 	__NR_settimeofday,
 #endif
@@ -3002,6 +3151,12 @@ static const long skip_syscalls[] = {
 #if defined(__NR_sigsuspend)
 	__NR_sigsuspend,
 #endif
+#if defined(__NR_sigtimedwait)
+	__NR_sigtimedwait,
+#endif
+#if defined(__NR_sigwaitinfo)
+	__NR_sigwaitinfo,
+#endif
 #if defined(__NR_socket)
 	__NR_socket,
 #endif
@@ -3086,6 +3241,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_syncfs)
 	__NR_syncfs,
 #endif
+#if defined(__NR_sys_debug_setcontext)
+	__NR_sys_debug_setcontext,
+#endif
 #if defined(__NR_syscall)
 	__NR_syscall,
 #endif
@@ -3107,9 +3265,6 @@ static const long skip_syscalls[] = {
 #if defined(__NR__sysctl)
 	__NR__sysctl,
 #endif
-#if defined(__NR_sys_debug_setcontext)
-	__NR_sys_debug_setcontext,
-#endif
 #if defined(__NR_sysfs)
 	__NR_sysfs,
 #endif
@@ -3140,6 +3295,21 @@ static const long skip_syscalls[] = {
 #if defined(__NR_timer_delete)
 	__NR_timer_delete,
 #endif
+#if defined(__NR_timer_getoverrun)
+	__NR_timer_getoverrun,
+#endif
+#if defined(__NR_timer_gettime)
+	__NR_timer_gettime,
+#endif
+#if defined(__NR_timer_gettime64)
+	__NR_timer_gettime64,
+#endif
+#if defined(__NR_timer_settime)
+	__NR_timer_settime,
+#endif
+#if defined(__NR_timer_settime64)
+	__NR_timer_settime64,
+#endif
 #if defined(__NR_timerfd)
 	__NR_timerfd,
 #endif
@@ -3149,17 +3319,14 @@ static const long skip_syscalls[] = {
 #if defined(__NR_timerfd_gettime)
 	__NR_timerfd_gettime,
 #endif
+#if defined(__NR_timerfd_gettime64)
+	__NR_timerfd_gettime64,
+#endif
 #if defined(__NR_timerfd_settime)
 	__NR_timerfd_settime,
 #endif
-#if defined(__NR_timer_getoverrun)
-	__NR_timer_getoverrun,
-#endif
-#if defined(__NR_timer_gettime)
-	__NR_timer_gettime,
-#endif
-#if defined(__NR_timer_settime)
-	__NR_timer_settime,
+#if defined(__NR_timerfd_settime64)
+	__NR_timerfd_settime64,
 #endif
 #if defined(__NR_times)
 	__NR_times,
@@ -3236,6 +3403,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_utimensat)
 	__NR_utimensat,
 #endif
+#if defined(__NR_utimensat_time64)
+	__NR_utimensat_time64,
+#endif
 #if defined(__NR_utimes)
 	__NR_utimes,
 #endif
@@ -3260,6 +3430,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_vserver)
 	__NR_vserver,
 #endif
+#if defined(__NR_wait3)
+	__NR_wait3,
+#endif
 #if defined(__NR_wait4)
 	__NR_wait4,
 #endif
@@ -3311,18 +3484,33 @@ static void NORETURN MLOCKED_TEXT stress_badhandler(int signum)
 	_exit(1);
 }
 
+static void NORETURN MLOCKED_TEXT stress_sigill_handler(int signum)
+{
+	(void)signum;
+
+	if (stress_call_type == CALL_UNDEFINED) {
+		/* No idea how we got here, bail out */
+		_exit(1);
+	}
+	siglongjmp(jmp_env, 1);
+}
+
 /*
  *  Call a system call in a child context so we don't clobber
  *  the parent
  */
-static inline int stress_do_syscall(const stress_args_t *args, const long number)
+static inline int stress_do_syscall(
+	const stress_args_t *args,
+	const unsigned long number,
+	const bool random)
 {
 	pid_t pid;
 	int rc = 0;
 
 	/* Check if this is a known non-ENOSYS syscall */
-	if (syscall_find(number))
+	if (syscall_find(number)) {
 		return rc;
+	}
 	if (!keep_stressing(args))
 		return 0;
 	pid = fork();
@@ -3347,6 +3535,8 @@ static inline int stress_do_syscall(const stress_args_t *args, const long number
 			if (stress_sighandler(args->name, sigs[i], stress_badhandler, NULL) < 0)
 				_exit(EXIT_FAILURE);
 		}
+		if (stress_sighandler(args->name, SIGILL, stress_sigill_handler, NULL) < 0)
+			_exit(EXIT_FAILURE);
 
 		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
@@ -3396,8 +3586,10 @@ static inline int stress_do_syscall(const stress_args_t *args, const long number
 		rc = WEXITSTATUS(status);
 
 		/* Add to known syscalls that don't return ENOSYS */
-		if (rc != ENOSYS)
-			syscall_add(number);
+		if (rc != ENOSYS) {
+			if ((!random) || (number < 65536))
+				syscall_add(number);
+		}
 		inc_counter(args);
 	}
 	return rc;
@@ -3412,9 +3604,8 @@ static int stress_enosys(const stress_args_t *args)
 	pid_t pid;
 
 #if defined(STRESS_EXERCISE_X86_SYSCALL)
-	stress_x86syscall_available = stress_check_x86syscall();
+	stress_x86syscall_available = stress_cpu_x86_has_syscall();
 #endif
-
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	if (!keep_stressing(args))
@@ -3484,45 +3675,45 @@ again:
 			syscall_add(skip_syscalls[j]);
 
 		do {
-			long number;
+			unsigned long number;
 
 			/* Low sequential syscalls */
 			for (number = 0; number < MAX_SYSCALL + 1024; number++) {
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, number);
+				stress_do_syscall(args, number, false);
 			}
 
 			/* Random syscalls */
 			for (j = 0; j < 1024; j++) {
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, stress_mwc8() & mask);
+				stress_do_syscall(args, stress_mwc8() & mask, true);
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, stress_mwc16() & mask);
+				stress_do_syscall(args, stress_mwc16() & mask, true);
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, stress_mwc32() & mask);
+				stress_do_syscall(args, stress_mwc32() & mask, true);
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, (long)(stress_mwc64() & mask));
+				stress_do_syscall(args, (long)(stress_mwc64() & mask), true);
 			}
 
 			/* Various bit masks */
 			for (number = 1; number; number <<= 1) {
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, number);
+				stress_do_syscall(args, number, false);
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, number | 1);
+				stress_do_syscall(args, number | 1, false);
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, number | (number << 1));
+				stress_do_syscall(args, number | (number << 1), false);
 				if (!keep_stressing(args))
 					goto finish;
-				stress_do_syscall(args, ~number);
+				stress_do_syscall(args, ~number, false);
 			}
 
 			/* Various high syscalls */
@@ -3532,7 +3723,7 @@ again:
 				for (n = 0; n < 0x100; n++) {
 					if (!keep_stressing(args))
 						goto finish;
-					stress_do_syscall(args, n + number);
+					stress_do_syscall(args, n + number, false);
 				}
 			}
 		} while (keep_stressing(args));
