@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
+
+#if defined(HAVE_SYS_UN_H)
+#include <sys/un.h>
+#endif
+
+#if defined(HAVE_SYS_EPOLL_H)
+#include <sys/epoll.h>
+#endif
+
+#define MIN_EPOLL_PORT		(1024)
+#define MAX_EPOLL_PORT		(65535)
+#define DEFAULT_EPOLL_PORT	(6000)
+
+#define MAX_EPOLL_EVENTS 	(1024)
+#define MAX_SERVERS		(4)
 
 static const stress_help_t help[] = {
 	{ NULL,	"epoll N",	  "start N workers doing epoll handled socket activity" },
@@ -32,9 +43,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		  NULL }
 };
 
-#define MAX_EPOLL_EVENTS 	(1024)
-#define MAX_SERVERS		(4)
-
 #if defined(HAVE_SYS_EPOLL_H) &&	\
     defined(HAVE_EPOLL_CREATE) &&	\
     defined(HAVE_LIB_RT) &&		\
@@ -188,8 +196,7 @@ static pid_t epoll_spawn(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() &&
-		    ((errno == EAGAIN) || (errno == ENOMEM)))
+		if (stress_redo_fork(errno))
 			goto again;
 		return -1;
 	}
@@ -637,7 +644,7 @@ retry:
 	if (addr && (epoll_domain == AF_UNIX)) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
 	if (connect_timeouts)
@@ -940,7 +947,7 @@ die:
 	if (addr && (epoll_domain == AF_UNIX)) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
 	free(events);
@@ -1021,6 +1028,7 @@ stressor_info_t stress_epoll_info = {
 	.stressor = stress_epoll,
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
