@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_EVENTFD_H)
+#include <sys/eventfd.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"eventfd N",	 "start N workers stressing eventfd read/writes" },
 	{ NULL,	"eventfd-ops N", "stop eventfd workers after N bogo operations" },
@@ -97,13 +96,16 @@ static int stress_eventfd(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() &&
-		    ((errno == EAGAIN) || (errno == ENOMEM)))
+		if (stress_redo_fork(errno))
 			goto again;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+
 		(void)close(fd1);
 		(void)close(fd2);
+
+		if (!keep_stressing(args))
+			return EXIT_SUCCESS;
+		pr_fail("%s: fork failed, errno=%d (%s)\n",
+			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
 		int n = 0;
@@ -147,7 +149,6 @@ again:
 			if (n++ >= 64) {
 				n = 0;
 
-
 				/* Exercise write using small buffer */
 				(void)memset(re, 0, sizeof(re));
 				ret = write(fd1, re, sizeof(re));
@@ -256,6 +257,7 @@ stressor_info_t stress_eventfd_info = {
 	.stressor = stress_eventfd,
 	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
