@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2021 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_FALLOCATE_BYTES	(1 * MB)
+#define MAX_FALLOCATE_BYTES	(MAX_FILE_LIMIT)
+#define DEFAULT_FALLOCATE_BYTES	(1 * GB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"fallocate N",		"start N workers fallocating 16MB files" },
 	{ NULL,	"fallocate-ops N",	"stop after N fallocate bogo operations" },
@@ -150,7 +149,7 @@ static int stress_fallocate(const stress_args_t *args)
 		(void)stress_temp_dir_rm_args(args);
 		return ret;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	pipe_ret = pipe(pipe_fds);
 
@@ -307,7 +306,15 @@ done:
 
 	if (fd != -1)
 		(void)close(fd);
-	(void)stress_temp_dir_rm_args(args);
+
+	/*
+	 *  We may reach here on a siglongjmp without the temp dirs
+	 *  having been created. Don't call the normal helper but
+	 *  use rmdir and ignore any error if the directories don't
+	 *  exist.
+	 */
+	(void)stress_temp_dir_args(args, filename, sizeof(filename));
+	(void)shim_rmdir(filename);
 
 	return EXIT_SUCCESS;
 }
@@ -316,6 +323,7 @@ stressor_info_t stress_fallocate_info = {
 	.stressor = stress_fallocate,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
