@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2012-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_SYS_FANOTIFY_H)
+#include <sys/fanotify.h>
+#endif
+
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"fanotify N",	  "start N workers exercising fanotify events" },
@@ -40,11 +45,11 @@ static const stress_help_t help[] = {
 
 /* fanotify stats */
 typedef struct {
-	uint64_t	open;
-	uint64_t	close_write;
-	uint64_t	close_nowrite;
-	uint64_t	access;
-	uint64_t	modify;
+	uint64_t open;		/* # opens */
+	uint64_t close_write;	/* # close writes */
+	uint64_t close_nowrite;	/* # close no writes */
+	uint64_t access;	/* # accesses */
+	uint64_t modify;	/* # modifications */
 } stress_fanotify_account_t;
 
 #endif
@@ -118,6 +123,9 @@ static const unsigned int fan_stress_settings[] = {
 #endif
 #if defined(FAN_Q_OVERFLOW)
 	FAN_Q_OVERFLOW,
+#endif
+#if defined(FAN_FS_ERROR)
+	FAN_FS_ERROR,
 #endif
 	0
 };
@@ -151,6 +159,9 @@ static const unsigned int init_flags[] =
 #if defined(FAN_REPORT_DIR_FID)
 	FAN_REPORT_DIR_FID,
 #endif
+#if defined(FAN_REPORT_PIDFD)
+	FAN_REPORT_PIDFD,
+#endif
 };
 
 static char *mnts[MAX_MNTS];
@@ -549,7 +560,7 @@ static int stress_fanotify(const stress_args_t *args)
 			(void)close(fd);
 
 			/* Force remove */
-			(void)unlink(filename);
+			(void)shim_unlink(filename);
 		} while (keep_stressing(args));
 
 		_exit(EXIT_SUCCESS);
@@ -680,7 +691,7 @@ tidy:
 		(void)kill(pid, SIGKILL);
 		(void)shim_waitpid(pid, &status, 0);
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 	stress_mount_free(mnts, n_mnts);
 
@@ -691,6 +702,7 @@ stressor_info_t stress_fanotify_info = {
 	.stressor = stress_fanotify,
 	.supported = stress_fanotify_supported,
 	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
