@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -67,10 +62,8 @@ static void check_return(const stress_args_t *args, const int ret, const char *c
 		}
 	}
 }
-
 #endif
 
-
 /*
  *  do_fcntl()
  */
@@ -93,6 +86,8 @@ static int do_fcntl(
 		ret = fcntl(fd, F_DUPFD, bad_fd);
 		(void)ret;
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(F_DUPFD) &&		\
@@ -105,6 +100,8 @@ static int do_fcntl(
 		if (ret > -1)
 			(void)close(ret);
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(F_GETFD)
@@ -127,11 +124,15 @@ static int do_fcntl(
 			ret = fcntl(fd, F_SETFD, new_flags);
 			check_return(args, ret, "F_SETFD");
 		}
+#else
+		UNEXPECTED
 #endif
 		/* Exercise invalid fd */
 		old_flags = fcntl(bad_fd, F_GETFD);
 		(void)old_flags;
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(F_GETFL)
@@ -154,8 +155,12 @@ static int do_fcntl(
 			ret = fcntl(fd, F_SETFL, new_flags);
 			check_return(args, ret, "F_SETFL");
 		}
+#else
+		UNEXPECTED
 #endif
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(F_SETOWN) &&	\
@@ -166,6 +171,8 @@ static int do_fcntl(
 #if defined(HAVE_GETPGRP)
 		ret = fcntl(fd, F_SETOWN, -getpgrp());
 		check_return(args, ret, "F_SETOWN");
+#else
+		UNEXPECTED
 #endif
 		ret = fcntl(fd, F_SETOWN, args->pid);
 		check_return(args, ret, "F_SETOWN");
@@ -182,6 +189,8 @@ static int do_fcntl(
 		ret = fcntl(fd, F_SETOWN, args->pid);
 		(void)ret;
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(F_GETOWN) &&	\
@@ -244,6 +253,8 @@ static int do_fcntl(
 		owner.pid = getpgrp();
 		ret = fcntl(fd, F_SETOWN_EX, &owner);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 #if defined(F_OWNER_TID) &&	\
     defined(__linux__)
@@ -251,6 +262,8 @@ static int do_fcntl(
 		owner.pid = shim_gettid();
 		ret = fcntl(fd, F_SETOWN_EX, &owner);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 	}
 #endif
@@ -268,8 +281,11 @@ static int do_fcntl(
 		owner.type = F_OWNER_PGRP;
 		ret = fcntl(fd, F_GETOWN_EX, &owner);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 #if defined(F_OWNER_GID)
+		/* deprecated, renamed to F_OWNER_PGRP */
 		owner.type = F_OWNER_GID;
 		ret = fcntl(fd, F_GETOWN_EX, &owner);
 		(void)ret;
@@ -279,6 +295,8 @@ static int do_fcntl(
 		owner.type = F_OWNER_TID;
 		ret = fcntl(fd, F_GETOWN_EX, &owner);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 	}
 #endif
@@ -301,6 +319,8 @@ static int do_fcntl(
 		ret = fcntl(fd, F_SETSIG, 0);
 		(void)ret;
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(F_GETSIG)
@@ -310,9 +330,12 @@ static int do_fcntl(
 		ret = fcntl(fd, F_GETSIG);
 		check_return(args, ret, "F_GETSIG");
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(F_GETOWNER_UIDS)
+	/* Not defined in libc fcntl.h */
 	{
 		int ret;
 		uid_t uids[2];
@@ -329,6 +352,8 @@ static int do_fcntl(
 		ret = fcntl(fd, F_GETLEASE);
 		check_return(args, ret, "F_GETLEASE");
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(F_GETLK) &&		\
@@ -574,6 +599,8 @@ lock_abort:	{ /* Nowt */ }
 		check_return(args, ret, "F_OFD_SETLK (F_UNLCK)");
 ofd_lock_abort:	{ /* Nowt */ }
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if (defined(F_GET_FILE_RW_HINT) && defined(F_SET_FILE_RW_HINT)) | \
@@ -618,6 +645,8 @@ ofd_lock_abort:	{ /* Nowt */ }
 		hint = ~0UL;
 		ret = fcntl(fd, F_SET_FILE_RW_HINT, &hint);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 #if defined(F_GET_RW_HINT) &&	\
     defined(F_SET_RW_HINT)
@@ -629,6 +658,8 @@ ofd_lock_abort:	{ /* Nowt */ }
 				(void)ret;
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 	}
@@ -658,6 +689,8 @@ ofd_lock_abort:	{ /* Nowt */ }
 		if (ret > -1)
 			(void)close(ret);
 	}
+#else
+	UNEXPECTED
 #endif
 #if defined(F_GETOWN)
 	{
@@ -668,6 +701,8 @@ ofd_lock_abort:	{ /* Nowt */ }
 		(void)ret;
 
 	}
+#else
+	UNEXPECTED
 #endif
 #else
 	(void)path_fd;
@@ -750,8 +785,8 @@ tidy:
 		(void)close(path_fd);
 	if (fd >= 0)
 		(void)close(fd);
-	(void)unlink(filename);
-	(void)rmdir(pathname);
+	(void)shim_unlink(filename);
+	(void)shim_rmdir(pathname);
 
 	return rc;
 }
@@ -759,5 +794,6 @@ tidy:
 stressor_info_t stress_fcntl_info = {
 	.stressor = stress_fcntl,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
