@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_FIEMAP_H)
+#include <linux/fiemap.h>
+#endif
+
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#endif
+
+#define MIN_FIEMAP_SIZE		(1 * MB)
+#define MAX_FIEMAP_SIZE		(MAX_FILE_LIMIT)
+#define DEFAULT_FIEMAP_SIZE	(16 * MB)
+
 #define MAX_FIEMAP_PROCS	(4)		/* Number of FIEMAP stressors */
 #define COUNT_MAX		(128)
 
@@ -53,7 +60,6 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
     defined(HAVE_LINUX_FIEMAP_H) && 	\
     defined(FS_IOC_FIEMAP)
 
-
 /*
  *  stress_fiemap_count()
  *     racy bogo counter across child processes, avoids
@@ -117,7 +123,7 @@ static int stress_fiemap_writer(
     defined(FALLOC_FL_KEEP_SIZE)
 		if (!punch_hole)
 			continue;
-		shim_usleep(1000);
+		(void)shim_usleep(1000);
 
 		offset = stress_mwc64() % len;
 		if (shim_fallocate(fd, FALLOC_FL_PUNCH_HOLE |
@@ -127,9 +133,11 @@ static int stress_fiemap_writer(
 			if (errno == EOPNOTSUPP)
 				punch_hole = false;
 		}
-		shim_usleep(1000);
+		(void)shim_usleep(1000);
 		if (!stress_fiemap_count(args, counters))
 			break;
+#else
+		UNEXPECTED
 #endif
 	} while (keep_stressing(args));
 	rc = EXIT_SUCCESS;
@@ -289,7 +297,7 @@ static int stress_fiemap(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto dir_clean;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	memset(&fiemap, 0, sizeof(fiemap));
 	fiemap.fm_length = ~0UL;
@@ -341,6 +349,7 @@ stressor_info_t stress_fiemap_info = {
 	.stressor = stress_fiemap,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
