@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
+
+#define MIN_FIFO_READERS	(1)
+#define MAX_FIFO_READERS	(64)
+#define DEFAULT_FIFO_READERS	(4)
+
 #if defined(HAVE_SYS_SELECT_H)
 static const uint64_t wrap_mask = 0xffff000000000000ULL;
 #endif
@@ -31,7 +34,7 @@ static const uint64_t wrap_mask = 0xffff000000000000ULL;
 static const stress_help_t help[] = {
 	{ NULL,	"fifo N",	  "start N workers exercising fifo I/O" },
 	{ NULL,	"fifo-ops N",	  "stop after N fifo bogo operations" },
-	{ NULL,	"fifo-readers N", "number of fifo reader stessors to start" },
+	{ NULL,	"fifo-readers N", "number of fifo reader stressors to start" },
 	{ NULL,	NULL,		  NULL }
 };
 
@@ -92,7 +95,7 @@ static void stress_fifo_reader(
 
 	fd = open(fifoname, O_RDONLY | O_NONBLOCK);
 	if (fd < 0) {
-		pr_err("%s: fifo read open failed: errno=%d (%s)\n",
+		pr_fail("%s: fifo read open failed: errno=%d (%s)\n",
 			name, errno, strerror(errno));
 		return;
 	}
@@ -132,25 +135,27 @@ redo:
 			ret = ioctl(fd, FIONREAD, &isz);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 		sz = read(fd, &val, sizeof(val));
 		if (sz < 0) {
 			if ((errno == EAGAIN) || (errno == EINTR))
 				continue;
-			pr_err("%s: fifo read failed: errno=%d (%s)\n",
+			pr_fail("%s: fifo read failed: errno=%d (%s)\n",
 				name, errno, strerror(errno));
 			break;
 		}
 		if (sz == 0)
 			break;
 		if (sz != sizeof(val)) {
-			pr_err("%s: fifo read did not get uint64\n",
+			pr_fail("%s: fifo read did not get uint64\n",
 				name);
 			break;
 		}
 		if ((val < lastval) &&
 		    ((~val & wrap_mask) && (lastval & wrap_mask))) {
-			pr_err("%s: fifo read did not get "
+			pr_fail("%s: fifo read did not get "
 				"expected value\n", name);
 			break;
 		}
@@ -205,7 +210,7 @@ static int stress_fifo(const stress_args_t *args)
 
 	if (mkfifo(fifoname, S_IRUSR | S_IWUSR) < 0) {
 		rc = exit_status(errno);
-		pr_err("%s: mkfifo failed: errno=%d (%s)\n",
+		pr_fail("%s: mkfifo failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
@@ -229,7 +234,7 @@ static int stress_fifo(const stress_args_t *args)
 			rc = 0;
 		} else {
 			rc = exit_status(fd);
-			pr_err("%s: fifo write open failed: "
+			pr_fail("%s: fifo write open failed: "
 				"errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 		}
@@ -266,13 +271,13 @@ reap:
 		if (pids[i] > 0) {
 			int status;
 
-			(void)kill(pids[i], SIGKILL);
+			(void)stress_killpid(pids[i]);
 			(void)shim_waitpid(pids[i], &status, 0);
 		}
 	}
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)unlink(fifoname);
+	(void)shim_unlink(fifoname);
 	(void)stress_temp_dir_rm_args(args);
 
 	return rc;
@@ -282,6 +287,7 @@ stressor_info_t stress_fifo_info = {
 	.stressor = stress_fifo,
 	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
