@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"file-ioctl N",		"start N workers exercising file specific ioctls" },
 	{ NULL,	"file-ioctl-ops N",	"stop after N file ioctl bogo operations" },
@@ -93,7 +92,7 @@ static int stress_file_ioctl(const stress_args_t *args)
 		(void)stress_temp_dir_rm_args(args);
 		return ret;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 #if defined(FICLONE) || defined(FICLONERANGE)
 	(void)stress_temp_filename_args(args, filename, sizeof(filename), rnd + 1);
@@ -105,7 +104,7 @@ static int stress_file_ioctl(const stress_args_t *args)
 		pr_err("%s: cannot create %s\n", args->name, filename);
 		return ret;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 #endif
 
 	(void)shim_fallocate(fd, 0, 0, file_sz);
@@ -128,6 +127,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 #if defined(FIONCLEX)
 		{
@@ -136,6 +137,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 #if defined(FIONBIO)
 		{
@@ -158,6 +161,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 #endif
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FIOASYNC)
@@ -181,6 +186,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 #endif
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FIOQSIZE)
@@ -198,6 +205,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 			}
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 /* Disable this at the moment, it is fragile */
@@ -213,6 +222,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 			exercised++;
 		}
 #endif
+#else
+		/* UNEXPECTED */
 #endif
 
 #if defined(FIGETBSZ)
@@ -226,6 +237,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FICLONE)
@@ -235,6 +248,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FICLONERANGE)
@@ -252,6 +267,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FIDEDUPERANGE)
@@ -286,6 +303,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FIONREAD)
@@ -305,18 +324,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 
 			exercised++;
 		}
-#endif
-
-
-#if defined(FIONWRITE)
-		{
-			int isz = 0;
-
-			ret = ioctl(fd, FIONWRITE, &isz);
-			(void)ret;
-
-			exercised++;
-		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FS_IOC_GETVERSION)
@@ -328,6 +337,8 @@ static int stress_file_ioctl(const stress_args_t *args)
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(_IOW) &&	\
@@ -397,6 +408,8 @@ struct shim_space_resv {
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FS_IOC_RESVSP64)
@@ -429,6 +442,8 @@ struct shim_space_resv {
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FS_IOC_UNRESVSP)
@@ -445,6 +460,8 @@ struct shim_space_resv {
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FS_IOC_UNRESVSP64)
@@ -461,6 +478,8 @@ struct shim_space_resv {
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(FS_IOC_ZERO_RANGE)
@@ -477,6 +496,8 @@ struct shim_space_resv {
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #endif
@@ -499,6 +520,8 @@ struct shim_space_resv {
 
 			exercised++;
 		}
+#else
+		UNEXPECTED
 #endif
 		if (!exercised) {	/* cppcheck-suppress knownConditionTrueFalse */
 			pr_inf("%s: no available file ioctls to exercise\n",
@@ -526,5 +549,6 @@ tidy:
 stressor_info_t stress_file_ioctl_info = {
 	.stressor = stress_file_ioctl,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
