@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_STATVFS_H)
+#include <sys/statvfs.h>
+#endif
+
 #define	STRESS_FILENAME_PROBE	(0)	/* Default */
 #define STRESS_FILENAME_POSIX	(1)	/* POSIX 2008.1 */
 #define STRESS_FILENAME_EXT	(2)	/* EXT* filesystems */
@@ -110,7 +109,7 @@ static int stress_filename_probe_length(
 			return -1;
 		}
 		(void)close(fd);
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		max = i;
 	}
 	*sz_max = max + 1;
@@ -174,7 +173,7 @@ static int stress_filename_probe(
 			}
 		} else {
 			(void)close(fd);
-			(void)unlink(filename);
+			(void)shim_unlink(filename);
 			allowed[j] = (char)i;
 			j++;
 		}
@@ -240,11 +239,11 @@ static void stress_filename_tidy(const char *path)
 				continue;
 			(void)stress_mk_filename(filename, sizeof(filename),
 				path, d->d_name);
-			(void)unlink(filename);
+			(void)shim_unlink(filename);
 		}
 		(void)closedir(dir);
 	}
-	(void)rmdir(path);
+	(void)shim_rmdir(path);
 }
 
 /*
@@ -281,6 +280,14 @@ static void stress_filename_test(
 	const bool should_pass)
 {
 	int fd;
+	int ret;
+	struct stat buf;
+
+	/* exercise dcache lookup of non-existent filename */
+	ret = stat(filename, &buf);
+	if (ret == 0)
+		pr_fail("%s: stat succeeded on non-existent file\n",
+			args->name);
 
 	if ((fd = creat(filename, S_IRUSR | S_IWUSR)) < 0) {
 		if ((!should_pass) && (errno == ENAMETOOLONG))
@@ -291,8 +298,20 @@ static void stress_filename_test(
 			args->name, sz_max, errno, strerror(errno));
 	} else {
 		(void)close(fd);
-		(void)unlink(filename);
+
+		/* exercise dcache lookup of existent filename */
+		ret = stat(filename, &buf);
+		(void)ret;
+
+		(void)shim_unlink(filename);
 	}
+
+	/* exercise dcache lookup of non-existent filename */
+	ret = stat(filename, &buf);
+	if (ret == 0)
+		pr_fail("%s: stat succeeded on non-existent unlinked file\n",
+			args->name);
+	(void)ret;
 }
 
 /*
@@ -335,6 +354,8 @@ static int stress_filename(const stress_args_t *args)
 			args->name, pathname, errno, strerror(errno));
 		goto tidy_dir;
 	}
+#else
+	UNEXPECTED
 #endif
 
 	(void)shim_strlcpy(filename, pathname, sizeof(filename) - 1);
@@ -554,5 +575,6 @@ stressor_info_t stress_filename_info = {
 	.stressor = stress_filename,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
