@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -56,6 +51,8 @@ static void stress_flock_child(
 			if (!cont)
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 		/*
@@ -81,10 +78,12 @@ static void stress_flock_child(
 		 */
 		ret = flock(fd, LOCK_NB);
 		if (ret == 0) {
-			pr_err("%s: flock failed expected EINVAL, instead got "
+			pr_fail("%s: flock failed expected EINVAL, instead got "
 				"errno=%d (%s)\n", args->name, errno, strerror(errno));
 			(void)flock(fd, LOCK_UN);
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(LOCK_SH)
@@ -98,6 +97,8 @@ static void stress_flock_child(
 			if (!cont)
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(LOCK_SH) &&		\
@@ -112,6 +113,8 @@ static void stress_flock_child(
 			if (!cont)
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(LOCK_MAND) &&	\
@@ -126,6 +129,8 @@ static void stress_flock_child(
 			if (!cont)
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(LOCK_MAND) &&	\
@@ -140,6 +145,8 @@ static void stress_flock_child(
 			if (!cont)
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(LOCK_EX) &&		\
@@ -155,13 +162,16 @@ static void stress_flock_child(
 			if (!cont)
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 #if defined(__linux__)
 		if ((i & 0xff) == 0) {
 			char buf[4096];
+			ssize_t rret;
 
-			ret = system_read("/proc/locks", buf, sizeof(buf));
-			(void)ret;
+			rret = system_read("/proc/locks", buf, sizeof(buf));
+			(void)rret;
 		}
 #endif
 	}
@@ -224,7 +234,7 @@ reap:
 		}
 	}
 
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 err:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)stress_temp_dir_rm_args(args);
