@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-pragma.h"
+
+#define MIN_FORKS		(1)
+#define MAX_FORKS		(16000)
+#define DEFAULT_FORKS		(1)
+
+#define MIN_VFORKS		(1)
+#define MAX_VFORKS		(16000)
+#define DEFAULT_VFORKS		(1)
 
 static const stress_help_t fork_help[] = {
 	{ "f N","fork N",	"start N workers spinning on fork() and exit()" },
@@ -142,16 +146,13 @@ static int stress_fork_fn(
 				break;
 			case STRESS_VFORK:
 				fork_fn_name = "vfork";
-STRESS_PRAGMA_PUSH
-STRESS_PRAGMA_WARN_OFF
-				pid = vfork();
-STRESS_PRAGMA_POP
+				pid = shim_vfork();
 				break;
 			default:
 				/* This should not happen */
 				fork_fn_name = "unknown";
 				pid = -1;
-				pr_err("%s: bad fork/vfork function, aborting\n", args->name);
+				pr_fail("%s: bad fork/vfork function, aborting\n", args->name);
 				errno = ENOSYS;
 				break;
 			}
@@ -160,6 +161,8 @@ STRESS_PRAGMA_POP
 #if defined(HAVE_GETPGID)
 				const pid_t my_pid = getpid();
 				const pid_t my_pgid = getpgid(my_pid);
+#else
+				UNEXPECTED
 #endif
 
 				/*
@@ -195,6 +198,8 @@ STRESS_PRAGMA_POP
 #if defined(HAVE_GETPGID)
 				ret = setpgid(my_pid, my_pgid);
 				(void)ret;
+#else
+				UNEXPECTED
 #endif
 
 				/* -ve pgid is EINVAL */
@@ -324,6 +329,7 @@ stressor_info_t stress_fork_info = {
 	.stressor = stress_fork,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = fork_opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = fork_help
 };
 
