@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+
+#if defined(HAVE_FENV_H)
+#include <fenv.h>
+#endif
+
+#if defined(HAVE_FLOAT_H)
+#include <float.h>
+#endif
+
 
 static const stress_help_t help[] = {
 	{ NULL,	"fp-error N",	  "start N workers exercising floating point errors" },
@@ -82,6 +87,7 @@ static void stress_fp_check(
 {
 #if defined(__linux__) &&		\
     !defined(STRESS_ARCH_M68K) &&	\
+    !defined(__ICC) &&			\
     NEED_GNUC(4,8,0)
 	if (stress_double_same(val, val_expected, is_nan, is_inf) &&
 	    (fetestexcept(excepts_expected) & excepts_expected) &&
@@ -158,6 +164,7 @@ static int stress_fp_error(const stress_args_t *args)
 #endif
 
 #if defined(FE_INEXACT)
+#if !defined(STRESS_ARCH_ALPHA)
 		/*
 		 * Use volatiles to force compiler to generate code
 		 * to perform run time computation of 1.0 / M_PI
@@ -167,6 +174,7 @@ static int stress_fp_error(const stress_args_t *args)
 		SET_VOLATILE(d2, M_PI);
 		stress_fp_check(args, "1.0 / M_PI", d1 / d2, d1 / d2,
 			false, false, 0, FE_INEXACT);
+#endif
 
 		/*
 		 * Use volatiles to force compiler to generate code
@@ -176,7 +184,7 @@ static int stress_fp_error(const stress_args_t *args)
 		SET_VOLATILE(d1, DBL_MAX);
 		SET_VOLATILE(d2, DBL_MAX / 2.0);
 		stress_fp_check(args, "DBL_MAX + DBL_MAX / 2.0",
-			DBL_MAX + DBL_MAX / 2.0, (double)INFINITY,
+			d1 + d2, (double)INFINITY,
 			false, true, 0, FE_OVERFLOW | FE_INEXACT);
 #endif
 
@@ -211,6 +219,7 @@ static int stress_fp_error(const stress_args_t *args)
 stressor_info_t stress_fp_error_info = {
 	.stressor = stress_fp_error,
 	.class = CLASS_CPU,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
