@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -101,7 +96,7 @@ static void stress_fstat_helper(const stress_ctxt_t *ctxt)
 	struct stat buf;
 #if defined(AT_EMPTY_PATH) &&	\
     defined(AT_SYMLINK_NOFOLLOW)
-	struct shim_statx bufx;
+	shim_statx_t bufx;
 #endif
 	stress_stat_info_t *si = ctxt->si;
 	int ret;
@@ -135,6 +130,8 @@ static void stress_fstat_helper(const stress_ctxt_t *ctxt)
 	/* invalid flags in statx */
 	ret = shim_statx(AT_EMPTY_PATH, si->path, AT_SYMLINK_NOFOLLOW, ~0, &bufx);
 	(void)ret;
+#else
+	UNEXPECTED
 #endif
 	/*
 	 *  Opening /dev files such as /dev/urandom
