@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-put.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"full N",	"start N workers exercising /dev/full" },
@@ -62,7 +58,7 @@ static int stress_full(const stress_args_t *args)
 		if ((fd = open("/dev/full", O_RDWR)) < 0) {
 			if (errno == ENOENT) {
 				if (args->instance == 0)
-					pr_inf("%s: /dev/full not available, skipping stress test\n",
+					pr_inf_skip("%s: /dev/full not available, skipping stress test\n",
 						args->name);
 				return EXIT_NOT_IMPLEMENTED;
 			}
@@ -121,6 +117,8 @@ try_read:
 			stress_uint8_put(*ptr);
 #if defined(MS_SYNC)
 			(void)msync((void *)ptr, args->page_size, MS_SYNC);
+#else
+			UNEXPECTED
 #endif
 			(void)munmap((void *)ptr, args->page_size);
 		}
@@ -155,6 +153,7 @@ try_read:
 stressor_info_t stress_full_info = {
 	.stressor = stress_full,
 	.class = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
