@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_FUTEX_H)
+#include <linux/futex.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"futex N",	"start N workers exercising a fast mutex" },
 	{ NULL,	"futex-ops N",	"stop after N fast mutex bogo operations" },
@@ -35,6 +34,54 @@ static const stress_help_t help[] = {
 
 #define THRESHOLD	(100000)
 
+/*
+ *  stress_futex_wait()
+ *     exercise futex_wait and every 16th time futex_waitv
+ */
+static int stress_futex_wait(uint32_t *futex, const int val, const uint64_t nsec)
+{
+	struct timespec t;
+
+#if defined(FUTEX_32) &&		\
+    defined(CLOCK_MONOTONIC)
+	static int try_futex_waitv = true;
+	static int try = 0;
+
+	if (try_futex_waitv && try++ > 16) {
+		struct shim_futex_waitv w;
+
+		w.val = (uint64_t)val;
+		w.uaddr = (uintptr_t)futex;
+		w.flags = FUTEX_32;
+		w.__reserved = 0;
+
+		try = 0;
+		if (clock_gettime(CLOCK_MONOTONIC, &t) == 0) {
+			int ret;
+
+			t.tv_nsec += nsec;
+			if (t.tv_nsec > 1000000000) {
+				t.tv_sec++;
+				t.tv_nsec -= 1000000000;
+			}
+
+			ret = shim_futex_waitv(&w, 1, 0, &t, CLOCK_MONOTONIC);
+			if ((ret < 0) && (ret == ENOSYS)) {
+				try_futex_waitv = false;
+			} else {
+				return ret;
+			}
+		}
+	}
+#else
+	/* UNEXPECTED */
+#endif
+	t.tv_sec = 0;
+	t.tv_nsec = nsec;
+
+	return shim_futex_wait(futex, val, &t);
+}
+
 /*
  *  stress_futex()
  *	stress system by futex calls. The intention is not to
@@ -96,14 +143,13 @@ again:
 
 		do {
 			/* Small timeout to force rapid timer wakeups */
-			const struct timespec t = { .tv_sec = 0, .tv_nsec = 5000 };
 			int ret;
 
 			/* Break early before potential long wait */
 			if (!keep_stressing_flag())
 				break;
 
-			ret = shim_futex_wait(futex, 0, &t);
+			ret = stress_futex_wait(futex, 0, 5000);
 
 			/* timeout, re-do, stress on stupid fast polling */
 			if ((ret < 0) && (errno == ETIMEDOUT)) {
@@ -134,6 +180,7 @@ finish:
 stressor_info_t stress_futex_info = {
 	.stressor = stress_futex,
 	.class = CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
