@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,17 +16,40 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
+#if defined(HAVE_LINUX_SYSCTL_H)
+#include <linux/sysctl.h>
+#endif
+
+#if defined(HAVE_SYS_STATVFS_H)
+#include <sys/statvfs.h>
+#endif
+
+#if defined(HAVE_SYS_TIMEX_H)
+#include <sys/timex.h>
+#endif
+
+#if defined(HAVE_SYS_UTSNAME_H)
+#include <sys/utsname.h>
+#endif
+
+#if defined(HAVE_SYS_VFS_H)
+#include <sys/vfs.h>
+#endif
+
+#if defined(HAVE_UCONTEXT_H)
+#include <ucontext.h>
+#endif
+
+#if !defined(_DEFAULT_SOURCE)
 #define _DEFAULT_SOURCE 1
+#endif
+#if !defined(_BSD_SOURCE)
 #define _BSD_SOURCE 1
+#endif
 
 #define GIDS_MAX 	(1024)
 #define MOUNTS_MAX	(256)
@@ -210,7 +234,11 @@ static int stress_get(const stress_args_t *args)
 			if (!keep_stressing_flag())
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETDOMAINNAME)
@@ -222,6 +250,8 @@ static int stress_get(const stress_args_t *args)
 			if (!keep_stressing_flag())
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETHOSTID)
@@ -233,6 +263,8 @@ static int stress_get(const stress_args_t *args)
 			if (!keep_stressing_flag())
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETHOSTNAME)
@@ -244,6 +276,8 @@ static int stress_get(const stress_args_t *args)
 			if (!keep_stressing_flag())
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 		ptr = getcwd(path, sizeof path);
@@ -300,8 +334,10 @@ static int stress_get(const stress_args_t *args)
 		/*
 		 *  libc may detect a -ve gidsetsize argument and not call
 		 *  the system call. Override this by directly calling the
-		 *  system call if possible
+		 *  system call if possible. Memset gids to zero to keep
+		 *  valgrind happy.
 		 */
+		(void)memset(gids, 0, sizeof(gids));
 		ret = (int)syscall(__NR_getgroups, -1, gids);
 		(void)ret;
 #endif
@@ -311,6 +347,8 @@ static int stress_get(const stress_args_t *args)
 #if defined(HAVE_GETPGRP)
 		pid = getpgrp();
 		(void)pid;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETPGID)
@@ -326,6 +364,8 @@ static int stress_get(const stress_args_t *args)
 
 		if (!keep_stressing_flag())
 			break;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETPRIORITY)
@@ -352,6 +392,8 @@ static int stress_get(const stress_args_t *args)
 			ret = getpriority(priorities[i], ~(id_t)0);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETRESGID)
@@ -365,6 +407,8 @@ static int stress_get(const stress_args_t *args)
 			if (!keep_stressing_flag())
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETRESUID)
@@ -378,6 +422,8 @@ static int stress_get(const stress_args_t *args)
 			if (!keep_stressing_flag())
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 		/* Invalid getrlimit syscall and ignoring failure */
 		(void)getrlimit(INT_MAX, &rlim);
@@ -438,6 +484,8 @@ static int stress_get(const stress_args_t *args)
 			if (!keep_stressing_flag())
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_LINUX_SYSCTL_H) &&	\
@@ -449,7 +497,7 @@ static int stress_get(const stress_args_t *args)
 			 */
 			int name[] = { KERN_VERSION };
 			char kern_version[64];
-			size_t kern_version_len;
+			size_t kern_version_len = 0;
 			struct __sysctl_args sysctl_args;
 
 			(void)memset(&sysctl_args, 0, sizeof(sysctl_args));
@@ -486,6 +534,8 @@ static int stress_get(const stress_args_t *args)
 		(void)ret;
 		if (!keep_stressing_flag())
 			break;
+#else
+		UNEXPECTED
 #endif
 
 		(void)shim_gettid();
@@ -606,16 +656,22 @@ static int stress_get(const stress_args_t *args)
 				pr_fail("%s: uname failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETPAGESIZE)
 		ret = getpagesize();
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETDTABLESIZE)
 		ret = getdtablesize();
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_LOOKUP_DCOOKIE)
@@ -625,6 +681,8 @@ static int stress_get(const stress_args_t *args)
 			ret = (int)syscall(__NR_lookup_dcookie, buf, sizeof(buf));
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_TIMEX_H) &&	\
@@ -638,6 +696,8 @@ static int stress_get(const stress_args_t *args)
 				pr_fail("%s: adjtimex failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_TIMEX_H) &&	\
@@ -647,6 +707,8 @@ static int stress_get(const stress_args_t *args)
 		if (cap_sys_time && verify && (ret < 0) && (errno != EPERM))
 			pr_fail("%s: adjtime failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+#else
+		UNEXPECTED
 #endif
 
 #if defined(__NR_lookup_dcookie)
@@ -705,6 +767,8 @@ static int stress_get(const stress_args_t *args)
 
 			statvfs(mnts[i], &buf);
 		}
+#else
+		UNEXPECTED
 #endif
 
 		inc_counter(args);
@@ -720,5 +784,6 @@ static int stress_get(const stress_args_t *args)
 stressor_info_t stress_get_info = {
 	.stressor = stress_get,
 	.class = CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
