@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_RANDOM_H)
+#include <linux/random.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"getrandom N",	   "start N workers fetching random data via getrandom()" },
 	{ NULL,	"getrandom-ops N", "stop after N getrandom bogo operations" },
@@ -52,7 +51,7 @@ static int stress_getrandom_supported(const char *name)
 
 	ret = shim_getrandom(buffer, sizeof(buffer), 0);
 	if ((ret < 0) && (errno == ENOSYS)) {
-		pr_inf("%s stressor will be skipped, getrandom() not supported\n", name);
+		pr_inf_skip("%s stressor will be skipped, getrandom() not supported\n", name);
 		return -1;
 	}
 	return 0;
@@ -127,7 +126,7 @@ static int stress_getrandom(const stress_args_t *args)
 				if (errno == ENOSYS) {
 					/* Should not happen.. */
 					if (args->instance == 0)
-						pr_inf("%s: stressor will be skipped, "
+						pr_inf_skip("%s: stressor will be skipped, "
 							"getrandom() not supported\n",
 							args->name);
 					return EXIT_NOT_IMPLEMENTED;
@@ -150,6 +149,7 @@ stressor_info_t stress_getrandom_info = {
 	.stressor = stress_getrandom,
 	.supported = stress_getrandom_supported,
 	.class = CLASS_OS | CLASS_CPU,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
