@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -118,6 +113,7 @@ static int get_mount_info(const stress_args_t *args)
 static int stress_handle_child(const stress_args_t *args, void *context)
 {
 	const int mounts = *((int *)context);
+	const int bad_fd = stress_get_bad_fd();
 
 	do {
 		struct file_handle *fhp, *tmp;
@@ -198,13 +194,24 @@ static int stress_handle_child(const stress_args_t *args, void *context)
 		fhp->handle_bytes = 0;
 		(void)name_to_handle_at(AT_FDCWD, "", fhp, &mount_id, 0);
 
-		/* Exercise with invalid mount_fd */
+		/* Exercise with invalid handle_bytes, EOVERFLOW */
+		fhp->handle_bytes = 1;
+		(void)name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, AT_EMPTY_PATH);
+
+		/* Exercise with invalid mount_fd, part 1*/
 		fhp->handle_bytes = 32;
 		(void)name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, 0);
 		fd = open_by_handle_at(-1, fhp, O_RDONLY);
 		if (fd >= 0)
 			(void)close(fd);
 
+		/* Exercise with invalid mount_fd, part 2 */
+		fhp->handle_bytes = 32;
+		(void)name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, 0);
+		fd = open_by_handle_at(bad_fd, fhp, O_RDONLY);
+		if (fd >= 0)
+			(void)close(fd);
+
 		/*
 		 *  Exercise with bad handle, unconstify f_handle,
 		 *  fill it with random garbage and cause an ESTALE
@@ -254,6 +261,7 @@ static int stress_handle(const stress_args_t *args)
 stressor_info_t stress_handle_info = {
 	.stressor = stress_handle,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
