@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_UIO_H)
+#include <sys/uio.h>
+#endif
+
+#if defined(HAVE_UTIME_H)
+#include <utime.h>
+#endif
+
+#define MIN_HDD_BYTES		(1 * MB)
+#define MAX_HDD_BYTES		(MAX_FILE_LIMIT)
+#define DEFAULT_HDD_BYTES	(1 * GB)
+
+#define MIN_HDD_WRITE_SIZE	(1)
+#define MAX_HDD_WRITE_SIZE	(4 * MB)
+#define DEFAULT_HDD_WRITE_SIZE	(64 * 1024)
+
 #define BUF_ALIGNMENT		(4096)
 #define HDD_IO_VEC_MAX		(16)		/* Must be power of 2 */
 
@@ -204,6 +215,8 @@ static ssize_t stress_hdd_write(
 #if defined(HAVE_FUTIMES)
 	if (hdd_flags & HDD_OPT_UTIMES)
 		stress_hdd_utimes(fd);
+#else
+	UNEXPECTED
 #endif
 	errno = 0;
 	if (hdd_flags & HDD_OPT_IOVEC) {
@@ -257,14 +270,20 @@ static ssize_t stress_hdd_write(
 #if defined(HAVE_FSYNC)
 	if (hdd_flags & HDD_OPT_FSYNC)
 		(void)shim_fsync(fd);
+#else
+	UNEXPECTED
 #endif
 #if defined(HAVE_FDATASYNC)
 	if (hdd_flags & HDD_OPT_FDATASYNC)
 		(void)shim_fdatasync(fd);
+#else
+	UNEXPECTED
 #endif
 #if defined(HAVE_SYNCFS)
 	if (hdd_flags & HDD_OPT_SYNCFS)
 		(void)syncfs(fd);
+#else
+	UNEXPECTED
 #endif
 
 	return ret;
@@ -284,6 +303,8 @@ static ssize_t stress_hdd_read(
 #if defined(HAVE_FUTIMES)
 	if (hdd_flags & HDD_OPT_UTIMES)
 		stress_hdd_utimes(fd);
+#else
+	UNEXPECTED
 #endif
 
 	errno = 0;
@@ -361,6 +382,8 @@ static void stress_hdd_invalid_read(const int fd, uint8_t *buf)
 	/* invalid preadv2 flags */
 	ret = preadv2(fd, iov, HDD_IO_VEC_MAX, 0, ~0);
 	(void)ret;
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_UIO_H) &&	\
@@ -372,6 +395,8 @@ static void stress_hdd_invalid_read(const int fd, uint8_t *buf)
 	/* invalid preadv offset */
 	ret = preadv(fd, iov, HDD_IO_VEC_MAX, -1);
 	(void)ret;
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_UIO_H)
@@ -405,6 +430,8 @@ static void stress_hdd_invalid_write(const int fd, uint8_t *buf)
 		iov[i].iov_len = (size_t)1;
 		data++;
 	}
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_UIO_H) &&	\
@@ -420,6 +447,8 @@ static void stress_hdd_invalid_write(const int fd, uint8_t *buf)
 	/* invalid pwritev2 flags */
 	ret = pwritev2(fd, iov, HDD_IO_VEC_MAX, 0, ~0);
 	(void)ret;
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_UIO_H) &&	\
@@ -431,12 +460,16 @@ static void stress_hdd_invalid_write(const int fd, uint8_t *buf)
 	/* invalid pwritev offset */
 	ret = pwritev(fd, iov, HDD_IO_VEC_MAX, -1);
 	(void)ret;
+#else
+	UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_UIO_H)
 	/* invalid writev fd */
 	ret = writev(-1, iov, HDD_IO_VEC_MAX);
 	(void)ret;
+#else
+	UNEXPECTED
 #endif
 
 	/* invalid write fd */
@@ -693,7 +726,7 @@ static int stress_hdd(const stress_args_t *args)
 			 *  not support O_DIRECT:
 			 *  https://bugzilla.kernel.org/show_bug.cgi?id=213041
 			 */
-			(void)unlink(filename);
+			(void)shim_unlink(filename);
 			goto finish;
 		}
 
@@ -736,7 +769,7 @@ static int stress_hdd(const stress_args_t *args)
 			ret = truncate("", (off_t)0);
 			(void)ret;
 		}
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 
 		if (!keep_stressing(args)) {
 			(void)close(fd);
@@ -759,13 +792,13 @@ static int stress_hdd(const stress_args_t *args)
 			w = stress_mwc32();
 			z = stress_mwc32();
 
-			stress_mwc_seed(w, z);
+			stress_mwc_set_seed(w, z);
 
 			for (i = 0; i < hdd_bytes; i += hdd_write_size) {
 				size_t j;
-				size_t offset = (i == 0) ?
+				size_t offset = ((i == 0) ?
 					hdd_bytes :
-					(stress_mwc64() % hdd_bytes) & ~511UL;
+					(stress_mwc64() % hdd_bytes)) & ~511UL;
 rnd_wr_retry:
 				if (!keep_stressing(args)) {
 					(void)close(fd);
@@ -794,7 +827,7 @@ rnd_wr_retry:
 				inc_counter(args);
 			}
 
-			stress_mwc_seed(w, z);
+			stress_mwc_set_seed(w, z);
 		}
 		/* Sequential Write */
 		if (hdd_flags & HDD_OPT_WR_SEQ) {
@@ -986,5 +1019,6 @@ stressor_info_t stress_hdd_info = {
 	.stressor = stress_hdd,
 	.class = CLASS_IO | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
