@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,21 +16,30 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"hrtimers N",	  "start N workers that exercise high resolution timers" },
 	{ NULL,	"hrtimers-ops N", "stop after N bogo high-res timer bogo operations" },
+	{ NULL, "hrtimers-adjust","adjust rate to try and maximum timer rate" },
 	{ NULL,	NULL,		  NULL }
 };
 
+static int stress_set_hrtimers_adjust(const char *opt)
+{
+        bool hrtimers_adjust = true;
+
+        (void)opt;
+
+        return stress_set_setting("hrtimers-adjust", TYPE_ID_BOOL, &hrtimers_adjust);
+}
+
+static const stress_opt_set_func_t opt_set_funcs[] = {
+	{ OPT_hrtimers_adjust,	stress_set_hrtimers_adjust },
+	{ 0,			NULL },
+};
+
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_TIMER_CREATE) &&	\
     defined(HAVE_TIMER_DELETE) &&	\
@@ -38,6 +48,7 @@ static volatile uint64_t *timer_counter;
 static uint64_t max_ops;
 static timer_t timerid;
 static double start;
+static long ns_delay;
 
 #define PROCS_MAX	(32)
 
@@ -47,8 +58,12 @@ static double start;
  */
 static void stress_hrtimers_set(struct itimerspec *timer)
 {
+	if (ns_delay < 0) {
+		timer->it_value.tv_nsec = (stress_mwc64() % 50000) + 1;
+	} else {
+		timer->it_value.tv_nsec = ns_delay;
+	}
 	timer->it_value.tv_sec = 0;
-	timer->it_value.tv_nsec = (stress_mwc64() % 50000) + 1;
 	timer->it_interval.tv_sec = timer->it_value.tv_sec;
 	timer->it_interval.tv_nsec = timer->it_value.tv_nsec;
 }
@@ -86,6 +101,7 @@ static void MLOCKED_TEXT stress_hrtimers_handler(int sig)
 		if ((stress_time_now() - start) > (double)g_opt_timeout)
 			goto cancel;
 	stress_hrtimers_set(&timer);
+	(void)timer_settime(timerid, 0, &timer, NULL);
 	return;
 
 cancel:
@@ -104,9 +120,13 @@ static int stress_hrtimer_process(const stress_args_t *args, uint64_t *counter)
 	struct sigevent sev;
 	struct itimerspec timer;
 	sigset_t mask;
+	static uint64_t last_count;
+	double previous_time, dt;
+	bool hrtimer_interrupt = false;
 	int ret;
 
 	timer_counter = counter;
+	last_count = *counter;
 
 	(void)sigemptyset(&mask);
 	(void)sigaddset(&mask, SIGINT);
@@ -136,11 +156,54 @@ static int stress_hrtimer_process(const stress_args_t *args, uint64_t *counter)
 		return EXIT_FAILURE;
 	}
 
+	previous_time = stress_time_now();
+
 	do {
-		/* The sleep will be interrupted on each hrtimer tick */
-		(void)sleep(1);
+		if (ns_delay < 0) {
+			ret = sleep(1);
+			if (ret == 0)
+				hrtimer_interrupt = true;
+		} else {
+			long ns_adjust = ns_delay >> 2;
+			double now;
+
+			/* The sleep will be interrupted on each hrtimer tick */
+			ret = sleep(1);
+			if (ret == 0)
+				hrtimer_interrupt = true;
+
+			now = stress_time_now();
+			dt = now - previous_time;
+
+			if (dt > 0.1) {
+				static double last_signal_rate = -1.0;
+				double signal_rate;
+				const uint64_t signal_count = *counter - last_count;
+
+				previous_time = now;
+
+				/* Handle unlikely time warping */
+				if (UNLIKELY(dt <= 0.0))
+					continue;
+
+				signal_rate = (double)signal_count / dt;
+				if (last_signal_rate > 0.0) {
+					if (last_signal_rate > signal_rate) {
+						ns_delay += ns_adjust;
+					} else {
+						ns_delay -= ns_adjust;
+					}
+				}
+				last_signal_rate = signal_rate;
+			}
+		}
 	} while (stress_hrtimers_keep_stressing());
 
+	if (!hrtimer_interrupt) {
+		pr_fail("%s: did not detect any hrtimer interrupts of a sleep\n",
+			args->name);
+	}
+
 	if (timer_delete(timerid) < 0) {
 		pr_fail("%s: timer_delete failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -156,8 +219,13 @@ static int stress_hrtimers(const stress_args_t *args)
 	const size_t page_size = args->page_size;
 	const size_t counters_sz = sizeof(*counters) * PROCS_MAX;
 	const size_t sz = (counters_sz + page_size) & ~(page_size - 1);
+        bool hrtimers_adjust = false;
+	double start_time, dt;
 	size_t i;
 
+        (void)stress_get_setting("hrtimers-adjust", &hrtimers_adjust);
+	ns_delay = hrtimers_adjust ? 50000 : -1;
+
 	max_ops = args->max_ops / PROCS_MAX;
 
 	(void)memset(pids, 0, sizeof(pids));
@@ -188,6 +256,8 @@ static int stress_hrtimers(const stress_args_t *args)
 		}
 	}
 
+	start_time = stress_time_now();
+
 	do {
 		struct timespec req;
 
@@ -200,6 +270,12 @@ static int stress_hrtimers(const stress_args_t *args)
 			add_counter(args, counters[i]);
 	} while (keep_stressing(args));
 
+	dt = stress_time_now() - start_time;
+	if (dt > 0.0) {
+		pr_inf("%s: hrtimer signals at %.3f MHz\n",
+			args->name, ((double)get_counter(args) / dt) / 1000000.0);
+	}
+
 
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -225,12 +301,15 @@ reap:
 stressor_info_t stress_hrtimers_info = {
 	.stressor = stress_hrtimers,
 	.class = CLASS_SCHEDULER,
+	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 stressor_info_t stress_hrtimers_info = {
 	.stressor = stress_not_implemented,
 	.class = CLASS_SCHEDULER,
+	.opt_set_funcs = opt_set_funcs,
 	.help = help
 };
 #endif
