@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_HSEARCH_SIZE	(1 * KB)
+#define MAX_HSEARCH_SIZE	(4 * MB)
+#define DEFAULT_HSEARCH_SIZE	(8 * KB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"hsearch N",	  "start N workers that exercise a hash table search" },
 	{ NULL,	"hsearch-ops N",  "stop after N hash search bogo operations" },
@@ -41,7 +40,7 @@ static int stress_set_hsearch_size(const char *opt)
 
 	hsearch_size = stress_get_uint64(opt);
 	stress_check_range("hsearch-size", hsearch_size,
-		MIN_TSEARCH_SIZE, MAX_TSEARCH_SIZE);
+		MIN_HSEARCH_SIZE, MAX_HSEARCH_SIZE);
 	return stress_set_setting("hsearch-size", TYPE_ID_UINT64, &hsearch_size);
 }
 
@@ -135,10 +134,10 @@ free_all:
 	 * so for now, don't free them and just let it
 	 * leak, the exit() will clean up the heap for us
 	 */
-	/*
+#if defined(__linux__)
 	for (i = 0; i < max; i++)
 		free(keys[i]);
-	*/
+#endif
 	free(keys);
 free_hash:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -156,5 +155,6 @@ stressor_info_t stress_hsearch_info = {
 	.stressor = stress_hsearch,
 	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
