@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2012-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,17 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
- * stress-inotify.c is derived from the eCryptfs inotify tests
- * that I authored in 2012.
  */
+
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_EPOLL_H)
+#include <sys/epoll.h>
+#endif
+
+#if defined(HAVE_SYS_INOTIFY_H)
+#include <sys/inotify.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"inotify N",	 "start N workers exercising inotify events" },
 	{ NULL,	"inotify-ops N", "stop inotify workers after N bogo operations" },
@@ -58,7 +60,8 @@ typedef struct {
  */
 static void exercise_inotify1(void)
 {
-#if defined(IN_NONBLOCK) && defined(IN_CLOEXEC)
+#if defined(IN_NONBLOCK) &&	\
+    defined(IN_CLOEXEC)
 	int fd;
 
 	fd = inotify_init1(IN_NONBLOCK);
@@ -73,6 +76,8 @@ static void exercise_inotify1(void)
 	fd = inotify_init1(~0);
 	if (fd >= 0)
 		(void)close(fd);
+#else
+	UNEXPECTED
 #endif
 }
 
@@ -101,7 +106,7 @@ static void exercise_inotify_add_watch(
 	if (wd >= 0)
 		(void)inotify_rm_watch(fd, wd);
 
-	wd = inotify_add_watch(fd, watchname, ~0);
+	wd = inotify_add_watch(fd, watchname, ~0U);
 	if (wd >= 0)
 		(void)inotify_rm_watch(fd, wd);
 
@@ -111,6 +116,8 @@ static void exercise_inotify_add_watch(
 	wd = inotify_add_watch(fd, watchname, IN_MASK_CREATE | IN_MASK_ADD);
 	if (wd >= 0)
 		(void)inotify_rm_watch(fd, wd);
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_MASK_CREATE) &&	\
@@ -125,6 +132,8 @@ static void exercise_inotify_add_watch(
 		(void)inotify_rm_watch(fd, wd);
 	if (wd2 >= 0)
 		(void)inotify_rm_watch(fd, wd2);
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_MASK_ADD)
@@ -132,6 +141,8 @@ static void exercise_inotify_add_watch(
 	wd = inotify_add_watch(bad_fd, watchname, IN_MASK_ADD);
 	if (wd >= 0)
 		(void)inotify_rm_watch(fd, wd);
+#else
+	UNEXPECTED
 #endif
 
 	(void)close(fd);
@@ -169,6 +180,8 @@ static void exercise_inotify_rm_watch(const int bad_fd)
 		return;
 	ret = inotify_rm_watch(fd, 1);
 	(void)ret;
+#else
+	UNEXPECTED
 #endif
 
 	(void)close(fd);
@@ -185,11 +198,12 @@ static void inotify_exercise(
 	const char *watchname,		/* File/directory to watch using inotify */
 	const char *matchname,		/* Filename for inotify event to report */
 	const stress_inotify_helper func, /* Helper func */
-	const int flags,		/* IN_* flags to watch for */
+	const uint32_t flags,		/* IN_* flags to watch for */
 	void *private,			/* Helper func private data */
 	const int bad_fd)		/* A bad file descriptor */
 {
-	int fd, wd, check_flags = flags, n = 0;
+	int fd, wd, n = 0;
+	uint32_t check_flags = flags;
 	char buffer[1024];
 
 	exercise_inotify1();
@@ -199,7 +213,7 @@ static void inotify_exercise(
 retry:
 	n++;
 	if ((fd = inotify_init()) < 0) {
-		if (!keep_stressing_flag())
+		if (!keep_stressing(args))
 			return;
 
 		/* This is just so wrong... */
@@ -255,7 +269,7 @@ retry:
 		}
 
 redo:
-		if (!keep_stressing_flag())
+		if (!keep_stressing(args))
 			break;
 		/*
 		 *  Exercise FIOREAD to get inotify code coverage up
@@ -288,9 +302,11 @@ redo:
 		while ((i >= 0) && (i <= len - (ssize_t)sizeof(struct inotify_event))) {
 			struct inotify_event *event =
 				(struct inotify_event *)&buffer[i];
-			int f = event->mask & (IN_DELETE_SELF | IN_MOVE_SELF |
-					       IN_MOVED_TO | IN_MOVED_FROM |
-					       IN_ATTRIB);
+			uint32_t f = event->mask & (IN_DELETE_SELF |
+						    IN_MOVE_SELF |
+						    IN_MOVED_TO |
+						    IN_MOVED_FROM |
+						    IN_ATTRIB);
 			if (event->len &&
 			    strcmp(event->name, matchname) == 0 &&
 			    flags & event->mask)
@@ -316,7 +332,7 @@ cleanup:
  */
 static int rm_file(const stress_args_t *args, const char *path)
 {
-	if ((unlink(path) < 0) && errno != ENOENT) {
+	if ((shim_unlink(path) < 0) && errno != ENOENT) {
 		pr_err("%s: cannot remove file %s: errno=%d (%s)\n",
 			args->name, path, errno, strerror(errno));
 		return -1;
@@ -348,7 +364,7 @@ static int rm_dir(const stress_args_t *args, const char *path)
 		}
 		(void)closedir(dp);
 	}
-	ret = rmdir(path);
+	ret = shim_rmdir(path);
 	if (ret < 0 && errno != ENOENT)
 		pr_err("%s: cannot remove directory %s: errno=%d (%s)\n",
 			args->name, path, errno, strerror(errno));
@@ -392,19 +408,20 @@ static int mk_file(const stress_args_t *args, const char *filename, const size_t
 	}
 
 	(void)memset(buffer, 'x', BUF_SIZE);
-	while (sz > 0) {
+	while (keep_stressing(args) && (sz > 0)) {
 		size_t n = (sz > BUF_SIZE) ? BUF_SIZE : sz;
-		int ret;
+		ssize_t ret;
 
-		if ((ret = write(fd, buffer, n)) < 0) {
+		ret = write(fd, buffer, n);
+		if (ret < 0) {
 			if (errno == ENOSPC)
-				continue;
+				break;
 			pr_err("%s: error writing to file %s: errno=%d (%s)\n",
 				args->name, filename, errno, strerror(errno));
 			(void)close(fd);
 			return -1;
 		}
-		sz -= ret;
+		sz -= (size_t)ret;
 	}
 
 	if (close(fd) < 0) {
@@ -445,6 +462,8 @@ static void inotify_attrib_file(
 		inotify_attrib_helper, IN_ATTRIB, NULL, bad_fd);
 	(void)rm_file(args, filepath);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_ACCESS)
@@ -466,7 +485,7 @@ static int inotify_access_helper(
 
 	/* Just want to force an access */
 do_access:
-	if (keep_stressing_flag() && (read(fd, buffer, 1) < 0)) {
+	if (keep_stressing(args) && (read(fd, buffer, 1) < 0)) {
 		if ((errno == EAGAIN) || (errno == EINTR))
 			goto do_access;
 		pr_err("%s: cannot read file %s: errno=%d (%s)\n",
@@ -492,6 +511,8 @@ static void inotify_access_file(
 		inotify_access_helper, IN_ACCESS, NULL, bad_fd);
 	(void)rm_file(args, filepath);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_MODIFY)
@@ -513,7 +534,7 @@ static int inotify_modify_helper(
 		goto remove;
 	}
 do_modify:
-	if (keep_stressing_flag() && (write(fd, buffer, 1) < 0)) {
+	if (keep_stressing(args) && (write(fd, buffer, 1) < 0)) {
 		if ((errno == EAGAIN) || (errno == EINTR))
 			goto do_modify;
 		pr_err("%s: cannot write to file %s: errno=%d (%s)\n",
@@ -537,6 +558,8 @@ static void inotify_modify_file(
 	inotify_exercise(args, filepath, path, "inotify_file",
 		inotify_modify_helper, IN_MODIFY, NULL, bad_fd);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_CREATE)
@@ -568,6 +591,8 @@ static void inotify_creat_file(
 		inotify_creat_helper, IN_CREATE, NULL, bad_fd);
 	(void)rm_file(args, filepath);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_OPEN)
@@ -602,6 +627,8 @@ static void inotify_open_file(
 		inotify_open_helper, IN_OPEN, NULL, bad_fd);
 	(void)rm_file(args, filepath);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_DELETE)
@@ -630,6 +657,8 @@ static void inotify_delete_file(
 	/* We remove (again) it just in case the test failed */
 	(void)rm_file(args, filepath);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_DELETE_SELF)
@@ -658,6 +687,8 @@ static void inotify_delete_self(
 	/* We remove (again) in case the test failed */
 	(void)rm_dir(args, filepath);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_MOVE_SELF)
@@ -675,6 +706,8 @@ static int inotify_move_self_helper(
 	}
 	return 0;
 }
+#else
+	UNEXPECTED
 #endif
 
 static void inotify_move_self(
@@ -732,6 +765,8 @@ static void inotify_moved_to(
 	(void)rm_file(args, newfile);
 	(void)rm_dir(args, olddir);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_MOVED_FROM)
@@ -771,6 +806,8 @@ static void inotify_moved_from(
 	(void)rm_file(args, oldfile);	/* In case rename failed */
 	(void)rm_dir(args, newdir);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_CLOSE_WRITE)
@@ -809,6 +846,8 @@ static void inotify_close_write_file(
 	(void)rm_file(args, filepath);
 	(void)close(fd);
 }
+#else
+	UNEXPECTED
 #endif
 
 #if defined(IN_CLOSE_NOWRITE)
@@ -848,6 +887,8 @@ static void inotify_close_nowrite_file(
 	(void)rm_file(args, filepath);
 	(void)close(fd);
 }
+#else
+	UNEXPECTED
 #endif
 
 static const stress_inotify_t inotify_stressors[] = {
@@ -908,7 +949,7 @@ static int stress_inotify(const stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		for (i = 0; keep_stressing_flag() && inotify_stressors[i].func; i++)
+		for (i = 0; keep_stressing(args) && inotify_stressors[i].func; i++)
 			inotify_stressors[i].func(args, pathname, bad_fd);
 		inc_counter(args);
 	} while (keep_stressing(args));
@@ -922,6 +963,7 @@ static int stress_inotify(const stress_args_t *args)
 stressor_info_t stress_inotify_info = {
 	.stressor = stress_inotify,
 	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
