@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 #include "io-uring.h"
 
+#if defined(HAVE_LINUX_IO_URING_H)
+#include <linux/io_uring.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"io-uring N",		"start N workers that issue io-uring I/O requests" },
 	{ NULL,	"io-uring-ops N",	"stop after N bogo io-uring I/O requests" },
@@ -40,9 +39,18 @@ static const stress_help_t help[] = {
     defined(HAVE_POSIX_MEMALIGN) &&	\
     (defined(HAVE_IORING_OP_WRITEV) ||	\
      defined(HAVE_IORING_OP_READV) ||	\
+     defined(HAVE_IORING_OP_WRITE) || 	\
+     defined(HAVE_IORING_OP_READ) || 	\
      defined(HAVE_IORING_OP_FSYNC) ||	\
      defined(HAVE_IORING_OP_NOP) ||	\
-     defined(HAVE_IORING_FALLOCATE))
+     defined(HAVE_IORING_OP_FALLOCATE) || \
+     defined(HAVE_IORING_OP_FADVISE) ||	\
+     defined(HAVE_IORING_OP_CLOSE) ||	\
+     defined(HAVE_IORING_OP_MADVISE) ||	\
+     defined(HAVE_IORING_OP_STATX) || 	\
+     defined(HAVE_IORING_OP_SYNC_FILE_RANGE))
+
+
 
 /*
  *  io uring file info
@@ -96,6 +104,17 @@ typedef struct {
 
 typedef void (*stress_io_uring_setup)(stress_io_uring_file_t *io_uring_file, struct io_uring_sqe *sqe);
 
+/*
+ *  opcode to human readable name lookup
+ */
+typedef struct {
+	const uint8_t opcode;			/* opcode */
+	const char *name;			/* stringified opcode name */
+	const stress_io_uring_setup setup_func;	/* setup function */
+} stress_io_uring_setup_info_t;
+
+static const char *stress_io_uring_opcode_name(const uint8_t opcode);
+
 /*
  *  shim_io_uring_setup
  *	wrapper for io_uring_setup()
@@ -166,7 +185,7 @@ static int stress_setup_io_uring(
 				args->name);
 			return EXIT_NOT_IMPLEMENTED;
 		}
-		pr_err("%s: io_uring_setup failed: errno=%d (%s)\n",
+		pr_fail("%s: io_uring_setup failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -264,7 +283,8 @@ static void stress_close_io_uring(stress_io_uring_submit_t *submit)
 static inline int stress_io_uring_complete(
 	const stress_args_t *args,
 	stress_io_uring_submit_t *submit,
-	const uint8_t opcode)
+	const uint8_t opcode,
+	bool *supported)
 {
 	stress_uring_io_cq_ring_t *cring = &submit->cq_ring;
 	struct io_uring_cqe *cqe;
@@ -282,9 +302,16 @@ static inline int stress_io_uring_complete(
 		if ((cqe->res < 0) && (opcode != IORING_OP_FALLOCATE)) {
 			const int err = abs(cqe->res);
 
-			pr_err("%s: completion uring io error, opcode=%d: %d (%s)\n",
-				args->name, opcode, err, strerror(err));
-			ret = EXIT_FAILURE;
+			/* Silently ignore EOPNOTSUPP completion errors */
+			if (errno == EOPNOTSUPP) {
+				*supported = false;
+			} else  {
+				pr_fail("%s: completion opcode=%d (%s), error=%d (%s)\n",
+					args->name, opcode,
+					stress_io_uring_opcode_name(opcode),
+					err, strerror(err));
+				ret = EXIT_FAILURE;
+			}
 		}
 		head++;
 	}
@@ -305,7 +332,8 @@ static int stress_io_uring_submit(
 	const stress_args_t *args,
 	stress_io_uring_setup setup_func,
 	stress_io_uring_file_t *io_uring_file,
-	stress_io_uring_submit_t *submit)
+	stress_io_uring_submit_t *submit,
+	bool *supported)
 {
 	stress_uring_io_sq_ring_t *sring = &submit->sq_ring;
 	unsigned index = 0, tail = 0, next_tail = 0;
@@ -336,12 +364,16 @@ static int stress_io_uring_submit(
 		/* Silently ignore ENOSPC failures */
 		if (errno == ENOSPC)	
 			return EXIT_SUCCESS;
-		pr_fail("%s: io_uring_enter failed, opcode=%d, errno=%d (%s)\n",
-			args->name, opcode, errno, strerror(errno));
+		pr_fail("%s: io_uring_enter failed, opcode=%d (%s), errno=%d (%s)\n",
+			args->name, opcode,
+			stress_io_uring_opcode_name(opcode),
+			errno, strerror(errno));
+		if (errno == EOPNOTSUPP)
+			*supported = false;
 		return EXIT_FAILURE;
 	}
 
-	return stress_io_uring_complete(args, submit, opcode);
+	return stress_io_uring_complete(args, submit, opcode, supported);
 }
 
 #if defined(HAVE_IORING_OP_READV)
@@ -546,39 +578,104 @@ static void stress_io_uring_madvise_setup(
 }
 #endif
 
-static const stress_io_uring_setup stress_io_uring_setups[] = {
+#if defined(HAVE_IORING_OP_STATX)
+/*
+ *  stress_io_uring_statx_setup()
+ *	setup statx submit over io_uring
+ */
+static void stress_io_uring_statx_setup(
+	stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe)
+{
+	static const char *pathname = "";
+	shim_statx_t statxbuf;
+
+	sqe->opcode = IORING_OP_STATX;
+	sqe->fd = io_uring_file->fd;
+	sqe->addr = (uintptr_t)pathname;
+	sqe->addr2 = (uintptr_t)&statxbuf;
+	sqe->statx_flags = AT_EMPTY_PATH;
+	sqe->ioprio = 0;
+	sqe->buf_index = 0;
+	sqe->flags = 0;
+}
+#endif
+
+#if defined(HAVE_IORING_OP_SYNC_FILE_RANGE)
+/*
+ *  stress_io_uring_sync_file_range_setup()
+ *	setup sync_file_range submit over io_uring
+ */
+static void stress_io_uring_sync_file_range_setup(
+	stress_io_uring_file_t *io_uring_file,
+	struct io_uring_sqe *sqe)
+{
+	sqe->opcode = IORING_OP_SYNC_FILE_RANGE;
+	sqe->fd = io_uring_file->fd;
+	sqe->off = stress_mwc16() & ~511UL;
+	sqe->len = stress_mwc32() & ~511UL;
+	sqe->flags = 0;
+	sqe->addr = 0;
+	sqe->ioprio = 0;
+	sqe->buf_index = 0;
+}
+#endif
+
+static const stress_io_uring_setup_info_t stress_io_uring_setups[] = {
 #if defined(HAVE_IORING_OP_READV)
-	stress_io_uring_readv_setup,
+	{ IORING_OP_READV,	"IORING_OP_READV", 	stress_io_uring_readv_setup },
 #endif
 #if defined(HAVE_IORING_OP_WRITEV)
-	stress_io_uring_writev_setup,
+	{ IORING_OP_WRITEV,	"IORING_OP_WRITEV",	stress_io_uring_writev_setup },
 #endif
 #if defined(HAVE_IORING_OP_READ)
-	stress_io_uring_read_setup,
+	{ IORING_OP_READ,	"IORING_OP_READ",	stress_io_uring_read_setup },
 #endif
 #if defined(HAVE_IORING_OP_WRITE)
-	stress_io_uring_write_setup,
+	{ IORING_OP_WRITE,	"IORING_OP_WRITE",	stress_io_uring_write_setup },
 #endif
 #if defined(HAVE_IORING_OP_FSYNC)
-	stress_io_uring_fsync_setup,
+	{ IORING_OP_FSYNC,	"IORING_OP_FSYNC",	stress_io_uring_fsync_setup },
 #endif
 #if defined(HAVE_IORING_OP_NOP)
-	stress_io_uring_nop_setup,
+	{ IORING_OP_NOP,	"IORING_OP_NOP",	stress_io_uring_nop_setup },
 #endif
 #if defined(HAVE_IORING_OP_FALLOCATE)
-	stress_io_uring_fallocate_setup,
+	{ IORING_OP_FALLOCATE,	"IORING_OP_FALLOCATE",	stress_io_uring_fallocate_setup },
 #endif
 #if defined(HAVE_IORING_OP_FADVISE)
-	stress_io_uring_fadvise_setup,
+	{ IORING_OP_FADVISE,	"IORING_OP_FADVISE",	stress_io_uring_fadvise_setup },
 #endif
 #if defined(HAVE_IORING_OP_CLOSE)
-	stress_io_uring_close_setup,
+	{ IORING_OP_CLOSE,	"IORING_OP_CLOSE",	stress_io_uring_close_setup },
 #endif
 #if defined(HAVE_IORING_OP_MADVISE)
-	stress_io_uring_madvise_setup,
+	{ IORING_OP_MADVISE,	"IORING_OP_MADVISE",	stress_io_uring_madvise_setup },
+#endif
+#if defined(HAVE_IORING_OP_STATX)
+	{ IORING_OP_STATX,	"IORING_OP_STATX",	stress_io_uring_statx_setup },
+#endif
+#if defined(HAVE_IORING_OP_SYNC_FILE_RANGE)
+	{ IORING_OP_SYNC_FILE_RANGE, "IORING_OP_SYNC_FILE_RANGE", stress_io_uring_sync_file_range_setup },
 #endif
 };
 
+/*
+ *  stress_io_uring_opcode_name()
+ *	lookup opcode -> name
+ */
+static const char *stress_io_uring_opcode_name(const uint8_t opcode)
+{
+	size_t i;
+
+	for (i = 0; i < SIZEOF_ARRAY(stress_io_uring_setups); i++) {
+		if (stress_io_uring_setups[i].opcode == opcode)
+			return stress_io_uring_setups[i].name;
+	}
+	return "unknown";
+}
+
+
 /*
  *  stress_io_uring
  *	stress asynchronous I/O
@@ -588,12 +685,13 @@ static int stress_io_uring(const stress_args_t *args)
 	int ret, rc;
 	char filename[PATH_MAX];
 	stress_io_uring_file_t io_uring_file;
-	size_t i;
+	size_t i, j;
 	const size_t blocks = 1024;
 	const size_t block_size = 512;
 	off_t file_size = (off_t)blocks * block_size;
 	stress_io_uring_submit_t submit;
 	const pid_t self = getpid();
+	bool supported[SIZEOF_ARRAY(stress_io_uring_setups)];
 
 	(void)memset(&submit, 0, sizeof(submit));
 	(void)memset(&io_uring_file, 0, sizeof(io_uring_file));
@@ -647,31 +745,32 @@ static int stress_io_uring(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto clean;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	/*
+	 *  Assume all opcodes are supported
+	 */
+	for (j = 0; j < SIZEOF_ARRAY(stress_io_uring_setups); j++) {
+		supported[i] = true;
+	}
+
 	rc = EXIT_SUCCESS;
 	i = 0;
 	do {
-		size_t j;
-
 		for (j = 0; j < SIZEOF_ARRAY(stress_io_uring_setups); j++) {
-			rc = stress_io_uring_submit(args, stress_io_uring_setups[j], &io_uring_file, &submit);
-			if ((rc != EXIT_SUCCESS) || !keep_stressing(args))
-				break;
+			if (supported[j]) {
+				rc = stress_io_uring_submit(args,
+					stress_io_uring_setups[j].setup_func,
+					&io_uring_file, &submit, &supported[j]);
+				if ((rc != EXIT_SUCCESS) || !keep_stressing(args))
+					break;
+			}
 		}
 
-		/*
-		 *  occasional sync and fdinfo reads
-		 */
 		if (i++ > 1024) {
 			i = 0;
-#if defined(HAVE_IORING_OP_FSYNC)
-			rc = stress_io_uring_submit(args, stress_io_uring_fsync_setup, &io_uring_file, &submit);
-			if ((rc != EXIT_SUCCESS) || !keep_stressing(args))
-				break;
-#endif
 			(void)stress_read_fdinfo(self, submit.io_uring_fd);
 		}
 	} while (keep_stressing(args));
@@ -689,6 +788,7 @@ clean:
 stressor_info_t stress_io_uring_info = {
 	.stressor = stress_io_uring,
 	.class = CLASS_IO | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
