@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyrignt (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-put.h"
+
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#endif
+#if defined(HAVE_SYS_SENDFILE_H)
+#include <sys/sendfile.h>
+#endif
+
+#define MIN_IOMIX_BYTES		(1 * MB)
+#define MAX_IOMIX_BYTES		(MAX_FILE_LIMIT)
+#define DEFAULT_IOMIX_BYTES	(1 * GB)
 
 typedef void (*stress_iomix_func)(const stress_args_t *args, const int fd, const off_t iomix_bytes);
 
@@ -52,6 +59,75 @@ static off_t stress_iomix_rnd_offset(const off_t max)
 	return (off_t)(stress_mwc64() % max);
 }
 
+/*
+ *  stress_iomix_fadvise_random_dontneed()
+ *	hint that the data at offset is not needed
+ *	and the I/O is random for more stress
+ */
+static void stress_iomix_fadvise_random_dontneed(
+	const int fd,
+	const off_t offset,
+	const off_t len)
+{
+#if defined(HAVE_POSIX_FADVISE) &&	\
+    (defined(POSIX_FADV_RANDOM) ||	\
+     defined(POSIX_FADV_DONTNEED))
+	int flag = 0;
+#if defined(POSIX_FADV_RANDOM)
+	flag |= POSIX_FADV_RANDOM;
+#endif
+#if defined(POSIX_FADV_DONTNEED)
+	flag |= POSIX_FADV_DONTNEED;
+#endif
+	(void)posix_fadvise(fd, offset, len, flag);
+#else
+	(void)fd;
+	(void)offset;
+	(void)len;
+#endif
+}
+
+/*
+ *  stress_iomix_fsync_min_1Hz()
+ *	sync written data at most every once a second while
+ *	trying to minimize the number time get calls
+ */
+static void stress_iomix_fsync_min_1Hz(const int fd)
+{
+	static double time_last = -1.0;
+	static int counter = 0;
+	static int counter_max = 1;
+
+	if (time_last <= 0.0)
+		time_last = stress_time_now() + 1.0;
+
+	if (counter++ >= counter_max) {
+		const double now = stress_time_now();
+		const double delta = now - time_last;
+
+		/* Less than 1Hz? try again */
+		if (delta < 1.0)
+			return;
+
+		counter_max = (int)((double)counter / delta);
+
+		counter = 0;
+		time_last = now;
+
+		switch (stress_mwc8() % 3) {
+		case 0:
+			(void)shim_fsync(fd);
+			break;
+		case 1:
+			(void)shim_fdatasync(fd);
+			break;
+		case 2:
+			(void)sync();
+			break;
+		}
+	}
+}
+
 /*
  *  stress_iomix_wr_seq_bursts()
  *	bursty sequential writes
@@ -69,7 +145,7 @@ static void stress_iomix_wr_seq_bursts(
 
 		posn = stress_iomix_rnd_offset(iomix_bytes);
 		ret = lseek(fd, posn, SEEK_SET);
-		if (ret < 0) {
+		if (ret == (off_t)-1) {
 			pr_fail("%s: lseek failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return;
@@ -78,6 +154,8 @@ static void stress_iomix_wr_seq_bursts(
     defined(POSIX_FADV_SEQUENTIAL)
 		if (posn < iomix_bytes)
 			(void)posix_fadvise(fd, posn, iomix_bytes - posn, POSIX_FADV_SEQUENTIAL);
+#else
+		UNEXPECTED
 #endif
 		for (i = 0; (i < n) && (posn < iomix_bytes); i++) {
 			char buffer[512];
@@ -98,6 +176,7 @@ static void stress_iomix_wr_seq_bursts(
 			if (!keep_stressing(args))
 				return;
 			inc_counter(args);
+			stress_iomix_fsync_min_1Hz(fd);
 		}
 		tv.tv_sec = 0;
 		tv.tv_usec = stress_mwc32() % 1000000;
@@ -117,6 +196,8 @@ static void stress_iomix_wr_rnd_bursts(
 #if defined(HAVE_POSIX_FADVISE) &&      \
     defined(POSIX_FADV_RANDOM)
 	(void)posix_fadvise(fd, 0, iomix_bytes, POSIX_FADV_RANDOM);
+#else
+	UNEXPECTED
 #endif
 	do {
 		const int n = stress_mwc8();
@@ -131,7 +212,7 @@ static void stress_iomix_wr_rnd_bursts(
 
 			posn = stress_iomix_rnd_offset(iomix_bytes);
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret < 0) {
+			if (ret == (off_t)-1) {
 				pr_fail("%s: lseek failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				return;
@@ -149,6 +230,7 @@ static void stress_iomix_wr_rnd_bursts(
 			if (!keep_stressing(args))
 				return;
 			inc_counter(args);
+			stress_iomix_fsync_min_1Hz(fd);
 		}
 		tv.tv_sec = stress_mwc32() % 2;
 		tv.tv_usec = stress_mwc32() % 1000000;
@@ -170,7 +252,7 @@ static void stress_iomix_wr_seq_slow(
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret < 0) {
+		if (ret == (off_t)-1) {
 			pr_fail("%s: lseek failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return;
@@ -179,6 +261,8 @@ static void stress_iomix_wr_seq_slow(
     defined(POSIX_FADV_SEQUENTIAL)
 		if (posn < iomix_bytes)
 			(void)posix_fadvise(fd, posn, iomix_bytes - posn, POSIX_FADV_SEQUENTIAL);
+#else
+		UNEXPECTED
 #endif
 		while (posn < iomix_bytes) {
 			char buffer[512];
@@ -200,6 +284,7 @@ static void stress_iomix_wr_seq_slow(
 			if (!keep_stressing(args))
 				return;
 			inc_counter(args);
+			stress_iomix_fsync_min_1Hz(fd);
 		}
 	} while (keep_stressing(args));
 }
@@ -221,7 +306,7 @@ static void stress_iomix_rd_seq_bursts(
 
 		posn = stress_iomix_rnd_offset(iomix_bytes);
 		ret = lseek(fd, posn, SEEK_SET);
-		if (ret < 0) {
+		if (ret == (off_t)-1) {
 			pr_fail("%s: lseek failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return;
@@ -231,6 +316,8 @@ static void stress_iomix_rd_seq_bursts(
     defined(POSIX_FADV_SEQUENTIAL)
 		if (posn < iomix_bytes)
 			(void)posix_fadvise(fd, posn, iomix_bytes - posn, POSIX_FADV_SEQUENTIAL);
+#else
+		UNEXPECTED
 #endif
 		for (i = 0; (i < n) && (posn < iomix_bytes); i++) {
 			char buffer[512];
@@ -247,6 +334,9 @@ static void stress_iomix_rd_seq_bursts(
 			if (!keep_stressing(args))
 				return;
 			inc_counter(args);
+
+			/* Add some unhelpful advice */
+			stress_iomix_fadvise_random_dontneed(fd, posn, 4096);
 		}
 		tv.tv_sec = 0;
 		tv.tv_usec = stress_mwc32() % 1000000;
@@ -275,12 +365,12 @@ static void stress_iomix_rd_rnd_bursts(
 			off_t ret, posn;
 
 			posn = stress_iomix_rnd_offset(iomix_bytes);
-#if defined(HAVE_POSIX_FADVISE) &&      \
-    defined(POSIX_FADV_RANDOM)
-			(void)posix_fadvise(fd, posn, (off_t)len, POSIX_FADV_RANDOM);
-#endif
+
+			/* Add some unhelpful advice */
+			stress_iomix_fadvise_random_dontneed(fd, posn, len);
+
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret < 0) {
+			if (ret == (off_t)-1) {
 				pr_fail("%s: lseek failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				return;
@@ -315,7 +405,7 @@ static void stress_iomix_rd_seq_slow(
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret < 0) {
+		if (ret == (off_t)-1) {
 			pr_fail("%s: lseek failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return;
@@ -324,12 +414,17 @@ static void stress_iomix_rd_seq_slow(
     defined(POSIX_FADV_SEQUENTIAL)
 		if (posn < iomix_bytes)
 			(void)posix_fadvise(fd, posn, iomix_bytes - posn, POSIX_FADV_SEQUENTIAL);
+#else
+		UNEXPECTED
 #endif
 		while (posn < iomix_bytes) {
 			char buffer[512];
 			ssize_t rc;
 			const size_t len = 1 + (stress_mwc32() & (sizeof(buffer) - 1));
 
+			/* Add some unhelpful advice */
+			stress_iomix_fadvise_random_dontneed(fd, posn, len);
+
 			rc = read(fd, buffer, len);
 			if (rc < 0) {
 				pr_fail("%s: read failed, errno=%d (%s)\n",
@@ -341,6 +436,7 @@ static void stress_iomix_rd_seq_slow(
 			if (!keep_stressing(args))
 				return;
 			inc_counter(args);
+			stress_iomix_fsync_min_1Hz(fd);
 		}
 	} while (keep_stressing(args));
 }
@@ -378,18 +474,27 @@ static void stress_iomix_sync(
 		(void)select(0, NULL, NULL, NULL, &tv);
 		if (!keep_stressing(args))
 			break;
+#else
+		UNEXPECTED
 #endif
 #if defined(HAVE_SYNC_FILE_RANGE) &&	\
     defined(SYNC_FILE_RANGE_WRITE)
-		(void)sync_file_range(fd, stress_iomix_rnd_offset(iomix_bytes),
-				65536, SYNC_FILE_RANGE_WRITE);
-		if (!keep_stressing(args))
-			break;
-		tv.tv_sec = stress_mwc32() % 4;
-		tv.tv_usec = stress_mwc32() % 1000000;
-		(void)select(0, NULL, NULL, NULL, &tv);
+		{
+			const off_t posn = stress_iomix_rnd_offset(iomix_bytes);
+
+			(void)sync_file_range(fd, posn, 65536,
+				SYNC_FILE_RANGE_WRITE);
+			stress_iomix_fadvise_random_dontneed(fd, posn, 65536);
+
+			if (!keep_stressing(args))
+				break;
+			tv.tv_sec = stress_mwc32() % 4;
+			tv.tv_usec = stress_mwc32() % 1000000;
+			(void)select(0, NULL, NULL, NULL, &tv);
+		}
 #else
 		(void)iomix_bytes;
+		UNEXPECTED
 #endif
 	} while (keep_stressing(args));
 }
@@ -478,7 +583,7 @@ static void stress_iomix_wr_bytes(
 		off_t ret, posn = 0;
 
 		ret = lseek(fd, 0, SEEK_SET);
-		if (ret < 0) {
+		if (ret == (off_t)-1) {
 			pr_fail("%s: lseek failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return;
@@ -499,6 +604,46 @@ static void stress_iomix_wr_bytes(
 			if (!keep_stressing(args))
 				return;
 			inc_counter(args);
+			stress_iomix_fsync_min_1Hz(fd);
+		}
+	} while (keep_stressing(args));
+}
+
+/*
+ *  stress_iomix_wr_rev_bytes()
+ *	lots of small 1 byte writes in reverse order
+ */
+static void stress_iomix_wr_rev_bytes(
+	const stress_args_t *args,
+	const int fd,
+	const off_t iomix_bytes)
+{
+	do {
+		off_t ret, posn = iomix_bytes;
+
+		ret = lseek(fd, 0, SEEK_SET);
+		if (ret == (off_t)-1) {
+			pr_fail("%s: lseek failed, errno=%d (%s)\n",
+				args->name, errno, strerror(errno));
+			return;
+		}
+		while (posn != 0) {
+			char buffer[1] = { (stress_mwc8() % 26) + 'A' };
+			ssize_t rc;
+
+			rc = write(fd, buffer, sizeof(buffer));
+			if (rc < 0) {
+				if (errno != EPERM) {
+					pr_fail("%s: write failed, errno=%d (%s)\n",
+						args->name, errno, strerror(errno));
+					return;
+			}	}
+			(void)shim_usleep(1000);
+			posn--;
+			if (!keep_stressing(args))
+				return;
+			inc_counter(args);
+			stress_iomix_fsync_min_1Hz(fd);
 		}
 	} while (keep_stressing(args));
 }
@@ -519,8 +664,11 @@ static void stress_iomix_rd_bytes(
 			char buffer[1];
 			ssize_t rc;
 
+			/* Add some unhelpful advice */
+			stress_iomix_fadvise_random_dontneed(fd, posn, sizeof(buffer));
+
 			ret = lseek(fd, posn, SEEK_SET);
-			if (ret < 0) {
+			if (ret == (off_t)-1) {
 				pr_fail("%s: lseek failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				return;
@@ -585,7 +733,11 @@ static void stress_iomix_inode_ioctl(
 	ret = ioctl(fd, FS_IOC_SETFLAGS, &attr);
 	if (ret < 0)
 		return;
+#else
+	UNEXPECTED
 #endif
+#else
+	UNEXPECTED
 #endif
 	*ok = true;
 }
@@ -608,39 +760,62 @@ static void stress_iomix_inode_flags(
 		bool ok = false;
 #if defined(FS_APPEND_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_APPEND_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_COMPR_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_COMPR_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_IMMUTABLE_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_IMMUTABLE_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_JOURNAL_DATA_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_JOURNAL_DATA_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_NOATIME_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_NOATIME_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_NOCOW_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_NOCOW_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_NODUMP_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_NODUMP_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_NOTAIL_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_NOTAIL_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_SECRM_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_SECRM_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_SYNC_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_SYNC_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 #if defined(FS_UNRM_FL)
 		stress_iomix_inode_ioctl(args, fd, FS_UNRM_FL, &ok);
+#else
+		UNEXPECTED
 #endif
 		if (!ok)
 			_exit(EXIT_SUCCESS);
+		stress_iomix_fsync_min_1Hz(fd);
 	} while (keep_stressing(args));
 }
 #endif
@@ -679,6 +854,76 @@ static void stress_iomix_drop_caches(
 }
 #endif
 
+#if defined(HAVE_COPY_FILE_RANGE)
+/*
+ *  stress_iomix_copy_file_range()
+ *	lots of copies with copy_file_range
+ */
+static void stress_iomix_copy_file_range(
+	const stress_args_t *args,
+	const int fd,
+	const off_t iomix_bytes)
+{
+	do {
+		off_t from = stress_mwc64() % iomix_bytes;
+		off_t to = stress_mwc64() % iomix_bytes;
+		const size_t size = stress_mwc16();
+		struct timeval tv;
+		int ret;
+
+		ret = copy_file_range(fd, &from, fd, &to, size, 0);
+		(void)ret;
+		ret = copy_file_range(fd, &to, fd, &from, size, 0);
+		(void)ret;
+
+		if (!keep_stressing(args))
+			return;
+		stress_iomix_fsync_min_1Hz(fd);
+
+		tv.tv_sec = 0;
+		tv.tv_usec = stress_mwc32() % 100000;
+		(void)select(0, NULL, NULL, NULL, &tv);
+	} while (keep_stressing(args));
+}
+#endif
+
+#if defined(HAVE_SYS_SENDFILE_H) &&	\
+    defined(HAVE_SENDFILE)
+/*
+ *  stress_iomix_sendfile()
+ *	lots of copies with copy_file_range
+ */
+static void stress_iomix_sendfile(
+	const stress_args_t *args,
+	const int fd,
+	const off_t iomix_bytes)
+{
+	do {
+		off_t from = stress_mwc64() % iomix_bytes;
+		off_t to = stress_mwc64() % iomix_bytes;
+		off_t ret;
+		const size_t size = stress_mwc16();
+		struct timeval tv;
+
+		ret = lseek(fd, to, SEEK_SET);
+		if (ret != (off_t)-1) {
+			int sret;
+
+			sret = sendfile(fd, fd, &from, size);
+			(void)sret;
+		}
+
+		if (!keep_stressing(args))
+			return;
+		stress_iomix_fsync_min_1Hz(fd);
+
+		tv.tv_sec = 0;
+		tv.tv_usec = stress_mwc32() % 130000;
+		(void)select(0, NULL, NULL, NULL, &tv);
+	} while (keep_stressing(args));
+}
+#endif
+
 static stress_iomix_func iomix_funcs[] = {
 	stress_iomix_wr_seq_bursts,
 	stress_iomix_wr_rnd_bursts,
@@ -694,12 +939,20 @@ static stress_iomix_func iomix_funcs[] = {
 #endif
 	stress_iomix_rd_wr_mmap,
 	stress_iomix_wr_bytes,
+	stress_iomix_wr_rev_bytes,
 	stress_iomix_rd_bytes,
 #if defined(__linux__)
 	stress_iomix_inode_flags,
 #endif
 #if defined(__linux__)
-	stress_iomix_drop_caches
+	stress_iomix_drop_caches,
+#endif
+#if defined(HAVE_COPY_FILE_RANGE)
+	stress_iomix_copy_file_range,
+#endif
+#if defined(HAVE_SYS_SENDFILE_H) &&	\
+    defined(HAVE_SENDFILE)
+	stress_iomix_sendfile,
 #endif
 };
 
@@ -753,7 +1006,7 @@ static int stress_iomix(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto unmap;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 #if defined(FALLOC_FL_ZERO_RANGE)
 	ret = shim_fallocate(fd, FALLOC_FL_ZERO_RANGE, 0, iomix_bytes);
@@ -843,5 +1096,6 @@ stressor_info_t stress_iomix_info = {
 	.stressor = stress_iomix,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
