@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+
+#if defined(HAVE_SYS_IO_H)
+#include <sys/io.h>
+#endif
 
 #define IO_PORT		0x80
 
@@ -85,10 +85,10 @@ static int stress_ioport_supported(const char *name)
 	if (ret < 0) {
 		switch (errno) {
 		case ENOMEM:
-			pr_inf("%s: ioperm out of memory, skipping stressor\n", name);
+			pr_inf_skip("%s: ioperm out of memory, skipping stressor\n", name);
 			return -1;
 		case EPERM:
-			pr_inf("%s has insufficient privilege, invoke with CAP_SYS_RAWIO privilege, skipping stressor\n", name);
+			pr_inf_skip("%s has insufficient privilege, invoke with CAP_SYS_RAWIO privilege, skipping stressor\n", name);
 			return -1;
 		case EINVAL:
 		case EIO:
@@ -221,6 +221,8 @@ static int stress_ioport(const stress_args_t *args)
 				(void)ret;
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 		inc_counter(args);
 	} while (keep_stressing(args));
