@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-io-priority.h"
+
+#if defined(HAVE_SYS_UIO_H)
+#include <sys/uio.h>
+#endif
+
+#if defined(__NR_ioprio_get)
+#define HAVE_IOPRIO_GET
+#endif
+
+#if defined(__NR_ioprio_set)
+#define HAVE_IOPRIO_SET
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"ioprio N",	"start N workers exercising set/get iopriority" },
@@ -62,7 +70,7 @@ static int stress_ioprio(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto cleanup_dir;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -102,6 +110,8 @@ static int stress_ioprio(const stress_args_t *args)
 		}
 		if (!keep_stressing(args))
 			break;
+#else
+		UNEXPECTED
 #endif
 		if (shim_ioprio_get(IOPRIO_WHO_PGRP, 0) < 0) {
 			if (errno != EINVAL) {
@@ -121,7 +131,7 @@ static int stress_ioprio(const stress_args_t *args)
 		(void)shim_ioprio_get(IOPRIO_WHO_PGRP, ~0);
 		(void)shim_ioprio_get(IOPRIO_WHO_USER, ~0);
 
-		if (shim_ioprio_get(IOPRIO_WHO_USER, uid) < 0) {
+		if (shim_ioprio_get(IOPRIO_WHO_USER, (int)uid) < 0) {
 			if (errno != EINVAL) {
 				pr_fail("%s: ioprio_get(OPRIO_WHO_USR, %d), "
 					"errno = %d (%s)\n",
@@ -251,6 +261,7 @@ cleanup_dir:
 stressor_info_t stress_ioprio_info = {
 	.stressor = stress_ioprio,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
