@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -53,21 +48,35 @@ static int stress_io(const stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		(void)sync();
+		sync();
 #if defined(HAVE_SYNCFS)
 		if ((fd != -1) && (syncfs(fd) < 0))
 			pr_fail("%s: syncfs failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 
 		/* try to sync on all the mount points */
-		for (i = 0; i < n_mnts; i++)
-			if (fds[i] != -1)
-				(void)syncfs(fds[i]);
+		for (i = 0; i < n_mnts; i++) {
+			if (fds[i] < 0)
+				continue;
+			if (syncfs(fds[i]) < 0) {
+				if ((errno != ENOSPC) &&
+				    (errno != EDQUOT) &&
+				    (errno != EINTR)) {
+					pr_fail("%s syncfs failed, errno=%d (%s)\n",
+						args->name, errno, strerror(errno));
+				}
+			}
+		}
 
 		/*
 		 *  exercise with an invalid fd
 		 */
-		(void)syncfs(bad_fd);
+		if (syncfs(bad_fd) == 0) {
+			pr_fail("%s: syncfs on invalid fd %d succeed\n",
+				args->name, bad_fd);
+		}
+#else
+		UNEXPECTED
 #endif
 		inc_counter(args);
 	} while (keep_stressing(args));
@@ -82,6 +91,8 @@ static int stress_io(const stress_args_t *args)
 			(void)close(fds[i]);
 
 	stress_mount_free(mnts, n_mnts);
+#else
+	UNEXPECTED
 #endif
 
 	return EXIT_SUCCESS;
@@ -90,5 +101,6 @@ static int stress_io(const stress_args_t *args)
 stressor_info_t stress_io_info = {
 	.stressor = stress_io,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
