@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_JUDY_H)
+#include <Judy.h>
+#endif
+
+#define MIN_JUDY_SIZE		(1 * KB)
+#define MAX_JUDY_SIZE		(4 * MB)
+#define DEFAULT_JUDY_SIZE	(256 * KB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"judy N",	"start N workers that exercise a judy array search" },
 	{ NULL,	"judy-ops N",	"stop after N judy array search bogo operations" },
@@ -56,9 +59,9 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *  generate a unique large index position into a Judy array
  *  from a known small index
  */
-static inline Word_t gen_index(const Word_t index)
+static inline Word_t gen_index(const Word_t idx)
 {
-	return ((~index & 0xff) << 24) | (index & 0x00ffffff);
+	return ((~idx & 0xff) << 24) | (idx & 0x00ffffff);
 }
 
 /*
@@ -145,6 +148,7 @@ stressor_info_t stress_judy_info = {
 	.stressor = stress_judy,
 	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
