@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+#include "core-net.h"
+
+#if defined(HAVE_SYS_EPOLL_H)
+#include <sys/epoll.h>
+#endif
+
+#if defined(__NR_kcmp)
+#define HAVE_KCMP
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"kcmp N",	"start N workers exercising kcmp" },
@@ -331,6 +336,7 @@ finish:
 stressor_info_t stress_kcmp_info = {
 	.stressor = stress_kcmp,
 	.class = CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
