@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,27 +16,48 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_KEYUTILS_H)
+#include <keyutils.h>
+#endif
+
+#if defined(HAVE_SYS_CAPABILITY_H)
+#include <sys/capability.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"key N",	"start N workers exercising key operations" },
 	{ NULL,	"key-ops N",	"stop after N key bogo operations" },
 	{ NULL,	NULL,		NULL }
 };
 
+#if defined(__NR_add_key)
+#define HAVE_ADD_KEY
+#endif
+
+#if defined(__NR_keyctl)
+#define HAVE_KEYCTL
+#endif
+
+#if defined(__NR_request_key)
+#define HAVE_REQUEST_KEY
+#endif
+
+
+/*
+ *  Note we don't need HAVE_REQUEST_KEY to be defined
+ *  as this is an optional extra system call that can
+ *  be overlooked in this stressor if it is not available
+ */
 #if defined(HAVE_KEYUTILS_H) && \
-    defined(HAVE_ADD_KEY) && \
+    defined(HAVE_ADD_KEY) &&	\
     defined(HAVE_KEYCTL)
 
-#define MAX_KEYS 	(256)
-#define KEYCTL_TIMEOUT	(2)
+#define MAX_KEYS 		(256)
+#define KEYCTL_TIMEOUT		(2)
+#define KEY_HUGE_DESC_SIZE	(65536)
 
 /*
  *  shim_keyctl()
@@ -99,6 +121,16 @@ static int stress_key(const stress_args_t *args)
 	int rc = EXIT_SUCCESS;
 	bool timeout_supported = true;
 	bool no_error = true;
+	char *huge_description;
+	const size_t key_huge_desc_size = STRESS_MAXIMUM(args->page_size, KEY_HUGE_DESC_SIZE) + 1024;
+
+	huge_description = malloc(key_huge_desc_size);
+	if (!huge_description) {
+		pr_inf("%s: cannot allocate %zd byte description string, skipping stressor\n",
+			args->name, key_huge_desc_size);
+		return EXIT_NO_RESOURCE;
+	}
+	stress_strnrnd(huge_description, key_huge_desc_size);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -109,14 +141,58 @@ static int stress_key(const stress_args_t *args)
 
 		/* Add as many keys as we are allowed */
 		for (n = 0; n < MAX_KEYS; n++) {
+			size_t payload_len;
+
+			(void)snprintf(payload, sizeof(payload),
+				"somedata-%zu", n);
+			payload_len = strlen(payload);
 			(void)snprintf(description, sizeof(description),
 				"stress-ng-key-%" PRIdMAX "-%" PRIu32
 				"-%zu", (intmax_t)ppid, args->instance, n);
-			(void)snprintf(payload, sizeof(payload),
-				"somedata-%zu", n);
+
+
+#if defined(KEYCTL_INVALIDATE)
+			/* Exericse add_key with invalid long description */
+			keys[n] = shim_add_key("user", huge_description, payload,
+					payload_len, KEY_SPEC_PROCESS_KEYRING);
+			if (keys[n] >= 0)
+				(void)shim_keyctl(KEYCTL_INVALIDATE, keys[n]);
+#endif
+
+#if defined(KEYCTL_INVALIDATE)
+			/* Exercise add_key with invalid empty description */
+			keys[n] = shim_add_key("user", "", payload,
+					payload_len, KEY_SPEC_PROCESS_KEYRING);
+			if (keys[n] >= 0)
+				(void)shim_keyctl(KEYCTL_INVALIDATE, keys[n]);
+#endif
+
+#if defined(KEYCTL_INVALIDATE)
+			/* Exercise add_key with invalid description for keyring */
+			keys[n] = shim_add_key("keyring", ".bad", payload,
+					payload_len, KEY_SPEC_PROCESS_KEYRING);
+			if (keys[n] >= 0)
+				(void)shim_keyctl(KEYCTL_INVALIDATE, keys[n]);
+#endif
+
+#if defined(KEYCTL_INVALIDATE)
+			/* Exercise add_key with invalid payload */
+			keys[n] = shim_add_key("user", description, "",
+					0, KEY_SPEC_PROCESS_KEYRING);
+			if (keys[n] >= 0)
+				(void)shim_keyctl(KEYCTL_INVALIDATE, keys[n]);
+#endif
+
+#if defined(KEYCTL_INVALIDATE)
+			/* Exercise add_key with invalid payload length */
+			keys[n] = shim_add_key("user", description, payload,
+					SIZE_MAX, KEY_SPEC_PROCESS_KEYRING);
+			if (keys[n] >= 0)
+				(void)shim_keyctl(KEYCTL_INVALIDATE, keys[n]);
+#endif
 
 			keys[n] = shim_add_key("user", description,
-				payload, strlen(payload),
+				payload, payload_len,
 				KEY_SPEC_PROCESS_KEYRING);
 			if (keys[n] < 0) {
 				if (errno == ENOSYS) {
@@ -234,6 +310,22 @@ static int stress_key(const stress_args_t *args)
 						args->name, errno, strerror(errno));
 				}
 			}
+
+			/* exercise invalid type */
+			(void)shim_request_key("_INVALID_TYPE_", description, NULL,
+				KEY_SPEC_PROCESS_KEYRING);
+
+			/* exercise invalid description */
+			(void)shim_request_key("user", huge_description, NULL,
+				KEY_SPEC_PROCESS_KEYRING);
+
+			/* exercise invalid callout info */
+			(void)shim_request_key("user", description, huge_description,
+				KEY_SPEC_PROCESS_KEYRING);
+
+			/* exercise invalid dest keyring id */
+			(void)shim_request_key("user", description, NULL, INT_MIN);
+
 			if (!keep_stressing_flag())
 				goto tidy;
 #endif
@@ -308,12 +400,15 @@ tidy:
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	free(huge_description);
+
 	return rc;
 }
 
 stressor_info_t stress_key_info = {
 	.stressor = stress_key,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
