@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2021 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -30,6 +25,15 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
+/*
+ *  stress_kill_handle_sigusr1()
+ *	handle SIGUSR1
+ */
+static void stress_kill_handle_sigusr1(int sig)
+{
+	(void)sig;
+}
+
 /*
  *  stress on sched_kill()
  *	stress system by rapid kills
@@ -37,14 +41,30 @@ static const stress_help_t help[] = {
 static int stress_kill(const stress_args_t *args)
 {
 	uint64_t udelay = 5000;
+	pid_t pid;
+	const pid_t ppid = getpid();
+	int ret;
 
 	if (stress_sighandler(args->name, SIGUSR1, SIG_IGN, NULL) < 0)
 		return EXIT_FAILURE;
 
+	pid = fork();
+	if (pid == 0) {
+		ret = stress_sighandler(args->name, SIGUSR1, stress_kill_handle_sigusr1, NULL);
+		(void)ret;
+
+		while (keep_stressing(args)) {
+			if (kill(ppid, 0) < 0)
+				break;
+			pause();
+		}
+		_exit(0);
+	}
+
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		int ret;
+		pid_t bad_pid;
 
 		/*
 		 *  With many kill stressors we get into a state
@@ -104,16 +124,47 @@ static int stress_kill(const stress_args_t *args)
 		ret = kill(INT_MIN, 0);
 		(void)ret;
 
+		/*
+		 * Send child process some signals to keep it busy
+		 */
+		if (pid > 1) {
+			ret = kill(pid, 0);
+			(void)ret;
+#if defined(SIGSTOP) && 	\
+    defined(SIGCONT)
+			ret = kill(pid, SIGSTOP);
+			(void)ret;
+			ret = kill(pid, SIGCONT);
+			(void)ret;
+#endif
+			ret = kill(pid, SIGUSR1);
+			(void)ret;
+		}
+
+		bad_pid = stress_get_unused_pid_racy(false);
+		ret = kill(bad_pid, 0);
+		(void)ret;
+
 		inc_counter(args);
 	} while (keep_stressing(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	if (pid != -1) {
+		int status;
+
+		ret = kill(pid, SIGKILL);
+		(void)ret;
+		ret = waitpid(pid, &status, 0);
+		(void)ret;
+	}
+
 	return EXIT_SUCCESS;
 }
 
 stressor_info_t stress_kill_info = {
 	.stressor = stress_kill,
 	.class = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
