@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(__NR_syslog)
+#define HAVE_SYSLOG
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"klog N",	"start N workers exercising kernel syslog interface" },
@@ -79,7 +79,7 @@ static int stress_klog(const stress_args_t *args)
 						"not permitted, skipping stressor\n",
 						args->name);
 			} else {
-				pr_err("%s: cannot determine syslog buffer "
+				pr_fail("%s: cannot determine syslog buffer "
 					"size: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
@@ -88,7 +88,7 @@ static int stress_klog(const stress_args_t *args)
 	}
 	if (len == 0) {
 		if (!args->instance)
-			pr_err("%s: zero sized syslog buffer, aborting.\n", args->name);
+			pr_inf("%s: zero sized syslog buffer, skipping stressor.\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
 	if (len > (ssize_t)(4 * MB)) {
@@ -107,6 +107,18 @@ static int stress_klog(const stress_args_t *args)
 	do {
 		int ret, buflen = (int)(stress_mwc32() % len) + 1;
 
+		/* Exercise illegal read size */
+		(void)shim_klogctl(SYSLOG_ACTION_READ, buffer, -1);
+		(void)shim_klogctl(SYSLOG_ACTION_READ_ALL, buffer, -1);
+
+		/* Exercise illegal buffer */
+		(void)shim_klogctl(SYSLOG_ACTION_READ, NULL, 0);
+		(void)shim_klogctl(SYSLOG_ACTION_READ_ALL, NULL, 0);
+
+		/* Exercise zero size read */
+		(void)shim_klogctl(SYSLOG_ACTION_READ, buffer, 0);
+		(void)shim_klogctl(SYSLOG_ACTION_READ_ALL, buffer, 0);
+
 		ret = shim_klogctl(SYSLOG_ACTION_READ_ALL, buffer, buflen);
 		if (ret < 0)
 			pr_fail("%s: syslog ACTION_READ_ALL failed, errno=%d (%s)\n",
@@ -168,6 +180,7 @@ stressor_info_t stress_klog_info = {
 	.stressor = stress_klog,
 	.class = CLASS_OS,
 	.help = help,
+	.verify = VERIFY_ALWAYS,
 	.supported = stress_klog_supported
 };
 #else
