@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -82,6 +77,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 static uint32_t stress_l1cache_ln2(uint32_t val)
 {
 	int ln2 = 0;
+
 	while (val >>= 1)
 		++ln2;
 
@@ -282,23 +278,48 @@ static int stress_l1cache(const stress_args_t *args)
 		cache_start = cache_aligned + set_offset;
 		cache_end = cache_start + (l1cache_size << 1);
 
-		for (i = 0; i < 1000000; i++) {
-			register volatile uint8_t *ptr;
+		/*
+		 * cycle around 2 x cache size to force evictions
+		 * in cache set steps to hit every cache line in
+		 * the set, reads then writes
+		 */
+		if (UNLIKELY(g_opt_flags & OPT_FLAGS_VERIFY)) {
+			/* Verify slow path */
+			for (i = 0; i < 1000000; i++) {
+				register volatile uint8_t *ptr;
 
-			/*
-			 * cycle around 2 x cache size to force evictions
-			 * in cache set steps to hit every cache line in
-			 * the set, reads then writes
-			 */
-			for (ptr = cache_start; ptr < cache_end; ptr += l1cache_set_size)
-				*(ptr);
+				for (ptr = cache_start; ptr < cache_end; ptr += l1cache_set_size)
+					*(ptr);
 
-			for (ptr = cache_start; ptr < cache_end; ptr += l1cache_set_size)
-				*(ptr) = (uint8_t)set;
+				for (ptr = cache_start; ptr < cache_end; ptr += l1cache_set_size)
+					*(ptr) = (uint8_t)set;
 
-			set++;
-			if (set >= l1cache_sets)
-				set = 0;
+				if (UNLIKELY(g_opt_flags & OPT_FLAGS_VERIFY)) {
+					for (ptr = cache_start; ptr < cache_end; ptr += l1cache_set_size)
+						if (*ptr != (uint8_t)set)
+							pr_fail("%s: cache value mismatch at offset %zd\n",
+								args->name, (size_t)(ptr - cache_start));
+				}
+
+				set++;
+				if (set >= l1cache_sets)
+					set = 0;
+			}
+		} else {
+			/* non-verify fast path */
+			for (i = 0; i < 1000000; i++) {
+				register volatile uint8_t *ptr;
+
+				for (ptr = cache_start; ptr < cache_end; ptr += l1cache_set_size)
+					*(ptr);
+
+				for (ptr = cache_start; ptr < cache_end; ptr += l1cache_set_size)
+					*(ptr) = (uint8_t)set;
+
+				set++;
+				if (set >= l1cache_sets)
+					set = 0;
+			}
 		}
 		add_counter(args, l1cache_sets);
 	} while (keep_stressing(args));
@@ -314,5 +335,6 @@ stressor_info_t stress_l1cache_info = {
 	.stressor = stress_l1cache,
 	.class = CLASS_CPU_CACHE,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
