@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_LANDLOCK_H)
+#include <linux/landlock.h>
+#endif
+
+#if defined(HAVE_SYS_PRCTL_H)
+#include <sys/prctl.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"landlock N",	  "start N workers stressing landlock file operations" },
 	{ NULL,	"landlock-ops N", "stop after N landlock bogo operations" },
@@ -213,7 +216,7 @@ static int stress_landlock_flag(const stress_args_t *args, void *ctxt)
 	 *  Got a valid landlocked restricted child process,
 	 *  so now sanity check it on some test files
 	 */
-	(void)snprintf(filename, sizeof(filename), "%s/landlock-%d", path, (int)pid);
+	(void)snprintf(filename, sizeof(filename), "%s/landlock-%" PRIdMAX, path, (intmax_t)pid);
 
 	fd = open(path, O_PATH | O_CLOEXEC);
 	if (fd > -1)
@@ -223,7 +226,7 @@ static int stress_landlock_flag(const stress_args_t *args, void *ctxt)
 	if (fd > -1) {
 		pr_fail("%s: failed to landlock writable file %s\n",
 			args->name, filename);
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		(void)close(fd);
 		rc = EXIT_FAILURE;
 		goto close_parent;
@@ -255,15 +258,11 @@ static void stress_landlock_test(
 	int status;
 	pid_t pid;
 
-retry:
-	if (!keep_stressing(args))
-		return;
-
+again:
 	pid = fork();
-
 	if (pid < 0) {
-		if ((errno == EAGAIN) || (errno == ENOMEM))
-			goto retry;
+		if (stress_redo_fork(errno))
+			goto again;
 		return;
 	} else if (pid == 0) {
 		_exit(func(args, ctxt));
@@ -349,6 +348,7 @@ stressor_info_t stress_landlock_info = {
 	.stressor = stress_landlock,
 	.class = CLASS_OS,
 	.supported = stress_landlock_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
