@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_LEASE_BREAKERS	(1)
+#define MAX_LEASE_BREAKERS	(64)
+#define DEFAULT_LEASE_BREAKERS	(1)
+
 #if defined(F_SETLEASE) && 	\
     defined(F_WRLCK) &&		\
     defined(F_UNLCK)
@@ -90,8 +89,7 @@ static pid_t stress_lease_spawn(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() &&
-		    ((errno == EAGAIN) || (errno == ENOMEM)))
+		if (stress_redo_fork(errno))
 			goto again;
 		return -1;
 	}
@@ -165,7 +163,7 @@ static int stress_try_lease(
 		if (!keep_stressing_flag())
 			break;
 		if (errno != EAGAIN) {
-			pr_err("%s: fcntl failed: errno=%d: (%s)\n",
+			pr_fail("%s: fcntl failed: errno=%d: (%s)\n",
 				args->name, errno, strerror(errno));
 			break;
 		}
@@ -250,7 +248,7 @@ reap:
 		}
 	}
 
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 
 	pr_dbg("%s: %" PRIu64 " lease sigio interrupts caught\n", args->name, lease_sigio);
@@ -267,6 +265,7 @@ stressor_info_t stress_lease_info = {
 	.stressor = stress_lease,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
