@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LIBGEN_H)
+#include <libgen.h>
+#endif
+
 static const stress_help_t hardlink_help[] = {
 	{ NULL,	"link N",	 "start N workers creating hard links" },
 	{ NULL,	"link-ops N",	 "stop after N link bogo operations" },
@@ -53,14 +52,14 @@ static void stress_link_unlink(
 
 		(void)stress_temp_filename_args(args,
 			path, sizeof(path), i);
-		(void)unlink(path);
+		(void)shim_force_unlink(path);
 	}
 	(void)sync();
 }
 
-static inline int random_mount(const int mounts_max)
+static inline size_t random_mount(const int mounts_max)
 {
-	return (int)stress_mwc32() % mounts_max;
+	return (size_t)stress_mwc32() % mounts_max;
 }
 
 /*
@@ -78,6 +77,8 @@ static int stress_link_generic(
 	bool symlink_func = (linkfunc == symlink);
 	char *mnts[MOUNTS_MAX];
 
+	memset(tmp_newpath, 0, sizeof(tmp_newpath));
+
 	(void)snprintf(tmp_newpath, sizeof(tmp_newpath),
 		"/tmp/stress-ng-%s-%d-%" PRIu64 "-link",
 		args->name, (int)getpid(), stress_mwc64());
@@ -123,7 +124,7 @@ static int stress_link_generic(
 					continue;
 				}
 				if (errno == EPERM) {
-					pr_inf("%s: link calls not allowed on "
+					pr_inf_skip("%s: link calls not allowed on "
 						"the filesystem, skipping "
 						"stressor\n", args->name);
 					rc = EXIT_NO_RESOURCE;
@@ -141,26 +142,26 @@ static int stress_link_generic(
 #if defined(O_DIRECTORY) &&	\
     defined(HAVE_READLINKAT)
 				{
-					char tmpfile[PATH_MAX], *filename;
+					char tmpfilename[PATH_MAX], *filename;
 					char tmpdir[PATH_MAX], *dir;
-					int dirfd;
+					int dir_fd;
 
-					shim_strlcpy(tmpfile, newpath, sizeof(tmpfile));
-					filename = basename(tmpfile);
+					shim_strlcpy(tmpfilename, newpath, sizeof(tmpfilename));
+					filename = basename(tmpfilename);
 					shim_strlcpy(tmpdir, newpath, sizeof(tmpdir));
 					dir = dirname(tmpdir);
 
 					/*
 					 *   Relatively naive readlinkat exercising
 					 */
-					dirfd = open(dir, O_DIRECTORY | O_RDONLY);
-					if (dirfd >= 0) {
-						rret = readlinkat(dirfd, filename, buf, sizeof(buf) - 1);
+					dir_fd = open(dir, O_DIRECTORY | O_RDONLY);
+					if (dir_fd >= 0) {
+						rret = readlinkat(dir_fd, filename, buf, sizeof(buf) - 1);
 						if ((rret < 0) && (errno != ENOSYS)) {
 							pr_fail("%s: readlinkat failed, errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
 						}
-						(void)close(dirfd);
+						(void)close(dir_fd);
 					}
 				}
 #endif
@@ -184,9 +185,9 @@ static int stress_link_generic(
 				/* Hard link, exercise illegal cross device link, EXDEV error */
 				if (mounts_max > 0) {
 					/* Try hard link on differet random mount point */
-					ret = linkfunc(mnts[random_mount(mounts_max)], tmp_newpath);
+					ret = linkfunc(mnts[random_mount((size_t)mounts_max)], tmp_newpath);
 					if (ret == 0)
-						(void)unlink(tmp_newpath);
+						(void)shim_unlink(tmp_newpath);
 				}
 			}
 			if (lstat(newpath, &stbuf) < 0) {
@@ -231,7 +232,7 @@ err_unlink:
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)unlink(oldpath);
+	(void)shim_unlink(oldpath);
 	(void)stress_temp_dir_rm_args(args);
 
 	stress_mount_free(mnts, mounts_max);
@@ -263,6 +264,7 @@ static int stress_symlink(const stress_args_t *args)
 stressor_info_t stress_link_info = {
 	.stressor = stress_link,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = hardlink_help
 };
 #else
@@ -276,5 +278,6 @@ stressor_info_t stress_link_info = {
 stressor_info_t stress_symlink_info = {
 	.stressor = stress_symlink,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = symlink_help
 };
