@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2021 Canonical, Ltd.
+ * Copyright (C)      2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,18 +16,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"loadavg N",	 "start N workers that create a large load average" },
-	{ NULL,	"loadavg-ops N", "stop load averge workers after N bogo operations" },
+	{ NULL,	"loadavg-ops N", "stop load average workers after N bogo operations" },
 	{ NULL,	NULL,		 NULL }
 };
 
@@ -120,11 +115,12 @@ static void *stress_loadavg_func(void *arg)
 
 	buf[0] = stress_mwc8();
 #endif
+	double t_end = stress_time_now() + (double)g_opt_timeout;
 
 	(void)arg;
 	(void)shim_nice(19);	/* be very nice */
 
-	while (keep_thread_running()) {
+	while ((stress_time_now() < t_end) && keep_thread_running()) {
 #if defined(LOADAVG_IO)
 		if (fd >= 0) {
 			ssize_t ret;
@@ -189,7 +185,7 @@ static int stress_loadavg(const stress_args_t *args)
 
 	/*  Not a failure if can't open a file */
 	fd = open(filename, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR);
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 #endif
 
 	/*
