@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -198,7 +193,7 @@ static int stress_locka_contention(
 
 		locka_info = stress_locka_info_new();
 		if (!locka_info) {
-			pr_fail("%s: calloc failed, out of memory\n", args->name);
+			pr_err("%s: calloc failed, out of memory\n", args->name);
 			return -1;
 		}
 		locka_info->offset = offset;
@@ -235,7 +230,7 @@ static int stress_locka(const stress_args_t *args)
 	if (mkdir(pathname, S_IRWXU) < 0) {
 		if (errno != EEXIST) {
 			ret = exit_status(errno);
-			pr_fail("%s: mkdir %s failed, errno=%d (%s)\n",
+			pr_err("%s: mkdir %s failed, errno=%d (%s)\n",
 				args->name, pathname, errno, strerror(errno));
 			return ret;
 		}
@@ -251,14 +246,14 @@ static int stress_locka(const stress_args_t *args)
 
 	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
 		ret = exit_status(errno);
-		pr_fail("%s: open %s failed, errno=%d (%s)\n",
+		pr_err("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
-		(void)rmdir(pathname);
+		(void)shim_rmdir(pathname);
 		return ret;
 	}
 
 	if (lseek(fd, 0, SEEK_SET) < 0) {
-		pr_fail("%s: lseek failed, errno=%d (%s)\n",
+		pr_err("%s: lseek failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
@@ -273,7 +268,7 @@ redo:
 			if ((errno == EAGAIN) || (errno == EINTR))
 				goto redo;
 			ret = exit_status(errno);
-			pr_fail("%s: write failed, errno=%d (%s)\n",
+			pr_err("%s: write failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto tidy;
 		}
@@ -287,7 +282,7 @@ again:
 			goto again;
 		if (!keep_stressing(args))
 			goto tidy;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
@@ -317,14 +312,15 @@ tidy:
 	stress_locka_info_free();
 
 	(void)close(fd);
-	(void)unlink(filename);
-	(void)rmdir(pathname);
+	(void)shim_unlink(filename);
+	(void)shim_rmdir(pathname);
 
 	return ret;
 }
 stressor_info_t stress_locka_info = {
 	.stressor = stress_locka,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
