@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -153,7 +148,7 @@ static int stress_lockf_unlock(const stress_args_t *args, const int fd)
 		return 0;
 
 	if (lseek(fd, lockf_infos.head->offset, SEEK_SET) < 0) {
-		pr_fail("%s: lseek failed, errno=%d (%s)\n",
+		pr_err("%s: lseek failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return -1;
 	}
@@ -195,7 +190,7 @@ static int stress_lockf_contention(
 
 		offset = stress_mwc64() % (LOCK_FILE_SIZE - LOCK_SIZE);
 		if (lseek(fd, offset, SEEK_SET) < 0) {
-			pr_fail("%s: lseek failed, errno=%d (%s)\n",
+			pr_err("%s: lseek failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return -1;
 		}
@@ -209,7 +204,7 @@ static int stress_lockf_contention(
 		/* Locked OK, add to lock list */
 		lockf_info = stress_lockf_info_new();
 		if (!lockf_info) {
-			pr_fail("%s: calloc failed, out of memory\n", args->name);
+			pr_err("%s: calloc failed, out of memory\n", args->name);
 			return -1;
 		}
 		lockf_info->offset = offset;
@@ -258,7 +253,7 @@ static int stress_lockf(const stress_args_t *args)
 	if (mkdir(pathname, S_IRWXU) < 0) {
 		if (errno != EEXIST) {
 			ret = exit_status(errno);
-			pr_fail("%s: mkdir %s failed, errno=%d (%s)\n",
+			pr_err("%s: mkdir %s failed, errno=%d (%s)\n",
 				args->name, pathname, errno, strerror(errno));
 			return ret;
 		}
@@ -274,14 +269,14 @@ static int stress_lockf(const stress_args_t *args)
 
 	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
 		ret = exit_status(errno);
-		pr_fail("%s: open %s failed, errno=%d (%s)\n",
+		pr_err("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
-		(void)rmdir(pathname);
+		(void)shim_rmdir(pathname);
 		return ret;
 	}
 
 	if (lseek(fd, 0, SEEK_SET) < 0) {
-		pr_fail("%s: lseek failed, errno=%d (%s)\n",
+		pr_err("%s: lseek failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
@@ -296,7 +291,7 @@ redo:
 			if ((errno == EAGAIN) || (errno == EINTR))
 				goto redo;
 			ret = exit_status(errno);
-			pr_fail("%s: write failed, errno=%d (%s)\n",
+			pr_err("%s: write failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto tidy;
 		}
@@ -306,13 +301,11 @@ redo:
 again:
 	cpid = fork();
 	if (cpid < 0) {
-		if (!keep_stressing_flag()) {
-			ret = EXIT_SUCCESS;
-			goto tidy;
-		}
-		if ((errno == EAGAIN) || (errno == ENOMEM))
+		if (stress_redo_fork(errno))
 			goto again;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		if (!keep_stressing(args))
+			goto tidy;
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto tidy;
 	}
@@ -342,8 +335,8 @@ tidy:
 	stress_lockf_info_free();
 
 	(void)close(fd);
-	(void)unlink(filename);
-	(void)rmdir(pathname);
+	(void)shim_unlink(filename);
+	(void)shim_rmdir(pathname);
 
 	return ret;
 }
@@ -352,6 +345,7 @@ stressor_info_t stress_lockf_info = {
 	.stressor = stress_lockf,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
