@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -32,6 +27,7 @@ typedef struct {
 } jmp_buf_check_t;
 
 static jmp_buf_check_t bufchk;
+static volatile bool longjmp_failed;
 
 static const stress_help_t help[] = {
 	{ NULL,	"longjmp N",	 "start N workers exercising setjmp/longjmp" },
@@ -44,6 +40,7 @@ static void OPTIMIZE1 NOINLINE NORETURN stress_longjmp_func(void)
 	bufchk.ts = stress_time_now();
 	longjmp(bufchk.buf, 1);	/* Jump out */
 
+	longjmp_failed = true;
 	_exit(EXIT_FAILURE);	/* Never get here */
 }
 
@@ -58,6 +55,9 @@ static int OPTIMIZE1 stress_longjmp(const stress_args_t *args)
 	static double t_total;
 	static uint64_t n = 0;
 
+	/* assume OK unless proven otherwise */
+	longjmp_failed = false;
+
 	check0 = stress_mwc32();
 	check1 = stress_mwc32();
 
@@ -78,11 +78,11 @@ static int OPTIMIZE1 stress_longjmp(const stress_args_t *args)
 		 *  before/after the jmpbuf
 		 */
 		if (bufchk.check0 != check0) {
-			pr_err("%s: memory corrupted before jmpbuf region\n",
+			pr_fail("%s: memory corrupted before jmpbuf region\n",
 				args->name);
 		}
 		if (bufchk.check1 != check1) {
-			pr_err("%s: memory corrupted before jmpbuf region\n",
+			pr_fail("%s: memory corrupted before jmpbuf region\n",
 				args->name);
 		}
 
@@ -94,6 +94,10 @@ static int OPTIMIZE1 stress_longjmp(const stress_args_t *args)
 	if (keep_stressing(args))
 		stress_longjmp_func();
 
+	if (longjmp_failed) {
+		pr_fail("%s failed, did not detect any successful longjmp calls\n", args->name);
+	}
+
 	if (n) {
 		const double rate = (double)STRESS_NANOSECOND * t_total / (double)n;
 		pr_dbg("%s: about %.3f nanoseconds per longjmp call\n",
@@ -108,5 +112,6 @@ static int OPTIMIZE1 stress_longjmp(const stress_args_t *args)
 stressor_info_t stress_longjmp_info = {
 	.stressor = stress_longjmp,
 	.class = CLASS_CPU,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
