@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_LOOP_H)
+#include <linux/loop.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"loop N",	"start N workers exercising loopback devices" },
@@ -114,7 +114,7 @@ static int stress_loop(const stress_args_t *args)
 			args->name, backing_file, errno, strerror(errno));
 		goto tidy;
 	}
-	(void)unlink(backing_file);
+	(void)shim_unlink(backing_file);
 	if (ftruncate(backing_fd, (off_t)backing_size) < 0) {
 		pr_fail("%s: ftruncate failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -142,7 +142,15 @@ static int stress_loop(const stress_args_t *args)
 #if defined(LOOP_SET_BLOCK_SIZE)
 		unsigned long blk_size;
 		static const uint16_t blk_sizes[] = {
-			512, 1024, 2048, 4096,
+			256,	/* Invalid */
+			512,	/* Valid */
+			1024,	/* Valid */
+			2048,	/* Valid */
+			4096,	/* Valid */
+			8192,	/* Invalid */
+			4000,	/* Invalid */
+			0,	/* Invalid */
+			65535	/* Invalid */
 		};
 #endif
 
@@ -275,8 +283,11 @@ static int stress_loop(const stress_args_t *args)
 		(void)ret;
 #endif
 
-		ret = fsync(backing_fd);
-		(void)ret;
+		/*
+		 *  Sync is required to avoid loop_set_block_size
+		 *  warning messages
+		 */
+		(void)sync();
 
 #if defined(LOOP_SET_BLOCK_SIZE)
 		/*
@@ -287,6 +298,7 @@ static int stress_loop(const stress_args_t *args)
 		blk_size = (unsigned long)blk_sizes[stress_mwc8() % SIZEOF_ARRAY(blk_sizes)];
 		ret = ioctl(loop_dev, LOOP_SET_BLOCK_SIZE, blk_size);
 		(void)ret;
+
 #endif
 
 #if defined(LOOP_SET_DIRECT_IO)
@@ -337,6 +349,7 @@ static int stress_loop(const stress_args_t *args)
 #if defined(LOOP_GET_STATUS)
 clr_loop:
 #endif
+
 		/*
 		 *  Disassociate backing store from loop device
 		 */
@@ -371,6 +384,13 @@ destroy_loop:
 				break;
 			}
 		}
+		/* Remove invalid loop device */
+		ret = ioctl(ctrl_dev, LOOP_CTL_REMOVE, -1);
+		(void)ret;
+
+		/* Exercise invalid ioctl */
+		ret = ioctl(ctrl_dev, (LOOP_SET_FD & 0xff00) | 0xff, -1);
+		(void)ret;
 next:
 		(void)close(ctrl_dev);
 #if defined(LOOP_SET_CAPACITY)
@@ -396,6 +416,7 @@ stressor_info_t stress_loop_info = {
 	.stressor = stress_loop,
 	.supported = stress_loop_supported,
 	.class = CLASS_OS | CLASS_DEV,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
