@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -44,6 +39,7 @@ typedef struct madvise_ctxt {
 	void *buf;
 	size_t sz;
 	bool  is_thread;
+	char *smaps;
 } madvise_ctxt_t;
 
 static sigjmp_buf jmp_env;
@@ -120,7 +116,7 @@ static const int madvise_options[] = {
 #if defined(MADV_AUTOSYNC)
 	MADV_AUTOSYNC,
 #endif
-/* FreeBSD */
+/* FreeBSD and DragonFlyBSD */
 #if defined(MADV_CORE)
 	MADV_CORE,
 #endif
@@ -136,6 +132,34 @@ static const int madvise_options[] = {
 #if defined(MADV_POPULATE_WRITE)
 	MADV_POPULATE_WRITE,
 #endif
+/* OpenBSD */
+#if defined(MADV_SPACEAVAIL)
+	MADV_SPACEAVAIL,
+#endif
+/* OS X */
+#if defined(MADV_ZERO_WIRED_PAGES)
+	MADV_ZERO_WIRED_PAGES,
+#endif
+/* Solaris */
+#if defined(MADV_ACCESS_DEFAULT)
+	MADV_ACCESS_DEFAULT,
+#endif
+/* Solaris */
+#if defined(MADV_ACCESS_LWP)
+	MADV_ACCESS_LWP,
+#endif
+/* Solaris */
+#if defined(MADV_ACCESS_MANY)
+	MADV_ACCESS_MANY,
+#endif
+/* DragonFlyBSD */
+#if defined(MADV_INVAL)
+	MADV_INVAL,
+#endif
+/* DragonFlyBSD */
+#if defined(MADV_NOCORE)
+	MADV_NOCORE,
+#endif
 };
 
 /*
@@ -151,6 +175,34 @@ static void NORETURN MLOCKED_TEXT stress_sigbus_handler(int signum)
 	siglongjmp(jmp_env, 1);
 }
 
+#if defined(MADV_FREE)
+/*
+ *  stress_read_proc_smaps()
+ *	read smaps file for extra kernel exercising
+ */
+static void stress_read_proc_smaps(const char *smaps)
+{
+	static bool ignore = false;
+	const size_t sz = 4096;
+	ssize_t ret;
+	char buffer[sz];
+	int fd;
+
+	if (ignore)
+		return;
+
+	fd = open(smaps, O_RDONLY);
+	if (fd < 0) {
+		ignore = true;
+		return;
+	}
+	do {
+		ret = read(fd, buffer, sz);
+	} while (ret == (ssize_t)sz);
+	(void)close(fd);
+}
+#endif
+
 /*
  *  stress_random_advise()
  *	get a random advise option
@@ -182,6 +234,7 @@ static int stress_random_advise(const stress_args_t *args)
 		poison_count++;
 	}
 #else
+	UNEXPECTED
 	(void)args;
 #endif
 
@@ -227,6 +280,10 @@ static void *stress_madvise_pages(void *arg)
 		void *ptr = (void *)(((uint8_t *)buf) + n);
 
 		(void)shim_madvise(ptr, page_size, advise);
+#if defined(MADV_FREE)
+		if (advise == MADV_FREE)
+			stress_read_proc_smaps(ctxt->smaps);
+#endif
 		(void)shim_msync(ptr, page_size, MS_ASYNC);
 	}
 	for (n = 0; n < sz; n += page_size) {
@@ -347,9 +404,12 @@ static int stress_madvise(const stress_args_t *args)
 	NOCLOBBER int num_mem_retries = 0;
 	char filename[PATH_MAX];
 	char page[page_size];
+	char smaps[PATH_MAX];
 	size_t n;
 	madvise_ctxt_t ctxt;
 
+	(void)snprintf(smaps, sizeof(smaps), "/proc/%" PRIdMAX "/smaps", (intmax_t)pid);
+
 	ret = sigsetjmp(jmp_env, 1);
 	if (ret) {
 		pr_fail("%s: sigsetjmp failed\n", args->name);
@@ -379,12 +439,12 @@ static int stress_madvise(const stress_args_t *args)
 		ret = exit_status(errno);
 		pr_fail("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		(void)stress_temp_dir_rm_args(args);
 		return ret;
 	}
 
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	for (n = 0; n < sz; n += page_size) {
 		ssize_t wret;
 
@@ -437,6 +497,7 @@ static int stress_madvise(const stress_args_t *args)
 		ctxt.args = args;
 		ctxt.buf = buf;
 		ctxt.sz = sz;
+		ctxt.smaps = smaps;
 
 #if defined(HAVE_LIB_PTHREAD)
 		{
