@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-put.h"
+#include "core-target-clones.h"
+
+#define MIN_MATRIX3D_SIZE	(16)
+#define MAX_MATRIX3D_SIZE	(1024)
+#define DEFAULT_MATRIX3D_SIZE	(64)
 
 static const stress_help_t help[] = {
 	{ NULL,	"matrix-3d N",		"start N workers exercising 3D matrix operations" },
@@ -444,7 +445,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_frobenius(
 				return;
 		}
 	}
-	stress_double_put(sum);
+	stress_float_put((float)sum);
 }
 
 /*
@@ -476,7 +477,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_frobenius(
 				return;
 		}
 	}
-	stress_double_put(sum);
+	stress_float_put((float)sum);
 }
 
 /*
@@ -556,7 +557,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_xyz_mean(
 			register size_t k;
 
 			for (k = 0; k < n; k++) {
-				r[i][j][k] = (a[i][j][k] + b[i][j][k]) / 2.0;
+				r[i][j][k] = (a[i][j][k] + b[i][j][k]) / (stress_matrix_3d_type_t)2.0;
 			}
 			if (UNLIKELY(!keep_stressing_flag()))
 				return;
@@ -583,7 +584,7 @@ static void OPTIMIZE3 TARGET_CLONES stress_matrix_3d_zyx_mean(
 			register size_t i;
 
 			for (i = 0; i < n; i++) {
-				r[i][j][k] = (a[i][j][k] + b[i][j][k]) / 2.0;
+				r[i][j][k] = (a[i][j][k] + b[i][j][k]) / (stress_matrix_3d_type_t)2.0;
 			}
 			if (UNLIKELY(!keep_stressing_flag()))
 				return;
@@ -876,6 +877,17 @@ static inline size_t round_up(size_t page_size, size_t n)
 	return (n + page_size - 1) & (~(page_size -1));
 }
 
+/*
+ *  stress_matrix_data()
+ *`	generate some random data scaled by v
+ */
+static inline stress_matrix_3d_type_t stress_matrix_data(const stress_matrix_3d_type_t v)
+{
+	const uint64_t r = stress_mwc64();
+
+	return v * (stress_matrix_3d_type_t)r;
+}
+
 static inline int stress_matrix_3d_exercise(
 	const stress_args_t *args,
 	const stress_matrix_3d_func func,
@@ -922,8 +934,8 @@ static inline int stress_matrix_3d_exercise(
 			register size_t k;
 
 			for (k = 0; k < n; k++) {
-				a[i][j][k] = (stress_matrix_3d_type_t)stress_mwc64() * v;
-				b[i][j][k] = (stress_matrix_3d_type_t)stress_mwc64() * v;
+				a[i][j][k] = stress_matrix_data(v);
+				b[i][j][k] = stress_matrix_data(v);
 				r[i][j][k] = 0.0;
 			}
 		}
@@ -954,7 +966,7 @@ tidy_ret:
  */
 static int stress_matrix(const stress_args_t *args)
 {
-	char *matrix_3d_method_name;
+	char *matrix_3d_method_name = NULL;
 	const stress_matrix_3d_method_info_t *matrix_3d_method;
 	stress_matrix_3d_func func;
 	size_t matrix_3d_size = 128;
@@ -978,9 +990,9 @@ static int stress_matrix(const stress_args_t *args)
 
 	if (!stress_get_setting("matrix-3d-size", &matrix_3d_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			matrix_3d_size = MAX_MATRIX_SIZE;
+			matrix_3d_size = MAX_MATRIX3D_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			matrix_3d_size = MIN_MATRIX_SIZE;
+			matrix_3d_size = MIN_MATRIX3D_SIZE;
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
