@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2018-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-cache.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"mcontend N",	  "start N workers that produce memory contention" },
@@ -42,7 +39,8 @@ static inline void mem_barrier(void)
 	asm volatile("": : :"memory");
 }
 
-#if defined(STRESS_ARCH_X86)
+#if defined(STRESS_ARCH_X86) &&	\
+    defined(HAVE_ASM_X86_PAUSE)
 static inline void cpu_relax(void)
 {
 	asm volatile("pause\n": : :"memory");
@@ -170,7 +168,6 @@ static inline HOT OPTIMIZE3 void stress_memory_contend(const stress_pthread_args
 		read64(data1);
 	}
 
-#if defined(STRESS_ARCH_X86)
 	for (i = 0; i < 1024; i++) {
 		vdata0[0] = (uint64_t)i;
 		vdata1[0] = (uint64_t)i;
@@ -194,6 +191,8 @@ static inline HOT OPTIMIZE3 void stress_memory_contend(const stress_pthread_args
 		read64(data1);
 	}
 
+#if defined(STRESS_ARCH_X86) &&	\
+    defined(HAVE_ASM_X86_PAUSE)
 	for (i = 0; i < 1024; i++) {
 		vdata0[0] = (uint64_t)i;
 		cpu_relax();
@@ -332,22 +331,21 @@ static int stress_mcontend(const stress_args_t *args)
 	if (fd < 0) {
 		pr_inf("%s: open failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		(void)stress_temp_dir_rm_args(args);
 		return EXIT_NO_RESOURCE;
 	}
+	(void)shim_unlink(filename);
+
 	rc = page_write_sync(fd, args->page_size);
 	if (rc < 0) {
 		pr_inf("%s: mmap backing file write failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd);
-		(void)unlink(filename);
 		(void)stress_temp_dir_rm_args(args);
 		return EXIT_NO_RESOURCE;
 
 	}
-	(void)unlink(filename);
-	(void)stress_temp_dir_rm_args(args);
 
 	pa.args = args;
 	/*
@@ -360,6 +358,7 @@ static int stress_mcontend(const stress_args_t *args)
 		pr_inf("%s: mmap failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd);
+		(void)stress_temp_dir_rm_args(args);
 		return EXIT_NO_RESOURCE;
 	}
 	data[1] = mmap(NULL, args->page_size , PROT_READ | PROT_WRITE,
@@ -369,6 +368,7 @@ static int stress_mcontend(const stress_args_t *args)
 			args->name, errno, strerror(errno));
 		(void)munmap(data[0], args->page_size);
 		(void)close(fd);
+		(void)stress_temp_dir_rm_args(args);
 		return EXIT_NO_RESOURCE;
 	}
 	(void)close(fd);
@@ -401,6 +401,9 @@ static int stress_mcontend(const stress_args_t *args)
 	(void)munmap(data[0], args->page_size);
 	(void)munmap(data[1], args->page_size);
 
+	(void)close(fd);
+	(void)stress_temp_dir_rm_args(args);
+
 	return EXIT_SUCCESS;
 }
 
