@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_MEMBARRIER_H)
+#include <linux/membarrier.h>
+#endif
+
+#if defined(__NR_membarrier)
+#define HAVE_MEMBARRIER
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"membarrier N",		"start N workers performing membarrier system calls" },
 	{ NULL,	"membarrier-ops N",	"stop after N membarrier bogo operations" },
@@ -53,6 +56,9 @@ enum membarrier_cmd {
 	MEMBARRIER_CMD_REGISTER_PRIVATE_EXPEDITED_RSEQ		= (1 << 8),
 };
 
+enum membarrier_cmd_flag {
+        MEMBARRIER_CMD_FLAG_CPU					= (1 << 0),
+};
 #endif
 
 static int stress_membarrier_exercise(const stress_args_t *args)
@@ -60,7 +66,7 @@ static int stress_membarrier_exercise(const stress_args_t *args)
 	int ret;
 	unsigned int i, mask;
 
-	ret = shim_membarrier(MEMBARRIER_CMD_QUERY, 0);
+	ret = shim_membarrier(MEMBARRIER_CMD_QUERY, 0, 0);
 	if (ret < 0) {
 		pr_fail("%s: membarrier CMD QUERY failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -69,19 +75,29 @@ static int stress_membarrier_exercise(const stress_args_t *args)
 	mask = (unsigned int)ret;
 	for (i = 1; i; i <<= 1) {
 		if (i & mask) {
-			ret = shim_membarrier((int)i, 0);
+			ret = shim_membarrier((int)i, 0, 0);
 			(void)ret;
 
 			/* Exercise illegal flags */
-			ret = shim_membarrier((int)i, ~0);
+			ret = shim_membarrier((int)i, ~0, 0);
 			(void)ret;
+
+			/* Exercise illegal cpu_id */
+			ret = shim_membarrier((int)i, 0, INT_MAX);
+			(void)ret;
+
+#if defined(MEMBARRIER_CMD_FLAG_CPU)
+			/* Exercise MEMBARRIER_CMD_FLAG_CPU flag */
+			ret = shim_membarrier((int)i, MEMBARRIER_CMD_FLAG_CPU, 0);
+			(void)ret;
+#endif
 		}
 	}
 
 	/* Exercise illegal command */
 	for (i = 1; i; i <<= 1) {
 		if (!(i & mask)) {
-			ret = shim_membarrier((int)i, 0);
+			ret = shim_membarrier((int)i, 0, 0);
 			(void)ret;
 			break;
 		}
@@ -120,7 +136,7 @@ static int stress_membarrier(const stress_args_t *args)
 	int pthread_ret[MAX_MEMBARRIER_THREADS];
 	stress_pthread_args_t pargs = { args, NULL, 0 };
 
-	ret = shim_membarrier(MEMBARRIER_CMD_QUERY, 0);
+	ret = shim_membarrier(MEMBARRIER_CMD_QUERY, 0, 0);
 	if (ret < 0) {
 		if (errno == ENOSYS) {
 			if (args->instance == 0)
@@ -129,7 +145,7 @@ static int stress_membarrier(const stress_args_t *args)
 					args->name);
 			return EXIT_NOT_IMPLEMENTED;
 		}
-		pr_err("%s: membarrier failed: errno=%d: (%s)\n",
+		pr_fail("%s: membarrier failed: errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -153,7 +169,7 @@ static int stress_membarrier(const stress_args_t *args)
 
 	do {
 		if (stress_membarrier_exercise(args) < 0) {
-			pr_err("%s: membarrier failed: errno=%d: (%s)\n",
+			pr_fail("%s: membarrier failed: errno=%d: (%s)\n",
 				args->name, errno, strerror(errno));
 		}
 		inc_counter(args);
