@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_MEMFD_BYTES		(2 * MB)
+#define MAX_MEMFD_BYTES		(MAX_MEM_LIMIT)
+#define DEFAULT_MEMFD_BYTES	(256 * MB)
+
+#define MIN_MEMFD_FDS		(8)
+#define MAX_MEMFD_FDS		(4096)
+#define DEFAULT_MEMFD_FDS	(256)
+
 static const stress_help_t help[] = {
 	{ NULL,	"memfd N",	 "start N workers allocating memory with memfd_create" },
 	{ NULL,	"memfd-bytes N", "allocate N bytes for each stress iteration" },
@@ -99,6 +102,11 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 	size_t size;
 	size_t memfd_bytes = DEFAULT_MEMFD_BYTES;
 	uint32_t memfd_fds = DEFAULT_MEMFD_FDS;
+	int mmap_flags = MAP_FILE | MAP_SHARED;
+
+#if defined(MAP_POPULATE)
+	mmap_flags |= MAP_POPULATE;
+#endif
 
 	(void)context;
 
@@ -115,7 +123,6 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 	(void)stress_get_setting("memfd-fds", &memfd_fds);
 
 	size = memfd_bytes / memfd_fds;
-
 	if (size < min_size)
 		size = min_size;
 
@@ -158,7 +165,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 				case ENOSYS:
 				case EFAULT:
 				default:
-					pr_err("%s: memfd_create failed: errno=%d (%s)\n",
+					pr_fail("%s: memfd_create failed: errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 					keep_stressing_set_flag(false);
 					goto clean;
@@ -170,6 +177,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 
 		for (i = 0; i < memfd_fds; i++) {
 			ssize_t ret;
+
 #if defined(FALLOC_FL_PUNCH_HOLE) &&	\
     defined(FALLOC_FL_KEEP_SIZE)
 			off_t whence;
@@ -197,9 +205,7 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 			 * ..and map it in, using MAP_POPULATE
 			 * to force page it in
 			 */
-			maps[i] = mmap(NULL, size, PROT_WRITE,
-				MAP_FILE | MAP_SHARED | MAP_POPULATE,
-				fds[i], 0);
+			maps[i] = mmap(NULL, size, PROT_WRITE, mmap_flags, fds[i], 0);
 			(void)stress_mincore_touch_pages(maps[i], size);
 			(void)stress_madvise_random(maps[i], size);
 
@@ -217,12 +223,11 @@ static int stress_memfd_child(const stress_args_t *args, void *context)
 				goto clean;
 		}
 
-
 		for (i = 0; i < memfd_fds; i++) {
 			if (fds[i] < 0)
 				continue;
 #if defined(SEEK_SET)
-			if (lseek(fds[i], (off_t)size>> 1, SEEK_SET) < 0) {
+			if (lseek(fds[i], (off_t)size >> 1, SEEK_SET) < 0) {
 				if (errno != ENXIO)
 					pr_fail("%s: lseek SEEK_SET failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -328,7 +333,6 @@ clean:
 	return EXIT_SUCCESS;
 }
 
-
 /*
  *  stress_memfd()
  *	stress memfd
@@ -342,6 +346,7 @@ stressor_info_t stress_memfd_info = {
 	.stressor = stress_memfd,
 	.class = CLASS_OS | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
