@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-cache.h"
+#include "core-nt-store.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"memthrash N",		"start N workers thrashing a 16MB memory buffer" },
@@ -172,6 +170,47 @@ static void stress_memthrash_memmove(
 #endif
 }
 
+static void HOT OPTIMIZE3 stress_memthrash_memset64(
+	const stress_args_t *args,
+	const size_t mem_size)
+{
+	register uint64_t *ptr = (uint64_t *)mem;
+	register const uint64_t *end = (uint64_t *)(((uint8_t *)mem) + mem_size);
+	register uint64_t val = stress_mwc64();
+
+	(void)args;
+
+#if defined(HAVE_NT_STORE64)
+	if (stress_cpu_x86_has_sse2()) {
+		while (LIKELY(ptr < end)) {
+			stress_nt_store64(ptr + 0, val);
+			stress_nt_store64(ptr + 1, val);
+			stress_nt_store64(ptr + 2, val);
+			stress_nt_store64(ptr + 3, val);
+			stress_nt_store64(ptr + 4, val);
+			stress_nt_store64(ptr + 5, val);
+			stress_nt_store64(ptr + 6, val);
+			stress_nt_store64(ptr + 7, val);
+			ptr += 8;
+		}
+		return;
+	}
+#endif
+	/* normal temporal stores, non-SSE fallback */
+
+	while (LIKELY(ptr < end)) {
+		*ptr++ = val;
+		*ptr++ = val;
+		*ptr++ = val;
+		*ptr++ = val;
+		*ptr++ = val;
+		*ptr++ = val;
+		*ptr++ = val;
+		*ptr++ = val;
+	}
+}
+
+
 static void HOT OPTIMIZE3 stress_memthrash_flip_mem(
 	const stress_args_t *args,
 	const size_t mem_size)
@@ -257,6 +296,7 @@ static void HOT OPTIMIZE3 stress_memthrash_prefetch(
 	}
 }
 
+#if defined(HAVE_ASM_X86_CLFLUSH)
 static void HOT OPTIMIZE3 stress_memthrash_flush(
 	const stress_args_t *args,
 	const size_t mem_size)
@@ -275,6 +315,7 @@ static void HOT OPTIMIZE3 stress_memthrash_flush(
 		shim_clflush(ptr);
 	}
 }
+#endif
 
 static void HOT OPTIMIZE3 stress_memthrash_mfence(
 	const stress_args_t *args,
@@ -375,13 +416,16 @@ static const stress_memthrash_method_info_t memthrash_methods[] = {
 	{ "chunk256",	stress_memthrash_random_chunk256 },
 	{ "chunkpage",	stress_memthrash_random_chunkpage },
 	{ "flip",	stress_memthrash_flip_mem },
+#if defined(HAVE_ASM_X86_CLFLUSH)
 	{ "flush",	stress_memthrash_flush },
+#endif
 #if defined(MEM_LOCK)
 	{ "lock",	stress_memthrash_lock },
 #endif
 	{ "matrix",	stress_memthrash_matrix },
 	{ "memmove",	stress_memthrash_memmove },
 	{ "memset",	stress_memthrash_memset },
+	{ "memset64",	stress_memthrash_memset64 },
 	{ "mfence",	stress_memthrash_mfence },
 	{ "prefetch",	stress_memthrash_prefetch },
 	{ "random",	stress_memthrash_random },
