@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -65,15 +60,15 @@ static int stress_mincore_file(const stress_args_t *args)
 	(void)stress_temp_filename_args(args, filename,
 					sizeof(filename), stress_mwc32());
 	fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	if (fd < 0) {
 		(void)stress_temp_dir_rm_args(args);
 		return -1;
 	}
 	ret = shim_fallocate(fd, 0, (off_t)0, (off_t)args->page_size);
 	if (ret < 0) {
-		(void)stress_temp_dir_rm_args(args);
 		(void)close(fd);
+		(void)stress_temp_dir_rm_args(args);
 		return -1;
 	}
 	return fd;
@@ -231,15 +226,15 @@ redo: 			errno = 0;
 err:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	if (fd >= 0) {
-		(void)stress_temp_dir_rm_args(args);
-		(void)close(fd);
-	}
 	if (fdmapped != MAP_FAILED)
 		(void)munmap((void *)fdmapped, page_size);
 	if (mapped != MAP_FAILED)
 		(void)munmap((void *)mapped, page_size);
 
+	if (fd >= 0) {
+		(void)close(fd);
+		(void)stress_temp_dir_rm_args(args);
+	}
 	return rc;
 }
 
@@ -247,6 +242,7 @@ stressor_info_t stress_mincore_info = {
 	.stressor = stress_mincore,
 	.class = CLASS_OS | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
