@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LIBGEN_H)
+#include <libgen.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"mknod N",	"start N workers that exercise mknod" },
 	{ NULL,	"mknod-ops N",	"stop after N mknod bogo operations" },
@@ -68,7 +67,7 @@ static void stress_mknod_tidy(
 
 		(void)stress_temp_filename_args(args,
 			path, sizeof(path), gray_code);
-		(void)unlink(path);
+		(void)shim_unlink(path);
 	}
 }
 
@@ -142,6 +141,7 @@ static int stress_mknod_check_errno(
 
 static int stress_do_mknod(
 	const int dir_fd,
+	const int bad_fd,
 	const char *path,
 	const mode_t mode,
 	const dev_t dev)
@@ -158,14 +158,22 @@ static int stress_do_mknod(
 		(void)shim_strlcpy(tmp, path, sizeof(tmp));
 		filename = basename(tmp);
 
+		(void)shim_force_unlink(path);
+		ret = mknodat(bad_fd, filename, mode, dev);
+		if (ret == 0)
+			(void)shim_unlink(path);
+
 		ret = mknodat(dir_fd, filename, mode, dev);
 	} else {
+		(void)shim_force_unlink(path);
 		ret = mknod(path, mode, dev);
 	}
 #else
+	(void)shim_force_unlink(path);
 	ret = mknod(path, mode, dev);
 #endif
 	(void)dir_fd;
+	(void)bad_fd;
 
 	return ret;
 }
@@ -177,6 +185,7 @@ static int stress_do_mknod(
 static void stress_mknod_test_dev(
 	const stress_args_t *args,
 	const int dir_fd,
+	const int bad_fd,
 	const mode_t mode,
 	const char *mode_str,
 	dev_t dev)
@@ -186,11 +195,11 @@ static void stress_mknod_test_dev(
 
 	(void)stress_temp_filename_args(args, path, sizeof(path), stress_mwc32());
 
-	ret = stress_do_mknod(dir_fd, path, mode, dev);
+	ret = stress_do_mknod(dir_fd, bad_fd, path, mode, dev);
 	if (ret < 0)
 		(void)stress_mknod_check_errno(args, mode_str, path, errno);
 
-	(void)unlink(path);
+	(void)shim_unlink(path);
 }
 
 /*
@@ -204,6 +213,7 @@ static int stress_mknod(const stress_args_t *args)
 	dev_t chr_dev, blk_dev;
 	int chr_dev_ret, blk_dev_ret;
 	int dir_fd = -1;
+	const int bad_fd = stress_get_bad_fd();
 #if defined(HAVE_MKNODAT) &&	\
     defined(O_DIRECTORY)
 	char pathname[PATH_MAX];
@@ -233,9 +243,9 @@ static int stress_mknod(const stress_args_t *args)
 		uint64_t i, n = DEFAULT_DIRS;
 
 		if (chr_dev_ret == 0)
-			stress_mknod_test_dev(args, dir_fd, S_IFCHR, "S_IFCHR", chr_dev);
+			stress_mknod_test_dev(args, dir_fd, bad_fd, S_IFCHR, "S_IFCHR", chr_dev);
 		if (blk_dev_ret == 0)
-			stress_mknod_test_dev(args, dir_fd, S_IFBLK, "S_IFBLK", blk_dev);
+			stress_mknod_test_dev(args, dir_fd, bad_fd, S_IFBLK, "S_IFBLK", blk_dev);
 
 		for (i = 0; keep_stressing(args) && (i < n); i++) {
 			char path[PATH_MAX];
@@ -244,7 +254,7 @@ static int stress_mknod(const stress_args_t *args)
 
 			(void)stress_temp_filename_args(args,
 				path, sizeof(path), gray_code);
-			if (stress_do_mknod(dir_fd, path, modes[j].mode | S_IRUSR | S_IWUSR, 0) < 0) {
+			if (stress_do_mknod(dir_fd, bad_fd, path, modes[j].mode | S_IRUSR | S_IWUSR, 0) < 0) {
 				if (stress_mknod_check_errno(args, modes[j].mode_str, path, errno) < 0)
 					continue;	/* Try again */
 				break;
@@ -271,6 +281,7 @@ static int stress_mknod(const stress_args_t *args)
 stressor_info_t stress_mknod_info = {
 	.stressor = stress_mknod,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
