@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(__NR_mmap2)
+#define HAVE_MMAP2
+#endif
+
+#define MIN_MMAP_BYTES		(4 * KB)
+#define MAX_MMAP_BYTES		(MAX_MEM_LIMIT)
+#define DEFAULT_MMAP_BYTES	(256 * MB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"mmap N",	 "start N workers stressing mmap and munmap" },
 	{ NULL,	"mmap-ops N",	 "stop after N mmap bogo operations" },
@@ -117,7 +120,31 @@ static const int mmap_flags[] = {
 #if defined(MAP_PREFAULT_READ)
 	MAP_PREFAULT_READ,
 #endif
-	0
+/* NetBSD */
+#if defined(MAP_WIRED)
+	MAP_WIRED,
+#endif
+/* NetBSD */
+#if defined(MAP_ALIGNMENT_64KB)
+	MAP_ALIGNMENT_64KB,
+#endif
+/* NetBSD */
+#if defined(MAP_ALIGNMENT_16MB)
+	MAP_ALIGNMENT_16MB,
+#endif
+/* OpenBSD */
+#if defined(MAP_CONCEAL)
+	MAP_CONCEAL,
+#endif
+/* OpenBSD */
+#if defined(MAP_COPY)
+	MAP_COPY,
+#endif
+/* Solaris */
+#if defined(MAP_LOW32)
+	MAP_LOW32,
+#endif
+	0,
 };
 
 /*
@@ -447,6 +474,17 @@ retry:
 				uint64_t page = (i + j) % pages4k;
 				if (mapped[page] == PAGE_MAPPED) {
 					mapped[page] = 0;
+#if defined(HAVE_MQUERY) &&	\
+    defined(MAP_FIXED)
+					{
+						/* Exercise OpenBSD mquery */
+						void *query;
+
+						query = mquery(mappings[page], page_size,
+								PROT_READ, MAP_FIXED, -1, 0);
+						(void)query;
+					}
+#endif
 					(void)stress_madvise_random(mappings[page], page_size);
 					stress_mmap_mprotect(args->name, mappings[page],
 						page_size, page_size, context->mmap_mprotect);
@@ -652,6 +690,7 @@ static int stress_mmap(const stress_args_t *args)
 #else
 			pr_inf("%s: --mmap-odirect selected by not supported by this system\n",
 				args->name);
+			UNEXPECTED
 #endif
 		}
 		if (mmap_osync) {
@@ -660,6 +699,7 @@ static int stress_mmap(const stress_args_t *args)
 #else
 			pr_inf("%s: --mmap-osync selected by not supported by this system\n",
 				args->name);
+			UNEXPECTED
 #endif
 		}
 
@@ -668,12 +708,12 @@ static int stress_mmap(const stress_args_t *args)
 			rc = exit_status(errno);
 			pr_fail("%s: open %s failed, errno=%d (%s)\n",
 				args->name, filename, errno, strerror(errno));
-			(void)unlink(filename);
+			(void)shim_unlink(filename);
 			(void)stress_temp_dir_rm_args(args);
 
 			return (int)rc;
 		}
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		if (lseek(context.fd, (off_t)(context.sz - args->page_size), SEEK_SET) < 0) {
 			pr_fail("%s: lseek failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -733,5 +773,6 @@ stressor_info_t stress_mmap_info = {
 	.stressor = stress_mmap,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
