@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -55,7 +50,7 @@ static int stress_mmapaddr_check(const stress_args_t *args, uint8_t *map_addr)
 	(void)val;
 
 	if (page_fault) {
-		pr_err("%s: read of mmap'd address %p SEGFAULTed\n",
+		pr_fail("%s: read of mmap'd address %p SEGFAULTed\n",
 			args->name, (void *)map_addr);
 		return -1;
 	}
@@ -63,7 +58,7 @@ static int stress_mmapaddr_check(const stress_args_t *args, uint8_t *map_addr)
 	vec[0] = 0;
 	ret = shim_mincore(map_addr, args->page_size, vec);
 	if (ret != 0) {
-		pr_err("%s: mincore on address %p failed, errno=%d (%s)\n",
+		pr_fail("%s: mincore on address %p failed, errno=%d (%s)\n",
 			args->name, (void *)map_addr, errno, strerror(errno));
 		return -1;
 	}
@@ -226,5 +221,6 @@ static int stress_mmapaddr(const stress_args_t *args)
 stressor_info_t stress_mmapaddr_info = {
 	.stressor = stress_mmapaddr,
 	.class = CLASS_VM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
