@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_MQUEUE_H)
+#include <mqueue.h>
+#endif
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#endif
+
+#define MIN_MQ_SIZE		(1)
+#define MAX_MQ_SIZE		(32)
+#define DEFAULT_MQ_SIZE		(10)
+
 static const stress_help_t help[] = {
 	{ NULL,	"mq N",		"start N workers passing messages using POSIX messages" },
 	{ NULL,	"mq-ops N",	"stop mq workers after N bogo messages" },
@@ -511,6 +518,7 @@ stressor_info_t stress_mq_info = {
 	.stressor = stress_mq,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
