@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define DEFAULT_MREMAP_BYTES	(256 * MB)
+#define MIN_MREMAP_BYTES	(4 * KB)
+#define MAX_MREMAP_BYTES	(MAX_MEM_LIMIT)
+#define DEFAULT_MREMAP_BYTES	(256 * MB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"mremap N",	  "start N workers stressing mremap" },
 	{ NULL,	"mremap-ops N",	  "stop after N mremap bogo operations" },
@@ -67,8 +67,12 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 static inline void *rand_mremap_addr(const size_t sz, int flags)
 {
 	void *addr;
+	int mask = MREMAP_FIXED | MAP_SHARED;
 
-	flags &= ~(MREMAP_FIXED | MAP_SHARED | MAP_POPULATE);
+#if defined(MAP_POPULATE)
+	mask |= MAP_POPULATE;
+#endif
+	flags &= ~(mask);
 	flags |= (MAP_PRIVATE | MAP_ANONYMOUS);
 
 	addr = mmap(NULL, sz, PROT_READ | PROT_WRITE, flags, -1, 0);
@@ -314,6 +318,7 @@ stressor_info_t stress_mremap_info = {
 	.stressor = stress_mremap,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
