@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_IPC_H)
+#include <sys/ipc.h>
+#endif
+
+#if defined(HAVE_SYS_MSG_H)
+#include <sys/msg.h>
+#endif
+
 #define STRESS_MAX_IDS		(1024)
 
 static const stress_help_t help[] = {
@@ -66,6 +69,11 @@ static int stress_msg_get_stats(const stress_args_t *args, const int msgq_id)
 				args->name, errno, strerror(errno));
 			return -errno;
 		}
+
+#if defined(IPC_SET)
+		/* exercise IPC_SET, ignore failures */
+		(void)msgctl(msgq_id, IPC_SET, &buf);
+#endif
 	}
 #endif
 
@@ -158,7 +166,7 @@ static void stress_msgsnd(const int msgq_id)
 	/* Invalid msgq_id */
 	msg.mtype = 1;
 	msg.value = 0;
-	ret = msgsnd(-1, &msg, sizeof(msg), 0);
+	ret = msgsnd(-1, &msg, sizeof(msg.value), 0);
 	(void)ret;
 
 	/* Zero msg length + 0 msg.type */
@@ -168,7 +176,7 @@ static void stress_msgsnd(const int msgq_id)
 
 	/* Illegal flags, may or may not succeed */
 	msg.mtype = 1;
-	ret = msgsnd(msgq_id, &msg, sizeof(msg), ~0);
+	ret = msgsnd(msgq_id, &msg, sizeof(msg.value), ~0);
 	(void)ret;
 }
 
@@ -268,9 +276,10 @@ static int stress_msg(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() &&
-		    ((errno == EAGAIN) || (errno == ENOMEM)))
+		if (stress_redo_fork(errno))
 			goto again;
+		if (!keep_stressing(args))
+			goto cleanup;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
@@ -301,6 +310,19 @@ again:
 					(void)ret;
 				}
 #endif
+
+				if ((i & 0x1ff) == 0) {
+					/* Exercise invalid msgrcv queue ID */
+					(void)msgrcv(-1, &msg, sizeof(msg.value), mtype, 0);
+
+					/* Exercise invalid msgrcv message size */
+					(void)msgrcv(msgq_id, &msg, -1, mtype, 0);
+					(void)msgrcv(msgq_id, &msg, 0, mtype, 0);
+
+					/* Exercise invalid msgrcv message flag */
+					(void)msgrcv(msgq_id, &msg, sizeof(msg.value), mtype, ~0);
+				}
+
 				if (msgrcv(msgq_id, &msg, sizeof(msg.value), mtype, 0) < 0) {
 					/*
 					 * Check for errors that can occur
@@ -400,6 +422,7 @@ stressor_info_t stress_msg_info = {
 	.stressor = stress_msg,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
