@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_MSYNC_BYTES		(1 * MB)  /* MUST NOT BE page size or less! */
+#define MAX_MSYNC_BYTES		(MAX_FILE_LIMIT)
+#define DEFAULT_MSYNC_BYTES	(256 * MB)
+
 #if defined(HAVE_MSYNC)
 static sigjmp_buf jmp_env;
 static uint64_t sigbus_count;
@@ -57,7 +56,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *	check if mmap'd data is sane
  */
 static int stress_page_check(
-	uint8_t *buf,
+	const uint8_t *buf,
 	const uint8_t val,
 	const size_t sz)
 {
@@ -143,12 +142,12 @@ static int stress_msync(const stress_args_t *args)
 		rc = exit_status(errno);
 		pr_fail("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		(void)stress_temp_dir_rm_args(args);
 
 		return (int)rc;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	if (ftruncate(fd, (off_t)sz) < 0) {
 		pr_err("%s: ftruncate failed, errno=%d (%s)\n",
@@ -299,6 +298,7 @@ stressor_info_t stress_msync_info = {
 	.stressor = stress_msync,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
