@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -110,7 +105,7 @@ static int stress_nanosleep(const stress_args_t *args)
 				break;
 			}
 			/* Something really unexpected */
-			pr_fail("%s: pthread create failed, errno=%d (%s)\n",
+			pr_err("%s: pthread create failed, errno=%d (%s)\n",
 				args->name, ret, strerror(ret));
 			ret = EXIT_NO_RESOURCE;
 			goto tidy;
