@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_SOCKIOS_H)
+#include <linux/sockios.h>
+#endif
+
+#if defined(HAVE_NET_IF_H)
+#include <net/if.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"netdev N",	"start N workers exercising netdevice ioctls" },
 	{ NULL,	"netdev-ops N",	"stop netdev workers after N bogo operations" },
@@ -34,6 +37,14 @@ static const stress_help_t help[] = {
     defined(SIOCGIFCONF) &&	\
     defined(HAVE_IFCONF)
 
+/*
+ *  As per man 7 netdevice advise, workaround glibc 2.1 missing
+ *  ifr_newname
+ */
+#ifndef ifr_newname
+#define ifr_newname     ifr_ifru.ifru_slave
+#endif
+
 /*
  *  stress_netdev_check()
  *	helper to perform netdevice ioctl and check for failure
@@ -48,13 +59,19 @@ static void stress_netdev_check(
 	if (ioctl(fd, cmd, ifr) < 0) {
 		if ((errno != ENOTTY) &&
 		    (errno != EINVAL) &&
-		    (errno != EADDRNOTAVAIL))
+		    (errno != EADDRNOTAVAIL) &&
+		    (errno != EOPNOTSUPP) &&
+		    (errno != EBUSY) &&
+		    (errno != EPERM))
 			pr_fail("%s: interface '%s' ioctl %s failed, errno=%d (%s)\n",
 				args->name, ifr->ifr_name, cmd_name,
 				errno, strerror(errno));
 	}
 }
 
+#define STRESS_NETDEV_CHECK(args, ifr, fd, cmd)	\
+	stress_netdev_check(args, ifr, fd, cmd, #cmd)
+
 /*
  *  stress_netdev
  *	stress netdev
@@ -90,7 +107,7 @@ static int stress_netdev(const stress_args_t *args)
 		n = ifc.ifc_len / (int)sizeof(struct ifreq);
 		if (!n) {
 			if (args->instance == 0)
-				pr_dbg("%s: no network interfaces found, skipping.\n",
+				pr_dbg_skip("%s: no network interfaces found, skipping.\n",
 					args->name);
 			break;
 		}
@@ -137,68 +154,81 @@ static int stress_netdev(const stress_args_t *args)
 
 #if defined(SIOCGIFFLAGS)
 			/* Get flags */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFFLAGS, "SIOCGIFFLAGS");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFFLAGS);
 #endif
 
 #if defined(SIOCGIFPFLAGS)
 			/* Get extended flags */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFPFLAGS, "SIOCGIFPFLAGS");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFPFLAGS);
 #endif
 
-#if defined(stress_netdev_check)
+#if defined(SIOCGIFADDR)
 			/* Get address */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFADDR, "SIOCGIFADDR");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFADDR);
 #endif
 
 #if defined(SIOCGIFNETMASK)
 			/* Get netmask */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFNETMASK, "SIOCGIFNETMASK");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFNETMASK);
 #endif
 
-#if defined(SIOCGIFNETMASK)
+#if defined(SIOCGIFMETRIC)
 			/* Get metric (currently not supported) */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFMETRIC, "SIOCGIFMETRIC");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFMETRIC);
 #endif
 
-#if defined(SIOCGIFNETMASK)
+#if defined(SIOCGIFMTU)
 			/* Get the MTU */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFMTU, "SIOCGIFMTU");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFMTU);
 #endif
 
-#if defined(SIOCGIFNETMASK)
+#if defined(SIOCGIFHWADDR)
 			/* Get the hardware address */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFHWADDR, "SIOCGIFHWADDR");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFHWADDR);
 #endif
 
 #if defined(SIOCGIFMAP)
 			/* Get the hardware parameters */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFMAP, "SIOCGIFMAP");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFMAP);
 #endif
 
 #if defined(SIOCGIFTXQLEN)
 			/* Get the transmit queue length */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFTXQLEN, "SIOCGIFTXQLEN");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFTXQLEN);
 #endif
 
 #if defined(SIOCGIFDSTADDR)
 			/* Get the destination address */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFDSTADDR, "SIOCGIFDSTADDR");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFDSTADDR);
 #endif
 
 #if defined(SIOCGIFBRDADDR)
 			/* Get the broadcast address */
-			stress_netdev_check(args, ifr, fd,
-				SIOCGIFBRDADDR, "SIOCGIFBRDADDR");
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFBRDADDR);
+#endif
+#if defined(SIOCGMIIPHY) && 0
+			/* Get from current PHY, disabled for now */
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGMIIPHY);
+#endif
+#if defined(SIOCGMIIREG) && 0
+			/* Get reg, disabled for now */
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGMIIREG);
+#endif
+#if defined(SIOCSIFFLAGS) && 0
+			/* Get flags, disabled for now */
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCSIFFLAGS);
+#endif
+#if defined(SIOCSIFMETRIC) && 0
+			/* Get metric, disabled for now */
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCSIFMETRIC);
+#endif
+#if defined(SIOCGIFMEM)
+			/* Get memory space, not implemented */
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFMEM);
+#endif
+#if defined(SIOCGIFLINK)
+			/* Get if link, not implemented */
+			STRESS_NETDEV_CHECK(args, ifr, fd, SIOCGIFLINK);
 #endif
 		}
 		free(ifc.ifc_buf);
@@ -215,6 +245,7 @@ static int stress_netdev(const stress_args_t *args)
 stressor_info_t stress_netdev_info = {
 	.stressor = stress_netdev,
 	.class = CLASS_NETWORK,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
