@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2017-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,29 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_CN_PROC_H)
+#include <linux/cn_proc.h>
+#endif
+
+#if defined(HAVE_LINUX_CONNECTOR_H)
+#include <linux/connector.h>
+#endif
+
+#if defined(HAVE_LINUX_NETLINK_H)
+#include <linux/netlink.h>
+#endif
+
+#if defined(HAVE_SYS_UIO_H)
+#include <sys/uio.h>
+#endif
+
+#if defined(HAVE_LINUX_VERSION_H)
+#include <linux/version.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"netlink-proc N",	"start N workers exercising netlink process events" },
@@ -226,7 +242,7 @@ static int stress_netlink_proc(const stress_args_t *args)
 			(void)close(sock);
 			return EXIT_NO_RESOURCE;
 		}
-		pr_err("%s: writev failed: errno=%d (%s)\n",
+		pr_fail("%s: writev failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(sock);
 		return EXIT_FAILURE;
