@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2017-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_CN_PROC_H)
+#include <linux/cn_proc.h>
+#endif
+
+#if defined(HAVE_LINUX_CONNECTOR_H)
+#include <linux/connector.h>
+#endif
+
+#if defined(HAVE_LINUX_TASKSTATS_H)
+#include <linux/taskstats.h>
+#endif
+
+#if defined(HAVE_LINUX_GENETLINK_H)
+#include <linux/genetlink.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"netlink-task N",	"start N workers exercising netlink tasks events" },
@@ -194,7 +206,7 @@ static int stress_netlink_taskstats_monitor(
 		ret = stress_netlink_sendcmd(args, sock, id, pid, TASKSTATS_CMD_GET,
 			TASKSTATS_CMD_ATTR_PID, &pid_data, sizeof(pid_data));
 		if (ret < 0) {
-			pr_err("%s: sendto TASKSTATS_CMD_GET failed: %d (%s)\n",
+			pr_fail("%s: sendto TASKSTATS_CMD_GET failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 			break;
 		}
@@ -262,18 +274,18 @@ static int stress_netlink_task(const stress_args_t *args)
 	ret = stress_netlink_sendcmd(args, sock, GENL_ID_CTRL, pid, CTRL_CMD_GETFAMILY,
 		CTRL_ATTR_FAMILY_NAME, (const void *)name, sizeof(name));
 	if (ret < 0) {
-		pr_err("%s: sendto CTRL_CMD_GETFAMILY failed: %d (%s)\n",
+		pr_fail("%s: sendto CTRL_CMD_GETFAMILY failed: %d (%s)\n",
 			args->name, errno, strerror(errno));
 	}
 	len = recv(sock, &nlmsg, sizeof(nlmsg), 0);
 	if (len < 0) {
-		pr_err("%s: recv failed: %d (%s)\n",
+		pr_fail("%s: recv failed: %d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(sock);
 		return EXIT_FAILURE;
 	}
 	if (!NLMSG_OK((&nlmsg.n), (unsigned int)len)) {
-		pr_err("%s: recv NLMSG error\n",
+		pr_fail("%s: recv NLMSG error\n",
 			args->name);
 		(void)close(sock);
 		return EXIT_FAILURE;
@@ -285,7 +297,7 @@ static int stress_netlink_task(const stress_args_t *args)
 
 		id = *id_ptr;
 	} else {
-		pr_err("%s: failed to get family id\n", args->name);
+		pr_fail("%s: failed to get family id\n", args->name);
 		(void)close(sock);
 		return EXIT_FAILURE;
 	}
@@ -312,6 +324,7 @@ stressor_info_t stress_netlink_task_info = {
 	.stressor = stress_netlink_task,
 	.supported = stress_netlink_task_supported,
 	.class = CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
