@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"nice N",	"start N workers that randomly re-adjust nice levels" },
@@ -35,7 +31,8 @@ static const stress_help_t help[] = {
 static void stress_nice_delay(void)
 {
 	double start = stress_time_now();
-	double delay = 0.01 + (double)stress_mwc16() / 3276800.0;
+	const uint16_t r = stress_mwc16();
+	double delay = 0.01 + (double)r / 3276800.0;
 
 	while (stress_time_now() - start < delay)
 		(void)shim_sched_yield();
@@ -132,7 +129,7 @@ static int stress_nice(const stress_args_t *args)
 				pid = getpid();
 				for (i = min_prio; (i <= max_prio) && keep_stressing(args); i++) {
 					errno = 0;
-					(void)setpriority(PRIO_PROCESS, pid, i);
+					(void)setpriority(PRIO_PROCESS, (id_t)pid, i);
 					if (!errno)
 						stress_nice_delay();
 					inc_counter(args);
