@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"null N",	"start N workers writing to /dev/null" },
 	{ NULL,	"null-ops N",	"stop after N /dev/null bogo write operations" },
@@ -38,7 +37,17 @@ static int stress_null(const stress_args_t *args)
 {
 	int fd;
 	char buffer[4096];
+	int fcntl_mask = 0;
 
+#if defined(O_APPEND)
+	fcntl_mask |= O_APPEND;
+#endif
+#if defined(O_ASYNC)
+	fcntl_mask |= O_ASYNC;
+#endif
+#if defined(O_NONBLOCK)
+	fcntl_mask |= O_NONBLOCK;
+#endif
 	if ((fd = open("/dev/null", O_RDWR)) < 0) {
 		pr_fail("%s: open /dev/null failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -51,8 +60,9 @@ static int stress_null(const stress_args_t *args)
 
 	do {
 		ssize_t ret;
-#if defined(__linux__)
+		int flag;
 		off_t off;
+#if defined(__linux__)
 		void *ptr;
 		const size_t page_size = args->page_size;
 #endif
@@ -70,6 +80,50 @@ static int stress_null(const stress_args_t *args)
 			continue;
 		}
 
+		off = lseek(fd, (off_t)0, SEEK_SET);
+		(void)off;
+		off = lseek(fd, (off_t)0, SEEK_END);
+		(void)off;
+		off = lseek(fd, (off_t)stress_mwc64(), SEEK_CUR);
+		(void)off;
+
+		/* Illegal fallocate, should return ENODEV */
+		ret = shim_fallocate(fd, 0, 0, 4096);
+		(void)ret;
+
+		/* Fdatasync, EINVAL? */
+		ret = shim_fdatasync(fd);
+		(void)ret;
+
+		flag = fcntl(fd, F_GETFL, 0);
+		if (flag >= 0) {
+			const int newflag = O_RDWR | (stress_mwc32() & fcntl_mask);
+
+			ret = fcntl(fd, F_SETFL, newflag);
+			(void)ret;
+			ret = fcntl(fd, F_SETFL, flag);
+			(void)ret;
+		}
+
+#if defined(FIGETBSZ)
+		{
+			int isz;
+
+			ret = ioctl(fd, FIGETBSZ, &isz);
+			(void)ret;
+		}
+#endif
+
+#if defined(FIONREAD)
+		{
+			int isz = 0;
+
+			/* Should return -ENOTTY for /dev/null */
+			ret = ioctl(fd, FIONREAD, &isz);
+			(void)ret;
+		}
+#endif
+
 #if defined(__linux__)
 		off = (off_t)stress_mwc64() & ~((off_t)page_size - 1);
 		ptr = mmap(NULL, page_size, PROT_WRITE,
@@ -93,5 +147,6 @@ static int stress_null(const stress_args_t *args)
 stressor_info_t stress_null_info = {
 	.stressor = stress_null,
 	.class = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
