@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+#include <linux/mempolicy.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"numa N",	"start N workers stressing NUMA interfaces" },
@@ -54,6 +54,9 @@ static const stress_help_t help[] = {
 #if !defined(MPOL_LOCAL)
 #define MPOL_LOCAL		(4)
 #endif
+#if !defined(MPOL_PREFERRED_MANY)
+#define MPOL_PREFERRED_MANY	(5)
+#endif
 
 #if !defined(MPOL_F_NODE)
 #define MPOL_F_NODE		(1 << 0)
@@ -255,6 +258,8 @@ static int stress_numa(const stress_args_t *args)
 		unsigned cpu, curr_node;
 		struct shim_getcpu_cache cache;
 
+		(void)memset(node_mask, 0, sizeof(node_mask));
+
 		/*
 		 *  Fetch memory policy
 		 */
@@ -322,7 +327,7 @@ static int stress_numa(const stress_args_t *args)
 			mode |= MPOL_F_RELATIVE_NODES;
 #endif
 
-		switch (stress_mwc8() % 9) {
+		switch (stress_mwc8() % 11) {
 		case 0:
 #if defined(MPOL_DEFAULT)
 			ret = shim_set_mempolicy(MPOL_DEFAULT | mode, NULL, max_nodes);
@@ -349,15 +354,27 @@ static int stress_numa(const stress_args_t *args)
 			break;
 #endif
 		case 5:
+#if defined(MPOL_PREFERRED_MANY)
+			ret = shim_set_mempolicy(MPOL_PREFERRED_MANY | mode, node_mask, max_nodes);
+			break;
+#endif
+		case 6:
 			ret = shim_set_mempolicy(0, node_mask, max_nodes);
 			break;
-		case 6:
+		case 7:
 			ret = shim_set_mempolicy(mode, node_mask, max_nodes);
 			break;
-		case 7:
+		case 8:
 			/* Invalid mode */
 			ret = shim_set_mempolicy(mode | MPOL_F_STATIC_NODES | MPOL_F_RELATIVE_NODES, node_mask, max_nodes);
 			break;
+#if defined(MPOL_F_NUMA_BALANCING) &&	\
+    defined(MPOL_LOCAL)
+		case 9:
+			/* Invalid  MPOL_F_NUMA_BALANCING | MPOL_LOCAL */
+			ret = shim_set_mempolicy(MPOL_F_NUMA_BALANCING | MPOL_LOCAL, node_mask, max_nodes);
+			break;
+#endif
 		default:
 			/* Intentionally invalid mode */
 			ret = shim_set_mempolicy(~0, node_mask, max_nodes);
@@ -600,6 +617,7 @@ numa_free:
 stressor_info_t stress_numa_info = {
 	.stressor = stress_numa,
 	.class = CLASS_CPU | CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
