@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -110,6 +105,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 		if (pipe(pfd) < 0) {
 			pfd[0] = -1;
 			pfd[1] = -1;
+			break;
 		} else {
 			if (fcntl(pfd[0], F_SETFL, O_NONBLOCK) < 0) {
 				pr_fail("%s: fcntl F_SET_FL O_NONBLOCK failed, errno=%d (%s)\n",
@@ -154,7 +150,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 			(void)fcntl(fd[0], F_SETPIPE_SZ, -1);
 			(void)fcntl(fd[1], F_SETPIPE_SZ, -1);
 		}
-		
+
 		/* Set to minimum size */
 		for (i = 0, fd = fds; keep_stressing(args) && (i < max_pipes); i++, fd += 2) {
 			if ((fd[0] < 0) || (fd[1] < 0))
@@ -209,6 +205,7 @@ static int stress_oom_pipe(const stress_args_t *args)
 stressor_info_t stress_oom_pipe_info = {
 	.stressor = stress_oom_pipe,
 	.class = CLASS_MEMORY | CLASS_OS | CLASS_PATHOLOGICAL,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
