@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_AUDIT_H)
+#include <linux/audit.h>
+#endif
+
+#if defined(HAVE_LINUX_FILTER_H)
+#include <linux/filter.h>
+#endif
+
+#if defined(HAVE_LINUX_SECCOMP_H)
+#include <linux/seccomp.h>
+#endif
+
+#if defined(HAVE_SYS_PRCTL_H)
+#include <sys/prctl.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"opcode N",	   "start N workers exercising random opcodes" },
 	{ NULL,	"opcode-ops N",	   "stop after N opcode bogo operations" },
@@ -372,7 +383,7 @@ again:
 			 * corruption since the child will
 			 * die soon anyhow
 			 */
-			fflush(NULL);
+			(void)fflush(NULL);
 			(void)close(fileno(stdin));
 			(void)close(fileno(stdout));
 			(void)close(fileno(stderr));
