@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_OPENAT2_H)
+#include <linux/openat2.h>
+#endif
+
+#if defined(HAVE_UTIME_H)
+#include <utime.h>
+#endif
+
 typedef int (*stress_open_func_t)(void);
 
 static const stress_help_t help[] = {
@@ -294,7 +297,7 @@ static int open_with_openat_cwd(void)
 		tv[1].tv_sec = 0;
 		(void)obsolete_futimesat(AT_FDCWD, filename, tv);
 
-		(void)unlink(filename);
+		(void)shim_force_unlink(filename);
 	}
 	return fd;
 }
@@ -319,7 +322,7 @@ static int open_with_openat_dir_fd(void)
 	fd = openat(dir_fd, filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
 	if (fd >= 0) {
 		(void)obsolete_futimesat(dir_fd, filename, NULL);
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 	}
 	(void)close(dir_fd);
 	return fd;
@@ -379,15 +382,16 @@ static int open_with_openat2_cwd(void)
 
 		/* Exercise illegal usize field */
 		fd = (int)syscall(__NR_openat2, AT_FDCWD, filename, &how, 0);
+
 		if (fd >= 0) {
 			/* Unxexpected, but handle it anyhow */
-			(void)unlink(filename);
+			(void)shim_unlink(filename);
 			return fd;
 		}
 
 		fd = (int)syscall(__NR_openat2, AT_FDCWD, filename, &how, sizeof(how));
 		if (fd >= 0) {
-			(void)unlink(filename);
+			(void)shim_unlink(filename);
 			return fd;
 		}
 
@@ -508,7 +512,7 @@ static int stress_open(const stress_args_t *args)
 
 	(void)stress_get_setting("open-fd", &open_fd);
 	if (open_fd) {
-		(void)snprintf(path, sizeof(path), "/proc/%d/fd", (int)mypid);
+		(void)snprintf(path, sizeof(path), "/proc/%" PRIdMAX "/fd", (intmax_t)mypid);
 		if ((stat(path, &statbuf) == 0) &&
 		    ((statbuf.st_mode & S_IFMT) == S_IFDIR)) {
 			pid = fork();
