@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2021 Canonical, Ltd.
+ * Copyright (C)      2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -105,7 +100,7 @@ static void stress_pci_info_free(stress_pci_info_t *pci_info_list)
 static stress_pci_info_t *stress_pci_info_get(void)
 {
 	struct dirent **list = NULL;
-	static const char *sys_devices = "/sys/devices";
+	static const char sys_devices[] = "/sys/devices";
 	int i, n;
 	stress_pci_info_t *pci_info_list = NULL;
 
