@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_PERSONALITY_H)
+#include <sys/personality.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"personality N",	"start N workers that change their personality" },
 	{ NULL,	"personality-ops N",	"stop after N bogo personality calls" },
@@ -44,7 +43,7 @@ static const unsigned long personalities[] = {
 static int stress_personality(const stress_args_t *args)
 {
 	const ssize_t n = SIZEOF_ARRAY(personalities);
-	bool failed[n];
+	bool failed[(n > 0) ? n : 1];
 
 	if (n == 0) {
 		pr_inf("%s: no personalities to stress test\n", args->name);
@@ -104,6 +103,7 @@ static int stress_personality(const stress_args_t *args)
 stressor_info_t stress_personality_info = {
 	.stressor = stress_personality,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
