@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"physpage N",	  "start N workers performing physical page lookup" },
@@ -74,7 +70,7 @@ static int stress_virt_to_phys(
 			args->name, (void *)virt_addr, errno, strerror(errno));
 		goto err;
 	} else if (n != (ssize_t)sizeof(pageinfo)) {
-		pr_err("%s: read address %p in /proc/self/pagemap returned %zd bytes, expected %zd\n",
+		pr_fail("%s: read address %p in /proc/self/pagemap returned %zd bytes, expected %zd\n",
 			args->name, (void *)virt_addr, (size_t)n, sizeof(pageinfo));
 		goto err;
 	}
@@ -103,7 +99,7 @@ static int stress_virt_to_phys(
 			goto err;
 		}
 		if (page_count < 1) {
-			pr_err("%s: got zero page count for physical address %p\n",
+			pr_fail("%s: got zero page count for physical address %p\n",
 				args->name, (void *)phys_addr);
 			goto err;
 		}
@@ -113,6 +109,8 @@ static int stress_virt_to_phys(
 		 *  Try to exercise /dev/mem, seek may work, read most probably
 		 *  will fail with -EPERM. Ignore any errors, the aim here is
 		 *  just to try and exercise this interface.
+		 *  Modern systems fail /dev/mem reads with -EPERM because
+		 *  of CONFIG_STRICT_DEVMEM being enabled.
 		 */
 		if (fd_mem >= 0) {
 			off_t offret;
