@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -87,7 +82,7 @@ static int stress_pidfd_supported(const char *name)
 			(void)close(pidfd);
 			return -1;
 		}
-		/* Something went wrong, but let stressor fail on that */
+		/* Something went wrong, but don't let stressor fail on that */
 	}
 
 	/* initialized info to be safe */
@@ -110,12 +105,13 @@ static void stress_pidfd_reap(pid_t pid, int pidfd)
 {
 	int status;
 
-	if (pidfd >= 0)
-		(void)close(pidfd);
 	if (pid) {
 		(void)kill(pid, SIGKILL);
+		(void)shim_process_mrelease(pidfd, 0);
 		(void)shim_waitpid(pid, &status, 0);
 	}
+	if (pidfd >= 0)
+		(void)close(pidfd);
 }
 
 /*
@@ -131,11 +127,14 @@ static int stress_pidfd(const stress_args_t *args)
 	while (keep_stressing(args)) {
 		pid_t pid;
 
+again:
 		pid = fork();
 		if (pid < 0) {
-			if (keep_stressing_flag() && (errno == EAGAIN))
-				continue;
-			pr_fail("%s: fork failed, errno=%d (%s)\n",
+			if (stress_redo_fork(errno))
+				goto again;
+			if (!keep_stressing(args))
+				goto finish;
+			pr_err("%s: fork failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		} else if (pid == 0) {
@@ -204,25 +203,26 @@ static int stress_pidfd(const stress_args_t *args)
 					stress_pidfd_reap(pid, pidfd);
 					return EXIT_NOT_IMPLEMENTED;
 				}
-				pr_err("%s: pidfd_send_signal failed: errno=%d (%s)\n",
+				pr_fail("%s: pidfd_send_signal failed: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				stress_pidfd_reap(pid, pidfd);
 				break;
 			}
 			ret = shim_pidfd_send_signal(pidfd, SIGSTOP, NULL, 0);
 			if (ret != 0) {
-				pr_err("%s: pidfd_send_signal (SIGSTOP), failed: errno=%d (%s)\n",
+				pr_fail("%s: pidfd_send_signal (SIGSTOP), failed: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
 			ret = shim_pidfd_send_signal(pidfd, SIGCONT, NULL, 0);
 			if (ret != 0) {
-				pr_err("%s: pidfd_send_signal (SIGCONT), failed: errno=%d (%s)\n",
+				pr_fail("%s: pidfd_send_signal (SIGCONT), failed: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			}
 			stress_pidfd_reap(pid, pidfd);
 		}
 		inc_counter(args);
 	}
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return EXIT_SUCCESS;
@@ -232,6 +232,7 @@ stressor_info_t stress_pidfd_info = {
 	.stressor = stress_pidfd,
 	.class = CLASS_INTERRUPT | CLASS_OS,
 	.supported = stress_pidfd_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
