@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -186,17 +181,21 @@ static int stress_pipe(const stress_args_t *args)
 		pipe_change_size(args, pipefds[0], pipe_size);
 		pipe_change_size(args, pipefds[1], pipe_size);
 	}
+#else
+	UNEXPECTED
 #endif
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() && (errno == EAGAIN))
+		if (stress_redo_fork(errno))
 			goto again;
 		(void)close(pipefds[0]);
 		(void)close(pipefds[1]);
 		free(buf);
+		if (!keep_stressing(args))
+			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
@@ -284,6 +283,7 @@ again:
 		(void)close(pipefds[1]);
 		free(buf);
 	}
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return EXIT_SUCCESS;
@@ -301,5 +301,6 @@ stressor_info_t stress_pipe_info = {
 	.stressor = stress_pipe,
 	.class = CLASS_PIPE_IO | CLASS_MEMORY | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
