@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -113,6 +108,9 @@ static int stress_pipeherd(const stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
+	for (i = 0; i < PIPE_HERD_MAX; i++)
+		pids[i] = -1;
+
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 #if defined(HAVE_GETRUSAGE) &&	\
@@ -121,7 +119,7 @@ static int stress_pipeherd(const stress_args_t *args)
     defined(HAVE_RUSAGE_RU_NVCSW)
 	t1 = stress_time_now();
 #endif
-	for (i = 0; i < PIPE_HERD_MAX; i++) {
+	for (i = 0; keep_stressing(args) && (i < PIPE_HERD_MAX); i++) {
 		pid_t pid;
 
 		pid = fork();
