@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ "P N", "poll N",	"start N workers exercising zero timeout polling" },
 	{ NULL,	"poll-ops N",	"stop after N poll bogo operations" },
@@ -141,8 +144,10 @@ static int stress_poll(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() && (errno == EAGAIN))
+		if (stress_redo_fork(errno))
 			goto again;
+		if (!keep_stressing(args))
+			goto tidy;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
@@ -285,6 +290,8 @@ abort:
 #endif
 
 #endif
+
+#if defined(HAVE_SYS_SELECT_H)
 			/* stress out select */
 			FD_ZERO(&rfds);
 			for (i = 0; i < max_fds; i++) {
@@ -314,6 +321,8 @@ abort:
 			}
 			if (!keep_stressing(args))
 				break;
+#endif
+
 #if defined(HAVE_PSELECT)
 			/* stress out pselect */
 			ts.tv_sec = 0;
@@ -377,6 +386,7 @@ stressor_info_t stress_poll_info = {
 	.stressor = stress_poll,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
