@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+
+#if defined(HAVE_ASM_PRCTL_H)
+#include <asm/prctl.h>
+#endif
+
+#if defined(HAVE_LINUX_SECCOMP_H)
+#include <linux/seccomp.h>
+#endif
+
+#if defined(HAVE_SYS_PRCTL_H)
+#include <sys/prctl.h>
+#endif
+
+#if defined(HAVE_SYS_CAPABILITY_H)
+#include <sys/capability.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"procfs N",	"start N workers reading portions of /proc" },
@@ -30,6 +42,21 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
+/*
+ *  scheduling scopes, added in to Linux on commit
+ *  61bc346ce64a3864ac55f5d18bdc1572cda4fb18
+ *  "uapi/linux/prctl: provide macro definitions for the PR_SCHED_CORE type argument"
+ */
+#if !defined(PR_SCHED_CORE_SCOPE_THREAD)
+#define PR_SCHED_CORE_SCOPE_THREAD		(0)
+#endif
+#if !defined(PR_SCHED_CORE_SCOPE_THREAD_GROUP)
+#define PR_SCHED_CORE_SCOPE_THREAD_GROUP	(1)
+#endif
+#if !defined(PR_SCHED_CORE_SCOPE_PROCESS_GROUP)
+#define PR_SCHED_CORE_SCOPE_PROCESS_GROUP	(2)
+#endif
+
 #if defined(HAVE_SYS_PRCTL_H) &&		\
     defined(HAVE_PRCTL) &&			\
     (defined(PR_SET_PDEATHSIG) ||		/* 1 */ \
@@ -97,7 +124,7 @@ static const stress_help_t help[] = {
 
 /*
  *  getauxv_addr()
- *	fine the address of the auxv vector. This
+ *	find the address of the auxv vector. This
  *	is located at the end of the environment.
  *	Returns NULL if not found.
  */
@@ -169,6 +196,47 @@ static inline void stress_arch_prctl(void)
 #endif
 	}
 #endif
+
+#if defined(HAVE_ASM_PRCTL_H) &&		\
+    defined(HAVE_SYS_PRCTL_H) &&		\
+    defined(ARCH_GET_XCOMP_SUPP) &&		\
+    (defined(__x86_64__) || defined(__x86_64))
+	{
+		uint64_t features;
+		int ret;
+
+		ret = shim_arch_prctl(ARCH_GET_XCOMP_SUPP, (unsigned long)&features);
+		(void)ret;
+	}
+#endif
+#if defined(HAVE_ASM_PRCTL_H) &&		\
+    defined(HAVE_SYS_PRCTL_H) &&		\
+    defined(ARCH_GET_XCOMP_PERM) &&		\
+    (defined(__x86_64__) || defined(__x86_64))
+	{
+		uint64_t features;
+		int ret;
+
+		ret = shim_arch_prctl(ARCH_GET_XCOMP_PERM, (unsigned long)&features);
+		(void)ret;
+	}
+#endif
+#if defined(HAVE_ASM_PRCTL_H) &&		\
+    defined(HAVE_SYS_PRCTL_H) &&		\
+    defined(ARCH_REQ_XCOMP_PERM) &&		\
+    (defined(__x86_64__) || defined(__x86_64))
+	{
+		unsigned long idx;
+		int ret;
+
+		for (idx = 0; idx < 255; idx++) {
+			errno = 0;
+			ret = shim_arch_prctl(ARCH_REQ_XCOMP_PERM, idx);
+			if ((ret < 0) && (errno == -EINVAL))
+				break;
+		}
+	}
+#endif
 }
 
 #if defined(PR_SET_SYSCALL_USER_DISPATCH) &&	\
@@ -240,7 +308,7 @@ static int stress_prctl_syscall_user_dispatch(const stress_args_t *args)
 		/* EINVAL will occur on kernels where this is not supported */
 		if ((errno == EINVAL) || (errno == ENOSYS))
 			return 0;
-		pr_err("%s: prctl PR_SET_SYSCALL_USER_DISPATCH enable failed, errno=%d (%s)\n",
+		pr_fail("%s: prctl PR_SET_SYSCALL_USER_DISPATCH enable failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto err;
 	}
@@ -253,19 +321,19 @@ static int stress_prctl_syscall_user_dispatch(const stress_args_t *args)
 	ret = prctl(PR_SET_SYSCALL_USER_DISPATCH,
 		PR_SYS_DISPATCH_OFF, 0, 0, 0);
 	if (ret < 0) {
-		pr_err("%s: prctl PR_SET_SYSCALL_USER_DISPATCH disable failed, errno=%d (%s)\n",
+		pr_fail("%s: prctl PR_SET_SYSCALL_USER_DISPATCH disable failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto err;
 	}
 
 	if (!prctl_sigsys_info.handled) {
-		pr_err("%s: prctl PR_SET_SYSCALL_USER_DISPATCH syscall emulation failed\n",
+		pr_fail("%s: prctl PR_SET_SYSCALL_USER_DISPATCH syscall emulation failed\n",
 			args->name);
 		goto err;
 	}
 
 	if (prctl_sigsys_info.syscall != __NR_kill) {
-		pr_err("%s: prctl PR_SET_SYSCALL_USER_DISPATCH syscall expected syscall 0x%x, got 0x%x instead\n",
+		pr_fail("%s: prctl PR_SET_SYSCALL_USER_DISPATCH syscall expected syscall 0x%x, got 0x%x instead\n",
 			args->name, __NR_kill, prctl_sigsys_info.syscall);
 		goto err;
 	}
@@ -748,16 +816,67 @@ static int stress_prctl_child(const stress_args_t *args, const pid_t mypid)
 
 #if defined(PR_GET_SPECULATION_CTRL)
 	{
+		int val;
+
 		/* exercise invalid args */
 		ret = prctl(PR_GET_SPECULATION_CTRL, ~0, ~0, ~0, ~0);
 		(void)ret;
+
 #if defined(PR_SPEC_STORE_BYPASS)
-		ret = prctl(PR_GET_SPECULATION_CTRL, PR_SPEC_STORE_BYPASS, 0, 0, 0);
-		(void)ret;
+		val = prctl(PR_GET_SPECULATION_CTRL, PR_SPEC_STORE_BYPASS, 0, 0, 0);
+		(void)val;
+
+		if (val & PR_SPEC_PRCTL) {
+			val &= ~PR_SPEC_PRCTL;
+#if defined(PR_SPEC_ENABLE)
+
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_STORE_BYPASS, PR_SPEC_ENABLE, 0, 0);
+			(void)ret;
 #endif
+#if defined(PR_SPEC_DISABLE)
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_STORE_BYPASS, PR_SPEC_DISABLE, 0, 0);
+			(void)ret;
+#endif
+			/* ..and restore */
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_STORE_BYPASS, val, 0, 0);
+			(void)ret;
+#endif
+		}
+
 #if defined(PR_SPEC_INDIRECT_BRANCH)
-		ret = prctl(PR_GET_SPECULATION_CTRL, PR_SPEC_INDIRECT_BRANCH, 0, 0, 0);
-		(void)ret;
+		val = prctl(PR_GET_SPECULATION_CTRL, PR_SPEC_INDIRECT_BRANCH, 0, 0, 0);
+		if (val & PR_SPEC_PRCTL) {
+			val &= ~PR_SPEC_PRCTL;
+#if defined(PR_SPEC_ENABLE)
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_INDIRECT_BRANCH, PR_SPEC_ENABLE, 0, 0);
+			(void)ret;
+#endif
+#if defined(PR_SPEC_DISABLE)
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_INDIRECT_BRANCH, PR_SPEC_DISABLE, 0, 0);
+			(void)ret;
+#endif
+			/* ..and restore */
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_INDIRECT_BRANCH, val, 0, 0);
+			(void)ret;
+		}
+#endif
+
+#if defined(PR_SPEC_L1D_FLUSH)
+		val = prctl(PR_GET_SPECULATION_CTRL, PR_SPEC_L1D_FLUSH, 0, 0, 0);
+		if (val & PR_SPEC_PRCTL) {
+			val &= ~PR_SPEC_PRCTL;
+#if defined(PR_SPEC_ENABLE)
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_L1D_FLUSH, PR_SPEC_ENABLE, 0, 0);
+			(void)ret;
+#endif
+#if defined(PR_SPEC_DISABLE)
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_L1D_FLUSH, PR_SPEC_DISABLE, 0, 0);
+			(void)ret;
+#endif
+			/* ..and restore */
+			ret = prctl(PR_SET_SPECULATION_CTRL, PR_SPEC_L1D_FLUSH, val, 0, 0);
+			(void)ret;
+		}
 #endif
 	}
 #endif
@@ -788,11 +907,28 @@ static int stress_prctl_child(const stress_args_t *args, const pid_t mypid)
 		unsigned long cookie = 0;
 		const pid_t bad_pid = stress_get_unused_pid_racy(false);
 
-		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, 0, SHIM_PIDTYPE_PID, &cookie);
+		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, 0,
+				PR_SCHED_CORE_SCOPE_THREAD, &cookie);
 		(void)ret;
-		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, getpid(), SHIM_PIDTYPE_PID, &cookie);
+
+		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, getpid(),
+				PR_SCHED_CORE_SCOPE_THREAD, &cookie);
 		(void)ret;
-		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, bad_pid, SHIM_PIDTYPE_PID, &cookie);
+
+		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, bad_pid,
+				PR_SCHED_CORE_SCOPE_THREAD, &cookie);
+		(void)ret;
+
+		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, 0,
+				PR_SCHED_CORE_SCOPE_THREAD_GROUP, &cookie);
+		(void)ret;
+
+		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, 0,
+				PR_SCHED_CORE_SCOPE_PROCESS_GROUP, &cookie);
+		(void)ret;
+
+		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_GET, getgid(),
+				PR_SCHED_CORE_SCOPE_PROCESS_GROUP, &cookie);
 		(void)ret;
 	}
 #endif
@@ -803,7 +939,9 @@ static int stress_prctl_child(const stress_args_t *args, const pid_t mypid)
 	{
 		const pid_t ppid = getppid();
 
-		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_CREATE, ppid, SHIM_PIDTYPE_PID, 0);
+		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_CREATE, ppid,
+				PR_SCHED_CORE_SCOPE_THREAD, 0);
+		(void)ret;
 	}
 #endif
 
@@ -813,7 +951,9 @@ static int stress_prctl_child(const stress_args_t *args, const pid_t mypid)
 	{
 		const pid_t ppid = getppid();
 
-		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_CREATE, ppid, SHIM_PIDTYPE_PID, 0);
+		ret = prctl(PR_SCHED_CORE, PR_SCHED_CORE_CREATE, ppid,
+				PR_SCHED_CORE_SCOPE_THREAD, 0);
+		(void)ret;
 	}
 #endif
 
@@ -851,9 +991,13 @@ static int stress_prctl(const stress_args_t *args)
 
 	do {
 		pid_t pid;
-
+again:
 		pid = fork();
 		if (pid == -1) {
+			if (stress_redo_fork(errno))
+				goto again;
+			if (!keep_stressing(args))
+				goto finish;
 			pr_fail("%s: fork failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			break;
@@ -888,6 +1032,7 @@ static int stress_prctl(const stress_args_t *args)
 		inc_counter(args);
 	} while (keep_stressing(args));
 
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return EXIT_SUCCESS;
