@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-put.h"
+
+#if defined(HAVE_ASM_MTRR_H)
+#include <asm/mtrr.h>
+#endif
+
+#if defined(HAVE_LINUX_PCI_H)
+#include <linux/pci.h>
+#endif
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"procfs N",	"start N workers reading portions of /proc" },
@@ -164,7 +172,6 @@ static inline void stress_proc_rw(
 		bool writeable = true;
 		size_t len;
 		ssize_t i;
-		off_t dec;
 		bool skip_fifo = true;
 
 		ret = shim_pthread_spin_lock(&lock);
@@ -333,21 +340,29 @@ static inline void stress_proc_rw(
 		}
 
 		/*
-		 *  exercise 16 one byte reads backwards through procfs file
+		 *  exercise 13 x 5 byte reads backwards through procfs file to
+		 *  ensure we perform some weird misaligned non-word sized reads
 		 */
 		if (!skip_fifo) {
+			off_t dec;
+
 			pos = lseek(fd, 0, SEEK_END);
 			if (pos < 0)
 				goto mmap_test;
-			dec = pos / 16;
+			dec = pos / 13;
 			if (dec < 1)
 				dec = 1;
-			while (--pos > 0) {
-				pos = lseek(fd, pos, SEEK_SET);
-				if (pos < 0)
-				break;
-				ret = read(fd, buffer, 1);
+			while (pos > 0) {
+				off_t seek_pos;
+
+				seek_pos = lseek(fd, pos, SEEK_SET);
+				if (seek_pos < 0)
+					break;
+				ret = read(fd, buffer, 5);
 				(void)ret;
+
+				if (dec > pos)
+					dec = pos;
 				pos -= dec;
 			}
 		}
@@ -658,7 +673,7 @@ static int stress_procfs(const stress_args_t *args)
 	n = scandir("/proc", &dlist, NULL, alphasort);
 	if (n <= 0) {
 		if (args->instance == 0)
-			pr_inf_skip("%s: no /sys entries found, skipping stressor\n", args->name);
+			pr_inf_skip("%s: no /proc entries found, skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
 	n = stress_dirent_proc_prune(dlist, n);
@@ -723,7 +738,7 @@ static int stress_procfs(const stress_args_t *args)
 
 	rc = shim_pthread_spin_lock(&lock);
 	if (rc) {
-		pr_dbg("%s: failed to lock spin lock for sysfs_path\n", args->name);
+		pr_dbg("%s: spin lock failed for %s\n", args->name, proc_path);
 	} else {
 		shim_strlcpy(proc_path, "", sizeof(proc_path));
 		rc = shim_pthread_spin_unlock(&lock);
