@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,37 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+
+#if defined(HAVE_MODIFY_LDT)
+#include <asm/ldt.h>
+#endif
+
+#if defined(HAVE_LINUX_FUTEX_H)
+#include <linux/futex.h>
+#endif
+
+#if defined(HAVE_SYS_PRCTL_H)
+#include <sys/prctl.h>
+#endif
+
+#define MIN_PTHREAD		(1)
+#define MAX_PTHREAD		(30000)
+#define DEFAULT_PTHREAD		(1024)
+
+#if defined(__NR_get_thread_area)
+#define HAVE_GET_THREAD_AREA
+#endif
+
+#if defined(__NR_get_robust_list)
+#define HAVE_GET_ROBUST_LIST
+#endif
+
+#if defined(__NR_set_robust_list)
+#define HAVE_SET_ROBUST_LIST
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"pthread N",	 "start N workers that create multiple threads" },
@@ -37,6 +61,7 @@ static const stress_help_t help[] = {
 typedef struct {
 	pthread_t pthread;	/* The pthread */
 	int	  ret;		/* pthread create return */
+	uint64_t index;		/* which pthread */
 #if defined(HAVE_PTHREAD_ATTR_SETSTACK)
 	void 	 *stack;	/* pthread stack */
 #endif
@@ -179,7 +204,13 @@ static void *stress_pthread_func(void *parg)
 	struct robust_list_head *head;
 	size_t len;
 #endif
-	const stress_args_t *args = ((stress_pthread_args_t *)parg)->args;
+	const stress_pthread_args_t *spa = (stress_pthread_args_t *)parg;
+	const stress_args_t *args = spa->args;
+	const stress_pthread_info_t *pthread_info = (stress_pthread_info_t *)spa->data;
+	char str[16];
+
+	snprintf(str, sizeof(str), "%" PRIu64, pthread_info->index);
+	stress_set_proc_state_str(args->name, str);
 
 #if defined(HAVE_GET_ROBUST_LIST) &&	\
     defined(HAVE_LINUX_FUTEX_H)
@@ -401,6 +432,14 @@ static int stress_pthread(const stress_args_t *args)
 #if defined(HAVE_PTHREAD_ATTR_SETSTACK)
 	const size_t stack_size = STRESS_MAXIMUM(DEFAULT_STACK_MIN, stress_min_pthread_stack_size());
 #endif
+#if defined(HAVE_PTHREAD_MUTEXATTR_T) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_INIT) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_DESTROY) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_SETPRIOCEILING) &&	\
+    defined(HAVE_PTHREAD_MUTEXATTR_SETPROTOCOL)
+	pthread_mutexattr_t mutex_attr;
+	bool mutex_attr_init;
+#endif
 
 	keep_running_flag = true;
 
@@ -432,6 +471,26 @@ static int stress_pthread(const stress_args_t *args)
 			args->name, ret, strerror(ret));
 		return EXIT_FAILURE;
 	}
+#if defined(HAVE_PTHREAD_MUTEXATTR_T) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_INIT) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_DESTROY) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_SETPRIOCEILING) &&	\
+    defined(HAVE_PTHREAD_MUTEXATTR_SETPROTOCOL)
+	/*
+	 *  Attempt to use priority inheritance on mutex
+	 */
+	ret = pthread_mutexattr_init(&mutex_attr);
+	if (ret == 0) {
+		mutex_attr_init = true;
+		ret = pthread_mutexattr_setprotocol(&mutex_attr, PTHREAD_PRIO_INHERIT);
+		(void)ret;
+		ret = pthread_mutexattr_setprioceiling(&mutex_attr, 127);
+		(void)ret;
+	} else {
+		mutex_attr_init = false;
+	}
+#endif
+
 	ret = pthread_mutex_init(&mutex, NULL);
 	if (ret) {
 		pr_fail("%s: pthread_mutex_init failed, errno=%d (%s)\n",
@@ -453,8 +512,10 @@ static int stress_pthread(const stress_args_t *args)
 			stop_running();
 			continue;
 		}
-		for (i = 0; i < pthread_max; i++)
+		for (i = 0; i < pthread_max; i++) {
 			pthreads[i].ret = -1;
+			pthreads[i].index = i;
+		}
 
 		for (i = 0; i < pthread_max; i++) {
 #if defined(HAVE_PTHREAD_ATTR_SETSTACK)
@@ -464,19 +525,15 @@ static int stress_pthread(const stress_args_t *args)
 
 #if defined(HAVE_PTHREAD_ATTR_SETSTACK)
 			/*
-			 *  We mmap our own per pthread stack to ensure we
+			 *  We allocate our own per pthread stack to ensure we
 			 *  have one available before the pthread is started
 			 *  and this allows us to exercise the pthread stack
 			 *  setting.
 			 */
-			pthreads[i].stack = mmap(NULL, stack_size, PROT_READ | PROT_WRITE,
-				MAP_SHARED | MAP_ANONYMOUS, -1, 0);
-			if (pthreads[i].stack == MAP_FAILED)
+			pthreads[i].stack = calloc(1, stack_size);
+			if (!pthreads[i].stack)
 				break;
 
-			/* Ensure stack is all in physical memory */
-			stress_mincore_touch_pages(pthreads[i].stack, stack_size);
-
 			ret = pthread_attr_init(&attr);
 			if (ret) {
 				pr_fail("%s: pthread_attr_init failed, errno=%d (%s)\n",
@@ -593,8 +650,8 @@ reap:
 				}
 			}
 #if defined(HAVE_PTHREAD_ATTR_SETSTACK)
-			if (pthreads[j].stack != MAP_FAILED)
-				(void)munmap(pthreads[j].stack, stack_size);
+			if (pthreads[j].stack != NULL)
+				free(pthreads[j].stack);
 #endif
 		}
 	} while (!locked && keep_running() && keep_stressing(args));
@@ -612,6 +669,14 @@ reap:
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+#if defined(HAVE_PTHREAD_MUTEXATTR_T) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_INIT) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_DESTROY) &&		\
+    defined(HAVE_PTHREAD_MUTEXATTR_SETPRIOCEILING) &&	\
+    defined(HAVE_PTHREAD_MUTEXATTR_SETPROTOCOL)
+	if (mutex_attr_init)
+		(void)pthread_mutexattr_destroy(&mutex_attr);
+#endif
 	(void)pthread_cond_destroy(&cond);
 	(void)pthread_mutex_destroy(&mutex);
 	(void)shim_pthread_spin_destroy(&spinlock);
@@ -623,6 +688,7 @@ stressor_info_t stress_pthread_info = {
 	.stressor = stress_pthread,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
