@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_PTRACE)
+#include <sys/ptrace.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"ptrace N",	"start N workers that trace a child using ptrace" },
 	{ NULL,	"ptrace-ops N",	"stop ptrace workers after N system calls are traced" },
@@ -221,6 +220,7 @@ finish:
 stressor_info_t stress_ptrace_info = {
 	.stressor = stress_ptrace,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
