@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_TERMIO_H)
+#include <termio.h>
+#endif
+
+#define MIN_PTYS		(8)
+#define MAX_PTYS		(65536)
+#define DEFAULT_PTYS		(65536)
+
 static const stress_help_t help[] = {
 	{ NULL,	"pty N",	"start N workers that exercise pseudoterminals" },
 	{ NULL,	"pty-ops N",	"stop pty workers after N pty bogo operations" },
@@ -377,6 +380,7 @@ stressor_info_t stress_pty_info = {
 	.stressor = stress_pty,
 	.class = CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
