@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_SYS_MOUNT_H)
+#include <sys/mount.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"ramfs N",	 "start N workers exercising ramfs mounts" },
@@ -193,12 +193,12 @@ static int stress_ramfs_fs_ops(const stress_args_t *args, const char *pathname)
 				args->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
 		}
-		if (unlink(symlinkname) < 0) {
+		if (shim_unlink(symlinkname) < 0) {
 			pr_fail("%s: cannot unlink symbolic file on ram based file system, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
 		}
-		if (unlink(filename) < 0) {
+		if (shim_unlink(filename) < 0) {
 			pr_fail("%s: cannot unlink file on ram based file system, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
@@ -216,7 +216,7 @@ static int stress_ramfs_fs_ops(const stress_args_t *args, const char *pathname)
 				args->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
 		}
-		if (rmdir(filename) < 0) {
+		if (shim_rmdir(filename) < 0) {
 			pr_fail("%s: cannot remove directory on ram based file system, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			rc = EXIT_FAILURE;
@@ -452,6 +452,7 @@ stressor_info_t stress_ramfs_info = {
 	.class = CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
 	.supported = stress_ramfs_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
