@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,40 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+#include "core-net.h"
+
+#if defined(HAVE_LINUX_IF_PACKET_H)
+#include <linux/if_packet.h>
+#endif
+
+#if defined(HAVE_LINUX_IF_TUN_H)
+#include <linux/if_tun.h>
+#endif
+
+#if defined(HAVE_LINUX_SOCKIOS_H)
+#include <linux/sockios.h>
+#endif
+
+#if defined(HAVE_LINUX_UDP_H)
+#include <linux/udp.h>
+#endif
+
+#if defined(HAVE_NET_IF_H)
+#include <net/if.h>
+#endif
+
+#if defined(HAVE_NETINET_IP_H)
+#include <netinet/ip.h>
+#endif
+
+#include <arpa/inet.h>
+
+#define MIN_RAWPKT_PORT		(1024)
+#define MAX_RAWPKT_PORT		(65535)
+#define DEFAULT_RAWPKT_PORT	(14000)
 
 #if !defined(SOL_UDP)
 #define SOL_UDP 	(17)
@@ -255,7 +282,7 @@ static void NORETURN stress_rawpkt_client(
 
 		n = sendto(fd, buf, sizeof(struct ethhdr) + ip->tot_len, 0, (struct sockaddr *)&sadr, sizeof(sadr));
 		if (n < 0) {
-			pr_err("%s: raw socket sendto failed on port %d, errno=%d (%s)\n",
+			pr_fail("%s: raw socket sendto failed on port %d, errno=%d (%s)\n",
 				args->name, port, errno, strerror(errno));
 		}
 #if defined(SIOCOUTQ)
@@ -435,6 +462,7 @@ stressor_info_t stress_rawpkt_info = {
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
 	.supported = stress_rawpkt_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
