@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+#include "core-net.h"
+
+#if defined(HAVE_LINUX_UDP_H)
+#include <linux/udp.h>
+#endif
+
+#if defined(HAVE_NETINET_IP_H)
+#include <netinet/ip.h>
+#endif
+
+#include <arpa/inet.h>
+
+#define MIN_RAWUDP_PORT		(1024)
+#define MAX_RAWUDP_PORT		(65535)
+#define DEFAULT_RAWUDP_PORT	(13000)
 
 #if !defined(SOL_UDP)
 #define SOL_UDP 	(17)
@@ -137,7 +148,7 @@ static void NORETURN stress_rawudp_client(
 
 		n = sendto(fd, buf, ip->tot_len, 0, (struct sockaddr *)&s_in, sizeof(s_in));
 		if (n < 0) {
-			pr_err("%s: raw socket sendto failed on port %d, errno=%d (%s)\n",
+			pr_fail("%s: raw socket sendto failed on port %d, errno=%d (%s)\n",
 				args->name, port, errno, strerror(errno));
 		}
 		(void)close(fd);
