@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"reboot N",	"start N workers that exercise bad reboot calls" },
@@ -70,7 +66,8 @@ static int reboot_clone_func(void *arg)
 		int ret;
 
 		errno = 0;
-		ret = shim_reboot(SHIM_LINUX_BOOT_MAGIC1, boot_magic[j],
+		ret = shim_reboot((int)SHIM_LINUX_BOOT_MAGIC1,
+			(int)boot_magic[j],
 			SHIM_LINUX_REBOOT_CMD_POWER_OFF, NULL);
 		j = (j + 1) % SIZEOF_ARRAY(boot_magic);
 		(void)ret;
@@ -89,9 +86,6 @@ static int stress_reboot(const stress_args_t *args)
 {
 	const bool reboot_capable = stress_check_capability(SHIM_CAP_SYS_BOOT);
 #if defined(HAVE_CLONE)
-	const ssize_t stack_offset =
-		stress_get_stack_direction() *
-		(CLONE_STACK_SIZE - 64);
 	char *stack;
 
 	stack = malloc(CLONE_STACK_SIZE);
@@ -106,7 +100,7 @@ static int stress_reboot(const stress_args_t *args)
 	do {
 		int ret;
 #if defined(HAVE_CLONE)
-		char *stack_top = stack + stack_offset;
+		char *stack_top = (char *)stress_get_stack_top((void *)stack, CLONE_STACK_SIZE);
 		pid_t pid;
 
 		pid = clone(reboot_clone_func, stress_align_stack(stack_top),
@@ -116,7 +110,7 @@ static int stress_reboot(const stress_args_t *args)
 
 			(void)stress_mwc8();
 
-			(void)shim_waitpid(pid, &status, __WCLONE);
+			(void)shim_waitpid(pid, &status, (int)__WCLONE);
 			ret = WEXITSTATUS(status);
 			if (WIFEXITED(status) && (ret != 0)) {
 				pr_fail("%s: reboot in PID namespace failed, errno = %d (%s)\n",
@@ -144,7 +138,8 @@ static int stress_reboot(const stress_args_t *args)
 
 			for (i = 0; i < SIZEOF_ARRAY(boot_magic); i++) {
 				errno = 0;
-				ret = shim_reboot(SHIM_LINUX_BOOT_MAGIC1, boot_magic[i],
+				ret = shim_reboot((int)SHIM_LINUX_BOOT_MAGIC1,
+					(int)boot_magic[i],
 					SHIM_LINUX_REBOOT_CMD_POWER_OFF, NULL);
 				if (errno == EINVAL)
 					continue;
@@ -170,6 +165,7 @@ static int stress_reboot(const stress_args_t *args)
 stressor_info_t stress_reboot_info = {
 	.stressor = stress_reboot,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 
