@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"remap N",	"start N workers exercising page remappings" },
@@ -30,7 +26,8 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-#if defined(HAVE_REMAP_FILE_PAGES)
+#if defined(HAVE_REMAP_FILE_PAGES) &&	\
+    !defined(STRESS_ARCH_SPARC)
 
 #define N_PAGES		(512)
 
@@ -211,19 +208,30 @@ static int stress_remap(const stress_args_t *args)
 		if (unmapped) {
 			ret = remap_file_pages((void *)unmapped, page_size, 0, 0, 0);
 			(void)ret;
+
 			/* Illegal flags */
 			ret = remap_file_pages((void *)unmapped, page_size, 0, 0, ~0);
 			(void)ret;
-			/* Locked page */
+
+			/* Invalid prot */
+			ret = remap_file_pages((void *)unmapped, page_size, ~0, order[0], 0);
+			(void)ret;
 		}
 		if (mapped) {
 			ret = remap_file_pages((void *)(mapped + page_size), page_size, 0, 0, 0);
 			(void)ret;
+
 			/* Illegal flags */
 			ret = remap_file_pages((void *)(mapped + page_size), page_size, 0, 0, ~0);
 			(void)ret;
+
+			/* Invalid prot */
+			ret = remap_file_pages((void *)(mapped + page_size), page_size, ~0, order[0], 0);
+			(void)ret;
 		}
 
+
+
 		inc_counter(args);
 	} while (keep_stressing(args));
 
@@ -241,6 +249,7 @@ static int stress_remap(const stress_args_t *args)
 stressor_info_t stress_remap_info = {
 	.stressor = stress_remap,
 	.class = CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
