@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -108,7 +103,7 @@ static int exercise_renameat2(
 	int ret, file_fd;
 
 	/* Exercise with invalid flags */
-	ret = renameat2(oldfd, old_name, newfd, new_name, ~0);
+	ret = renameat2(oldfd, old_name, newfd, new_name, (unsigned int)~0);
 	if (ret >= 0) {
 		pr_fail("%s: reanameat2 with illegal flags "
 			"unexpectedly succeeded\n",
@@ -277,8 +272,8 @@ restart:
 		(void)stress_temp_filename(newname, PATH_MAX,
 			args->name, args->pid, inst2, i++);
 		if (rename(oldname, newname) < 0) {
-			(void)unlink(oldname);
-			(void)unlink(newname);
+			(void)shim_unlink(oldname);
+			(void)shim_unlink(newname);
 			goto restart;
 		}
 
@@ -292,8 +287,8 @@ restart:
 		(void)stress_temp_filename(newname, PATH_MAX,
 			args->name, args->pid, inst1, i++);
 		if (rename(oldname, newname) < 0) {
-			(void)unlink(oldname);
-			(void)unlink(newname);
+			(void)shim_unlink(oldname);
+			(void)shim_unlink(newname);
 			goto restart;
 		}
 
@@ -316,14 +311,14 @@ restart:
 			new = stress_basename(newname);
 
 			if (exercise_renameat(args, old, tmp_fd, new, tmp_fd, bad_fd) < 0) {
-				(void)unlink(oldname);
-				(void)unlink(newname);
+				(void)shim_unlink(oldname);
+				(void)shim_unlink(newname);
 				goto restart;
 			}
 
 			if (renameat(tmp_fd, old, tmp_fd, new) < 0) {
-				(void)unlink(oldname);
-				(void)unlink(newname);
+				(void)shim_unlink(oldname);
+				(void)shim_unlink(newname);
 				goto restart;
 			}
 			(void)shim_fsync(tmp_fd);
@@ -349,14 +344,14 @@ restart:
 			new = stress_basename(newname);
 
 			if (exercise_renameat2(args, old, tmp_fd, new, tmp_fd, bad_fd) < 0) {
-				(void)unlink(oldname);
-				(void)unlink(newname);
+				(void)shim_unlink(oldname);
+				(void)shim_unlink(newname);
 				goto restart;
 			}
 
 			if (renameat2(tmp_fd, old, tmp_fd, new, RENAME_NOREPLACE) < 0) {
-				(void)unlink(oldname);
-				(void)unlink(newname);
+				(void)shim_unlink(oldname);
+				(void)shim_unlink(newname);
 				goto restart;
 			}
 			tmpname = oldname;
@@ -378,9 +373,9 @@ restart:
 		(void)close(tmp_fd);
 #endif
 	if (*oldname)
-		(void)unlink(oldname);
+		(void)shim_unlink(oldname);
 	if (*newname)
-		(void)unlink(newname);
+		(void)shim_unlink(newname);
 	(void)stress_temp_dir_rm(args->name, args->pid, inst1);
 	(void)stress_temp_dir_rm(args->name, args->pid, inst2);
 
@@ -390,5 +385,6 @@ restart:
 stressor_info_t stress_rename_info = {
 	.stressor = stress_rename,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
