@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,45 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(__NR_userfaultfd)
+#define HAVE_USERFAULTFD
+#endif
+
+#if defined(HAVE_SYS_EVENTFD_H)
+#include <sys/eventfd.h>
+#endif
+
+#if defined(HAVE_SYS_INOTIFY_H)
+#include <sys/inotify.h>
+#endif
+
+#if defined(HAVE_SYS_IPC_H)
+#include <sys/ipc.h>
+#endif
+
+#if defined(HAVE_SYS_MSG_H)
+#include <sys/msg.h>
+#endif
+
+#if defined(HAVE_SEM_SYSV)
+#include <sys/sem.h>
+#endif
+
+#if defined(HAVE_SYS_TIMERFD_H)
+#include <sys/timerfd.h>
+#endif
+
+#if defined(HAVE_MQUEUE_H)
+#include <mqueue.h>
+#endif
+
+#if defined(HAVE_SEMAPHORE_H)
+#include <semaphore.h>
+#endif
+
 #define RESOURCE_FORKS 	(1024)
 #define MAX_LOOPS	(1024)
 
@@ -70,8 +101,8 @@ typedef struct {
 	int wd_inotify;
 #endif
 #if defined(HAVE_PTSNAME)
-	int pty_master;
-	int pty_slave;
+	int pty_mtx;
+	int pty;
 #endif
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_TIMER_CREATE) &&	\
@@ -166,6 +197,7 @@ static void NORETURN waste_resources(
 		}
 	}
 #else
+	UNEXPECTED
 	mlock_size = page_size * MAX_LOOPS;
 #endif
 
@@ -236,8 +268,8 @@ static void NORETURN waste_resources(
 		info[i].fd_inotify = -1;
 #endif
 #if defined(HAVE_PTSNAME)
-		info[i].pty_slave = -1;
-		info[i].pty_master = -1;
+		info[i].pty = -1;
+		info[i].pty_mtx = -1;
 #endif
 #if defined(HAVE_LIB_PTHREAD) &&	\
     defined(HAVE_SEM_POSIX)
@@ -325,6 +357,7 @@ static void NORETURN waste_resources(
 			(void)fcntl(info[i].fd_pipe[1], F_SETPIPE_SZ, pipe_size);
 		}
 #else
+		UNEXPECTED
 		(void)pipe_size;
 #endif
 		if (!keep_stressing_flag())
@@ -336,6 +369,8 @@ static void NORETURN waste_resources(
 		info[i].fd_ev = eventfd(0, 0);
 		if (!keep_stressing_flag())
 			break;
+#else
+		UNEXPECTED
 #endif
 #if defined(HAVE_MEMFD_CREATE)
 		(void)snprintf(name, sizeof(name), "memfd-%" PRIdMAX "-%zu",
@@ -352,6 +387,8 @@ static void NORETURN waste_resources(
 		}
 		if (!keep_stressing_flag())
 			break;
+#else
+		UNEXPECTED
 #endif
 #if defined(__NR_memfd_secret)
 		info[i].fd_memfd_secret = shim_memfd_secret(0);
@@ -384,6 +421,8 @@ static void NORETURN waste_resources(
 		info[i].fd_uf = shim_userfaultfd(0);
 		if (!keep_stressing_flag())
 			break;
+#else
+		UNEXPECTED
 #endif
 #if defined(O_TMPFILE)
 		info[i].fd_tmp = open("/tmp", O_TMPFILE | O_RDWR | flag,
@@ -411,7 +450,11 @@ static void NORETURN waste_resources(
 				(void)fcntl(info[i].fd_tmp, F_SETLK, &f);
 			}
 		}
+#else
+		UNEXPECTED
 #endif
+#else
+		UNEXPECTED
 #endif
 #if defined(HAVE_SYS_INOTIFY_H)
 		info[i].fd_inotify = inotify_init();
@@ -461,13 +504,13 @@ static void NORETURN waste_resources(
 			break;
 #endif
 #if defined(HAVE_PTSNAME)
-		info[i].pty_master = open("/dev/ptmx", O_RDWR | flag);
-		info[i].pty_slave = -1;
-		if (info[i].pty_master >= 0) {
-			const char *slavename = ptsname(info[i].pty_master);
+		info[i].pty_mtx = open("/dev/ptmx", O_RDWR | flag);
+		info[i].pty = -1;
+		if (info[i].pty_mtx >= 0) {
+			const char *ptyname = ptsname(info[i].pty_mtx);
 
-			if (slavename)
-				info[i].pty_slave = open(slavename, O_RDWR | flag);
+			if (ptyname)
+				info[i].pty = open(ptyname, O_RDWR | flag);
 		}
 		if (!keep_stressing_flag())
 			break;
@@ -651,10 +694,10 @@ static void NORETURN waste_resources(
 #endif
 
 #if defined(HAVE_PTSNAME)
-		if (info[i].pty_slave != -1)
-			(void)close(info[i].pty_slave);
-		if (info[i].pty_master != -1)
-			(void)close(info[i].pty_master);
+		if (info[i].pty != -1)
+			(void)close(info[i].pty);
+		if (info[i].pty_mtx != -1)
+			(void)close(info[i].pty_mtx);
 #endif
 
 #if defined(HAVE_LIB_PTHREAD) &&	\
@@ -696,7 +739,7 @@ static void NORETURN waste_resources(
 		if (info[i].pid > 0) {
 			int status;
 
-			(void)kill(info[i].pid, SIGKILL);
+			(void)stress_killpid(info[i].pid);
 			(void)shim_waitpid(info[i].pid, &status, 0);
 		}
 	}
