@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_REVIO_BYTES		(1 * MB)
+#define MAX_REVIO_BYTES		(MAX_FILE_LIMIT)
+#define DEFAULT_REVIO_BYTES	(1 * GB)
+
 #define BUF_ALIGNMENT		(4096)
 
 #define DEFAULT_REVIO_WRITE_SIZE (1024)
@@ -157,18 +156,26 @@ static ssize_t stress_revio_write(
 #endif
 
 	ret = write(fd, buf, count);
+	if (!keep_stressing_flag())
+		return ret;
 
 #if defined(HAVE_FSYNC)
 	if (revio_flags & REVIO_OPT_FSYNC)
 		(void)shim_fsync(fd);
+	if (!keep_stressing_flag())
+		return ret;
 #endif
 #if defined(HAVE_FDATASYNC)
 	if (revio_flags & REVIO_OPT_FDATASYNC)
 		(void)shim_fdatasync(fd);
+	if (!keep_stressing_flag())
+		return ret;
 #endif
 #if defined(HAVE_SYNCFS)
 	if (revio_flags & REVIO_OPT_SYNCFS)
 		(void)syncfs(fd);
+	if (!keep_stressing_flag())
+		return ret;
 #endif
 
 	return ret;
@@ -252,7 +259,7 @@ static int stress_revio_advise(const stress_args_t *args, const int fd, const in
 	if (!(flags & REVIO_OPT_FADV_MASK))
 		return 0;
 
-	for (i = 0; i < SIZEOF_ARRAY(revio_opts); i++) {
+	for (i = 0; keep_stressing(args) && (i < SIZEOF_ARRAY(revio_opts)); i++) {
 		if (revio_opts[i].flag & flags) {
 			if (posix_fadvise(fd, 0, 0, revio_opts[i].advice) < 0) {
 				pr_fail("%s: posix_fadvise failed, errno=%d (%s)\n",
@@ -269,27 +276,6 @@ static int stress_revio_advise(const stress_args_t *args, const int fd, const in
 	return 0;
 }
 
-static inline size_t stress_revio_get_extents(const int fd)
-{
-#if defined(FS_IOC_FIEMAP) &&	\
-    defined(HAVE_LINUX_FIEMAP_H)
-	struct fiemap fiemap;
-
-	(void)memset(&fiemap, 0, sizeof(fiemap));
-	fiemap.fm_length = ~0UL;
-
-	/* Find out how many extents there are */
-	if (ioctl(fd, FS_IOC_FIEMAP, &fiemap) < 0)
-		return 0;
-
-	return fiemap.fm_mapped_extents;
-#else
-	(void)fd;
-
-	return 0;
-#endif
-}
-
 /*
  *  stress_revio
  *	stress I/O via writes in reverse
@@ -389,7 +375,7 @@ static int stress_revio(const stress_args_t *args)
 				args->name, filename, errno, strerror(errno));
 			goto finish;
 		}
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		if (ftruncate(fd, (off_t)revio_bytes) < 0) {
 			pr_fail("%s: ftruncate failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -437,7 +423,7 @@ seq_wr_retry:
 			inc_counter(args);
 		}
 		iterations++;
-		extents = stress_revio_get_extents(fd);
+		extents = stress_get_extents(fd);
 		avg_extents += (double)extents;
 		(void)close(fd);
 	} while (keep_stressing(args));
@@ -467,5 +453,6 @@ stressor_info_t stress_revio_info = {
 	.stressor = stress_revio,
 	.class = CLASS_IO | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
