@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -174,7 +169,6 @@ static int stress_rlimit_child(const stress_args_t *args, void *ctxt)
 		 *  Exercise all known good resource ids
 		 */
 		for (i = 0; i < SIZEOF_ARRAY(resource_ids); i++) {
-
 			ret = getrlimit(resource_ids[i].resource, &rlim);
 			if (ret < 0)
 				continue;
@@ -308,7 +302,7 @@ static int stress_rlimit(const stress_args_t *args)
 		(void)stress_temp_dir_rm_args(args);
 		return EXIT_FAILURE;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	for (i = 0; i < SIZEOF_ARRAY(limits); i++) {
 		limits[i].ret = getrlimit(limits[i].resource, &limits[i].old_limit);
@@ -329,5 +323,6 @@ static int stress_rlimit(const stress_args_t *args)
 stressor_info_t stress_rlimit_info = {
 	.stressor = stress_rlimit,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
