@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -159,18 +154,18 @@ static int stress_rmap(const stress_args_t *args)
 
 	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
 		rc = exit_status(errno);
-		pr_fail("%s: open %s failed, errno=%d (%s)\n",
+		pr_err("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 		(void)stress_temp_dir_rm_args(args);
 		(void)munmap((void *)counters, counters_sz);
 
 		return (int)rc;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	if (shim_fallocate(fd, 0, 0, (off_t)sz) < 0) {
-		pr_fail("%s: fallocate failed, errno=%d (%s)\n",
+		pr_err("%s: fallocate failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd);
 		(void)stress_temp_dir_rm_args(args);
@@ -252,7 +247,7 @@ cleanup:
 	for (i = 0; i < RMAP_CHILD_MAX; i++) {
 		if (pids[i] <= 0)
 			continue;
-		(void)kill(pids[i], SIGKILL);
+		(void)stress_killpid(pids[i]);
 	}
 	for (i = 0; i < RMAP_CHILD_MAX; i++) {
 		int status, ret;
@@ -266,15 +261,11 @@ cleanup:
 				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 			(void)kill(pids[i], SIGTERM);
-			(void)kill(pids[i], SIGKILL);
+			(void)stress_killpid(pids[i]);
 			(void)shim_waitpid(pids[i], &status, 0);
 		}
 	}
 
-	(void)munmap((void *)counters, counters_sz);
-	(void)close(fd);
-	(void)stress_temp_dir_rm_args(args);
-
 	for (i = 0; i < MAPPINGS_MAX; i++) {
 		if (mappings[i] != MAP_FAILED)
 			(void)munmap((void *)mappings[i], MAPPING_PAGES * page_size);
@@ -282,6 +273,10 @@ cleanup:
 			(void)munmap((void *)paddings[i], page_size);
 	}
 
+	(void)munmap((void *)counters, counters_sz);
+	(void)close(fd);
+	(void)stress_temp_dir_rm_args(args);
+
 	return EXIT_SUCCESS;
 }
 
