@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -14,15 +15,14 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-pragma.h"
+
+#if defined(HAVE_LINUX_RSEQ_H)
+#include <linux/rseq.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"rseq N",	"start N workers that exercise restartable sequences" },
@@ -30,12 +30,12 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-
 #if defined(HAVE_LINUX_RSEQ_H) &&	\
     defined(HAVE_ASM_NOP) &&		\
     defined(__NR_rseq) &&		\
     defined(__GNUC__) &&		\
-    !defined(__clang__)
+    !defined(__clang__) &&		\
+    !defined(__ICC)
 
 #define STRESS_ACCESS_ONCE(x)     (*(__volatile__  __typeof__(x) *)&(x))
 
@@ -47,13 +47,25 @@ static const stress_help_t help[] = {
 #define OPTIMIZE0       __attribute__((optimize("-O0")))
 #endif
 
-#define NOPS()		\
-asm volatile(		\
-	"nop\n"		\
-	"nop\n"		\
-	"nop\n"		\
-	"nop\n"		\
-);
+static inline void stress_op_nop(void)
+{
+#if defined(STRESS_ARCH_KVX)
+	/*
+	 * Extra ;; required for KVX to indicate end of
+	 * a VLIW instruction bundle
+	 */
+	__asm__ __volatile__("nop\n;;\n");
+#else
+	__asm__ __volatile__("nop;\n");
+#endif
+}
+
+#define NOPS()	do {		\
+	stress_op_nop();	\
+	stress_op_nop();	\
+	stress_op_nop();	\
+	stress_op_nop();	\
+} while (0);
 
 typedef struct {
 	uint64_t crit_count;		/* critical path entry count */
@@ -234,8 +246,8 @@ static int stress_rseq_oomable(const stress_args_t *args, void *context)
 	(void)memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = sigsegv_handler;
 	if (sigaction(SIGSEGV, &sa, NULL) < 0) {
-		fprintf(stderr, "Failed to set SIGSEGV hhandler\n");
-		exit(1);
+		pr_inf("%s: failed to set SIGSEGV handler\n", args->name);
+		_exit(EXIT_FAILURE);
 	}
 
 	do {
@@ -279,6 +291,11 @@ unreg:
 		if (ret == 0)
 			(void)rseq_unregister(&restartable_seq, valid_signature);
 
+		/* invalid flags */
+		ret = shim_rseq(&restartable_seq, sizeof(invalid_seq), ~RSEQ_FLAG_UNREGISTER, valid_signature);
+		if (ret == 0)
+			(void)rseq_unregister(&restartable_seq, valid_signature);
+
 		/* Invalid alignment, EINVAL */
 		ret = rseq_register(misaligned_seq, valid_signature);
 		if (ret == 0)
