@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_RTC_H)
+#include <linux/rtc.h>
+#endif
+
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"rtc N",	"start N workers that exercise the RTC interfaces" },
 	{ NULL,	"rtc-ops N",	"stop after N RTC bogo operations" },
@@ -154,6 +157,7 @@ static inline int stress_rtc_dev(const stress_args_t *args)
 	}
 #endif
 
+#if defined(HAVE_SYS_SELECT_H)
 	/*
 	 *  Very short delay select on the device
 	 *  that should normally always timeout because
@@ -164,6 +168,7 @@ static inline int stress_rtc_dev(const stress_args_t *args)
 	FD_ZERO(&rfds);
 	FD_SET(fd, &rfds);
 	(void)select(fd + 1, &rfds, NULL, NULL, &timeout);
+#endif
 
 #if defined(RTC_VL_READ)
 	if (ioctl(fd, RTC_VL_READ, &tmp) < 0) {
@@ -268,6 +273,7 @@ static int stress_rtc(const stress_args_t *args)
 stressor_info_t stress_rtc_info = {
 	.stressor = stress_rtc,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
