@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(__NR_sched_getattr)
+#define HAVE_SCHED_GETATTR
+#endif
+
+#if defined(__NR_sched_setattr)
+#define HAVE_SCHED_SETATTR
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"schedpolicy N",	"start N workers that exercise scheduling policy" },
@@ -58,7 +62,7 @@ static const int policies[] = {
 
 static int stress_schedpolicy(const stress_args_t *args)
 {
-	int policy = 0;
+	int policy = args->instance % SIZEOF_ARRAY(policies);
 #if defined(_POSIX_PRIORITY_SCHEDULING)
 	const bool root_or_nice_capability = stress_check_capability(SHIM_CAP_SYS_NICE);
 #endif
@@ -69,6 +73,7 @@ static int stress_schedpolicy(const stress_args_t *args)
 	uint32_t sched_util_max_value = 0;
 	int counter = 0;
 #endif
+
 #if defined(_POSIX_PRIORITY_SCHEDULING)
 	int n = 0;
 #endif
@@ -88,6 +93,8 @@ static int stress_schedpolicy(const stress_args_t *args)
 #if defined(HAVE_SCHED_GETATTR) &&	\
     defined(HAVE_SCHED_SETATTR)
 		struct shim_sched_attr attr;
+#else
+		UNEXPECTED
 #endif
 		struct sched_param param;
 		int ret = 0;
@@ -96,29 +103,45 @@ static int stress_schedpolicy(const stress_args_t *args)
 		const pid_t pid = stress_mwc1() ? 0 : args->pid;
 		const char *new_policy_name = stress_get_sched_name(new_policy);
 
+		if (!keep_stressing(args))
+			break;
+
+		shim_sched_yield();
+		errno = 0;
+
 		switch (new_policy) {
 #if defined(SCHED_DEADLINE) &&		\
     defined(HAVE_SCHED_GETATTR) &&	\
     defined(HAVE_SCHED_SETATTR)
 		case SCHED_DEADLINE:
-			attr.size = sizeof(attr);
-			attr.sched_flags = 0;
-			attr.sched_nice = 0;
-			attr.sched_priority = 0;
-			attr.sched_policy = SCHED_DEADLINE;
-			attr.sched_runtime = 10 * 1000 * 1000;
-			attr.sched_period = 30 * 1000 * 1000;
-			attr.sched_deadline = 30 * 1000 * 1000;
-			ret = shim_sched_setattr(0, &attr, 0);
-			(void)ret;
-			break;
+			/*
+			 *  Only have 1 RT deadline instance running
+			 */
+			if (args->instance == 0) {
+				(void)memset(&attr, 0, sizeof(attr));
+				attr.size = sizeof(attr);
+				attr.sched_flags = 0;
+				attr.sched_nice = 0;
+				attr.sched_priority = 0;
+				attr.sched_policy = SCHED_DEADLINE;
+				/* runtime <= deadline <= period */
+				attr.sched_runtime = 64 * 1000000;
+				attr.sched_deadline = 128 * 1000000;
+				attr.sched_period = 256 * 1000000;
+
+				ret = shim_sched_setattr(0, &attr, 0);
+				break;
+			}
+			CASE_FALLTHROUGH;
 #endif
 
 #if defined(SCHED_IDLE)
 		case SCHED_IDLE:
+			CASE_FALLTHROUGH;
 #endif
 #if defined(SCHED_BATCH)
 		case SCHED_BATCH:
+			CASE_FALLTHROUGH;
 #endif
 #if defined(SCHED_OTHER)
 		case SCHED_OTHER:
@@ -190,6 +213,7 @@ static int stress_schedpolicy(const stress_args_t *args)
 			 */
 			if ((errno != EPERM) &&
 			    (errno != EINVAL) &&
+			    (errno != EINTR) &&
 			    (errno != EBUSY)) {
 				pr_fail("%s: sched_setscheduler "
 					"failed: errno=%d (%s) "
@@ -214,23 +238,25 @@ static int stress_schedpolicy(const stress_args_t *args)
 		if (n++ >= 1024) {
 			n = 0;
 
-			/* Exercise invalid sched_getparam syscall*/
-			(void)memset(&param, 0, sizeof param);
+			/* Exercise invalid sched_getparam syscall */
+			(void)memset(&param, 0, sizeof(param));
 			ret = sched_getparam(-1, &param);
 			(void)ret;
 
 #if defined(__linux__)
 			/* Linux allows NULL param, will return EFAULT */
+			(void)memset(&param, 0, sizeof(param));
 			ret = sched_getparam(pid, NULL);
 			(void)ret;
 #endif
 
 			/* Exercise bad pid, ESRCH error */
+			(void)memset(&param, 0, sizeof(param));
 			ret = sched_getparam(stress_get_unused_pid_racy(false), &param);
 			(void)ret;
 
 			/* Exercise invalid sched_setparam syscall */
-			(void)memset(&param, 0, sizeof param);
+			(void)memset(&param, 0, sizeof(param));
 			ret = sched_setparam(-1, &param);
 			(void)ret;
 
@@ -258,7 +284,7 @@ static int stress_schedpolicy(const stress_args_t *args)
 		ret = sched_getscheduler(stress_get_unused_pid_racy(false));
 		(void)ret;
 
-		(void)memset(&param, 0, sizeof param);
+		(void)memset(&param, 0, sizeof(param));
 		ret = sched_getparam(pid, &param);
 		if ((ret < 0) && ((errno != EINVAL) && (errno != EPERM)))
 			pr_fail("%s: sched_getparam failed, errno=%d (%s)\n",
@@ -285,6 +311,7 @@ static int stress_schedpolicy(const stress_args_t *args)
 		}
 
 		/* Exercise invalid sched_getattr syscalls */
+		(void)memset(&attr, 0, sizeof(attr));
 		ret = shim_sched_getattr(pid, &attr, sizeof(attr), ~0U);
 		(void)ret;
 
@@ -362,9 +389,9 @@ static int stress_schedpolicy(const stress_args_t *args)
 			sched_util_max_value--;
 			counter = 0;
 		}
+#else
+		UNEXPECTED
 #endif
-
-		(void)shim_sched_yield();
 		policy++;
 		policy %= SIZEOF_ARRAY(policies);
 		inc_counter(args);
@@ -378,6 +405,7 @@ static int stress_schedpolicy(const stress_args_t *args)
 stressor_info_t stress_schedpolicy_info = {
 	.stressor = stress_schedpolicy,
 	.class = CLASS_INTERRUPT | CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
