@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,33 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
+
+#if defined(HAVE_SYS_UN_H)
+#include <sys/un.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_NETINET_SCTP_H)
+#include <netinet/sctp.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_NETINET_TCP_H)
+#include <netinet/tcp.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_SCTP_PORT		(1024)
+#define MAX_SCTP_PORT		(65535)
+#define DEFAULT_SCTP_PORT	(9000)
+
+#define SOCKET_BUF		(8192)	/* Socket I/O buffer size */
 
 static const stress_help_t help[] = {
 	{ NULL,	"sctp N",	 "start N workers performing SCTP send/receives " },
@@ -41,6 +61,8 @@ static const stress_help_t help[] = {
 #endif
 
 static uint64_t	sigpipe_count;
+#else
+UNEXPECTED
 #endif
 
 /*
@@ -104,36 +126,58 @@ static void stress_sctp_sockopts(const int fd)
 {
 #if defined(SCTP_RTOINFO)
 	STRESS_SCTP_SOCKOPT(SCTP_RTOINFO, struct sctp_rtoinfo)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_ASSOCINFO)
 	STRESS_SCTP_SOCKOPT(SCTP_ASSOCINFO, struct sctp_assocparams)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_INITMSG)
 	STRESS_SCTP_SOCKOPT(SCTP_INITMSG, struct sctp_initmsg)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_NODELAY)
 	STRESS_SCTP_SOCKOPT(SCTP_NODELAY, int)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_PRIMARY_ADDR)
 	STRESS_SCTP_SOCKOPT(SCTP_PRIMARY_ADDR, struct sctp_prim)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_PEER_ADDR_PARAMS)
 	STRESS_SCTP_SOCKOPT(SCTP_PEER_ADDR_PARAMS, struct sctp_paddrparams)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_EVENTS)
 	STRESS_SCTP_SOCKOPT(SCTP_EVENTS, struct sctp_event_subscribe)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_MAXSEG)
 	STRESS_SCTP_SOCKOPT(SCTP_MAXSEG, struct sctp_assoc_value)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_STATUS)
 	STRESS_SCTP_SOCKOPT(SCTP_STATUS, struct sctp_status)
+#else
+	UNEXPECTED
 #endif
 #if defined(SCTP_GET_PEER_ADDR_INFO) && 0
 	STRESS_SCTP_SOCKOPT(SCTP_GET_PEER_ADDR_INFO, struct sctp_paddrinfo)
+#else
+	/* UNEXPECTED */
 #endif
 #if defined(SCTP_GET_ASSOC_STATS)
 	STRESS_SCTP_SOCKOPT(SCTP_GET_ASSOC_STATS, struct sctp_assoc_stats)
+#else
+	UNEXPECTED
 #endif
 }
 
@@ -225,8 +269,10 @@ retry:
 	if (sctp_domain == AF_UNIX) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
+#else
+	UNEXPECTED
 #endif
 	/* Inform parent we're all done */
 	(void)kill(getppid(), SIGALRM);
@@ -303,17 +349,19 @@ static int stress_sctp_server(
 		if (sfd >= 0) {
 			size_t i;
 
-#if defined(SOCKET_NODELAY)
+#if defined(TCP_NODELAY)
 			int one = 1;
 
-			if (opt_flags & OPT_FLAGS_SOCKET_NODELAY) {
+			if (g_opt_flags & OPT_FLAGS_SOCKET_NODELAY) {
 				if (setsockopt(fd, SOL_TCP, TCP_NODELAY, &one, sizeof(one)) < 0) {
 					pr_inf("%s: setsockopt TCP_NODELAY "
 						"failed and disabled, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
-					opt_flags &= ~OPT_FLAGS_SOCKET_NODELAY;
+					g_opt_flags &= ~OPT_FLAGS_SOCKET_NODELAY;
 				}
 			}
+#else
+			UNEXPECTED
 #endif
 
 			(void)memset(buf, 'A' + (get_counter(args) % 26), sizeof(buf));
@@ -342,8 +390,10 @@ die:
 	if (addr && sctp_domain == AF_UNIX) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
+#else
+	UNEXPECTED
 #endif
 	if (pid) {
 		(void)kill(pid, SIGKILL);
@@ -385,8 +435,10 @@ static int stress_sctp(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() && (errno == EAGAIN))
+		if (stress_redo_fork(errno))
 			goto again;
+		if (!keep_stressing(args))
+			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
@@ -399,6 +451,7 @@ again:
 			sctp_port, sctp_domain);
 	}
 
+finish:
 	if (sigpipe_count)
 		pr_dbg("%s: caught %" PRIu64 " SIGPIPE signals\n", args->name, sigpipe_count);
 
