@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,33 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_AUDIT_H)
+#include <linux/audit.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_LINUX_FILTER_H)
+#include <linux/filter.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_LINUX_SECCOMP_H)
+#include <linux/seccomp.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_PRCTL_H)
+#include <sys/prctl.h>
+#else
+UNEXPECTED
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"seccomp N",	 "start N workers performing seccomp call filtering" },
 	{ NULL,	"seccomp-ops N", "stop after N seccomp bogo operations" },
@@ -126,6 +145,36 @@ static struct sock_fprog prog_random = {
 	.filter = filter_random
 };
 
+#if defined(SECCOMP_GET_ACTION_AVAIL)
+static uint32_t seccomp_actions[] = {
+#if defined(SECCOMP_RET_KILL_PROCESS)
+	SECCOMP_RET_KILL_PROCESS,
+#endif
+#if defined(SECCOMP_RET_KILL_THREAD)
+	SECCOMP_RET_KILL_THREAD,
+#endif
+#if defined(SECCOMP_RET_TRAP)
+	SECCOMP_RET_TRAP,
+#endif
+#if defined(SECCOMP_RET_ERRNO)
+	SECCOMP_RET_ERRNO,
+#endif
+#if defined(SECCOMP_RET_USER_NOTIF)
+	SECCOMP_RET_USER_NOTIF,
+#endif
+#if defined(SECCOMP_RET_TRACE)
+	SECCOMP_RET_TRACE,
+#endif
+#if defined(SECCOMP_RET_LOG)
+	SECCOMP_RET_LOG,
+#endif
+#if defined(SECCOMP_RET_LOG)
+	SECCOMP_RET_ALLOW,
+#endif
+	~0,	/* Invalid */
+};
+#endif
+
 static int stress_seccomp_supported(const char *name)
 {
 	pid_t pid;
@@ -229,7 +278,6 @@ next:
 	return 0;
 }
 
-
 /*
  *  stress_seccomp_set_filter()
  *	set up a seccomp filter, allow writes
@@ -240,7 +288,6 @@ static int stress_seccomp_set_filter(
 	const bool allow_write,
 	bool do_random)
 {
-
 	/*
 	 *  Depending on allow_write we either use the
 	 *  filter that allows writes or the filter
@@ -268,6 +315,44 @@ static int stress_seccomp_set_filter(
 	 *  Try using the newer seccomp syscall first
 	 */
 	if (use_seccomp) {
+#if defined(HAVE_SECCOMP_NOTIF_SIZES)
+		{
+			struct seccomp_notif_sizes sizes;
+
+			/*
+			 *  Exercise SECCOMP_GET_NOTIF_SIZES
+			 */
+			(void)shim_seccomp(SECCOMP_GET_NOTIF_SIZES, 0, &sizes);
+			/* Invalid flags, EINVAL */
+			(void)shim_seccomp(SECCOMP_GET_NOTIF_SIZES, ~0, &sizes);
+		}
+#else
+		UNEXPECTED
+#endif
+#if defined(SECCOMP_GET_ACTION_AVAIL)
+		{
+			for (i = 0; i < SIZEOF_ARRAY(seccomp_actions); i++) {
+				/* Valid flags */
+				(void)shim_seccomp(SECCOMP_GET_ACTION_AVAIL, 0, (void *)&seccomp_actions[i]);
+				/* Invalid flags, EINVAL */
+				(void)shim_seccomp(SECCOMP_GET_ACTION_AVAIL, ~0, (void *)&seccomp_actions[i]);
+			}
+		}
+#else
+		UNEXPECTED
+#endif
+#if defined(SECCOMP_SET_MODE_STRICT)
+		if (stress_mwc8() < 16) {
+			(void)shim_seccomp(SECCOMP_SET_MODE_STRICT, 0, NULL);
+			(void)shim_seccomp(SECCOMP_SET_MODE_STRICT, ~0, NULL);
+			(void)shim_seccomp(SECCOMP_SET_MODE_STRICT, 0, &i);
+		}
+#else
+		UNEXPECTED
+#endif
+		/* Exercise invalid op */
+		(void)shim_seccomp(~0, 0, NULL);
+
 redo_seccomp:
 		if (shim_seccomp(SECCOMP_SET_MODE_FILTER, 0, p) == 0)
 			return 0;
@@ -300,7 +385,6 @@ redo_prctl:
 	return 0;
 }
 
-
 /*
  *  stress_seccomp()
  *	stress seccomp
@@ -399,6 +483,7 @@ stressor_info_t stress_seccomp_info = {
 	.stressor = stress_seccomp,
 	.supported = stress_seccomp_supported,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
