@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_SEEK_SIZE		(1 * MB)
+#define MAX_SEEK_SIZE		(MAX_FILE_LIMIT)
+#define DEFAULT_SEEK_SIZE	(16 * MB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"seek N",	"start N workers performing random seek r/w IO" },
 	{ NULL,	"seek-ops N",	"stop after N seek bogo operations" },
@@ -74,10 +73,14 @@ static int stress_seek(const stress_args_t *args)
 	char filename[PATH_MAX];
 	uint8_t buf[512];
 	const off_t bad_off_t = max_off_t();
-#if defined(OPT_SEEK_PUNCH)
+#if defined(HAVE_OFF64_T) &&	\
+    defined(HAVE_LSEEK64)
+	off64_t	offset64 = (off64_t)0;
+#endif
+#if defined(FALLOC_FL_PUNCH_HOLE)
 	bool seek_punch = false;
 
-	(void)stress_get_setting("seek-punch", TYPE_ID_BOOL, &seek_punch);
+	(void)stress_get_setting("seek-punch", &seek_punch);
 #endif
 
 	if (!stress_get_setting("seek-size", &seek_size)) {
@@ -102,7 +105,7 @@ static int stress_seek(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto finish;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	/* Generate file with hole at the end */
 	if (lseek(fd, (off_t)len, SEEK_SET) < 0) {
 		rc = exit_status(errno);
@@ -180,6 +183,8 @@ re_read:
 				pr_fail("%s: lseek SEEK_END failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 		}
+#else
+		UNEXPECTED
 #endif
 #if defined(SEEK_CUR)
 		if (lseek(fd, 0, SEEK_CUR) < 0) {
@@ -187,6 +192,44 @@ re_read:
 				pr_fail("%s: lseek SEEK_CUR failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 		}
+#else
+		UNEXPECTED
+#endif
+
+#if defined(HAVE_OFF64_T) &&	\
+    defined(HAVE_LSEEK64)
+#if defined(SEEK_SET)
+		/*
+		 *  exercise 64 bit lseek, on 64 arches this calls lseek,
+		 *  but it is worth checking anyhow just in case it is
+		 *  broken on 32 or 64 bit systems
+		 */
+		if (lseek64(fd, offset64, SEEK_SET) < 0) {
+			if (errno != EINVAL)
+				pr_fail("%s: lseek64 SEEK_SET failed, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+		}
+#else
+		UNEXPECTED
+#endif
+#if defined(SEEK_END)
+		if (lseek64(fd, offset64, SEEK_END) < 0) {
+			if (errno != EINVAL)
+				pr_fail("%s: lseek64 SEEK_END failed, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+		}
+#else
+		UNEXPECTED
+#endif
+#if defined(SEEK_CUR)
+		if (lseek64(fd, offset64, SEEK_CUR) < 0) {
+			if (errno != EINVAL)
+				pr_fail("%s: lseek64 SEEK_CUR failed, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+		}
+#else
+		UNEXPECTED
+#endif
 #endif
 #if defined(SEEK_HOLE) &&	\
     !defined(__APPLE__)
@@ -195,6 +238,8 @@ re_read:
 				pr_fail("%s: lseek SEEK_HOLE failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 		}
+#else
+		UNEXPECTED
 #endif
 #if defined(SEEK_DATA) &&	\
     !defined(__APPLE__)
@@ -203,6 +248,8 @@ re_read:
 				pr_fail("%s: lseek SEEK_DATA failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 		}
+#else
+		UNEXPECTED
 #endif
 #if defined(SEEK_HOLE) &&	\
     defined(SEEK_DATA) &&	\
@@ -221,16 +268,18 @@ re_read:
 			}
 		}
 #endif
-#if defined(OPT_SEEK_PUNCH)
-		if (!seek_punch_hole)
+#if defined(FALLOC_FL_PUNCH_HOLE)
+		if (!seek_punch)
 			continue;
 
 		offset = (off_t)(stress_mwc64() % len);
 		if (shim_fallocate(fd, FALLOC_FL_PUNCH_HOLE |
 				  FALLOC_FL_KEEP_SIZE, offset, 8192) < 0) {
 			if (errno == EOPNOTSUPP)
-				seek_punch_hole = false;
+				seek_punch = false;
 		}
+#else
+		UNEXPECTED
 #endif
 		/*
 		 *  Exercise lseek on an invalid fd
@@ -292,5 +341,6 @@ stressor_info_t stress_seek_info = {
 	.stressor = stress_seek,
 	.class = CLASS_IO | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
