@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SEM_SYSV)
+#include <sys/sem.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_SEM_SYSV_PROCS     (2)
+#define MAX_SEM_SYSV_PROCS     (64)
+#define DEFAULT_SEM_SYSV_PROCS (2)
+
 #define STRESS_MAX_SEMS		(100)
 
 static const stress_help_t help[] = {
@@ -48,7 +53,7 @@ static int stress_set_semaphore_sysv_procs(const char *opt)
 
 	semaphore_sysv_procs = stress_get_uint64(opt);
 	stress_check_range("sem-sysv-procs", semaphore_sysv_procs,
-		MIN_SEMAPHORE_PROCS, MAX_SEMAPHORE_PROCS);
+		MIN_SEM_SYSV_PROCS, MAX_SEM_SYSV_PROCS);
 	return stress_set_setting("sem-sysv-procs", TYPE_ID_UINT64, &semaphore_sysv_procs);
 }
 
@@ -64,7 +69,27 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  */
 static void stress_semaphore_sysv_init(void)
 {
-	int count = 0;
+	int count = 0, sem_id;
+
+	/* Exercise invalid nsems, EINVAL */
+	sem_id = semget((key_t)stress_mwc16(), -1, IPC_CREAT | S_IRUSR | S_IWUSR);
+	if (sem_id != -1)
+		(void)semctl(sem_id, 0, IPC_RMID);
+
+	/* Exercise invalid nsems, EINVAL */
+	sem_id = semget((key_t)stress_mwc16(), INT_MAX, IPC_CREAT | S_IRUSR | S_IWUSR);
+	if (sem_id != -1)
+		(void)semctl(sem_id, 0, IPC_RMID);
+
+	/* Exercise invalid semget semflg, EINVAL */
+	sem_id = semget((key_t)stress_mwc16(), 0, ~0);
+	if (sem_id != -1)
+		(void)semctl(sem_id, 0, IPC_RMID);
+
+	/* Exercise without IPC_CREAT, ENOENT */
+	sem_id = semget((key_t)stress_mwc16(), 1, S_IRUSR | S_IWUSR);
+	if (sem_id != -1)
+		(void)semctl(sem_id, 0, IPC_RMID);
 
 	while (count < 100) {
 		g_shared->sem_sysv.key_id = (key_t)stress_mwc16();
@@ -94,7 +119,7 @@ static void stress_semaphore_sysv_init(void)
 			"(%s), skipping semaphore stressor\n",
 			errno, strerror(errno));
 	} else {
-		pr_err("semaphore init (System V) failed: errno=%d: "
+		pr_fail("semaphore init (System V) failed: errno=%d: "
 			"(%s)\n", errno, strerror(errno));
 		_exit(EXIT_FAILURE);
 	}
@@ -183,6 +208,8 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 			ret = semtimedop(sem_id, sems, STRESS_MAX_SEMS * 3, &timeout);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 		for (i = 0; i < 1000; i++) {
@@ -254,6 +281,8 @@ timed_out:
 				s.buf = &ds;
 				ret = semctl(sem_id, 2, IPC_SET, &s);
 				(void)ret;
+#else
+				UNEXPECTED
 #endif
 			}
 
@@ -279,6 +308,8 @@ timed_out:
 			}
 #endif
 		}
+#else
+		UNEXPECTED
 #endif
 #if defined(SEM_STAT) &&	\
     defined(__linux__)
@@ -379,6 +410,8 @@ timed_out:
 		/* Exercise with an illegal sem number */
 		ret = semctl(sem_id, ~0, GETVAL, 0);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 #if defined(GETPID)
 		ret = semctl(sem_id, 0, GETPID, 0);
@@ -391,6 +424,8 @@ timed_out:
 		/* Exercise with an illegal sem number */
 		ret = semctl(sem_id, ~0, GETPID, 0);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 #if defined(GETNCNT)
 		ret = semctl(sem_id, 0, GETNCNT, 0);
@@ -403,6 +438,8 @@ timed_out:
 		/* Exercise with an illegal sem number */
 		ret = semctl(sem_id, ~0, GETNCNT, 0);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 #if defined(GETZCNT)
 		ret = semctl(sem_id, 0, GETZCNT, 0);
@@ -415,6 +452,8 @@ timed_out:
 		/* Exercise with an illegal sem number */
 		ret = semctl(sem_id, ~0, GETZCNT, 0);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 		/*
 		 * Now exercise invalid options and arguments
@@ -434,11 +473,43 @@ timed_out:
 			semwait.sem_num = 0;
 			semwait.sem_op = -1;
 			semwait.sem_flg = SEM_UNDO;
-
 			ret = semtimedop(sem_id, &semwait, 1, &timeout);
 			(void)ret;
+
+			/*
+			 *  Exercise invalid semid, EINVAL
+			 */
+			timeout.tv_sec = 0;
+			timeout.tv_nsec = 10000;
+			ret = semtimedop(-1, &semwait, 1, &timeout);
+			(void)ret;
+
+			/*
+			 *  Exercise invalid nsops, E2BIG
+			 */
+			timeout.tv_sec = 0;
+			timeout.tv_nsec = 10000;
+			ret = semtimedop(sem_id, &semwait, -1, &timeout);
+			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
+		{
+			struct sembuf semwait;
+			/*
+			 *  Exercise invalid semid, EINVAL
+			 */
+			ret = semop(-1, &semwait, 1);
+			(void)ret;
+
+			/*
+			 *  Exercise invalid nsops, E2BIG
+			 */
+			ret = semop(sem_id, &semwait, -1);
+			(void)ret;
+		}
+
 		/*
 		 *  Exercise illegal semwait
 		 */
@@ -452,10 +523,20 @@ timed_out:
 			ret = semop(sem_id, &semwait, 1);
 			(void)ret;
 		}
+
+		/*
+		 *  Exercise illegal cmd, use direct system call
+		 *  if available as some libcs detect that the cmd
+		 *  is invalid without doing the actual system call
+		 */
+		(void)semctl(sem_id, 0, INT_MAX, 0);
+#if defined(__NR_semctl)
+		(void)syscall(__NR_semctl, sem_id, 0, INT_MAX, 0);
+#endif
 	} while ((rc == EXIT_SUCCESS)  && keep_stressing(args));
 
 	if (rc == EXIT_FAILURE)
-		kill(getppid(), SIGALRM);
+		(void)kill(getppid(), SIGALRM);
 
 	return rc;
 }
@@ -471,7 +552,7 @@ static pid_t semaphore_sysv_spawn(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() && (errno == EAGAIN))
+		if (stress_redo_fork(errno))
 			goto again;
 		return -1;
 	}
@@ -497,16 +578,16 @@ static void stress_sem_sysv_sigchild(int sig)
  */
 static int stress_sem_sysv(const stress_args_t *args)
 {
-	pid_t pids[MAX_SEMAPHORE_PROCS];
+	pid_t pids[MAX_SEM_SYSV_PROCS];
 	uint64_t i;
-	uint64_t semaphore_sysv_procs = DEFAULT_SEMAPHORE_PROCS;
+	uint64_t semaphore_sysv_procs = DEFAULT_SEM_SYSV_PROCS;
 	int rc = EXIT_SUCCESS;
 
 	if (!stress_get_setting("sem-sysv-procs", &semaphore_sysv_procs)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			semaphore_sysv_procs = MAX_SEMAPHORE_PROCS;
+			semaphore_sysv_procs = MAX_SEM_SYSV_PROCS;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			semaphore_sysv_procs = MIN_SEMAPHORE_PROCS;
+			semaphore_sysv_procs = MIN_SEM_SYSV_PROCS;
 	}
 
 	if (!g_shared->sem_sysv.init) {
@@ -534,7 +615,7 @@ reap:
 
 	for (i = 0; i < semaphore_sysv_procs; i++) {
 		if (pids[i] > 0)
-			(void)kill(pids[i], SIGKILL);
+			(void)stress_killpid(pids[i]);
 	}
 	for (i = 0; i < semaphore_sysv_procs; i++) {
 		if (pids[i] > 0) {
@@ -557,6 +638,7 @@ stressor_info_t stress_sem_sysv_info = {
 	.deinit = stress_semaphore_sysv_deinit,
 	.class = CLASS_OS | CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
