@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SEMAPHORE_H)
+#include <semaphore.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_SEM_POSIX_PROCS     (2)
+#define MAX_SEM_POSIX_PROCS     (64)
+#define DEFAULT_SEM_POSIX_PROCS (2)
+
 static const stress_help_t help[] = {
 	{ NULL,	"sem N",	"start N workers doing semaphore operations" },
 	{ NULL,	"sem-ops N",	"stop after N semaphore bogo operations" },
@@ -37,7 +42,7 @@ static int stress_set_semaphore_posix_procs(const char *opt)
 
 	semaphore_posix_procs = stress_get_uint64(opt);
 	stress_check_range("sem-procs", semaphore_posix_procs,
-		MIN_SEMAPHORE_PROCS, MAX_SEMAPHORE_PROCS);
+		MIN_SEM_POSIX_PROCS, MAX_SEM_POSIX_PROCS);
 	return stress_set_setting("sem-procs", TYPE_ID_UINT64, &semaphore_posix_procs);
 }
 
@@ -51,8 +56,8 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
     defined(HAVE_SEM_POSIX)
 
 static sem_t sem;
-static pthread_t pthreads[MAX_SEMAPHORE_PROCS];
-static int p_ret[MAX_SEMAPHORE_PROCS];
+static pthread_t pthreads[MAX_SEM_POSIX_PROCS];
+static int p_ret[MAX_SEM_POSIX_PROCS];
 
 /*
  *  semaphore_posix_thrash()
@@ -116,7 +121,7 @@ static void *semaphore_posix_thrash(void *arg)
 			else
 				(void)shim_usleep(0);
 		}
-	} while (keep_stressing());
+	} while (keep_stressing(args));
 
 	return &nowt;
 }
@@ -127,21 +132,21 @@ static void *semaphore_posix_thrash(void *arg)
  */
 static int stress_sem(const stress_args_t *args)
 {
-	uint64_t semaphore_posix_procs = DEFAULT_SEMAPHORE_PROCS;
+	uint64_t semaphore_posix_procs = DEFAULT_SEM_POSIX_PROCS;
 	uint64_t i;
 	bool created = false;
 	stress_pthread_args_t p_args;
 
 	if (!stress_get_setting("sem-procs", &semaphore_posix_procs)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			semaphore_posix_procs = MAX_SEMAPHORE_PROCS;
+			semaphore_posix_procs = MAX_SEM_POSIX_PROCS;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			semaphore_posix_procs = MIN_SEMAPHORE_PROCS;
+			semaphore_posix_procs = MIN_SEM_POSIX_PROCS;
 	}
 
 	/* create a semaphore */
 	if (sem_init(&sem, 0, 1) < 0) {
-		pr_err("semaphore init (POSIX) failed: errno=%d: "
+		pr_fail("semaphore init (POSIX) failed: errno=%d: "
 			"(%s)\n", errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -149,6 +154,8 @@ static int stress_sem(const stress_args_t *args)
 	(void)memset(pthreads, 0, sizeof(pthreads));
 	(void)memset(p_ret, 0, sizeof(p_ret));
 
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	for (i = 0; i < semaphore_posix_procs; i++) {
 		p_args.args = args;
 		p_args.data = NULL;
@@ -170,9 +177,11 @@ static int stress_sem(const stress_args_t *args)
 	}
 
 	/* Wait for termination */
-	while (keep_stressing())
+	while (keep_stressing(args))
 		pause();
 
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	for (i = 0; i < semaphore_posix_procs; i++) {
 		int ret;
 
@@ -191,6 +200,7 @@ stressor_info_t stress_sem_info = {
 	.stressor = stress_sem,
 	.class = CLASS_OS | CLASS_SCHEDULER,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
