@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_SENDFILE_H)
+#include <sys/sendfile.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_SENDFILE_SIZE	(1 * KB)
+#define MAX_SENDFILE_SIZE	(1 * GB)
+#define DEFAULT_SENDFILE_SIZE	(4 * MB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"sendfile N",	   "start N workers exercising sendfile" },
 	{ NULL,	"sendfile-ops N",  "stop after N bogo sendfile operations" },
@@ -47,6 +52,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 };
 
 #if defined(HAVE_SYS_SENDFILE_H) &&	\
+    defined(HAVE_SENDFILE) &&		\
     NEED_GLIBC(2,1,0)
 
 /*
@@ -77,7 +83,7 @@ static int stress_sendfile(const stress_args_t *args)
 
 	if ((fdin = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
 		rc = exit_status(errno);
-		pr_fail("%s: open %s failed, errno=%d (%s)\n",
+		pr_err("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto dir_out;
 	}
@@ -88,20 +94,20 @@ static int stress_sendfile(const stress_args_t *args)
 #endif
 	if (ret < 0) {
 		rc = exit_status(errno);
-		pr_fail("%s: fallocate failed, errno=%d (%s)\n",
+		pr_err("%s: fallocate failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto dir_out;
 	}
 	(void)close(fdin);
 	if ((fdin = open(filename, O_RDONLY)) < 0) {
 		rc = exit_status(errno);
-		pr_fail("%s: open %s failed, errno=%d (%s)\n",
+		pr_err("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto dir_out;
 	}
 
 	if ((fdout = open("/dev/null", O_WRONLY)) < 0) {
-		pr_fail("%s: open /dev/null failed, errno=%d (%s)\n",
+		pr_err("%s: open /dev/null failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 		goto close_in;
@@ -117,7 +123,7 @@ static int stress_sendfile(const stress_args_t *args)
 		if (sendfile(fdout, fdin, &offset, sz) < 0) {
 			if (errno == ENOSYS) {
 				if (args->instance == 0)
-					pr_inf("%s: skipping stressor, sendfile not implemented\n",
+					pr_inf_skip("%s: skipping stressor, sendfile not implemented\n",
 						args->name);
 				rc = EXIT_NOT_IMPLEMENTED;
 				goto close_out;
@@ -173,7 +179,7 @@ close_out:
 close_in:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fdin);
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 dir_out:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)stress_temp_dir_rm_args(args);
@@ -185,6 +191,7 @@ stressor_info_t stress_sendfile_info = {
 	.stressor = stress_sendfile,
 	.class = CLASS_PIPE_IO | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
