@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -184,7 +179,9 @@ static int stress_session(const stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
-	while (keep_stressing()) {
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
+	while (keep_stressing(args)) {
 		pid_t pid;
 
 		pid = fork();
@@ -227,6 +224,7 @@ static int stress_session(const stress_args_t *args)
 		}
 		inc_counter(args);
 	}
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fds[0]);
 	(void)close(fds[1]);
 
