@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,18 +16,28 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
+#if defined(HAVE_SYS_FSUID_H)
+#include <sys/fsuid.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_GRP_H)
+#include <grp.h>
+#else
+UNEXPECTED
+#endif
+
+#if !defined(_DEFAULT_SOURCE)
 #define _DEFAULT_SOURCE 1
+#endif
+#if !defined(_BSD_SOURCE)
 #define _BSD_SOURCE 1
-
+#endif
 
 #define GIDS_MAX 	(1024)
 
@@ -132,6 +143,8 @@ static int stress_set(const stress_args_t *args)
 		uid_t uid;
 #if defined(HAVE_SETREUID)
 		uid_t bad_uid;
+#else
+		UNEXPECTED
 #endif
 		struct rlimit rlim;
 
@@ -180,6 +193,8 @@ static int stress_set(const stress_args_t *args)
 			if (!keep_stressing(args))
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_SETTIMEOFDAY)
@@ -198,6 +213,8 @@ static int stress_set(const stress_args_t *args)
 				}
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_GETPGRP) &&	\
@@ -210,6 +227,8 @@ static int stress_set(const stress_args_t *args)
 			if (!keep_stressing(args))
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 		/* getuid always succeeds */
@@ -245,6 +264,8 @@ static int stress_set(const stress_args_t *args)
 				(void)ret;
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_SETREUID)
@@ -262,7 +283,10 @@ static int stress_set(const stress_args_t *args)
 					args->name, errno, strerror(errno));
 			}
 		}
+#else
+		UNEXPECTED
 #endif
+
 #if defined(HAVE_SETREGID)
 #if defined(HAVE_GETRESGID)
 		{
@@ -287,9 +311,13 @@ static int stress_set(const stress_args_t *args)
 				}
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 		ret = setregid((gid_t)-1, (gid_t)-1);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_SETRESUID)
@@ -328,9 +356,13 @@ static int stress_set(const stress_args_t *args)
 				(void)ret;
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 		ret = setresuid((uid_t)-1, (uid_t)-1, (uid_t)-1);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_SETRESGID)
@@ -368,9 +400,13 @@ static int stress_set(const stress_args_t *args)
 				(void)ret;
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 		ret = setresgid((gid_t)-1, (gid_t)-1, (gid_t)-1);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_SETFSGID) && 	\
@@ -401,6 +437,8 @@ static int stress_set(const stress_args_t *args)
 				}
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_SETFSUID) && 	\
@@ -431,6 +469,8 @@ static int stress_set(const stress_args_t *args)
 				}
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(__NR_sgetmask) &&	\
@@ -466,6 +506,8 @@ static int stress_set(const stress_args_t *args)
 			if (!keep_stressing(args))
 				break;
 		}
+#else
+		UNEXPECTED
 #endif
 		/*
 		 *  Invalid setrlimit syscall with invalid
@@ -542,5 +584,6 @@ static int stress_set(const stress_args_t *args)
 stressor_info_t stress_set_info = {
 	.stressor = stress_set,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
