@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_SHELLSORT_SIZE	(1 * KB)
+#define MAX_SHELLSORT_SIZE	(4 * MB)
+#define DEFAULT_SHELLSORT_SIZE	(256 * KB)
+
 static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
 
@@ -180,7 +179,7 @@ static int stress_shellsort(const stress_args_t *args)
 	n = (size_t)shellsort_size;
 
 	if ((data = calloc(n, sizeof(*data))) == NULL) {
-		pr_fail("%s: malloc failed, out of memory\n", args->name);
+		pr_err("%s: malloc failed, out of memory\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -200,7 +199,7 @@ static int stress_shellsort(const stress_args_t *args)
 
 	/* This is expensive, do it once */
 	for (ptr = data, i = 0; i < n; i++) {
-		*ptr++ = stress_mwc32();
+		*ptr++ = (int32_t)stress_mwc32();
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -285,5 +284,6 @@ stressor_info_t stress_shellsort_info = {
 	.stressor = stress_shellsort,
 	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
