@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,36 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_MEMPOLICY_H)
+#include <linux/mempolicy.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_IPC_H)
+#include <sys/ipc.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_SHM_H)
+#include <sys/shm.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_SHM_SYSV_BYTES	(1 * MB)
+#define MAX_SHM_SYSV_BYTES	(256 * MB)
+#define DEFAULT_SHM_SYSV_BYTES	(8 * MB)
+
+#define MIN_SHM_SYSV_SEGMENTS	(1)
+#define MAX_SHM_SYSV_SEGMENTS	(128)
+#define DEFAULT_SHM_SYSV_SEGMENTS (8)
 
 static const stress_help_t help[] = {
 	{ NULL,	"shm-sysv N",		"start N workers that exercise System V shared memory" },
@@ -136,7 +159,10 @@ static int stress_shm_sysv_check(
  *  exercise_shmat()
  *	exercise shmat syscall with all possible values of arguments
  */
-static void exercise_shmat(const int shm_id, const size_t page_size)
+static void exercise_shmat(
+	const int shm_id,
+	const size_t page_size,
+	const size_t sz)
 {
 	void *addr;
 	uint64_t buffer[(page_size / sizeof(uint64_t)) + 1];
@@ -159,18 +185,53 @@ static void exercise_shmat(const int shm_id, const size_t page_size)
 	addr = shmat(shm_id, NULL, SHM_RDONLY);
 	if (addr != (void *)-1)
 		(void)shmdt(addr);
+#else
+	UNEXPECTED
 #endif
 
 #if defined(SHM_EXEC)
 	addr = shmat(shm_id, NULL, SHM_EXEC);
 	if (addr != (void *)-1)
 		(void)shmdt(addr);
+#else
+	UNEXPECTED
 #endif
 
 #if defined(SHM_RND)
 	addr = shmat(shm_id, NULL, SHM_RND);
-	if (addr != (void *)-1)
+	if (addr != (void *)-1) {
 		(void)shmdt(addr);
+
+#if defined(SHM_REMAP)
+		/*  Exercise valid remap */
+		addr = shmat(shm_id, addr, SHM_REMAP);
+		if (addr != (void *)-1) {
+			/* Exercise remap onto itself */
+			void *remap = shmat(shm_id, addr, SHM_REMAP | SHM_RDONLY);
+			if (remap != (void *)-1)
+				(void)shmdt(remap);
+			if (addr != remap)
+				(void)shmdt(addr);
+		}
+#else
+	UNEXPECTED
+#endif
+	}
+#else
+	UNEXPECTED
+#endif
+
+#if defined(SHM_RND)
+	addr = mmap(NULL, sz, PROT_READ,
+		    MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (addr != MAP_FAILED) {
+		(void)munmap(addr, sz);
+		addr = shmat(shm_id, addr, SHM_RND);
+		if (addr != (void *)-1)
+			(void)shmdt(addr);
+	}
+#else
+	UNEXPECTED
 #endif
 
 	/* Exercise shmat with SHM_REMAP flag on NULL address */
@@ -179,6 +240,8 @@ static void exercise_shmat(const int shm_id, const size_t page_size)
 	if (addr != (void *)-1) {
 		(void)shmdt(addr);
 	}
+#else
+	UNEXPECTED
 #endif
 
 	/* Exercise invalid shmat with unaligned page address */
@@ -198,6 +261,8 @@ static void exercise_shmat(const int shm_id, const size_t page_size)
 	addr = shmat(shm_id, unaligned, SHM_RND);
 	if (addr != (void *) -1)
 		(void)shmdt(addr);
+#else
+	UNEXPECTED
 #endif
 }
 
@@ -276,6 +341,8 @@ static void exercise_shmctl(const size_t sz, const stress_args_t *args)
 			pr_fail("%s: shmctl IPC_STAT unexpectedly succeeded on non-existent shared "
 				"memory segment, errno=%d (%s)\n", args->name, errno, strerror(errno));
 	}
+#else
+	UNEXPECTED
 #endif
 }
 
@@ -334,6 +401,8 @@ static void exercise_shmget(const size_t sz, const char *name, const bool cap_ip
 			"size argument, errno=%d (%s)\n", name, errno, strerror(errno));
 		(void)shmctl(shm_id, IPC_RMID, NULL);
 	}
+#else
+	/* UNEXPECTED */
 #endif
 
 #if defined(SHMMAX)
@@ -343,6 +412,8 @@ static void exercise_shmget(const size_t sz, const char *name, const bool cap_ip
 			"size argument, errno=%d (%s)\n", name, errno, strerror(errno));
 		(void)shmctl(shm_id, IPC_RMID, NULL);
 	}
+#else
+	/* UNEXPECTED */
 #endif
 
 #if defined(SHM_HUGETLB)
@@ -363,6 +434,8 @@ static void exercise_shmget(const size_t sz, const char *name, const bool cap_ip
 	shm_id = shmget(IPC_PRIVATE, sz, IPC_CREAT);
 	if (shm_id >= 0)
 		(void)shmctl(shm_id, IPC_RMID, NULL);
+#else
+	UNEXPECTED
 #endif
 
 	shm_id = shmget(key, sz, IPC_EXCL);
@@ -518,7 +591,7 @@ static int stress_shm_sysv_child(
 				goto reap;
 
 			for (count = 0; count < KEY_GET_RETRIES; count++) {
-				bool unique = true;
+				bool unique;
 				const int rnd =
 					stress_mwc32() % SIZEOF_ARRAY(shm_flags); /* cppcheck-suppress moduloofone */
 				const int rnd_flag = shm_flags[rnd] & mask;
@@ -529,6 +602,7 @@ static int stress_shm_sysv_child(
 				/* Get a unique key */
 				do {
 					size_t j;
+					unique = true;
 
 					if (!keep_stressing_flag())
 						goto reap;
@@ -543,7 +617,6 @@ static int stress_shm_sysv_child(
 					}
 					if (!keep_stressing_flag())
 						goto reap;
-
 				} while (!unique);
 
 				shm_id = shmget(key, sz,
@@ -583,7 +656,7 @@ static int stress_shm_sysv_child(
 				goto reap;
 			}
 
-			exercise_shmat(shm_id, args->page_size);
+			exercise_shmat(shm_id, args->page_size, sz);
 
 			addr = shmat(shm_id, NULL, 0);
 			if (addr == (char *) -1) {
@@ -625,6 +698,8 @@ static int stress_shm_sysv_child(
 					(void)ret;
 				}
 			}
+#else
+			UNEXPECTED
 #endif
 #if defined(IPC_STAT) &&	\
     defined(HAVE_SHMID_DS)
@@ -641,8 +716,12 @@ static int stress_shm_sysv_child(
 					ret = shmctl(shm_id, SHM_SET, &ds);
 					(void)ret;
 				}
+#else
+				/* UNEXPECTED */
 #endif
 			}
+#else
+			UNEXPECTED
 #endif
 #if defined(IPC_INFO) &&	\
     defined(HAVE_SHMINFO)
@@ -653,6 +732,8 @@ static int stress_shm_sysv_child(
 					pr_fail("%s: shmctl IPC_INFO failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 			}
+#else
+			UNEXPECTED
 #endif
 #if defined(SHM_INFO) &&	\
     defined(HAVE_SHMINFO)
@@ -663,6 +744,8 @@ static int stress_shm_sysv_child(
 					pr_fail("%s: shmctl SHM_INFO failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 			}
+#else
+			UNEXPECTED
 #endif
 #if defined(SHM_LOCK) &&	\
     defined(SHM_UNLOCK)
@@ -673,6 +756,8 @@ static int stress_shm_sysv_child(
 				(void)ret;
 
 			}
+#else
+			UNEXPECTED
 #endif
 			/*
 			 *  Exercise NUMA mem_policy on shm
@@ -711,6 +796,8 @@ static int stress_shm_sysv_child(
 					ret = shmctl(shm_ids[i], IPC_STAT, &ds);
 					(void)ret;
 				}
+#else
+				UNEXPECTED
 #endif
 				ret = shmdt(addrs[i]);
 				(void)ret;
@@ -938,6 +1025,7 @@ stressor_info_t stress_shm_sysv_info = {
 	.stressor = stress_shm_sysv,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
