@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
-#define SHM_NAME_LEN	128
+#define MIN_SHM_POSIX_BYTES	(1 * MB)
+#define MAX_SHM_POSIX_BYTES	(1 * GB)
+#define DEFAULT_SHM_POSIX_BYTES	(8 * MB)
+
+#define MIN_SHM_POSIX_OBJECTS	(1)
+#define MAX_SHM_POSIX_OBJECTS	(128)
+#define DEFAULT_SHM_POSIX_OBJECTS (32)
+
+#define SHM_NAME_LEN		(128)
 
 typedef struct {
 	ssize_t	index;
@@ -134,6 +137,7 @@ static int stress_shm_posix_child(
 	/* Make sure this is killable by OOM killer */
 	stress_set_oom_adjustment(args->name, true);
 
+	(void)memset(&msg, 0, sizeof(msg));
 	(void)memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = SIG_IGN;
 #if defined(SA_NOCLDWAIT)
@@ -429,7 +433,7 @@ again:
 				msg.shm_name[SHM_NAME_LEN - 1] = '\0';
 				(void)shim_strlcpy(shm_name, msg.shm_name, SHM_NAME_LEN);
 			}
-			(void)kill(pid, SIGKILL);
+			(void)stress_killpid(pid);
 			(void)shim_waitpid(pid, &status, 0);
 			if (WIFSIGNALED(status)) {
 				if ((WTERMSIG(status) == SIGKILL) ||
@@ -485,6 +489,7 @@ stressor_info_t stress_shm_info = {
 	.stressor = stress_shm,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
