@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_SIGNALFD_H)
+#include <sys/signalfd.h>
+#else
+UNEXPECTED
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"sigfd N",	"start N workers reading signals via signalfd reads " },
 	{ NULL,	"sigfd-ops N",	"stop after N bogo signalfd reads" },
@@ -62,7 +63,7 @@ static int stress_sigfd(const stress_args_t *args)
 	(void)sigemptyset(&mask);
 	(void)sigaddset(&mask, SIGRTMIN);
 	if (sigprocmask(SIG_BLOCK, &mask, NULL) < 0) {
-		pr_fail("%s: sigprocmask failed, errno=%d (%s)\n",
+		pr_err("%s: sigprocmask failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -109,7 +110,7 @@ again:
 			goto again;
 		if (!keep_stressing(args))
 			goto finish;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -190,6 +191,7 @@ finish:
 stressor_info_t stress_sigfd_info = {
 	.stressor = stress_sigfd,
 	.class = CLASS_INTERRUPT | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
