@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-put.h"
+
+#if defined(HAVE_FENV_H)
+#include <fenv.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_FLOAT_H)
+#include <float.h>
+#else
+UNEXPECTED
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"sigfpe N",	"start N workers generating floating point math faults" },
@@ -135,30 +144,44 @@ static int stress_sigfpe(const stress_args_t *args)
 	 */
 	static const stress_fpe_err_t fpe_errs[] = {
 #if defined(FPE_INTDIV)
-		{ SNG_INTDIV,	FPE_INTDIV },
+		{ SNG_INTDIV,	FPE_INTDIV },	/* can be trapped */
+#else
+		UNEXPECTED
 #endif
 #if defined(FPE_FLTDIV)
-		{ SNG_FLTDIV,	FPE_FLTDIV },
+		{ SNG_FLTDIV,	FPE_FLTDIV },	/* NaN or Inf, no trap */
+#else
+		UNEXPECTED
 #endif
 #if defined(FE_DIVBYZERO) &&	\
     defined(FPE_FLTDIV)
-		{ FE_DIVBYZERO,	FPE_FLTDIV },
+		{ FE_DIVBYZERO,	FPE_FLTDIV },	/* Nan or Inf, no trap */
+#else
+		UNEXPECTED
 #endif
 #if defined(FE_INEXACT) &&	\
     defined(FPE_FLTRES)
-		{ FE_INEXACT,	FPE_FLTRES },
+		{ FE_INEXACT,	FPE_FLTRES },	/* Floating-point inexact result, no trap */
+#else
+		UNEXPECTED
 #endif
 #if defined(FE_INVALID) &&	\
     defined(FPE_FLTINV)
-		{ FE_INVALID,	FPE_FLTINV },
+		{ FE_INVALID,	FPE_FLTINV },	/* Invalid floating-point operation */
+#else
+		UNEXPECTED
 #endif
 #if defined(FE_OVERFLOW) &&	\
     defined(FPE_FLTOVF)
-		{ FE_OVERFLOW,	FPE_FLTOVF },
+		{ FE_OVERFLOW,	FPE_FLTOVF },	/* Floating-point overflow  */
+#else
+		UNEXPECTED
 #endif
 #if defined(FE_UNDERFLOW) &&	\
     defined(FPE_FLTUND)
-		{ FE_UNDERFLOW,	FPE_FLTUND },
+		{ FE_UNDERFLOW,	FPE_FLTUND },	/* Floating-point underflow */
+#else
+		UNEXPECTED
 #endif
 	};
 
@@ -176,7 +199,7 @@ static int stress_sigfpe(const stress_args_t *args)
 
 	ret = sigaction(SIGFPE, &action, NULL);
 	if (ret < 0) {
-		pr_fail("%s: sigaction SIGFPE: errno=%d (%s)\n",
+		pr_err("%s: sigaction SIGFPE: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -229,6 +252,11 @@ static int stress_sigfpe(const stress_args_t *args)
 
 			switch (exception) {
 			case SNG_FLTDIV:
+				/*
+				 * This is undefined behaviour, so NaN or infinity should
+				 * occur, should not generate a division by zero trap but
+				 * exercise it anyhow just to cover this scenario
+				 */
 				stress_float_put((float)1.0 / (float)zero);
 				break;
 			case SNG_INTDIV:
@@ -239,6 +267,9 @@ static int stress_sigfpe(const stress_args_t *args)
 				(void)feraiseexcept((int)exception);
 				break;
 			}
+#if defined(STRESS_ARCH_M68K)
+			shim_sched_yield();
+#endif
 		}
 		i++;
 		i %= SIZEOF_ARRAY(fpe_errs);
@@ -253,6 +284,7 @@ static int stress_sigfpe(const stress_args_t *args)
 stressor_info_t stress_sigfpe_info = {
 	.stressor = stress_sigfpe,
 	.class = CLASS_INTERRUPT | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
