@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#else
+UNEXPECTED
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"sigio N",	"start N workers that exercise SIGIO signals" },
 	{ NULL,	"sigio-ops N",	"stop after N bogo sigio signals" },
@@ -39,6 +40,7 @@ static const stress_help_t help[] = {
 #define BUFFER_SIZE	(4096)
 
 static volatile int got_err;
+static volatile uint64_t async_sigs;
 static int rd_fd;
 static const stress_args_t *sigio_args;
 static pid_t pid;
@@ -54,19 +56,28 @@ static void MLOCKED_TEXT stress_sigio_handler(int signum)
 
 	(void)signum;
 
-	if (!keep_stressing_flag() || (stress_time_now() > time_end))
-		return;
+	async_sigs++;
+
 
 	if (rd_fd > 0) {
-		ssize_t ret;
+		/*
+		 *  Data is ready, so drain as much as possible
+		 */
+		while (keep_stressing_flag() &&  (stress_time_now() < time_end)) {
+			ssize_t ret;
 
-		got_err = 0;
-		ret = read(rd_fd, buffer, sizeof(buffer));
-		if ((ret < 0) && (errno != EAGAIN))
-			got_err = errno;
-		else if (sigio_args)
-			inc_counter(sigio_args);
-		(void)shim_sched_yield();
+			got_err = 0;
+			errno = 0;
+			ret = read(rd_fd, buffer, sizeof(buffer));
+			if (ret < 0) {
+				if (errno != EAGAIN)
+					got_err = errno;
+				break;
+			}
+
+			if (sigio_args)
+				inc_counter(sigio_args);
+		}
 	}
 }
 
@@ -80,21 +91,27 @@ static int stress_sigio(const stress_args_t *args)
 
 	rd_fd = -1;
 	sigio_args = args;
+	double t_start, t_delta;
 	pid = -1;
 
 	time_end = stress_time_now() + (double)g_opt_timeout;
 	if (pipe(fds) < 0) {
-		pr_fail("%s: pipe failed, errno=%d (%s)\n",
+		pr_err("%s: pipe failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return rc;
 	}
 	rd_fd = fds[0];
 
+#if defined(F_SETPIPE_SZ)
+	(void)fcntl(fds[0], F_SETPIPE_SZ, BUFFER_SIZE * 2);
+	(void)fcntl(fds[1], F_SETPIPE_SZ, BUFFER_SIZE * 2);
+#endif
+
 #if !defined(__minix__)
 	ret = fcntl(fds[0], F_SETOWN, getpid());
 	if (ret < 0) {
 		if (errno != EINVAL) {
-			pr_fail("%s: fcntl F_SETOWN failed, errno=%d (%s)\n",
+			pr_err("%s: fcntl F_SETOWN failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto err;
 		}
@@ -102,20 +119,26 @@ static int stress_sigio(const stress_args_t *args)
 #endif
 	flags = fcntl(fds[0], F_GETFL);
 	if (flags < 0) {
-		pr_fail("%s: fcntl F_GETFL failed, errno=%d (%s)\n",
+		pr_err("%s: fcntl F_GETFL failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto err;
 	}
 
+	async_sigs = 0;
+
+again:
 	pid = fork();
 	if (pid < 0) {
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		if (stress_redo_fork(errno))
+			goto again;
+		if (!keep_stressing(args))
+			goto finish;
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto err;
 	} else if (pid == 0) {
 		/* Child */
-
-		char buffer[BUFFER_SIZE >> 4];
+		char buffer[BUFFER_SIZE];
 
 		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
@@ -133,12 +156,10 @@ static int stress_sigio(const stress_args_t *args)
 			n = write(fds[1], buffer, sizeof buffer);
 			if (n < 0)
 				break;
-			(void)shim_sched_yield();
 		}
 		(void)close(fds[1]);
 		_exit(1);
 	}
-
 	/* Parent */
 	(void)close(fds[1]);
 	fds[1] = -1;
@@ -150,10 +171,11 @@ static int stress_sigio(const stress_args_t *args)
 
 	ret = fcntl(fds[0], F_SETFL, flags | O_ASYNC | O_NONBLOCK);
 	if (ret < 0) {
-		pr_fail("%s: fcntl F_SETFL failed, errno=%d (%s)\n",
+		pr_err("%s: fcntl F_SETFL failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto reap;
 	}
+	t_start = stress_time_now();
 	do {
 		struct timeval timeout;
 
@@ -162,12 +184,20 @@ static int stress_sigio(const stress_args_t *args)
 
 		(void)select(0, NULL, NULL, NULL, &timeout);
 		if (got_err) {
-			pr_inf("%s: read error, errno=%d (%s)\n",
-				args->name, got_err, strerror(got_err));
+			if (got_err != EINTR)
+				pr_inf("%s: read error, errno=%d (%s)\n",
+					args->name, got_err, strerror(got_err));
 			break;
 		}
 	} while (keep_stressing(args));
 
+	t_delta = stress_time_now() - t_start;
+
+	if (t_delta > 0.0) 
+		pr_inf("%s: %.2f SIGIO signals/sec\n",
+			args->name, (double)async_sigs / t_delta);
+
+finish:
 	/*  And ignore IO signals from now on */
 	ret = stress_sighandler(args->name, SIGIO, SIG_IGN, NULL);
 	(void)ret;
