@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -50,8 +45,16 @@ typedef void (*shim_sighandler_t)(int);
  */
 static shim_sighandler_t shim_signal(int signum, shim_sighandler_t handler)
 {
-#if defined(__NR_signal)
-	return (shim_sighandler_t)syscall(__NR_signal, signum, handler);
+#if defined(__NR_signal) &&	\
+    !defined(__sparc__)
+	shim_sighandler_t ret;
+
+	ret = (shim_sighandler_t)syscall(__NR_signal, signum, handler);
+	if ((ret == SIG_ERR) && (errno == ENOSYS)) {
+		errno = 0;
+		ret = signal(signum, handler);
+	}
+	return ret;
 #else
 	return signal(signum, handler);
 #endif
@@ -113,7 +116,7 @@ static int stress_signal(const stress_args_t *args)
 			break;
 		}
 		if (tmp != counter) {
-			pr_err("%s: setting of SIG_DFL unexpectedly triggered a SIGCHLD\n",
+			pr_fail("%s: setting of SIG_DFL unexpectedly triggered a SIGCHLD\n",
 				args->name);
 		}
 
