@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -83,7 +78,7 @@ again:
 			goto again;
 		if (!keep_stressing(args))
 			goto finish;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -97,7 +92,7 @@ again:
 		(void)sigemptyset(&mask);
 		(void)sigaddset(&mask, SIGUSR1);
 		if (sigprocmask(SIG_SETMASK, &mask, NULL) < 0) {
-			pr_fail("%s: sigprocmask failed, errno=%d (%s)\n",
+			pr_err("%s: sigprocmask failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			_exit(EXIT_FAILURE);
 		}
@@ -159,6 +154,9 @@ again:
 				info.si_uid = myuid;
 				info.si_value = s;
 				(void)shim_rt_sigqueueinfo(pid, SIGUSR1, &info);
+
+				/* Exercise invalid signal */
+				(void)shim_rt_sigqueueinfo(pid, 0, &info);
 			}
 #endif
 			inc_counter(args);
