@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -66,9 +61,11 @@ static int stress_sigrt(const stress_args_t *args)
 again:
 		pids[i] = fork();
 		if (pids[i] < 0) {
-			if (keep_stressing_flag() && (errno == EAGAIN))
+			if (stress_redo_fork(errno))
 				goto again;
-			pr_fail("%s: fork failed, errno=%d (%s)\n",
+			if (!keep_stressing(args))
+				goto reap;
+			pr_err("%s: fork failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto reap;
 		} else if (pids[i] == 0) {
