@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-cpu.h"
+#include "core-nt-store.h"
 
 static sigjmp_buf jmp_env;
 #if defined(SA_SIGINFO)
@@ -42,7 +40,7 @@ static const stress_help_t help[] = {
  *	SEGV handler
  */
 #if defined(SA_SIGINFO)
-static void MLOCKED_TEXT stress_segvhandler(
+static void NORETURN MLOCKED_TEXT stress_segvhandler(
 	int num,
 	siginfo_t *info,
 	void *ucontext)
@@ -57,7 +55,7 @@ static void MLOCKED_TEXT stress_segvhandler(
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
 }
 #else
-static void MLOCKED_TEXT stress_segvhandler(int signum)
+static void NORETURN MLOCKED_TEXT stress_segvhandler(int signum)
 {
 	(void)signum;
 
@@ -79,7 +77,8 @@ static NOINLINE OPTIMIZE0 void stress_sigsegv_x86_trap(void)
 {
 	int a = 1, b = 2;
 
-	asm(".byte 0xf2\n\t"
+	 __asm__ __volatile__(
+	    ".byte 0xf2\n\t"
 	    ".byte 0xf2\n\t"
 	    ".byte 0xf2\n\t"
 	    ".byte 0xf2\n\t"
@@ -116,6 +115,53 @@ static NOINLINE OPTIMIZE0 void stress_sigsegv_x86_trap(void)
 }
 #endif
 
+#if defined(STRESS_ARCH_X86) &&	\
+    defined(__linux__)
+/*
+ *  stress_sigsegv_x86_int88()
+ *	making an illegal int trap causes a SIGSEGV on
+ *	x86 linux implementations, so exercise this
+ */
+static NOINLINE OPTIMIZE0 void stress_sigsegv_x86_int88(void)
+{
+	__asm__ __volatile__("int $88\n");
+	/*
+	 *  Should not get here
+	 */
+}
+#endif
+
+#if defined(STRESS_ARCH_X86) &&	\
+    defined(__linux__)
+static void stress_sigsegv_rdmsr(void)
+{
+	uint32_t ecx = 0x00000010, eax, edx;
+
+	__asm__ __volatile__("rdmsr" : "=a" (eax), "=d" (edx) : "c" (ecx));
+	/*
+	 *  Should not get here
+	 */
+}
+#endif
+
+#if defined(STRESS_ARCH_X86) &&		\
+    defined(__linux__) &&		\
+    defined(HAVE_NT_STORE128) &&	\
+    defined(HAVE_INT128_T)
+static void stress_sigsegv_misaligned128nt(void)
+{
+	/* Misaligned non-temporal 128 bit store */
+
+	__uint128_t buffer[2];
+	__uint128_t *ptr = (__uint128_t *)(((uint8_t *)buffer) + 1);
+
+	stress_nt_store128(ptr, ~(__uint128_t)0);
+	/*
+	 *  Should not get here
+	 */
+}
+#endif
+
 /*
  *  stress_sigsegv
  *	stress by generating segmentation faults by
@@ -127,6 +173,14 @@ static int stress_sigsegv(const stress_args_t *args)
 	NOCLOBBER int rc = EXIT_FAILURE;
 #if defined(SA_SIGINFO)
 	const bool verify = (g_opt_flags & OPT_FLAGS_VERIFY);
+#endif
+#if defined(STRESS_ARCH_X86) &&		\
+   defined(__linux__)	
+	const bool has_msr = stress_cpu_x86_has_msr();
+#if defined(HAVE_NT_STORE128) &&	\
+    defined(HAVE_INT128_T)
+	const bool has_sse2 = stress_cpu_x86_has_sse2();
+#endif
 #endif
 
 	/* Allocate read only page */
@@ -186,7 +240,7 @@ static int stress_sigsegv(const stress_args_t *args)
 #if defined(SA_SIGINFO)
 			if (verify && fault_addr && fault_addr != ptr) {
 				pr_fail("%s: expecting fault address %p, got %p instead\n",
-					args->name, fault_addr, ptr);
+					args->name, fault_addr, (void *)ptr);
 			}
 			if (verify &&
 			    (signo != -1) &&
@@ -208,13 +262,34 @@ static int stress_sigsegv(const stress_args_t *args)
 			code = -1;
 			fault_addr = 0;
 #endif
-			/* Trip a SIGSEGV/SIGILL/SIGBUS */
+			switch (stress_mwc8() >> 6) {
 #if defined(STRESS_ARCH_X86) &&	\
     defined(__linux__)
-			if (stress_mwc8() > 250)
+			case 0:
+				/* Trip a SIGSEGV/SIGILL/SIGBUS */
 				stress_sigsegv_x86_trap();
+				CASE_FALLTHROUGH;
+			case 1:
+				/* Illegal int $88 */
+				stress_sigsegv_x86_int88();
+				CASE_FALLTHROUGH;
+			case 2:
+				/* Privileged instruction -> SIGSEGV */
+				if (has_msr)
+					stress_sigsegv_rdmsr();
+				CASE_FALLTHROUGH;
+#if defined(HAVE_NT_STORE128) &&	\
+    defined(HAVE_INT128_T)
+			case 3:
+				if (has_sse2)
+					stress_sigsegv_misaligned128nt();
+				CASE_FALLTHROUGH;
 #endif
-			*ptr = 0;
+#endif
+			default:
+				*ptr = 0;
+				break;
+			}
 		}
 	}
 	rc = EXIT_SUCCESS;
@@ -229,5 +304,8 @@ tidy:
 stressor_info_t stress_sigsegv_info = {
 	.stressor = stress_sigsegv,
 	.class = CLASS_INTERRUPT | CLASS_OS,
+#if defined(SA_SIGINFO)
+	.verify = VERIFY_OPTIONAL,
+#endif
 	.help = help
 };
