@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,23 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MAX_SIGSUSPEND_PIDS	(4)
+#define CACHE_STRIDE_SHIFT	(6)
+
 static const stress_help_t help[] = {
 	{ NULL,	"sigsuspend N",	    "start N workers exercising sigsuspend" },
 	{ NULL,	"sigsuspend-ops N", "stop after N bogo sigsuspend wakes" },
 	{ NULL,	NULL,		    NULL }
 };
 
-#define CACHE_STRIDE_SHIFT	(6)
-
 /*
  *  stress_usr1_handler()
  *      SIGUSR1 handler
@@ -63,7 +59,7 @@ static int stress_sigsuspend(const stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (counters == MAP_FAILED) {
-		pr_fail("%s: mmap failed, errno=%d (%s)\n",
+		pr_err("%s: mmap failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
@@ -78,9 +74,11 @@ static int stress_sigsuspend(const stress_args_t *args)
 again:
 		pid[n] = fork();
 		if (pid[n] < 0) {
-			if (keep_stressing_flag() && (errno == EAGAIN))
+			if (stress_redo_fork(errno))
 				goto again;
-			pr_fail("%s: fork failed, errno=%d (%s)\n",
+			if (!keep_stressing(args))
+				goto reap;
+			pr_err("%s: fork failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto reap;
 		} else if (pid[n] == 0) {
