@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -11,19 +12,24 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_SLEEP		(1)
+#define MAX_SLEEP		(30000)
+#define DEFAULT_SLEEP		(1024)
+
 #if defined(HAVE_LIB_PTHREAD)
 
 typedef struct {
@@ -82,7 +88,7 @@ static void *stress_pthread_func(void *c)
 
 	while (keep_stressing(args) &&
 	       !thread_terminate &&
-	       (!max_ops || ctxt->counter < max_ops)) {
+	       (!max_ops || (ctxt->counter < max_ops))) {
 		struct timespec tv;
 #if defined(HAVE_SYS_SELECT_H)
 		struct timeval timeout;
@@ -110,6 +116,26 @@ static void *stress_pthread_func(void *c)
 			break;
 		if (shim_usleep(10000) < 0)
 			break;
+#if defined(HAVE_PSELECT)
+		tv.tv_sec = 0;
+		tv.tv_nsec = 1;
+		if (pselect(0, NULL, NULL, NULL, &tv, NULL) < 0)
+			goto skip_pselect;
+		tv.tv_sec = 0;
+		tv.tv_nsec = 10;
+		if (pselect(0, NULL, NULL, NULL, &tv, NULL) < 0)
+			goto skip_pselect;
+		tv.tv_sec = 0;
+		tv.tv_nsec = 100;
+		if (pselect(0, NULL, NULL, NULL, &tv, NULL) < 0)
+			goto skip_pselect;
+		tv.tv_sec = 0;
+		tv.tv_nsec = 1000;
+		if (pselect(0, NULL, NULL, NULL, &tv, NULL) < 0)
+			goto skip_pselect;
+
+skip_pselect:
+#endif
 
 #if defined(HAVE_SYS_SELECT_H)
 		timeout.tv_sec = 0;
