@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2021 Canonical, Ltd.
+ * Copyright (C) 2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-cpu.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_SYS_IO_H)
+#include <sys/io.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"smi N",	"start N workers that trigger SMIs" },
@@ -31,46 +33,13 @@ static const stress_help_t help[] = {
 };
 
 #if defined(STRESS_ARCH_X86) &&		\
-    defined(HAVE_LIBKMOD_H) &&		\
-    defined(HAVE_LIB_KMOD) &&		\
-    defined(HAVE_CPUID_H) &&		\
-    defined(HAVE___GET_CPUID) &&	\
     defined(HAVE_IOPORT) &&		\
     defined(HAVE_SYS_IO_H) &&		\
     defined(__linux__)
 
 #define MSR_SMI_COUNT   (0x00000034)
 #define APM_PORT	(0xb2)
-
-/*
- *  stress_smi_zero_regs
- *	inline helper to clear a..d
- */
-static inline void stress_smi_zero_regs(
-	uint32_t *a,
-	uint32_t *b,
-	uint32_t *c,
-	uint32_t *d)
-{
-	*a = 0;
-	*b = 0;
-	*c = 0;
-	*d = 0;
-}
-
-/*
- *  stress_smi_cpu_has_msr()
- *	return non-zero if CPU has MSR support
- */
-static inline int stress_smi_cpu_has_msr(void)
-{
-	uint32_t a, b, c, d;
-
-	stress_smi_zero_regs(&a, &b, &c, &d);
-	__get_cpuid(1, &a, &b, &c, &d);
-	return d & (1 << 5);
-
-}
+#define STRESS_SMI_NOP	(0x90)	/* SMI No-op command */
 
 /*
  *  stress_smi_supported()
@@ -96,7 +65,7 @@ static int stress_smi_supported(const char *name)
                         "rights for this stressor\n", name);
                 return -1;
 	}
-	if (!stress_smi_cpu_has_msr()) {
+	if (!stress_cpu_x86_has_msr()) {
                 pr_inf_skip("%s stressor will be skipped, "
                         "CPU cannot read model specific registers (MSR)\n",
                         name);
@@ -116,7 +85,7 @@ static int stress_smi_readmsr64(const int cpu, const uint32_t reg, uint64_t *val
 	int ret;
 
 	*val = ~0;
-	snprintf(buffer, sizeof(buffer), "/dev/cpu/%d/msr", cpu);
+	(void)snprintf(buffer, sizeof(buffer), "/dev/cpu/%d/msr", cpu);
 	if ((fd = open(buffer, O_RDONLY)) < 0)
 		return -1;
 
@@ -160,14 +129,25 @@ static int stress_smi(const stress_args_t *args)
 	int ret, rc = EXIT_SUCCESS;
 	bool already_loaded = false;
 	bool read_msr_ok = true;
-	uint64_t s1 = 0;
+	bool load_module = false;
+	uint64_t s1 = 0, val;
 	double d1 = 0.0;
 	const int cpus = stress_get_processors_online();
 
-	if (args->instance == 0) {
+	/*
+	 *  If MSR can't be read maybe we need to load
+	 *  the module to do so
+	 */
+	if (stress_smi_readmsr64(0, MSR_SMI_COUNT, &val) < 0)
+		load_module = true;
+
+	/*
+	 *  Module load failure is not a problem, it just means
+	 *  we can't get the SMI count
+	 */
+	if (load_module && (args->instance == 0)) {
 		ret = stress_module_load(args->name, "msr", NULL, &already_loaded);
-		if (ret < 0)
-			return EXIT_NO_RESOURCE;
+		(void)ret;
 	}
 
 	if (ioperm(APM_PORT, 2, 1) < 0) {
@@ -185,7 +165,7 @@ static int stress_smi(const stress_args_t *args)
 	}
 
 	do {
-		outb(1, 0xb2);
+		outb(STRESS_SMI_NOP, APM_PORT);
 		inc_counter(args);
 	} while (keep_stressing(args));
 
@@ -219,8 +199,10 @@ static int stress_smi(const stress_args_t *args)
 				args->name);
 		}
 
-		ret = stress_module_unload(args->name, "msr", already_loaded);
-		(void)ret;
+		if (load_module) {
+			ret = stress_module_unload(args->name, "msr", already_loaded);
+			(void)ret;
+		}
 	}
 
 	return rc;
