@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,46 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
+
+#if defined(HAVE_LINUX_SOCKIOS_H)
+#include <linux/sockios.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_NET_IF_H)
+#include <net/if.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_NETINET_TCP_H)
+#include <netinet/tcp.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UN_H)
+#include <sys/un.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UTSNAME_H)
+#include <sys/utsname.h>
+#else
+UNEXPECTED
+#endif
+
+#include <netinet/in.h>
+#include <arpa/inet.h>
+
+#define MIN_SOCKET_PORT		(1024)
+#define MAX_SOCKET_PORT		(65535)
+#define DEFAULT_SOCKET_PORT	(5000)
 
 #define MMAP_BUF_SIZE		(65536)
 #define MMAP_IO_SIZE		(8192)	/* Must be less or equal to 8192 */
@@ -102,6 +135,8 @@ static int stress_set_socket_opts(const char *opt)
 		{ "sendmsg",	SOCKET_OPT_SENDMSG },
 #if defined(HAVE_SENDMMSG)
 		{ "sendmmsg",	SOCKET_OPT_SENDMMSG },
+#else
+		UNEXPECTED
 #endif
 		{ NULL,		0 }
 	};
@@ -719,7 +754,6 @@ retry:
 			struct msghdr msg;
 			struct iovec vec[MMAP_IO_SIZE / 16];
 #if defined(HAVE_RECVMMSG)
-			unsigned int msg_len = 0;
 			struct mmsghdr msgvec[MSGVEC_SIZE];
 			const int max_opt = 3;
 #else
@@ -788,7 +822,6 @@ retry:
 				for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
 					vec[j].iov_base = buf;
 					vec[j].iov_len = i;
-					msg_len += i;
 				}
 				for (i = 0; i < MSGVEC_SIZE; i++) {
 					msgvec[i].msg_hdr.msg_iov = vec;
@@ -837,7 +870,7 @@ retry:
 	if (socket_domain == AF_UNIX) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
 	/* Inform parent we're all done */
@@ -961,7 +994,6 @@ static int stress_sock_server(
 			struct iovec vec[MMAP_IO_SIZE / 16];
 #if defined(HAVE_SENDMMSG)
 			struct mmsghdr msgvec[MSGVEC_SIZE];
-			unsigned int msg_len = 0;
 #endif
 
 			len = sizeof(saddr);
@@ -1060,7 +1092,6 @@ static int stress_sock_server(
 				for (j = 0, i = 16; i < MMAP_IO_SIZE; i += 16, j++) {
 					vec[j].iov_base = buf;
 					vec[j].iov_len = i;
-					msg_len += i;
 				}
 				for (i = 0; i < MSGVEC_SIZE; i++) {
 					msgvec[i].msg_hdr.msg_iov = vec;
@@ -1119,7 +1150,7 @@ die:
 	if (addr && (socket_domain == AF_UNIX)) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
 	if (pid) {
@@ -1214,7 +1245,7 @@ again:
 			rc = EXIT_SUCCESS;
 			goto finish;
 		}
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -1250,5 +1281,6 @@ stressor_info_t stress_sock_info = {
 	.stressor = stress_sock,
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
