@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +16,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
 
+#if defined(HAVE_SYS_XATTR_H)
+#include <sys/xattr.h>
+#undef HAVE_ATTR_XATTR_H
+#elif defined(HAVE_ATTR_XATTR_H)
+#include <attr/xattr.h>
+#endif
+/*  Sanity check */
+#if defined(HAVE_SYS_XATTR_H) &&        \
+    defined(HAVE_ATTR_XATTR_H)
+#error cannot have both HAVE_SYS_XATTR_H and HAVE_ATTR_XATTR_H
+#endif
+
+#define DEFAULT_SOCKABUSE_PORT	(12000)
 #define MSGVEC_SIZE		(4)
+#define SOCKET_BUF		(8192)	/* Socket I/O buffer size */
 
 static const stress_help_t help[] = {
 	{ "S N", "sock N",		"start N workers exercising socket I/O" },
@@ -66,16 +76,22 @@ static void stress_sockabuse_fd(const int fd)
 	VOID_RET(int, fchown(fd, uid, gid));
 #if defined(F_GETFD)
 	VOID_RET(int, fcntl(fd, F_GETFD));
+#else
+	UNEXPECTED
 #endif
 #if defined(HAVE_FLOCK) &&      \
     defined(LOCK_UN)
 	VOID_RET(int, flock(fd, LOCK_UN));
+#else
+	UNEXPECTED
 #endif
 #if (defined(HAVE_SYS_XATTR_H) ||       \
      defined(HAVE_ATTR_XATTR_H)) &&     \
     defined(HAVE_SETXATTR) &&		\
     defined(XATTR_CREATE)
 	VOID_RET(ssize_t, shim_fsetxattr(fd, "test", "value", 5, XATTR_CREATE));
+#else
+	UNEXPECTED
 #endif
 	VOID_RET(int, fstat(fd, &statbuf));
 	VOID_RET(int, ftruncate(fd, 0));
@@ -87,6 +103,8 @@ static void stress_sockabuse_fd(const int fd)
 
 		VOID_RET(ssize_t, shim_flistxattr(fd, list, sizeof(list)));
 	}
+#else
+	UNEXPECTED
 #endif
 #if defined(HAVE_FUTIMENS)
 	{
@@ -101,6 +119,8 @@ static void stress_sockabuse_fd(const int fd)
 			VOID_RET(int, futimens(fd, timespec));
 		}
 	}
+#else
+	UNEXPECTED
 #endif
 	addrlen = sizeof(addr);
 	VOID_RET(int, getpeername(fd, &addr, &addrlen));
@@ -110,9 +130,13 @@ static void stress_sockabuse_fd(const int fd)
 
 		VOID_RET(int, ioctl(fd, FIONREAD, &n));
 	}
+#else
+	UNEXPECTED
 #endif
 #if defined(SEEK_SET)
 	VOID_RET(off_t, lseek(fd, 0, SEEK_SET));
+#else
+	UNEXPECTED
 #endif
 	VOID_RET(int, shim_pidfd_send_signal(fd, SIGUSR1, NULL, 0));
 	ptr = mmap(NULL, 4096, PROT_READ, MAP_SHARED, fd, 0);
@@ -363,9 +387,13 @@ static int stress_sockabuse(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() && (errno == EAGAIN))
+		if (stress_redo_fork(errno))
 			goto again;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		if (!keep_stressing(args)) {
+			rc = EXIT_SUCCESS;
+			goto finish; 
+		}
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -373,16 +401,18 @@ again:
 			socket_port);
 		_exit(rc);
 	} else {
-
 		rc = stress_sockabuse_server(args, pid, ppid,
 			socket_port);
-		stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	}
+finish:
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	return rc;
 }
 
 stressor_info_t stress_sockabuse_info = {
 	.stressor = stress_sockabuse,
 	.class = CLASS_NETWORK | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
