@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
+
+#if defined(HAVE_SYS_UN_H)
+#include <sys/un.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_SOCKET_FD_PORT	(1024)
+#define MAX_SOCKET_FD_PORT	(65535)
+#define DEFAULT_SOCKET_FD_PORT	(8000)
 
 static const stress_help_t help[] = {
 	{ NULL,	"sockfd N",	 "start N workers sending file descriptors over sockets" },
@@ -227,8 +233,10 @@ retry:
 	if (addr) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
+#else
+	UNEXPECTED
 #endif
 
 	ret = EXIT_SUCCESS;
@@ -351,7 +359,7 @@ die:
 	if (addr) {
 		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-		(void)unlink(addr_un->sun_path);
+		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
 
@@ -398,12 +406,13 @@ static int stress_sockfd(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (errno == EAGAIN) {
-			if (keep_stressing_flag())
-				goto again;
-			return EXIT_NO_RESOURCE;
+		if (stress_redo_fork(errno))
+			goto again;
+		if (!keep_stressing(args)) {
+			ret = EXIT_SUCCESS;
+			goto finish;
 		}
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -413,6 +422,7 @@ again:
 		ret = stress_socket_server(args, pid, ppid, max_fd, socket_fd_port);
 	}
 
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return ret;
@@ -422,6 +432,7 @@ stressor_info_t stress_sockfd_info = {
 	.stressor = stress_sockfd,
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
