@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
 
+#if defined(HAVE_NETINET_TCP_H)
+#include <netinet/tcp.h>
+#else
+UNEXPECTED
+#endif
+
+#define DEFAULT_SOCKET_MANY_PORT (11000)
 #define SOCKET_MANY_BUF		(8)
 #define SOCKET_MANY_FDS		(100000)
 
@@ -239,6 +242,8 @@ static int stress_sockmany_server(
 				ret = setsockopt(fd, SOL_TCP, TCP_QUICKACK, &one, sizeof(one));
 				(void)ret;
 			}
+#else
+			UNEXPECTED
 #endif
 			(void)memset(buf, 'A' + (msgs % 26), sizeof(buf));
 			sret = send(sfd, buf, sizeof(buf), 0);
@@ -299,9 +304,13 @@ static int stress_sockmany(const stress_args_t *args)
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (keep_stressing_flag() && (errno == EAGAIN))
+		if (stress_redo_fork(errno))
 			goto again;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		if (!keep_stressing(args)) {
+			rc = EXIT_SUCCESS;
+			goto finish;
+		}
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -312,6 +321,7 @@ again:
 	}
 	pr_dbg("%s: %d sockets opened at one time\n", args->name, sock_fds->max_fd);
 
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	(void)munmap((void *)sock_fds, args->page_size);
@@ -321,5 +331,6 @@ again:
 stressor_info_t stress_sockmany_info = {
 	.stressor = stress_sockmany,
 	.class = CLASS_NETWORK | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
