@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +16,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
 #define MAX_SOCKET_PAIRS	(32768)
+#define SOCKET_PAIR_BUF         (64)	/* Socket pair I/O buffer size */
 
 static const stress_help_t help[] = {
 	{ NULL,	"sockpair N",	  "start N workers exercising socket pair I/O activity" },
@@ -82,13 +78,15 @@ static void socket_pair_close(
  *	this stressor needs to be oom-able in the parent
  *	and child cases
  */
-static int stress_sockpair_oomable(const stress_args_t *args)
+static int stress_sockpair_oomable(const stress_args_t *args, void *context)
 {
 	pid_t pid;
 	static int socket_pair_fds[MAX_SOCKET_PAIRS][2];
 	int socket_pair_fds_bad[2];
 	int i, max, ret;
 
+	(void)context;
+
 	/* exercise invalid socketpair domain */
 	socket_pair_fds_bad[0] = -1;
 	socket_pair_fds_bad[1] = -1;
@@ -176,7 +174,7 @@ again:
 
 		if (!keep_stressing(args))
 			goto finish;
-		pr_fail("%s: fork failed, errno=%d (%s)\n",
+		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
@@ -203,6 +201,8 @@ again:
 						goto abort;
 					else if (errno == EPERM)  /* Occurs on socket closure */
 						goto abort;
+					else if (errno == EPIPE)  /* Pipe broke */
+						goto abort;
 					else if (errno) {
 						pr_fail("%s: read failed, errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
@@ -224,7 +224,11 @@ abort:
 		uint8_t buf[SOCKET_PAIR_BUF];
 		int val = 0, status;
 
-		(void)setpgid(pid, g_pgrp);
+		stress_set_oom_adjustment(args->name, true);
+		(void)setpgid(0, g_pgrp);
+		stress_parent_died_alarm();
+		(void)sched_settings_apply(true);
+
 		/* Parent */
 		socket_pair_close(socket_pair_fds, max, 0);
 		do {
@@ -234,7 +238,9 @@ abort:
 				socket_pair_memset(buf, (uint8_t)val++, sizeof(buf));
 				wret = write(socket_pair_fds[i][1], buf, sizeof(buf));
 				if (wret <= 0) {
-					if ((errno == EAGAIN) || (errno == EINTR) || (errno == EPIPE))
+					if (errno == EPIPE)
+						break;
+					if ((errno == EAGAIN) || (errno == EINTR))
 						continue;
 					if (errno) {
 						pr_fail("%s: write failed, errno=%d (%s)\n",
@@ -259,7 +265,6 @@ abort:
 		socket_pair_close(socket_pair_fds, max, 1);
 	}
 finish:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return EXIT_SUCCESS;
 }
@@ -270,13 +275,23 @@ finish:
  */
 static int stress_sockpair(const stress_args_t *args)
 {
+	int rc;
+
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
-	return stress_sockpair_oomable(args);
+	if (stress_sighandler(args->name, SIGPIPE, stress_sighandler_nop, NULL) < 0)
+		return EXIT_NO_RESOURCE;
+
+	rc = stress_oomable_child(args, NULL, stress_sockpair_oomable, STRESS_OOMABLE_DROP_CAP);
+
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
+	return rc;
 }
 
 stressor_info_t stress_sockpair_info = {
 	.stressor = stress_sockpair,
 	.class = CLASS_NETWORK | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
