@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"softlockup N",     "start N workers that cause softlockups" },
@@ -114,6 +110,7 @@ static int stress_softlockup(const stress_args_t *args)
 	size_t policy = 0;
 	int max_prio = 0;
 	bool good_policy = false;
+	const bool first_instance = (args->instance == 0);
 	const uint32_t cpus_online = (uint32_t)stress_get_processors_online();
 	const uint32_t num_instances = args->num_instances;
 	struct sigaction old_action_xcpu;
@@ -126,14 +123,13 @@ static int stress_softlockup(const stress_args_t *args)
 	timeout = g_opt_timeout;
 	(void)memset(&param, 0, sizeof(param));
 
-	if (!args->instance) {
-		if (SIZEOF_ARRAY(policies) == (0)) {
-			if (args->instance == 0)
-				pr_inf_skip("%s: no scheduling policies "
+	if (SIZEOF_ARRAY(policies) == 0) {
+		if (first_instance) {
+			pr_inf_skip("%s: no scheduling policies "
 					"available, skipping test\n",
 					args->name);
-			return EXIT_NOT_IMPLEMENTED;
 		}
+		return EXIT_NOT_IMPLEMENTED;
 	}
 
 	/* Get the max priorities for each sched policy */
@@ -152,30 +148,24 @@ static int stress_softlockup(const stress_args_t *args)
 	 *  policies, so check for this
 	 */
 	if (!good_policy) {
-		if (args->instance == 0)
+		if (first_instance) {
 			pr_inf_skip("%s: cannot get valid maximum priorities for the "
 				"scheduling policies, skipping test\n",
 					args->name);
+		}
 		return EXIT_NOT_IMPLEMENTED;
 	}
 
-	if (!args->instance) {
-		if (max_prio < 1)
-			pr_inf("%s: running with a low maximum priority of %d\n",
-				args->name, max_prio);
+	if ((max_prio < 1) && (args->instance == 0)) {
+		pr_inf("%s: running with a low maximum priority of %d\n",
+			args->name, max_prio);
 	}
 
-	if (num_instances < cpus_online) {
+	if ((num_instances < cpus_online) && (first_instance)) {
 		pr_inf("%s: for best results, run with at least %d instances "
 			"of this stressor\n", args->name, cpus_online);
 	}
 
-	if (g_opt_timeout == TIMEOUT_NOT_SET) {
-		timeout = 60;
-		pr_inf("%s: timeout has not been set, forcing timeout to "
-			"be %" PRIu64 " seconds\n", args->name, timeout);
-	}
-
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 again:
