@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -60,7 +55,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *  stress_segvhandler()
  *	SEGV handler
  */
-static void MLOCKED_TEXT stress_segvhandler(int signum)
+static void MLOCKED_TEXT NORETURN stress_segvhandler(int signum)
 {
 	(void)signum;
 
@@ -105,7 +100,7 @@ static void stress_stack_alloc(
 	}
 #if defined(HAVE_MLOCK)
 	if (stack_mlock) {
-		intptr_t ptr = ((intptr_t)data) + (page_size - 1);
+		intptr_t ptr = ((intptr_t)data) + ((intptr_t)page_size - 1);
 		ssize_t mlock_sz = (uint8_t *)start - (uint8_t *)ptr;
 
 		if (mlock_sz < 0)
@@ -115,12 +110,14 @@ static void stress_stack_alloc(
 			int ret;
 
 			ptr &= ~(page_size - 1);
-			ret = shim_mlock((void *)ptr, mlock_sz - last_size);
+			ret = shim_mlock((void *)ptr, (size_t)(mlock_sz - last_size));
 			if (ret < 0)
 				stack_mlock = false;
 			last_size = mlock_sz;
 		}
 	}
+#else
+	UNEXPECTED
 #endif
 	inc_counter(args);
 
