@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-put.h"
+
+#if defined(HAVE_UCONTEXT_H)
+#include <ucontext.h>
+#else
+UNEXPECTED
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"stackmmap N",	   "start N workers exercising a filebacked stack" },
@@ -68,6 +70,8 @@ static void stress_stackmmap_push_msync(void)
 	}
 	if (keep_stressing_flag())
 		stress_stackmmap_push_msync();
+
+	stress_uint64_put((uint64_t)waste[1]);
 }
 
 /*
@@ -106,7 +110,7 @@ static int stress_stackmmap(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto tidy_dir;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	if (ftruncate(fd, MMAPSTACK_SIZE) < 0) {
 		pr_fail("%s: ftruncate failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -153,6 +157,8 @@ static int stress_stackmmap(const stress_args_t *args)
 	 */
 	(void)mprotect((void *)stack_mmap, page_size, PROT_NONE);
 	(void)mprotect((void *)(stack_mmap + MMAPSTACK_SIZE - page_size), page_size, PROT_NONE);
+#else
+	UNEXPECTED
 #endif
 
 	(void)memset(&c_test, 0, sizeof(c_test));
@@ -196,7 +202,7 @@ again:
 					pr_dbg("%s: waitpid(): errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
 				(void)kill(pid, SIGTERM);
-				(void)kill(pid, SIGKILL);
+				(void)stress_killpid(pid);
 				(void)shim_waitpid(pid, &status, 0);
 			}
 		} else if (pid == 0) {
