@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -28,7 +23,7 @@
  *  the STR stress test has different classes of string stressors
  */
 typedef void (*stress_str_func)(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -39,7 +34,7 @@ typedef void (*stress_str_func)(
 typedef struct {
 	const char 		*name;	/* human readable form of stressor */
 	const stress_str_func	func;	/* the stressor function */
-	const void 		*libc_func;
+	void 		*libc_func;
 } stress_str_method_info_t;
 
 static const stress_help_t help[] = {
@@ -75,7 +70,7 @@ static inline void strchk(
  *	stress on strcasecmp
  */
 static void stress_strcasecmp(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -83,8 +78,10 @@ static void stress_strcasecmp(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_strcasecmp_t)(const char *s1, const char *s2);
+
 	register size_t i;
-	int (*test_strcasecmp)(const char *s1, const char *s2) = libc_func;
+	test_strcasecmp_t test_strcasecmp = (test_strcasecmp_t)libc_func;
 
 	(void)len2;
 
@@ -111,7 +108,7 @@ static void stress_strcasecmp(
  *	stress on strncasecmp
  */
 static void stress_strncasecmp(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -119,8 +116,10 @@ static void stress_strncasecmp(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_strncasecmp_t)(const char *s1, const char *s2, size_t n);
+
 	register size_t i;
-	int (*test_strncasecmp)(const char *s1, const char *s2, size_t n) = libc_func;
+	test_strncasecmp_t test_strncasecmp = (test_strncasecmp_t)libc_func;
 
 	(void)len2;
 
@@ -147,7 +146,7 @@ static void stress_strncasecmp(
  *	stress on index
  */
 static void stress_index(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -155,8 +154,10 @@ static void stress_index(
 	const size_t len2,
 	bool *failed)
 {
+	typedef char * (*test_index_t)(const char *s, int c);
+
 	register size_t i;
-	char * (*test_index)(const char *s, int c) = libc_func;
+	test_index_t test_index = (test_index_t)libc_func;
 
 	(void)len2;
 
@@ -176,7 +177,7 @@ static void stress_index(
  *	stress on rindex
  */
 static void stress_rindex(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -184,8 +185,10 @@ static void stress_rindex(
 	const size_t len2,
 	bool *failed)
 {
+	typedef char * (*test_rindex_t)(const char *s, int c);
+
 	register size_t i;
-	char * (*test_rindex)(const char *s, int c) = libc_func;
+	test_rindex_t test_rindex = (test_rindex_t)libc_func;
 
 	(void)len2;
 
@@ -199,14 +202,14 @@ static void stress_rindex(
 }
 #endif
 
-#if defined(HAVE_STRLCPY) &&	\
+#if defined(HAVE_BSD_STRLCPY) &&	\
     !defined(BUILD_STATIC)
 /*
  *  stress_strlcpy()
  *	stress on strlcpy
  */
 static void stress_strlcpy(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -214,8 +217,10 @@ static void stress_strlcpy(
 	const size_t len2,
 	bool *failed)
 {
+	typedef size_t (*test_strlcpy_t)(char *dest, const char *src, size_t len);
+
 	register size_t i;
-	size_t (*test_strlcpy)(char *dest, const char *src, size_t len) = libc_func;
+	test_strlcpy_t test_strlcpy = (test_strlcpy_t)libc_func;
 
 	char buf[len1 + len2 + 1];
 	const size_t buf_len = sizeof(buf);
@@ -233,7 +238,7 @@ static void stress_strlcpy(
  *	stress on strcpy
  */
 static void stress_strcpy(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -254,14 +259,14 @@ static void stress_strcpy(
 #endif
 
 
-#if defined(HAVE_STRLCAT) &&	\
+#if defined(HAVE_BSD_STRLCAT) &&	\
     !defined(BUILD_STATIC)
 /*
  *  stress_strlcat()
  *	stress on strlcat
  */
 static void stress_strlcat(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -269,8 +274,10 @@ static void stress_strlcat(
 	const size_t len2,
 	bool *failed)
 {
+	typedef size_t (*test_strlcat_t)(char *dest, const char *src, size_t len);
+
+	test_strlcat_t test_strlcat = (test_strlcat_t)libc_func;
 	register size_t i;
-	size_t (*test_strlcat)(char *dest, const char *src, size_t len) = libc_func;
 
 	char buf[len1 + len2 + 1];
 	const size_t buf_len = sizeof(buf);
@@ -297,7 +304,7 @@ static void stress_strlcat(
  *	stress on strcat
  */
 static void stress_strcat(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -305,8 +312,10 @@ static void stress_strcat(
 	const size_t len2,
 	bool *failed)
 {
+	typedef char * (*test_strcat_t)(char *dest, const char *src);
+
 	register size_t i;
-	char * (*test_strcat)(char *dest, const char *src) = libc_func;
+	test_strcat_t test_strcat = (test_strcat_t)libc_func;
 
 	char buf[len1 + len2 + 1];
 
@@ -330,7 +339,7 @@ static void stress_strcat(
  *	stress on strncat
  */
 static void stress_strncat(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -338,8 +347,10 @@ static void stress_strncat(
 	const size_t len2,
 	bool *failed)
 {
+	typedef char * (*test_strncat_t)(char *dest, const char *src, size_t n);
+
 	register size_t i;
-	char * (*test_strncat)(char *dest, const char *src, size_t n) = libc_func;
+	test_strncat_t test_strncat = (test_strncat_t)libc_func;
 	char buf[len1 + len2 + 1];
 
 	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
@@ -361,7 +372,7 @@ static void stress_strncat(
  *	stress on strchr
  */
 static void stress_strchr(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -369,8 +380,10 @@ static void stress_strchr(
 	const size_t len2,
 	bool *failed)
 {
+	typedef char * (*test_strchr_t)(const char *s, int c);
+
 	register size_t i;
-	char * (*test_strchr)(const char *s, int c) = libc_func;
+	test_strchr_t test_strchr = (test_strchr_t)libc_func;
 
 	(void)len2;
 
@@ -388,7 +401,7 @@ static void stress_strchr(
  *	stress on strrchr
  */
 static void stress_strrchr(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -396,8 +409,10 @@ static void stress_strrchr(
 	const size_t len2,
 	bool *failed)
 {
+	typedef char * (*test_strrchr_t)(const char *s, int c);
+
 	register size_t i;
-	char * (*test_strrchr)(const char *s, int c) = libc_func;
+	test_strrchr_t test_strrchr = (test_strrchr_t)libc_func;
 
 	(void)len2;
 
@@ -415,7 +430,7 @@ static void stress_strrchr(
  *	stress on strcmp
  */
 static void stress_strcmp(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -423,8 +438,10 @@ static void stress_strcmp(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_strcmp_t)(const char *s1, const char *s2);
+
 	register size_t i;
-	int (*test_strcmp)(const char *s1, const char *s2) = libc_func;
+	test_strcmp_t test_strcmp = (test_strcmp_t)libc_func;
 
 	(void)len2;
 
@@ -449,7 +466,7 @@ static void stress_strcmp(
  *	stress on strncmp
  */
 static void stress_strncmp(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -457,8 +474,10 @@ static void stress_strncmp(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_strncmp_t)(const char *s1, const char *s2, size_t n);
+
 	register size_t i;
-	int (*test_strncmp)(const char *s1, const char *s2, size_t n) = libc_func;
+	test_strncmp_t test_strncmp = (test_strncmp_t)libc_func;
 
 	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
 		STRCHK(name, 0 == test_strncmp(str1, str1, len1), failed);
@@ -480,7 +499,7 @@ static void stress_strncmp(
  *	stress on strcoll
  */
 static void stress_strcoll(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -488,8 +507,10 @@ static void stress_strcoll(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_strcoll_t)(const char *s1, const char *s2);
+
 	register size_t i;
-	int (*test_strcoll)(const char *s1, const char *s2) = libc_func;
+	test_strcoll_t test_strcoll = (test_strcoll_t)libc_func;
 
 	(void)len2;
 
@@ -514,7 +535,7 @@ static void stress_strcoll(
  *	stress on strlen
  */
 static void stress_strlen(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -522,8 +543,10 @@ static void stress_strlen(
 	const size_t len2,
 	bool *failed)
 {
+	typedef size_t (*test_strlen_t)(const char *s);
+
 	register size_t i;
-	size_t (*test_strlen)(const char *s) = libc_func;
+	test_strlen_t test_strlen = (test_strlen_t)libc_func;
 
 
 	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
@@ -542,7 +565,7 @@ static void stress_strlen(
  *	stress on strxfrm
  */
 static void stress_strxfrm(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -550,9 +573,11 @@ static void stress_strxfrm(
 	const size_t len2,
 	bool *failed)
 {
+	typedef size_t (*test_strxfrm_t)(char *dest, const char *src, size_t n);
+
 	register size_t i;
 	char buf[len1 + len2];
-	size_t (*test_strxfrm)(char *dest, const char *src, size_t n) = libc_func;
+	test_strxfrm_t test_strxfrm = (test_strxfrm_t)libc_func;
 
 	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
 		*buf = '\0';
@@ -574,7 +599,7 @@ static void stress_strxfrm(
  *	iterate over all string stressors
  */
 static void stress_str_all(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	char *str1,
 	const size_t len1,
@@ -603,7 +628,7 @@ static const stress_str_method_info_t str_methods[] = {
 	{ "rindex",		stress_rindex,		rindex  },
 	{ "strcasecmp",		stress_strcasecmp,	strcasecmp },
 #endif
-#if defined(HAVE_STRLCAT) &&	\
+#if defined(HAVE_BSD_STRLCAT) &&	\
     !defined(BUILD_STATIC)
 	{ "strlcat",		stress_strlcat,		strlcat },
 #else
@@ -612,7 +637,7 @@ static const stress_str_method_info_t str_methods[] = {
 	{ "strchr",		stress_strchr,		strchr },
 	{ "strcoll",		stress_strcoll,		strcoll },
 	{ "strcmp",		stress_strcmp,		strcmp },
-#if defined(HAVE_STRLCPY) &&	\
+#if defined(HAVE_BSD_STRLCPY) &&	\
     !defined(BUILD_STATIC)
 	{ "strlcpy",		stress_strlcpy,		strlcpy },
 #else
@@ -661,7 +686,7 @@ static int stress_str(const stress_args_t *args)
 {
 	const stress_str_method_info_t *str_method = &str_methods[0];
 	stress_str_func func;
-	const void *libc_func;
+	void *libc_func;
 	bool failed = false;
 	char ALIGN64 str1[256], ALIGN64 str2[128];
 	register char *ptr1, *ptr2;
@@ -719,5 +744,6 @@ stressor_info_t stress_str_info = {
 	.set_default = stress_str_set_default,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
