@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  * This stressor is loosely based on the STREAM Sustainable
  * Memory Bandwidth In High Performance Computers tool.
  *   https://www.cs.virginia.edu/stream/
@@ -34,6 +29,18 @@
  *
  */
 #include "stress-ng.h"
+#include "core-cpu.h"
+#include "core-nt-store.h"
+
+#define MIN_STREAM_L3_SIZE	(4 * KB)
+#define MAX_STREAM_L3_SIZE	(MAX_MEM_LIMIT)
+#define DEFAULT_STREAM_L3_SIZE	(4 * MB)
+
+#if defined(HAVE_NT_STORE_DOUBLE)
+#define NT_STORE(dst, src)		stress_nt_store_double(&dst, src)
+#endif
+
+#define STORE(dst, src)			dst = src
 
 typedef struct {
 	const char *name;
@@ -112,46 +119,59 @@ static inline void OPTIMIZE3 stress_stream_copy_index0(
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		c[i] = a[i];
+		STORE(c[i], a[i]);
 }
 
-static inline void OPTIMIZE3 stress_stream_copy_index1(
+#if defined(HAVE_NT_STORE_DOUBLE)
+static inline void OPTIMIZE3 stress_stream_copy_index0_nt(
 	double *RESTRICT c,
 	const double *RESTRICT a,
-	size_t *RESTRICT idx1,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		c[idx1[i]] = a[idx1[i]];
+		NT_STORE(c[i], a[i]);
+}
+#endif
+
+static inline void OPTIMIZE3 stress_stream_copy_index1(
+	double *RESTRICT c,
+	const double *RESTRICT a,
+	const size_t *RESTRICT idx1,
+	const uint64_t n)
+{
+	register uint64_t i;
+
+	for (i = 0; i < n; i++)
+		STORE(c[idx1[i]], a[idx1[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_copy_index2(
 	double *RESTRICT c,
 	const double *RESTRICT a,
-	size_t *RESTRICT idx1,
-	size_t *RESTRICT idx2,
+	const size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx2,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		c[idx1[i]] = a[idx2[i]];
+		STORE(c[idx1[i]], a[idx2[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_copy_index3(
 	double *RESTRICT c,
 	const double *RESTRICT a,
-	size_t *RESTRICT idx1,
-	size_t *RESTRICT idx2,
-	size_t *RESTRICT idx3,
+	const size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx2,
+	const size_t *RESTRICT idx3,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		c[idx3[idx1[i]]] = a[idx2[i]];
+		STORE(c[idx3[idx1[i]]], a[idx2[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_scale_index0(
@@ -163,49 +183,63 @@ static inline void OPTIMIZE3 stress_stream_scale_index0(
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		b[i] = q * c[i];
+		STORE(b[i], q * c[i]);
 }
 
-static inline void OPTIMIZE3 stress_stream_scale_index1(
+#if defined(HAVE_NT_STORE_DOUBLE)
+static inline void OPTIMIZE3 stress_stream_scale_index0_nt(
 	double *RESTRICT b,
 	const double *RESTRICT c,
 	const double q,
-	size_t *RESTRICT idx1,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		b[idx1[i]] = q * c[idx1[i]];
+		NT_STORE(b[i], q * c[i]);
+}
+#endif
+
+static inline void OPTIMIZE3 stress_stream_scale_index1(
+	double *RESTRICT b,
+	const double *RESTRICT c,
+	const double q,
+	const size_t *RESTRICT idx1,
+	const uint64_t n)
+{
+	register uint64_t i;
+
+	for (i = 0; i < n; i++)
+		STORE(b[idx1[i]], q * c[idx1[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_scale_index2(
 	double *RESTRICT b,
 	const double *RESTRICT c,
 	const double q,
-	size_t *RESTRICT idx1,
-	size_t *RESTRICT idx2,
+	const size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx2,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		b[idx1[i]] = q * c[idx2[i]];
+		STORE(b[idx1[i]], q * c[idx2[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_scale_index3(
 	double *RESTRICT b,
 	const double *RESTRICT c,
 	const double q,
-	size_t *RESTRICT idx1,
-	size_t *RESTRICT idx2,
-	size_t *RESTRICT idx3,
+	const size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx2,
+	const size_t *RESTRICT idx3,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		b[idx3[idx1[i]]] = q * c[idx2[i]];
+		STORE(b[idx3[idx1[i]]], q * c[idx2[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_add_index0(
@@ -217,49 +251,63 @@ static inline void OPTIMIZE3 stress_stream_add_index0(
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		c[i] = a[i] + b[i];
+		STORE(c[i], a[i] + b[i]);
 }
 
-static inline void OPTIMIZE3 stress_stream_add_index1(
+#if defined(HAVE_NT_STORE_DOUBLE)
+static inline void OPTIMIZE3 stress_stream_add_index0_nt(
 	const double *RESTRICT a,
 	const double *RESTRICT b,
 	double *RESTRICT c,
-	size_t *RESTRICT idx1,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		c[idx1[i]] = a[idx1[i]] + b[idx1[i]];
+		NT_STORE(c[i], a[i] + b[i]);
+}
+#endif
+
+static inline void OPTIMIZE3 stress_stream_add_index1(
+	const double *RESTRICT a,
+	const double *RESTRICT b,
+	double *RESTRICT c,
+	const size_t *RESTRICT idx1,
+	const uint64_t n)
+{
+	register uint64_t i;
+
+	for (i = 0; i < n; i++)
+		STORE(c[idx1[i]], a[idx1[i]] + b[idx1[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_add_index2(
 	const double *RESTRICT a,
 	const double *RESTRICT b,
 	double *RESTRICT c,
-	size_t *RESTRICT idx1,
-	size_t *RESTRICT idx2,
+	const size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx2,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		c[idx1[i]] = a[idx2[i]] + b[idx1[i]];
+		STORE(c[idx1[i]], a[idx2[i]] + b[idx1[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_add_index3(
 	const double *RESTRICT a,
 	const double *RESTRICT b,
 	double *RESTRICT c,
-	size_t *RESTRICT idx1,
-	size_t *RESTRICT idx2,
-	size_t *RESTRICT idx3,
+	const size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx2,
+	const size_t *RESTRICT idx3,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		c[idx1[i]] = a[idx2[i]] + b[idx3[i]];
+		STORE(c[idx1[i]], a[idx2[i]] + b[idx3[i]]);
 }
 
 static inline void OPTIMIZE3 stress_stream_triad_index0(
@@ -272,21 +320,36 @@ static inline void OPTIMIZE3 stress_stream_triad_index0(
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		a[i] = b[i] + (c[i] * q);
+		STORE(a[i], b[i] + (c[i] * q));
 }
 
+#if defined(HAVE_NT_STORE_DOUBLE)
+static inline void OPTIMIZE3 stress_stream_triad_index0_nt(
+	double *RESTRICT a,
+	const double *RESTRICT b,
+	const double *RESTRICT c,
+	const double q,
+	const uint64_t n)
+{
+	register uint64_t i;
+
+	for (i = 0; i < n; i++)
+		NT_STORE(a[i], b[i] + (c[i] * q));
+}
+#endif
+
 static inline void OPTIMIZE3 stress_stream_triad_index1(
 	double *RESTRICT a,
 	const double *RESTRICT b,
 	const double *RESTRICT c,
 	const double q,
-	size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx1,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		a[idx1[i]] = b[idx1[i]] + (c[idx1[i]] * q);
+		STORE(a[idx1[i]], b[idx1[i]] + (c[idx1[i]] * q));
 }
 
 static inline void OPTIMIZE3 stress_stream_triad_index2(
@@ -294,14 +357,14 @@ static inline void OPTIMIZE3 stress_stream_triad_index2(
 	const double *RESTRICT b,
 	const double *RESTRICT c,
 	const double q,
-	size_t *RESTRICT idx1,
-	size_t *RESTRICT idx2,
+	const size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx2,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		a[idx1[i]] = b[idx2[i]] + (c[idx1[i]] * q);
+		STORE(a[idx1[i]], b[idx2[i]] + (c[idx1[i]] * q));
 }
 
 static inline void OPTIMIZE3 stress_stream_triad_index3(
@@ -309,15 +372,15 @@ static inline void OPTIMIZE3 stress_stream_triad_index3(
 	const double *RESTRICT b,
 	const double *RESTRICT c,
 	const double q,
-	size_t *RESTRICT idx1,
-	size_t *RESTRICT idx2,
-	size_t *RESTRICT idx3,
+	const size_t *RESTRICT idx1,
+	const size_t *RESTRICT idx2,
+	const size_t *RESTRICT idx3,
 	const uint64_t n)
 {
 	register uint64_t i;
 
 	for (i = 0; i < n; i++)
-		a[idx1[i]] = b[idx2[i]] + (c[idx3[i]] * q);
+		STORE(a[idx1[i]], b[idx2[i]] + (c[idx3[i]] * q));
 }
 
 static void stress_stream_init_data(
@@ -330,7 +393,7 @@ static void stress_stream_init_data(
 	for (i = 0; i < n; i++) {
 		const uint32_t r1 = stress_mwc32();
 
-		data[i] = (double)r1 * divisor;
+		STORE(data[i], (double)r1 * divisor);
 	}
 }
 
@@ -361,6 +424,8 @@ static inline void *stress_stream_mmap(const stress_args_t *args, uint64_t sz)
 
 		ret = madvise(ptr, sz, advice);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 	}
 	return ptr;
@@ -446,6 +511,9 @@ static int stress_stream(const stress_args_t *args)
 	uint64_t L3, sz, n, sz_idx;
 	uint64_t stream_L3_size = DEFAULT_STREAM_L3_SIZE;
 	bool guess = false;
+#if defined(HAVE_NT_STORE_DOUBLE)
+	const bool has_sse2 = stress_cpu_x86_has_sse2();
+#endif
 
 	if (stress_get_setting("stream-L3-size", &stream_L3_size))
 		L3 = stream_L3_size;
@@ -551,6 +619,15 @@ static int stress_stream(const stress_args_t *args)
 			break;
 		case 0:
 		default:
+#if defined(HAVE_NT_STORE_DOUBLE)
+			if (has_sse2) {
+				stress_stream_copy_index0_nt(c, a, n);
+				stress_stream_scale_index0_nt(b, c, q, n);
+				stress_stream_add_index0_nt(c, b, a, n);
+				stress_stream_triad_index0_nt(a, b, c, q, n);
+				break;
+			}
+#endif
 			stress_stream_copy_index0(c, a, n);
 			stress_stream_scale_index0(b, c, q, n);
 			stress_stream_add_index0(c, b, a, n);
