@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(__sun__)
+/* Disable for SunOs/Solaris because */
+#undef HAVE_SYS_SWAP_H
+#endif
+#if defined(HAVE_SYS_SWAP_H)
+#include <sys/swap.h>
+#endif
+
+#define SHIM_EXT2_IOC_GETFLAGS		_IOR('f', 1, long)
+#define SHIM_EXT2_IOC_SETFLAGS		_IOW('f', 2, long)
+#define SHIM_FS_NOCOW_FL		0x00800000 /* No Copy-on-Write file */
 
 static const stress_help_t help[] = {
 	{ NULL,	"swap N",	"start N workers exercising swapon/swapoff" },
@@ -90,6 +98,25 @@ static int stress_swap_supported(const char *name)
 	return 0;
 }
 
+/*
+ *  stress_swapoff()
+ *	swapoff and retry if EINTR occurs
+ */
+static int stress_swapoff(const char *path)
+{
+	int ret, i;
+
+	for (i = 0; i < 25; i++) {
+		errno = 0;
+		ret = swapoff(path);
+		if (ret == 0)
+			break;
+		if ((ret < 0) && (errno != EINTR))
+			break;
+	}
+	return ret;
+}
+
 static int stress_swap_zero(
 	const stress_args_t *args,
 	const int fd,
@@ -117,7 +144,7 @@ static int stress_swap_zero(
 static int stress_swap_set_size(
 	const stress_args_t *args,
 	const int fd,
-	const uint32_t npages,
+	const size_t npages,
 	const int bad_flags)
 {
 	char signature[] = SWAP_SIGNATURE;
@@ -182,6 +209,31 @@ static int stress_swap_set_size(
 	return 0;
 }
 
+static void stress_swap_check_swapped(
+	void *addr,
+	const size_t page_size,
+	const size_t npages,
+	uint64_t *swapped_out,
+	uint64_t *swapped_total)
+{
+	unsigned char *vec;
+	register size_t n = 0;
+	register size_t i;
+
+	*swapped_total += npages;
+
+	vec = calloc(npages, sizeof(*vec));
+	if (!vec)
+		return;
+
+	shim_mincore(addr, page_size * npages, vec);
+	for (i = 0; i < npages; i++)
+		n += ((vec[i] & 1) == 0);
+
+	*swapped_out += n;
+	free(vec);
+}
+
 /*
  *  stress_swap()
  *	stress swap operations
@@ -191,8 +243,12 @@ static int stress_swap(const stress_args_t *args)
 	char filename[PATH_MAX];
 	int fd, ret;
 	uint8_t *page;
+	uint64_t swapped_out = 0;
+	uint64_t swapped_total = 0;
+	const size_t page_size = args->page_size;
+	double swapped_percent;
 
-	page = calloc(1, args->page_size);
+	page = calloc(1, page_size);
 	if (!page) {
 		pr_err("%s: failed to allocate 1 page: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -216,6 +272,23 @@ static int stress_swap(const stress_args_t *args)
 		goto tidy_rm;
 	}
 
+#if defined(__linux__)
+	{
+		unsigned long flags;
+
+		/*
+		 *  Disable Copy-on-Write on file where possible, since
+		 *  file systems such as btrfs have CoW enabled by default
+		 *  and we swap does not support this feature.
+		 */
+		if (ioctl(fd, SHIM_EXT2_IOC_GETFLAGS, &flags) == 0) {
+			flags |= SHIM_FS_NOCOW_FL;
+			ret = ioctl(fd, SHIM_EXT2_IOC_SETFLAGS, &flags);
+			(void)ret;
+		}
+	}
+#endif
+
 	if (stress_swap_zero(args, fd, MAX_SWAP_PAGES, page) < 0) {
 		ret = EXIT_FAILURE;
 		goto tidy_close;
@@ -226,8 +299,10 @@ static int stress_swap(const stress_args_t *args)
 	do {
 		int swapflags = 0;
 		int bad_flags;
-		uint32_t npages = (stress_mwc32() % (MAX_SWAP_PAGES - MIN_SWAP_PAGES)) +
+		char *ptr;
+		size_t npages = (size_t)(stress_mwc32() % (MAX_SWAP_PAGES - MIN_SWAP_PAGES)) +
 				  MIN_SWAP_PAGES;
+		const size_t mmap_size = npages * page_size;
 
 #if defined(SWAP_FLAG_PREFER)
 		if (stress_mwc1()) {
@@ -264,6 +339,8 @@ static int stress_swap(const stress_args_t *args)
 					args->name);
 				ret = EXIT_NO_RESOURCE;
 				break;
+			case EBUSY:
+				continue;
 			default:
 				pr_fail("%s: swapon failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
@@ -273,23 +350,55 @@ static int stress_swap(const stress_args_t *args)
 			goto tidy_close;
 		}
 
-		ret = swapoff(filename);
+		ptr = mmap(NULL, mmap_size, PROT_READ | PROT_WRITE,
+				MAP_SHARED | MAP_ANONYMOUS, -1, 0);
+		if (ptr != MAP_FAILED) {
+			size_t i;
+			const char *p_end = ptr + mmap_size;
+			char *p;
+
+			/* Add simple check value to start of each page */
+			for (i = 0, p = ptr; p < p_end; p += page_size, i++) {
+				uintptr_t *up = (uintptr_t *)p;
+
+				(void)memset(p, i, page_size);
+				*up = (uintptr_t)p;
+			}
+#if defined(MADV_PAGEOUT)
+			(void)shim_madvise(ptr, mmap_size, MADV_PAGEOUT);
+#endif
+			stress_swap_check_swapped(ptr, page_size, npages,
+				&swapped_out, &swapped_total);
+
+			/* Check page has check address value */
+			for (i = 0, p = ptr; p < p_end; p += page_size, i++) {
+				uintptr_t *up = (uintptr_t *)p;
+
+				if (*up != (uintptr_t)p) {
+					pr_fail("%s: failed: address %p contains "
+						"%" PRIuPTR " and not %" PRIuPTR "\n",
+						args->name, p, *up, (uintptr_t)p);
+				}
+			}
+			(void)munmap(ptr, mmap_size);
+		}
+
+		ret = stress_swapoff(filename);
 		if ((bad_flags == SWAP_HDR_SANE) && (ret < 0)) {
 			pr_fail("%s: swapoff failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			ret = EXIT_FAILURE;
-			(void)stress_thrash_stop();
 			goto tidy_close;
 		}
 
 		/* Exercise illegal swapon filename */
 		ret = swapon("", swapflags);
 		if (ret == 0)
-			ret = swapoff("");	/* Should never happen */
+			ret = stress_swapoff("");	/* Should never happen */
 		(void)ret;
 
 		/* Exercise illegal swapoff filename */
-		ret = swapoff("");
+		ret = stress_swapoff("");
 		if (ret == 0)
 			ret = swapon("", swapflags);	/* Should never happen */
 		(void)ret;
@@ -297,19 +406,24 @@ static int stress_swap(const stress_args_t *args)
 		/* Exercise illegal swapon flags */
 		ret = swapon(filename, ~0);
 		if (ret == 0)
-			ret = swapoff(filename);	/* Should never happen */
+			ret = stress_swapoff(filename);	/* Should never happen */
 		(void)ret;
 
 		inc_counter(args);
 	} while (keep_stressing(args));
 
+	swapped_percent = (swapped_total == 0) ?
+		0.0 : (100.0 * (double)swapped_out) / (double)swapped_total;
+	pr_inf("%s: %" PRIu64 " of %" PRIu64 " (%.2f%%) pages were swapped out\n",
+		args->name, swapped_out, swapped_total, swapped_percent);
+
 	ret = EXIT_SUCCESS;
 tidy_close:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fd);
 tidy_rm:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 tidy_free:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -323,6 +437,7 @@ stressor_info_t stress_swap_info = {
 	.stressor = stress_swap,
 	.supported = stress_swap_supported,
 	.class = CLASS_VM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
