@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_SYNC_FILE_BYTES	(1 * MB)
+#define MAX_SYNC_FILE_BYTES	(MAX_FILE_LIMIT)
+#define DEFAULT_SYNC_FILE_BYTES	(1 * GB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"sync-file N",	     "start N workers exercise sync_file_range" },
 	{ NULL,	"sync-file-ops N",   "stop after N sync_file_range bogo operations" },
@@ -40,6 +39,8 @@ static const unsigned int sync_modes[] = {
 	SYNC_FILE_RANGE_WAIT_AFTER,
 	0	/* No-op */
 };
+#else
+UNEXPECTED
 #endif
 
 static int stress_set_sync_file_bytes(const char *opt)
@@ -82,10 +83,12 @@ static int stress_sync_allocate(
 	if (ret < 0) {
 		if ((errno == ENOSPC) || (errno == EINTR))
 			return -errno;
-		pr_err("%s: fdatasync failed: errno=%d (%s)\n",
+		pr_fail("%s: fdatasync failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return -errno;
 	}
+#else
+	UNEXPECTED
 #endif
 
 	ret = shim_fallocate(fd, 0, (off_t)0, sync_file_bytes);
@@ -128,7 +131,7 @@ static int stress_sync_file(const stress_args_t *args)
 		filename, sizeof(filename), stress_mwc32());
 	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
 		if ((errno == ENFILE) || (errno == ENOMEM) || (errno == ENOSPC)) {
-			pr_inf("%s: cannot create file to sync on, skipping stressor: errno=%d (%s)\n",
+			pr_inf_skip("%s: cannot create file to sync on, skipping stressor: errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
 		}
@@ -139,7 +142,7 @@ static int stress_sync_file(const stress_args_t *args)
 		(void)stress_temp_dir_rm_args(args);
 		return ret;
 	}
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
@@ -161,7 +164,7 @@ static int stress_sync_file(const stress_args_t *args)
 			ret = shim_sync_file_range(fd, offset, sz, mode);
 			if (ret < 0) {
 				if (errno == ENOSYS) {
-					pr_inf("%s: skipping stressor, sync_file_range is not implemented\n",
+					pr_inf_skip("%s: skipping stressor, sync_file_range is not implemented\n",
 						args->name);
 					goto err;
 				}
@@ -205,7 +208,7 @@ static int stress_sync_file(const stress_args_t *args)
 			ret = shim_sync_file_range(fd, sync_file_bytes - offset, sz, mode);
 			if (ret < 0) {
 				if (errno == ENOSYS) {
-					pr_inf("%s: skipping stressor, sync_file_range is not implemented\n",
+					pr_inf_skip("%s: skipping stressor, sync_file_range is not implemented\n",
 						args->name);
 					goto err;
 				}
@@ -226,12 +229,11 @@ static int stress_sync_file(const stress_args_t *args)
 		}
 		for (i = 0; keep_stressing_flag() && i <
 		     (shim_off64_t)sync_file_bytes / (shim_off64_t)(128 * KB); i++) {
-			
 			offset = (stress_mwc64() % sync_file_bytes) & ~((128 * KB) - 1);
 			ret = shim_sync_file_range(fd, offset, 128 * KB, mode);
 			if (ret < 0) {
 				if (errno == ENOSYS) {
-					pr_inf("%s: skipping stressor, sync_file_range is not implemented\n",
+					pr_inf_skip("%s: skipping stressor, sync_file_range is not implemented\n",
 						args->name);
 					goto err;
 				}
@@ -255,6 +257,7 @@ stressor_info_t stress_sync_file_info = {
 	.stressor = stress_sync_file,
 	.class = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
