@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,55 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_STATFS_H)
+#include <sys/statfs.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UIO_H)
+#include <sys/uio.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UTSNAME_H)
+#include <sys/utsname.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_XATTR_H)
+#include <sys/xattr.h>
+#undef HAVE_ATTR_XATTR_H
+#elif defined(HAVE_ATTR_XATTR_H)
+#include <attr/xattr.h>
+#endif
+/*  Sanity check */
+#if defined(HAVE_SYS_XATTR_H) &&        \
+    defined(HAVE_ATTR_XATTR_H)
+#error cannot have both HAVE_SYS_XATTR_H and HAVE_ATTR_XATTR_H
+#endif
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_UTIME_H)
+#include <utime.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_PTRACE)
+#include <sys/ptrace.h>
+#endif
+
 typedef void *(*stress_bad_addr_t)(const stress_args_t *args);
 typedef int (*stress_bad_syscall_t)(void *addr);
 
@@ -420,6 +461,8 @@ static int bad_ioctl(void *addr)
 {
 	return ioctl(0, TCGETS, addr);
 }
+#else
+UNEXPECTED
 #endif
 
 static int bad_lchown(void *addr)
@@ -442,6 +485,8 @@ static int bad_madvise(void *addr)
 {
 	return madvise((void *)addr, 8192, MADV_NORMAL);
 }
+#else
+UNEXPECTED
 #endif
 
 #if defined(HAVE_MEMFD_CREATE)
@@ -473,6 +518,8 @@ static int bad_mlock(void *addr)
 {
 	return shim_mlock((void *)addr, 4096);
 }
+#else
+UNEXPECTED
 #endif
 
 #if defined(HAVE_MLOCK2)
@@ -511,6 +558,8 @@ static int bad_msync(void *addr)
 {
 	return shim_msync((void *)addr, 4096, MS_SYNC);
 }
+#else
+UNEXPECTED
 #endif
 
 #if defined(HAVE_NANOSLEEP)
@@ -519,6 +568,8 @@ static int bad_nanosleep(void *addr)
 	return nanosleep((struct timespec *)addr,
 		(struct timespec *)inc_addr(addr, 1));
 }
+#else
+UNEXPECTED
 #endif
 
 static int bad_open(void *addr)
@@ -564,6 +615,8 @@ static int bad_poll(void *addr)
 {
 	return poll((struct pollfd *)addr, (nfds_t)16, (int)1);
 }
+#else
+UNEXPECTED
 #endif
 
 static int bad_pwrite(void *addr)
@@ -622,6 +675,8 @@ static int bad_sched_getaffinity(void *addr)
 {
 	return sched_getaffinity(getpid(), (size_t)8192, (cpu_set_t *)addr);
 }
+#else
+UNEXPECTED
 #endif
 
 static int bad_select(void *addr)
@@ -660,6 +715,8 @@ static int bad_statfs(void *addr)
 {
 	return statfs(".", (struct statfs *)addr);
 }
+#else
+UNEXPECTED
 #endif
 
 #if defined(HAVE_SYS_SYSINFO_H) && 	\
@@ -668,6 +725,8 @@ static int bad_sysinfo(void *addr)
 {
 	return sysinfo((struct sysinfo *)addr);
 }
+#else
+UNEXPECTED
 #endif
 
 static int bad_time(void *addr)
@@ -684,6 +743,8 @@ static int bad_timer_create(void *addr)
 	timerid++;
 	return timer_create(CLOCK_MONOTONIC, (struct sigevent *)addr, timerid);
 }
+#else
+UNEXPECTED
 #endif
 
 static int bad_times(void *addr)
@@ -702,6 +763,8 @@ static int bad_uname(void *addr)
 {
 	return uname((struct utsname *)addr);
 }
+#else
+UNEXPECTED
 #endif
 
 static int bad_ustat(void *addr)
@@ -744,6 +807,8 @@ static int bad_waitid(void *addr)
 {
 	return waitid(P_PID, (id_t)getpid(), (siginfo_t *)addr, 0);
 }
+#else
+UNEXPECTED
 #endif
 
 static int bad_write(void *addr)
@@ -772,6 +837,8 @@ static int bad_writev(void *addr)
 	}
 	return (int)ret;
 }
+#else
+UNEXPECTED
 #endif
 
 static stress_bad_syscall_t bad_syscalls[] = {
@@ -1074,7 +1141,7 @@ static int stress_sysbadaddr(const stress_args_t *args)
 	rx_page = mmap(NULL, page_size, PROT_EXEC | PROT_READ,
 		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (rx_page == MAP_FAILED) {
-		pr_inf("%s: cannot mmap anonymous excute-only page: "
+		pr_inf("%s: cannot mmap anonymous execute-only page: "
 		       "errno=%d (%s)\n", args->name, errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
