@@ -1,4 +1,7 @@
 /*
+ * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -13,14 +16,29 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-put.h"
+#include "core-capabilities.h"
+#include "core-hash.h"
+
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UTSNAME_H)
+#include <sys/utsname.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#else
+UNEXPECTED
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"sysfs N",	"start N workers reading files from /sys" },
@@ -42,7 +60,7 @@ static shim_pthread_spinlock_t lock;
 static uint32_t mixup;
 static volatile bool drain_kmsg = false;
 static volatile uint32_t counter = 0;
-static char signum_path[] = "/sys/kernel/notes";
+static const char signum_path[] = "/sys/kernel/notes";
 static uint32_t os_release;
 static stress_hash_table_t *sysfs_hash_table;
 
@@ -219,11 +237,6 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 			(void)close(fd);
 			goto drain;
 		}
-/*
-if (stress_mwc8() > 240)
-	kill(getpid(), SIGKILL);
-*/
-
 		/*
 		 *  mmap it
 		 */
@@ -260,6 +273,8 @@ if (stress_mwc8() > 240)
 			if (stress_time_now() - t_start > threshold)
 				goto next;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_PPOLL)
@@ -279,6 +294,8 @@ if (stress_mwc8() > 240)
 			ret = ppoll(fds, 1, &ts, &sigmask);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 		/*
@@ -297,6 +314,8 @@ if (stress_mwc8() > 240)
 			ret = ioctl(fd, FIGETBSZ, &isz);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 #if defined(FIONREAD)
 		{
@@ -305,6 +324,8 @@ if (stress_mwc8() > 240)
 			ret = ioctl(fd, FIONREAD , &isz);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 		if (stress_kmsg_drain(ctxt->kmsgfd)) {
@@ -517,7 +538,7 @@ static void stress_sys_dir(
 		(void)shim_pthread_spin_unlock(&lock);
 		drain_kmsg = false;
 		time_start = stress_time_now();
-		time_end = time_start + ((double)DURATION_PER_SYSFS_FILE / 1000000.0);
+		time_end = time_start + ((double)DURATION_PER_SYSFS_FILE * ONE_MILLIONTH);
 		time_out = time_start + 1.0;
 		/*
 		 *  wait for a timeout, or until woken up
@@ -653,6 +674,8 @@ static int stress_sysfs(const stress_args_t *args)
 				os_release = (major * 100) + minor;
 		}
 	}
+#else
+	UNEXPECTED
 #endif
 	sysfs_hash_table = stress_hash_create(1021);
 	if (!sysfs_hash_table) {
@@ -703,13 +726,14 @@ static int stress_sysfs(const stress_args_t *args)
 		pid_t pid;
 
 again:
-		if (!keep_stressing(args))
-			break;
-
 		pid = fork();
 		if (pid < 0) {
-			if ((errno == EAGAIN) || (errno == ENOMEM))
+			if (stress_redo_fork(errno))
 				goto again;
+			if (!keep_stressing(args)) {
+				rc = EXIT_SUCCESS;
+				goto finish;
+			}
 		} else if (pid > 0) {
 			int status;
 
@@ -792,6 +816,7 @@ again:
 		}
 	} while (keep_stressing(args));
 
+finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	stress_hash_delete(sysfs_hash_table);
@@ -808,6 +833,7 @@ again:
 stressor_info_t stress_sysfs_info = {
 	.stressor = stress_sysfs,
 	.class = CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
