@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,33 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_SYSMACROS_H)
+#include <sys/sysmacros.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_STATFS_H)
+#include <sys/statfs.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_STATVFS_H)
+#include <sys/statvfs.h>
+#else
+UNEXPECTED
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"sysinfo N",	 "start N workers reading system information" },
 	{ NULL,	"sysinfo-ops N", "stop after sysinfo bogo operations" },
@@ -125,6 +144,8 @@ static int stress_sysinfo(const stress_args_t *args)
 					ret = ioctl(fd, FS_IOC_GETFSLABEL, label);
 					(void)ret;
 				}
+#else
+				UNEXPECTED
 #endif
 
 				ret = fstatfs(fd, &statfs_buf);
@@ -247,5 +268,6 @@ static int stress_sysinfo(const stress_args_t *args)
 stressor_info_t stress_sysinfo_info = {
 	.stressor = stress_sysinfo,
 	.class = CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
