@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_TIMER_FREQ		(1)
+#define MAX_TIMER_FREQ		(100000000)
+#define DEFAULT_TIMER_FREQ	(1000000)
+
 static const stress_help_t help[] = {
 	{ "T N", "timer N",	"start N workers producing timer events" },
 	{ NULL, "timer-ops N",	"stop after N timer bogo events" },
@@ -163,6 +162,7 @@ static void MLOCKED_TEXT stress_timer_handler(int sig)
 		if (ret > 0)
 			overruns += (uint64_t)ret;
 		stress_timer_set(&timer);
+		(void)timer_settime(timerid, 0, &timer, NULL);
 		return;
 	}
 
@@ -247,7 +247,6 @@ static int stress_timer(const stress_args_t *args)
 		req.tv_nsec = 10000000;
 		(void)nanosleep(&req, NULL);
 		set_counter(args, timer_counter);
-
 	} while (keep_stressing(args));
 
 	if (timer_delete(timerid) < 0) {
