@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_SYS_TIMERFD_H)
+#include <sys/timerfd.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_TIMERFD_FREQ	(1)
+#define MAX_TIMERFD_FREQ	(100000000)
+#define DEFAULT_TIMERFD_FREQ	(1000000)
 
 static const stress_help_t help[] = {
 	{ NULL,	"timerfd N",	  "start N workers producing timerfd events" },
@@ -212,7 +224,7 @@ static int stress_timerfd(const stress_args_t *args)
 		rc = exit_status(errno);
 		goto close_file_fd;
 	}
-	(void)unlink(file_fd_name);
+	(void)shim_unlink(file_fd_name);
 
 #if defined(CLOCK_REALTIME_ALARM)
 	/* Check timerfd_create cannot succeed without capability */
