@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-cpu.h"
+
+#if defined(HAVE_SYS_PLATFORM_PPC_H)
+#include <sys/platform/ppc.h>
+#endif
+
+#if defined(HAVE_SYS_CAPABILITY_H)
+#include <sys/capability.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"tsc N",	"start N workers reading the time stamp counter" },
@@ -31,7 +36,7 @@ static const stress_help_t help[] = {
 };
 
 
-#if defined(STRESS_ARCH_RISC_V) &&	\
+#if defined(STRESS_ARCH_RISCV) &&	\
     defined(SIGILL)
 
 #define HAVE_STRESS_TSC_CAPABILITY
@@ -85,10 +90,9 @@ static int stress_tsc_supported(const char *name)
 }
 #endif
 
-#if defined(HAVE_CPUID_H) &&	\
-    defined(STRESS_ARCH_X86) && 	\
-    defined(HAVE_CPUID) &&	\
-    NEED_GNUC(4,6,0)
+#if defined(STRESS_ARCH_X86) &&		\
+    !defined(__PCC__) &&		\
+    !defined(__TINYC__)
 
 #define HAVE_STRESS_TSC_CAPABILITY
 
@@ -100,8 +104,6 @@ static bool tsc_supported = false;
  */
 static int stress_tsc_supported(const char *name)
 {
-	uint32_t eax, ebx, ecx, edx;
-
 	/* Intel CPU? */
 	if (!stress_cpu_is_x86()) {
 		pr_inf_skip("%s stressor will be skipped, "
@@ -109,8 +111,7 @@ static int stress_tsc_supported(const char *name)
 		return -1;
 	}
 	/* ..and supports tsc? */
-	__cpuid(1, eax, ebx, ecx, edx);
-	if (!(edx & 0x10)) {
+	if (!stress_cpu_x86_has_tsc()) {
 		pr_inf_skip("%s stressor will be skipped, CPU "
 			"does not support the tsc instruction\n", name);
 		return -1;
