@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-net.h"
+
+#if defined(HAVE_LINUX_SOCKIOS_H)
+#include <linux/sockios.h>
+#else
+UNEXPECTED
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"udp-flood N",		"start N workers that performs a UDP flood attack" },
@@ -108,6 +110,8 @@ static int stress_udp_flood(const stress_args_t *args)
 
 			(void)ioctl(fd, SIOCOUTQ, &pending);
 		}
+#else
+		UNEXPECTED
 #endif
 
 		if (!keep_stressing(args))
