@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,36 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-cpu.h"
+#include "core-net.h"
+
+#if defined(HAVE_LINUX_SOCKIOS_H)
+#include <linux/sockios.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_LINUX_UDP_H)
+#include <linux/udp.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_SYS_UN_H)
+#include <sys/un.h>
+#else
+UNEXPECTED
+#endif
+
+#include <netinet/in.h>
+
+#define MIN_UDP_PORT		(1024)
+#define MAX_UDP_PORT		(65535)
+#define DEFAULT_UDP_PORT	(7000)
+
+#define UDP_BUF			(1024)	/* UDP I/O buffer size */
 
 /* See bugs section of udplite(7) */
 #if !defined(SOL_UDPLITE)
@@ -110,7 +133,7 @@ static int stress_udp(const stress_args_t *args)
 #endif
 
 	pr_dbg("%s: process [%d] using udp port %d\n",
-		args->name, (int)args->pid, udp_port + args->instance);
+		args->name, (int)args->pid, udp_port + (int)args->instance);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
@@ -145,7 +168,8 @@ again:
 			stress_set_sockaddr(args->name, args->instance, ppid,
 				udp_domain, udp_port,
 				&addr, &len, NET_ADDR_ANY);
-#if defined(IPPROTO_UDPLITE) && defined(UDPLITE_SEND_CSCOV)
+#if defined(IPPROTO_UDPLITE) &&	\
+    defined(UDPLITE_SEND_CSCOV)
 			if (proto == IPPROTO_UDPLITE) {
 				int val = 8;	/* Just the 8 byte header */
 				socklen_t slen;
@@ -162,7 +186,8 @@ again:
 				(void)getsockopt(fd, SOL_UDPLITE, UDPLITE_SEND_CSCOV, &val, &slen);
 			}
 #endif
-#if defined(IPPROTO_UDPLITE) && defined(UDPLITE_RECV_CSCOV)
+#if defined(IPPROTO_UDPLITE) &&	\
+    defined(UDPLITE_RECV_CSCOV)
 			if (proto == IPPROTO_UDPLITE) {
 				int val;
 				socklen_t slen = sizeof(val);
@@ -181,7 +206,10 @@ again:
 					slen = sizeof(val);
 					ret = setsockopt(fd, proto, UDP_CORK, &val, slen);
 				}
+				(void)ret;
 			}
+#else
+			UNEXPECTED
 #endif
 #if defined(UDP_ENCAP)
 			{
@@ -193,7 +221,10 @@ again:
 					slen = sizeof(val);
 					ret = setsockopt(fd, proto, UDP_ENCAP, &val, slen);
 				}
+				(void)ret;
 			}
+#else
+			UNEXPECTED
 #endif
 #if defined(UDP_NO_CHECK6_TX)
 			{
@@ -205,7 +236,10 @@ again:
 					slen = sizeof(val);
 					ret = setsockopt(fd, proto, UDP_NO_CHECK6_TX, &val, slen);
 				}
+				(void)ret;
 			}
+#else
+			UNEXPECTED
 #endif
 #if defined(UDP_NO_CHECK6_RX)
 			{
@@ -216,7 +250,10 @@ again:
 					slen = sizeof(val);
 					ret = setsockopt(fd, proto, UDP_NO_CHECK6_RX, &val, slen);
 				}
+				(void)ret;
 			}
+#else
+			UNEXPECTED
 #endif
 #if defined(UDP_SEGMENT)
 			{
@@ -228,7 +265,10 @@ again:
 					slen = sizeof(val);
 					ret = setsockopt(fd, proto, UDP_SEGMENT, &val, slen);
 				}
+				(void)ret;
 			}
+#else
+			UNEXPECTED
 #endif
 			do {
 				size_t i;
@@ -250,6 +290,8 @@ again:
 
 					(void)ioctl(fd, SIOCOUTQ, &pending);
 				}
+#else
+			UNEXPECTED
 #endif
 			} while (keep_stressing(args));
 			(void)close(fd);
@@ -260,7 +302,7 @@ again:
 		if ((udp_domain == AF_UNIX) && addr) {
 			struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-			(void)unlink(addr_un->sun_path);
+			(void)shim_unlink(addr_un->sun_path);
 		}
 #endif
 		/* Inform parent we're all done */
@@ -323,7 +365,6 @@ again:
 			}
 		}
 #endif
-
 		do {
 			socklen_t len = addr_len;
 			ssize_t n;
@@ -333,6 +374,8 @@ again:
 
 				(void)ioctl(fd, SIOCINQ, &pending);
 			}
+#else
+			UNEXPECTED
 #endif
 
 			n = recvfrom(fd, buf, sizeof(buf), 0, addr, &len);
@@ -357,7 +400,7 @@ die:
 		if ((udp_domain == AF_UNIX) && addr) {
 			struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
-			(void)unlink(addr_un->sun_path);
+			(void)shim_unlink(addr_un->sun_path);
 		}
 #endif
 		if (pid) {
