@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -186,7 +181,7 @@ static int stress_unshare(const stress_args_t *args)
 			if (pids[i] > 0) {
 				int ret;
 
-				ret = kill(pids[i], SIGKILL);
+				ret = stress_killpid(pids[i]);
 				if (ret == 0) {
 					if (shim_waitpid(pids[i], &status, 0) < 0) {
 						if (errno != EINTR)
@@ -198,7 +193,7 @@ static int stress_unshare(const stress_args_t *args)
 		}
 		inc_counter(args);
 	} while (keep_stressing(args));
-	
+
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	return EXIT_SUCCESS;
@@ -207,6 +202,7 @@ static int stress_unshare(const stress_args_t *args)
 stressor_info_t stress_unshare_info = {
 	.stressor = stress_unshare,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
