@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"uprobe N",	"start N workers that generate uprobe events" },
@@ -58,7 +54,7 @@ static int stress_uprobe_supported(const char *name)
  *  stress_uprobe_write
  *     write to a uprobe sysfs file a string, open using flags setting in flags
  */
-static int stress_uprobe_write(const char *path, int flags, const char *str)
+static int stress_uprobe_write(const char *path, const int flags, const char *str)
 {
 	int fd, rc = 0;
 
@@ -85,7 +81,7 @@ static void *stress_uprobe_libc_start(const pid_t pid, char *libc_path)
 	uint64_t start, end, offset, dev_major, dev_minor, inode;
 	void *addr = NULL;
 
-	(void)snprintf(path, sizeof(path), "/proc/%d/maps", (int)pid);
+	(void)snprintf(path, sizeof(path), "/proc/%" PRIdMAX "/maps", (intmax_t)pid);
 	fp = fopen(path, "r");
 	if (!fp)
 		return addr;
@@ -99,12 +95,15 @@ static void *stress_uprobe_libc_start(const pid_t pid, char *libc_path)
 			&inode, libc_path);
 
 		/*
-		 *  name *libc-*.so found?
+		 *  name /libc-*.so or /libc.so found?
 		 */
-		if ((n == 8) && strstr(libc_path, "/libc-") &&
-		    strstr(libc_path, ".so") && !strncmp(perm, "r-xp", 4)) {
-			addr = (void *)(intptr_t)(start - offset);
-			break;
+		if ((n == 8) && !strncmp(perm, "r-xp", 4) &&
+		    strstr(libc_path, ".so")) {
+			if (strstr(libc_path, "/libc-") ||
+			    strstr(libc_path, "/libc.so")) {
+				addr = (void *)(intptr_t)(start - offset);
+				break;
+			}
 		}
 	}
 	(void)fclose(fp);
@@ -245,7 +244,7 @@ clear_events:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	/* Remove uprobe */
-	snprintf(buf, sizeof(buf), "-:%s\n", event);
+	(void)snprintf(buf, sizeof(buf), "-:%s\n", event);
 	ret = stress_uprobe_write("/sys/kernel/debug/tracing/uprobe_events",
 		O_WRONLY | O_APPEND, buf);
 	(void)ret;
