@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
+
+#if defined(HAVE_LINUX_RANDOM_H)
+#include <linux/random.h>
+#else
+UNEXPECTED
+#endif
 
 static const stress_help_t help[] = {
 	{ "u N","urandom N",	 "start N workers reading /dev/urandom" },
@@ -98,7 +100,7 @@ static int stress_urandom(const stress_args_t *args)
 
 	if ((fd_urnd < 0) && (fd_rnd < 0)) {
 		if (args->instance == 0)
-			pr_inf("%s: random device(s) do not exist, skipping stressor\n",
+			pr_inf_skip("%s: random device(s) do not exist, skipping stressor\n",
 				args->name);
 #if defined(__linux__)
 		if (fd_rnd_wr >= 0)
@@ -142,6 +144,8 @@ static int stress_urandom(const stress_args_t *args)
 			/* Try to avoid emptying entropy pool */
 			if (val < 128)
 				goto next;
+#else
+			UNEXPECTED
 #endif
 			ret = read(fd_rnd, buffer, 1);
 			if (ret < 0) {
@@ -170,10 +174,14 @@ next:
 #if defined(RNDCLEARPOOL)
 				ret = ioctl(fd_rnd, RNDCLEARPOOL, NULL);
 				check_eperm(args, ret, errno);
+#else
+				UNEXPECTED
 #endif
 #if defined(RNDZAPENTCNT)
 				ret = ioctl(fd_rnd, RNDZAPENTCNT, NULL);
 				check_eperm(args, ret, errno);
+#else
+				UNEXPECTED
 #endif
 #if defined(RNDADDTOENTCNT)
 				{
@@ -186,10 +194,14 @@ next:
 					ret = ioctl(fd_rnd, RNDADDTOENTCNT, &count);
 					check_eperm(args, ret, errno);
 				}
+#else
+				UNEXPECTED
 #endif
 #if defined(RNDRESEEDCRNG)
 				ret = ioctl(fd_rnd, RNDRESEEDCRNG, stress_mwc32());
 				check_eperm(args, ret, errno);
+#else
+				UNEXPECTED
 #endif
 				/*
 				 *  Exercise invalid ioctl command
@@ -266,5 +278,6 @@ err:
 stressor_info_t stress_urandom_info = {
 	.stressor = stress_urandom,
 	.class = CLASS_DEV | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
