@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,27 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(__NR_userfaultfd)
+#define HAVE_USERFAULTFD
+#endif
+
+#if defined(HAVE_LINUX_USERFAULTFD_H)
+#include <linux/userfaultfd.h>
+#endif
+
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_USERFAULT_BYTES	(4 * KB)
+#define MAX_USERFAULT_BYTES	(MAX_MEM_LIMIT)
+#define DEFAULT_USERFAULT_BYTES	(256 * MB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"userfaultfd N",	"start N page faulting workers with userspace handling" },
 	{ NULL,	"userfaultfd-ops N",	"stop after N page faults have been handled" },
@@ -55,7 +68,7 @@ static int stress_set_userfaultfd_bytes(const char *opt)
 
 	userfaultfd_bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
 	stress_check_range_bytes("userfaultfd-bytes", userfaultfd_bytes,
-		MIN_MMAP_BYTES, MAX_MEM_LIMIT);
+		MIN_USERFAULT_BYTES, MAX_MEM_LIMIT);
 	return stress_set_setting("userfaultfd-bytes", TYPE_ID_SIZE_T, &userfaultfd_bytes);
 }
 
@@ -183,7 +196,7 @@ static int stress_userfaultfd_child(const stress_args_t *args, void *context)
 	bool do_poll = true;
 	static uint8_t stack[STACK_SIZE]; /* Child clone stack */
 	uint8_t *stack_top = (uint8_t *)stress_get_stack_top((void *)stack, STACK_SIZE);
-	size_t userfaultfd_bytes = DEFAULT_MMAP_BYTES;
+	size_t userfaultfd_bytes = DEFAULT_USERFAULT_BYTES;
 
 	(void)context;
 
@@ -191,11 +204,11 @@ static int stress_userfaultfd_child(const stress_args_t *args, void *context)
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			userfaultfd_bytes = MAX_32;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			userfaultfd_bytes = MIN_MMAP_BYTES;
+			userfaultfd_bytes = MIN_USERFAULT_BYTES;
 	}
 	userfaultfd_bytes /= args->num_instances;
-	if (userfaultfd_bytes < MIN_MMAP_BYTES)
-		userfaultfd_bytes = MIN_MMAP_BYTES;
+	if (userfaultfd_bytes < MIN_USERFAULT_BYTES)
+		userfaultfd_bytes = MIN_USERFAULT_BYTES;
 	if (userfaultfd_bytes < args->page_size)
 		userfaultfd_bytes = args->page_size;
 
@@ -214,8 +227,14 @@ static int stress_userfaultfd_child(const stress_args_t *args, void *context)
 		goto free_zeropage;
 	}
 
+	/* Exercise invalid flags */
+	fd = shim_userfaultfd(~0);
+	if (fd >= 0)
+		(void)close(fd);
+
 	/* Get userfault fd */
-	if ((fd = shim_userfaultfd(0)) < 0) {
+	fd = shim_userfaultfd(0);
+	if (fd < 0) {
 		switch (errno) {
 		case EPERM:
 			if (args->instance == 0)
@@ -232,7 +251,7 @@ static int stress_userfaultfd_child(const stress_args_t *args, void *context)
 			break;
 		default:
 			rc = exit_status(errno);
-			pr_err("%s: userfaultfd failed, errno = %d (%s)\n",
+			pr_fail("%s: userfaultfd failed, errno = %d (%s)\n",
 				args->name, errno, strerror(errno));
 			break;
 		}
@@ -247,13 +266,13 @@ static int stress_userfaultfd_child(const stress_args_t *args, void *context)
 	api.api = UFFD_API;
 	api.features = 0;
 	if (ioctl(fd, UFFDIO_API, &api) < 0) {
-		pr_err("%s: ioctl UFFDIO_API failed, errno = %d (%s)\n",
+		pr_fail("%s: ioctl UFFDIO_API failed, errno = %d (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 		goto unmap_data;
 	}
 	if (api.api != UFFD_API) {
-		pr_err("%s: ioctl UFFDIO_API API check failed\n",
+		pr_fail("%s: ioctl UFFDIO_API API check failed\n",
 			args->name);
 		rc = EXIT_FAILURE;
 		goto unmap_data;
@@ -265,7 +284,7 @@ static int stress_userfaultfd_child(const stress_args_t *args, void *context)
 	reg.range.len = sz;
 	reg.mode = UFFDIO_REGISTER_MODE_MISSING;
 	if (ioctl(fd, UFFDIO_REGISTER, &reg) < 0) {
-		pr_err("%s: ioctl UFFDIO_REGISTER failed, errno = %d (%s)\n",
+		pr_fail("%s: ioctl UFFDIO_REGISTER failed, errno = %d (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 		goto unmap_data;
@@ -273,14 +292,14 @@ static int stress_userfaultfd_child(const stress_args_t *args, void *context)
 
 	/* OK, so do we have copy supported? */
 	if ((reg.ioctls & uffdio_copy) != uffdio_copy) {
-		pr_err("%s: ioctl UFFDIO_REGISTER did not support _UFFDIO_COPY\n",
+		pr_fail("%s: ioctl UFFDIO_REGISTER did not support _UFFDIO_COPY\n",
 			args->name);
 		rc = EXIT_FAILURE;
 		goto unmap_data;
 	}
 	/* OK, so do we have zeropage supported? */
 	if ((reg.ioctls & uffdio_zeropage) != uffdio_zeropage) {
-		pr_err("%s: ioctl UFFDIO_REGISTER did not support _UFFDIO_ZEROPAGE\n",
+		pr_fail("%s: ioctl UFFDIO_REGISTER did not support _UFFDIO_ZEROPAGE\n",
 			args->name);
 		rc = EXIT_FAILURE;
 		goto unmap_data;
@@ -395,7 +414,7 @@ do_read:
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	/* Run it over, zap child */
-	(void)kill(pid, SIGKILL);
+	(void)stress_killpid(pid);
 	if (shim_waitpid(pid, &status, 0) < 0) {
 		pr_dbg("%s: waitpid failed, errno = %d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -403,7 +422,7 @@ do_read:
 unreg:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	if (ioctl(fd, UFFDIO_UNREGISTER, &reg) < 0) {
-		pr_err("%s: ioctl UFFDIO_UNREGISTER failed, errno = %d (%s)\n",
+		pr_fail("%s: ioctl UFFDIO_UNREGISTER failed, errno = %d (%s)\n",
 			args->name, errno, strerror(errno));
 		rc = EXIT_FAILURE;
 		goto unmap_data;
@@ -433,6 +452,7 @@ stressor_info_t stress_userfaultfd_info = {
 	.stressor = stress_userfaultfd,
 	.class = CLASS_VM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
