@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-pragma.h"
+
+#if defined(HAVE_UTIME_H)
+#include <utime.h>
+#else
+UNEXPECTED
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"utime N",	"start N workers updating file timestamps" },
@@ -55,6 +57,7 @@ static int stress_utime(const stress_args_t *args)
 	char hugename[PATH_MAX + 16];
 	int ret, fd;
 	bool utime_fsync = false;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
 	(void)stress_get_setting("utime-fsync", &utime_fsync);
 
@@ -67,6 +70,8 @@ static int stress_utime(const stress_args_t *args)
     defined(UTIME_NOW)
 	(void)stress_temp_dir_args(args, path, sizeof(path));
 	dir_fd = open(path, O_DIRECTORY | O_PATH);
+#else
+	UNEXPECTED
 #endif
 
 	(void)stress_temp_filename_args(args,
@@ -92,13 +97,12 @@ static int stress_utime(const stress_args_t *args)
 		struct timespec ts[2];
 #endif
 
-		if (gettimeofday(&timevals[0], NULL) == 0) {
-			timevals[1] = timevals[0];
-			if (utimes(filename, timevals) < 0) {
-				pr_dbg("%s: utimes failed: errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
-				break;
-			}
+		(void)gettimeofday(&timevals[0], NULL);
+		timevals[1] = timevals[0];
+		if (utimes(filename, timevals) < 0) {
+			pr_dbg("%s: utimes failed: errno=%d (%s)\n",
+				args->name, errno, strerror(errno));
+			break;
 		}
 		if (utimes(filename, NULL) < 0) {
 			pr_dbg("%s: utimes failed: errno=%d (%s)\n",
@@ -139,6 +143,8 @@ static int stress_utime(const stress_args_t *args)
 				args->name, errno, strerror(errno));
 			break;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(UTIME_OMIT)
@@ -149,6 +155,8 @@ static int stress_utime(const stress_args_t *args)
 				args->name, errno, strerror(errno));
 			break;
 		}
+#else
+		UNEXPECTED
 #endif
 #endif
 
@@ -171,11 +179,18 @@ static int stress_utime(const stress_args_t *args)
 		/* Exercise huge filename, ENAMETOOLONG */
 		ret = utimensat(AT_FDCWD, hugename, ts, 0);
 		(void)ret;
+
+		/* Exercise invalid flags */
+		ret = utimensat(AT_FDCWD, filename, ts, ~0);
+		(void)ret;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(O_DIRECTORY) &&	\
     defined(O_PATH) &&		\
-    defined(UTIME_NOW)
+    defined(UTIME_NOW) &&	\
+    defined(HAVE_PRAGMA_INSIDE)
 STRESS_PRAGMA_PUSH
 STRESS_PRAGMA_WARN_OFF
 		if (dir_fd >= 0) {
@@ -218,23 +233,41 @@ STRESS_PRAGMA_POP
 		}
 #endif
 #endif
+#else
+		UNEXPECTED
 #endif
 
 #if defined(HAVE_UTIME_H)
 		{
 			struct utimbuf utbuf;
-			const double actime = stress_time_now();
+			struct timeval tv;
 
-			utbuf.actime = (time_t)actime;
+			(void)gettimeofday(&tv, NULL);
+			utbuf.actime = (time_t)tv.tv_sec;
 			utbuf.modtime = utbuf.actime;
 
 			if (utime(filename, &utbuf) < 0) {
-				pr_dbg("%s: utime failed: errno=%d (%s)\n",
+				pr_fail("%s: utime failed: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				break;
 			}
+
+			if (verify) {
+				struct stat statbuf;
+
+				if (stat(filename, &statbuf) == 0) {
+					if (statbuf.st_atime < tv.tv_sec) {
+						pr_fail("%s: utime failed: access time is less than expected time\n",
+							args->name);
+					}
+					if (statbuf.st_mtime < tv.tv_sec) {
+						pr_fail("%s: utime failed: modified time is less than expected time\n",
+							args->name);
+					}
+				}
+			}
 			if (utime(filename, NULL) < 0) {
-				pr_dbg("%s: utime failed: errno=%d (%s)\n",
+				pr_fail("%s: utime failed: errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				break;
 			}
@@ -247,6 +280,8 @@ STRESS_PRAGMA_POP
 			ret = utime(hugename, &utbuf);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 		/* forces metadata writeback */
 		if (utime_fsync) {
@@ -265,7 +300,7 @@ STRESS_PRAGMA_POP
 		(void)close(dir_fd);
 #endif
 	(void)close(fd);
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 
 	return EXIT_SUCCESS;
@@ -280,5 +315,6 @@ stressor_info_t stress_utime_info = {
 	.stressor = stress_utime,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
