@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+#include "core-put.h"
+#include "core-target-clones.h"
+#include "core-vecmath.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"vecmath N",	 "start N workers performing vector math ops" },
@@ -30,29 +29,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		 NULL }
 };
 
-/*
- *  Clang 5.0 is the lowest version of clang that
- *  can build this without issues (clang 4.0 seems
- *  to spend forever optimizing this and causes the build
- *  to never complete)
- */
-#if defined(__clang__) && \
-    defined(__clang_major__) && \
-    __clang_major__ < 5
-#undef HAVE_VECMATH
-#endif
-
-/*
- *  gcc 5.x or earlier breaks on 128 bit vector maths on
- *  PPC64 for some reason with some flavours of the toolchain
- *  so disable this test for now
- */
-#if defined(STRESS_ARCH_PPC64) && \
-    defined(__GNUC__) && \
-    __GNUC__ < 6
-#undef HAVE_VECMATH
-#endif
-
 #if defined(HAVE_VECMATH)
 
 typedef int8_t  stress_vint8_t  __attribute__ ((vector_size (16)));
