@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_FS_H)
+#include <linux/fs.h>
+#else
+UNEXPECTED
+#endif
+
+#if defined(HAVE_LINUX_FSVERITY_H)
+#include <linux/fsverity.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"verity N",		"start N workers exercising file verity ioctls" },
 	{ NULL,	"verity-ops N",		"stop after N file verity bogo operations" },
@@ -94,6 +99,8 @@ static int stress_verity(const stress_args_t *args)
 #if defined(FS_IOC_READ_VERITY_METADATA)
 		struct shim_fsverity_read_metadata_arg md_arg;
 		char md_buf[4096];
+#else
+		UNEXPECTED
 #endif
 
 		fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
@@ -149,6 +156,7 @@ static int stress_verity(const stress_args_t *args)
 			case EINVAL:
 			case ENOTTY:
 			case EOPNOTSUPP:
+			case ENOSYS:
 				if (args->instance == 0)
 					pr_inf_skip("%s: verity is not supported on the "
 						"file system or by the kernel, skipping stress test\n",
@@ -198,6 +206,8 @@ static int stress_verity(const stress_args_t *args)
 					args->name);
 			}
 		}
+#else
+		UNEXPECTED
 #endif
 		(void)close(fd);
 
@@ -242,10 +252,12 @@ static int stress_verity(const stress_args_t *args)
 		md_arg.length = (uint64_t)sizeof(md_buf);
 
 		(void)ioctl(fd, FS_IOC_READ_VERITY_METADATA, &md_arg);
+#else
+		UNEXPECTED
 #endif
 
 		(void)close(fd);
-		(void)unlink(filename);
+		(void)shim_unlink(filename);
 
 		inc_counter(args);
 	} while (keep_stressing(args));
@@ -255,7 +267,7 @@ static int stress_verity(const stress_args_t *args)
 clean:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 
 	return ret;
