@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,42 +16,44 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #if !defined(STRESS_VERSION_H)
 #define STRESS_VERSION_H
 
-#define _VER_(major, minor, patchlevel)			\
+#define STRESS_VERSION_NUMBER(major, minor, patchlevel)		\
 	((major * 10000) + (minor * 100) + patchlevel)
 
-#if defined(__GLIBC__) && defined(__GLIBC_MINOR__)
+#if defined(__GLIBC__) &&	\
+    defined(__GLIBC_MINOR__)
 #define NEED_GLIBC(major, minor, patchlevel) 			\
-	_VER_(major, minor, patchlevel) <= _VER_(__GLIBC__, __GLIBC_MINOR__, 0)
+	STRESS_VERSION_NUMBER(major, minor, patchlevel) <=	\
+	STRESS_VERSION_NUMBER(__GLIBC__, __GLIBC_MINOR__, 0)
 #else
 #define NEED_GLIBC(major, minor, patchlevel) 	(0)
 #endif
 
-#if defined(__GNUC__) && defined(__GNUC_MINOR__)
+#if defined(__GNUC__) &&	\
+    defined(__GNUC_MINOR__)
 #if defined(__GNUC_PATCHLEVEL__)
 #define NEED_GNUC(major, minor, patchlevel) 			\
-	_VER_(major, minor, patchlevel) <= _VER_(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__)
+	STRESS_VERSION_NUMBER(major, minor, patchlevel) <= 	\
+	STRESS_VERSION_NUMBER(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__)
 #else
 #define NEED_GNUC(major, minor, patchlevel) 			\
-	_VER_(major, minor, patchlevel) <= _VER_(__GNUC__, __GNUC_MINOR__, 0)
+	STRESS_VERSION_NUMBER(major, minor, patchlevel) <=	\
+	STRESS_VERSION_NUMBER(__GNUC__, __GNUC_MINOR__, 0)
 #endif
 #else
 #define NEED_GNUC(major, minor, patchlevel) 	(0)
 #endif
 
-#if defined(__clang__) && defined(__clang_major__) && defined(__clang_minor__) && \
+#if defined(__clang__) &&	\
+    defined(__clang_major__) &&	\
+    defined(__clang_minor__) && \
     defined(__clang_patchlevel__)
-#define NEED_CLANG(major, minor, patchlevel)	\
-	_VER_(major, minor, patchlevel) <= _VER_(__clang_major__, __clang_minor__, __clang_patchlevel__)
+#define NEED_CLANG(major, minor, patchlevel)			\
+	STRESS_VERSION_NUMBER(major, minor, patchlevel) <=	\
+	STRESS_VERSION_NUMBER(__clang_major__, __clang_minor__, __clang_patchlevel__)
 #else
 #define NEED_CLANG(major, minor, patchlevel)	(0)
 #endif
