@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2017-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -193,10 +188,7 @@ vfork_again:
 				if (pid >= 0)
 					start_pid = getpid();
 			} else {
-STRESS_PRAGMA_PUSH
-STRESS_PRAGMA_WARN_OFF
-				pid = vfork();
-STRESS_PRAGMA_POP
+				pid = shim_vfork();
 				inc_counter(args);
 			}
 			if (pid < 0) {
@@ -272,7 +264,7 @@ STRESS_PRAGMA_POP
 			if (errno == EINTR)
 				continue;
 			/* Something went wrong, kill */
-			(void)kill(chpid, SIGKILL);
+			(void)stress_killpid(chpid);
 		}
 	}
 finish:
