@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_UIO_H)
+#include <sys/uio.h>
+#else
+UNEXPECTED
+#endif
+
+#define MIN_VM_RW_BYTES		(4 * KB)
+#define MAX_VM_RW_BYTES		(MAX_MEM_LIMIT)
+#define DEFAULT_VM_RW_BYTES	(16 * MB)
+
 static const stress_help_t help[] = {
 	{ NULL,	"vm-rw N",	 "start N vm read/write process_vm* copy workers" },
 	{ NULL,	"vm-rw-bytes N", "transfer N bytes of memory per bogo operation" },
@@ -32,7 +37,7 @@ static const stress_help_t help[] = {
 };
 
 #if defined(HAVE_PROCESS_VM_READV) &&	\
-    defined(HAVE_PROCESS_VM_READV) &&	\
+    defined(HAVE_PROCESS_VM_WRITEV) &&	\
     defined(HAVE_CLONE) &&		\
     defined(CLONE_VM)
 
@@ -71,7 +76,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 };
 
 #if defined(HAVE_PROCESS_VM_READV) &&	\
-    defined(HAVE_PROCESS_VM_READV) &&	\
+    defined(HAVE_PROCESS_VM_WRITEV) &&	\
     defined(HAVE_CLONE) &&		\
     defined(CLONE_VM)
 
@@ -343,7 +348,7 @@ fail:
 	}
 	(void)close(ctxt->pipe_wr[0]);
 	(void)close(ctxt->pipe_rd[1]);
-	(void)kill(ctxt->pid, SIGKILL);
+	(void)stress_killpid(ctxt->pid);
 	(void)shim_waitpid(ctxt->pid, &status, 0);
 	(void)munmap(localbuf, ctxt->sz);
 
@@ -416,6 +421,7 @@ stressor_info_t stress_vm_rw_info = {
 	.stressor = stress_vm_rw,
 	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
