@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-cache.h"
+
+#if defined(HAVE_PTRACE)
+#include <sys/ptrace.h>
+#else
+UNEXPECTED
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"vm-segv N",	 "start N workers that unmap their address space" },
@@ -39,7 +41,9 @@ static NOINLINE void vm_unmap_child(const size_t page_size)
 	while (len > page_size) {
 		(void)munmap((void *)0, len - page_size);
 		len >>= 1;
+#if !defined(__DragonFly__)
 		shim_clflush(addr);
+#endif
 		shim_flush_icache(addr, (void *)(((uint8_t *)addr) + 64));
 	}
 }
@@ -49,7 +53,9 @@ static NOINLINE void vm_unmap_self(const size_t page_size)
 	void *addr = stress_align_address((void *)vm_unmap_self, page_size);
 
 	(void)munmap(addr, page_size);
+#if !defined(__DragonFly__)
 	shim_clflush(addr);
+#endif
 	shim_flush_icache(addr, (void *)(((uint8_t *)addr) + 64));
 }
 
@@ -61,6 +67,7 @@ static NOINLINE void vm_unmap_self(const size_t page_size)
  */
 static int stress_vm_segv(const stress_args_t *args)
 {
+	bool test_valid = false;
 	stress_set_oom_adjustment(args->name, true);
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -78,15 +85,17 @@ again:
 			pr_err("%s: fork failed: errno=%d: (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
-		} else if (pid > 0) {
+		}else if (pid > 0) {
 			int status, ret;
-
+	
 			(void)setpgid(pid, g_pgrp);
 			/* Parent, wait for child */
 
 			ret = shim_waitpid(pid, &status, 0);
 			if (ret < 0)
 				goto kill_child;
+
+			test_valid = true;
 #if !defined(HAVE_PTRACE)
 			if (WTERMSIG(status) == SIGSEGV) {
 				inc_counter(args);
@@ -120,7 +129,7 @@ again:
 #endif
 kill_child:
 			(void)kill(pid, SIGTERM);
-			(void)kill(pid, SIGKILL);
+			(void)stress_killpid(pid);
 			(void)shim_waitpid(pid, &status, 0);
 		} else if (pid == 0) {
 			/* Child */
@@ -158,11 +167,15 @@ kill_child:
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	if (test_valid && (get_counter(args) == 0))
+		pr_fail("%s: no SIGSEGV signals detected\n", args->name);
+
 	return EXIT_SUCCESS;
 }
 
 stressor_info_t stress_vm_segv_info = {
 	.stressor = stress_vm_segv,
 	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
