@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-cache.h"
+#include "core-target-clones.h"
+#include "core-nt-store.h"
+#include "core-vecmath.h"
+
+#define MIN_VM_BYTES		(4 * KB)
+#define MAX_VM_BYTES		(MAX_MEM_LIMIT)
+#define DEFAULT_VM_BYTES	(256 * MB)
+
+#define MIN_VM_HANG		(0)
+#define MAX_VM_HANG		(3600)
+#define DEFAULT_VM_HANG		(~0ULL)
 
 /*
  *  For testing, set this to 1 to simulate random memory errors
@@ -319,7 +326,7 @@ static inline size_t stress_vm_count_bits(uint64_t v)
 
 /*
  *  stress_vm_moving_inversion()
- *	work sequentially through memory setting 8 bytes at at a time
+ *	work sequentially through memory setting 8 bytes at a time
  *	with a random value, then check if it is correct, invert it and
  *	then check if that is correct.
  */
@@ -339,12 +346,12 @@ static size_t TARGET_CLONES stress_vm_moving_inversion(
 	w = stress_mwc32();
 	z = stress_mwc32();
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint64_t *)buf; ptr < (uint64_t *)buf_end; ) {
 		*(ptr++) = stress_mwc64();
 	}
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (bit_errors = 0, ptr = (uint64_t *)buf; ptr < (uint64_t *)buf_end; ) {
 		uint64_t val = stress_mwc64();
 
@@ -362,7 +369,7 @@ static size_t TARGET_CLONES stress_vm_moving_inversion(
 
 	inject_random_bit_errors(buf, sz);
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (bit_errors = 0, ptr = (uint64_t *)buf; ptr < (uint64_t *)buf_end; ) {
 		uint64_t val = stress_mwc64();
 
@@ -375,7 +382,7 @@ static size_t TARGET_CLONES stress_vm_moving_inversion(
 	if (UNLIKELY(!keep_stressing_flag()))
 		goto ret;
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint64_t *)buf_end; ptr > (uint64_t *)buf; ) {
 		*--ptr = stress_mwc64();
 	}
@@ -385,7 +392,7 @@ static size_t TARGET_CLONES stress_vm_moving_inversion(
 	inject_random_bit_errors(buf, sz);
 
 	(void)stress_mincore_touch_pages(buf, sz);
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint64_t *)buf_end; ptr > (uint64_t *)buf; ) {
 		uint64_t val = stress_mwc64();
 
@@ -399,7 +406,7 @@ static size_t TARGET_CLONES stress_vm_moving_inversion(
 	if (UNLIKELY(!keep_stressing_flag()))
 		goto ret;
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint64_t *)buf_end; ptr > (uint64_t *)buf; ) {
 		uint64_t val = stress_mwc64();
 
@@ -712,6 +719,65 @@ static size_t TARGET_CLONES stress_vm_gray(
 	return bit_errors;
 }
 
+/*
+ *  stress_vm_grayflip()
+ *	fill all of memory with a gray code based pattern that
+ *	flips as many bits as possible on each write.
+ */
+static size_t TARGET_CLONES stress_vm_grayflip(
+	void *buf,
+	void *buf_end,
+	const size_t sz,
+	const stress_args_t *args,
+	const uint64_t max_ops)
+{
+	static uint8_t val;
+	uint8_t v;
+	volatile uint8_t *ptr;
+	size_t bit_errors = 0;
+	const uint64_t c_orig = get_counter(args);
+	uint64_t c;
+
+	for (c = c_orig, v = val, ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; v++) {
+		register uint8_t gray;
+
+		if (UNLIKELY(!keep_stressing_flag()))
+			return 0;
+
+		gray = (v >> 1) ^ v;
+		*ptr++ = gray;
+		gray = ~gray;
+		*ptr++ = gray;
+	}
+	(void)stress_mincore_touch_pages(buf, sz);
+	inject_random_bit_errors(buf, sz);
+
+	for (v = val, ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; v++) {
+		register uint8_t gray;
+
+		if (UNLIKELY(!keep_stressing_flag()))
+			break;
+
+		gray = (v >> 1) ^ v;
+		if (UNLIKELY(*(ptr++) != gray))
+			bit_errors++;
+		c++;
+		gray = ~gray;
+		if (UNLIKELY(*(ptr++) != gray))
+			bit_errors++;
+		c++;
+		if (UNLIKELY(max_ops && c >= max_ops))
+			break;
+	}
+	val++;
+
+	stress_vm_check("gray code", bit_errors);
+	set_counter(args, c);
+
+	return bit_errors;
+}
+
+
 /*
  *  stress_vm_incdec()
  *	work through memory incrementing it and then decrementing
@@ -788,7 +854,7 @@ static size_t TARGET_CLONES stress_vm_prime_incdec(
 		ptr[i] += val;
 		c++;
 		if (UNLIKELY(max_ops && c >= max_ops))
-			break;
+			return 0;
 	}
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
@@ -801,7 +867,7 @@ static size_t TARGET_CLONES stress_vm_prime_incdec(
 		ptr[j % sz] -= val;
 		c++;
 		if (UNLIKELY(max_ops && c >= max_ops))
-			break;
+			return 0;
 	}
 
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr++) {
@@ -847,7 +913,7 @@ static size_t TARGET_CLONES stress_vm_swap(
 		swaps[i] = (stress_mwc64() % chunks) * chunk_sz;
 	}
 
-	stress_mwc_seed(w1, z1);
+	stress_mwc_set_seed(w1, z1);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
 		uint8_t val = stress_mwc8();
 		(void)memset((void *)ptr, val, chunk_sz);
@@ -895,7 +961,7 @@ static size_t TARGET_CLONES stress_vm_swap(
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
-	stress_mwc_seed(w1, z1);
+	stress_mwc_set_seed(w1, z1);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
 		volatile uint8_t *p = (volatile uint8_t *)ptr;
 		const volatile uint8_t *p_end = (volatile uint8_t *)ptr + chunk_sz;
@@ -909,9 +975,9 @@ static size_t TARGET_CLONES stress_vm_swap(
 		if (UNLIKELY(!keep_stressing_flag()))
 			break;
 	}
+	stress_vm_check("swap bytes", bit_errors);
 abort:
 	free(swaps);
-	stress_vm_check("swap bytes", bit_errors);
 	set_counter(args, c);
 
 	return bit_errors;
@@ -939,7 +1005,7 @@ static size_t TARGET_CLONES stress_vm_rand_set(
 	w = stress_mwc32();
 	z = stress_mwc32();
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
 		uint8_t val = stress_mwc8();
 
@@ -961,7 +1027,7 @@ static size_t TARGET_CLONES stress_vm_rand_set(
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
 		uint8_t val = stress_mwc8();
 
@@ -976,8 +1042,8 @@ static size_t TARGET_CLONES stress_vm_rand_set(
 		if (UNLIKELY(!keep_stressing_flag()))
 			break;
 	}
-abort:
 	stress_vm_check("rand-set", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1007,7 +1073,7 @@ static size_t TARGET_CLONES stress_vm_ror(
 	w = stress_mwc32();
 	z = stress_mwc32();
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
 		uint8_t val = stress_mwc8();
 
@@ -1047,7 +1113,7 @@ static size_t TARGET_CLONES stress_vm_ror(
 
 	inject_random_bit_errors(buf, sz);
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
 		uint8_t val = stress_mwc8();
 		ROR8(val);
@@ -1063,8 +1129,8 @@ static size_t TARGET_CLONES stress_vm_ror(
 		if (UNLIKELY(!keep_stressing_flag()))
 			break;
 	}
-abort:
 	stress_vm_check("ror", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1095,7 +1161,7 @@ static size_t TARGET_CLONES stress_vm_flip(
 	w = stress_mwc32();
 	z = stress_mwc32();
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
 		uint8_t val = stress_mwc8();
 
@@ -1144,7 +1210,7 @@ static size_t TARGET_CLONES stress_vm_flip(
 
 	inject_random_bit_errors(buf, sz);
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += chunk_sz) {
 		uint8_t val = stress_mwc8();
 
@@ -1167,8 +1233,8 @@ static size_t TARGET_CLONES stress_vm_flip(
 			break;
 	}
 
-abort:
 	stress_vm_check("flip", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1229,8 +1295,8 @@ static size_t TARGET_CLONES stress_vm_zero_one(
 		if (UNLIKELY(!keep_stressing_flag()))
 			break;
 	}
-abort:
 	stress_vm_check("zero-one", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1433,8 +1499,8 @@ static size_t TARGET_CLONES stress_vm_inc_nybble(
 			break;
 	}
 
-abort:
 	stress_vm_check("inc-nybble", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1464,7 +1530,7 @@ static size_t TARGET_CLONES stress_vm_rand_sum(
 	w = stress_mwc32();
 	z = stress_mwc32();
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint64_t *)buf; ptr < (uint64_t *)buf_end; ptr += chunk_sz) {
 		*(ptr + 0) = stress_mwc64();
 		*(ptr + 1) = stress_mwc64();
@@ -1484,7 +1550,7 @@ static size_t TARGET_CLONES stress_vm_rand_sum(
 	(void)stress_mincore_touch_pages(buf, sz);
 	inject_random_bit_errors(buf, sz);
 
-	stress_mwc_seed(w, z);
+	stress_mwc_set_seed(w, z);
 	for (ptr = (uint64_t *)buf; ptr < (uint64_t *)buf_end; ptr += chunk_sz) {
 		bit_errors += stress_vm_count_bits(*(ptr + 0) ^ stress_mwc64());
 		bit_errors += stress_vm_count_bits(*(ptr + 1) ^ stress_mwc64());
@@ -1497,8 +1563,8 @@ static size_t TARGET_CLONES stress_vm_rand_sum(
 		if (UNLIKELY(!keep_stressing_flag()))
 			break;
 	}
-abort:
 	stress_vm_check("rand-sum", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1557,8 +1623,8 @@ static size_t TARGET_CLONES stress_vm_prime_zero(
 		bit_errors += stress_vm_count_bits8(ptr8[i]);
 	}
 
-abort:
 	stress_vm_check("prime-zero", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1618,8 +1684,8 @@ static size_t TARGET_CLONES stress_vm_prime_one(
 		if (UNLIKELY(!keep_stressing_flag()))
 			break;
 	}
-abort:
 	stress_vm_check("prime-one", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1689,8 +1755,8 @@ static size_t TARGET_CLONES stress_vm_prime_gray_zero(
 		if (UNLIKELY(!keep_stressing_flag()))
 			break;
 	}
-abort:
 	stress_vm_check("prime-gray-zero", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1761,8 +1827,8 @@ static size_t TARGET_CLONES stress_vm_prime_gray_one(
 		if (UNLIKELY(!keep_stressing_flag()))
 			break;
 	}
-abort:
 	stress_vm_check("prime-gray-one", bit_errors);
+abort:
 	set_counter(args, c);
 
 	return bit_errors;
@@ -1832,6 +1898,76 @@ static size_t TARGET_CLONES stress_vm_write64(
 	return 0;
 }
 
+#if defined(HAVE_NT_STORE64)
+/*
+ *  stress_vm_write_64nt()
+ *	64 bit non-temporal write, no read check
+ */
+static size_t TARGET_CLONES stress_vm_write64nt(
+	void *buf,
+	void *buf_end,
+	const size_t sz,
+	const stress_args_t *args,
+	const uint64_t max_ops)
+{
+	if (stress_cpu_x86_has_sse2()) {
+		static uint64_t val;
+		uint64_t *ptr = (uint64_t *)buf;
+		register uint64_t v = val;
+		register size_t i = 0, n = sz / (sizeof(*ptr) * 32);
+
+		(void)buf_end;
+
+		while (i < n) {
+			stress_nt_store64(&ptr[0x00], v);
+			stress_nt_store64(&ptr[0x01], v);
+			stress_nt_store64(&ptr[0x02], v);
+			stress_nt_store64(&ptr[0x03], v);
+			stress_nt_store64(&ptr[0x04], v);
+			stress_nt_store64(&ptr[0x05], v);
+			stress_nt_store64(&ptr[0x06], v);
+			stress_nt_store64(&ptr[0x07], v);
+
+			stress_nt_store64(&ptr[0x08], v);
+			stress_nt_store64(&ptr[0x09], v);
+			stress_nt_store64(&ptr[0x0a], v);
+			stress_nt_store64(&ptr[0x0b], v);
+			stress_nt_store64(&ptr[0x0c], v);
+			stress_nt_store64(&ptr[0x0d], v);
+			stress_nt_store64(&ptr[0x0e], v);
+			stress_nt_store64(&ptr[0x0f], v);
+
+			stress_nt_store64(&ptr[0x10], v);
+			stress_nt_store64(&ptr[0x11], v);
+			stress_nt_store64(&ptr[0x12], v);
+			stress_nt_store64(&ptr[0x13], v);
+			stress_nt_store64(&ptr[0x14], v);
+			stress_nt_store64(&ptr[0x15], v);
+			stress_nt_store64(&ptr[0x16], v);
+			stress_nt_store64(&ptr[0x17], v);
+
+			stress_nt_store64(&ptr[0x18], v);
+			stress_nt_store64(&ptr[0x19], v);
+			stress_nt_store64(&ptr[0x1a], v);
+			stress_nt_store64(&ptr[0x1b], v);
+			stress_nt_store64(&ptr[0x1c], v);
+			stress_nt_store64(&ptr[0x1d], v);
+			stress_nt_store64(&ptr[0x1e], v);
+			stress_nt_store64(&ptr[0x1f], v);
+
+			i++;
+			if (UNLIKELY(!keep_stressing_flag() || (max_ops && (i >= max_ops))))
+				break;
+		}
+		add_counter(args, i);
+		val++;
+		return 0;
+	}
+	return stress_vm_write64(buf, buf_end, sz, args, max_ops);
+}
+#endif
+
+
 /*
  *  stress_vm_read_64()
  *	simple 64 bit read
@@ -1843,48 +1979,52 @@ static size_t TARGET_CLONES stress_vm_read64(
 	const stress_args_t *args,
 	const uint64_t max_ops)
 {
-	volatile uint64_t *ptr = (uint64_t *)buf;
+	uint64_t *ptr = (uint64_t *)buf;
 	register size_t i = 0, n = sz / (sizeof(*ptr) * 32);
 
 	(void)buf_end;
 
 	while (i < n) {
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
+		shim_builtin_prefetch((uint8_t *)ptr + 1024, 0, 3);
+		(void)*(volatile uint64_t *)&ptr[0];
+		(void)*(volatile uint64_t *)&ptr[1];
+		(void)*(volatile uint64_t *)&ptr[2];
+		(void)*(volatile uint64_t *)&ptr[3];
+		(void)*(volatile uint64_t *)&ptr[4];
+		(void)*(volatile uint64_t *)&ptr[5];
+		(void)*(volatile uint64_t *)&ptr[6];
+		(void)*(volatile uint64_t *)&ptr[7];
+		ptr += 8;
 
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
+		(void)*(volatile uint64_t *)&ptr[0];
+		(void)*(volatile uint64_t *)&ptr[1];
+		(void)*(volatile uint64_t *)&ptr[2];
+		(void)*(volatile uint64_t *)&ptr[3];
+		(void)*(volatile uint64_t *)&ptr[4];
+		(void)*(volatile uint64_t *)&ptr[5];
+		(void)*(volatile uint64_t *)&ptr[6];
+		(void)*(volatile uint64_t *)&ptr[7];
+		ptr += 8;
 
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
-		(void)*(ptr++);
+		(void)*(volatile uint64_t *)&ptr[0];
+		(void)*(volatile uint64_t *)&ptr[1];
+		(void)*(volatile uint64_t *)&ptr[2];
+		(void)*(volatile uint64_t *)&ptr[3];
+		(void)*(volatile uint64_t *)&ptr[4];
+		(void)*(volatile uint64_t *)&ptr[5];
+		(void)*(volatile uint64_t *)&ptr[6];
+		(void)*(volatile uint64_t *)&ptr[7];
+		ptr += 8;
 
+		(void)*(volatile uint64_t *)&ptr[0];
+		(void)*(volatile uint64_t *)&ptr[1];
+		(void)*(volatile uint64_t *)&ptr[2];
+		(void)*(volatile uint64_t *)&ptr[3];
+		(void)*(volatile uint64_t *)&ptr[4];
+		(void)*(volatile uint64_t *)&ptr[5];
+		(void)*(volatile uint64_t *)&ptr[6];
+		(void)*(volatile uint64_t *)&ptr[7];
+		ptr += 8;
 		i++;
 		if (UNLIKELY(!keep_stressing_flag() || (max_ops && (i >= max_ops))))
 			break;
@@ -1894,6 +2034,43 @@ static size_t TARGET_CLONES stress_vm_read64(
 	return 0;
 }
 
+#if defined(HAVE_VECMATH)
+typedef int8_t stress_vint8w1024_t      __attribute__ ((vector_size(1024 / 8)));
+
+/*
+ *  stress_vm_write_1024v()
+ *	vector 1024 bit write, no read check
+ */
+static size_t TARGET_CLONES stress_vm_write1024v(
+	void *buf,
+	void *buf_end,
+	const size_t sz,
+	const stress_args_t *args,
+	const uint64_t max_ops)
+{
+	static uint64_t val;
+	stress_vint8w1024_t *ptr = (stress_vint8w1024_t *)buf;
+	stress_vint8w1024_t v;
+	register size_t i = 0, n = sz / sizeof(*ptr);
+
+	for (i = 0; i < sizeof(v) / sizeof(uint64_t); i++)
+		((uint64_t *)&v)[i] = val;
+
+	(void)buf_end;
+
+	while (i < n) {
+		*ptr++ = v;
+		i++;
+		if (UNLIKELY(!keep_stressing_flag() || (max_ops && (i >= max_ops))))
+			break;
+	}
+	add_counter(args, i);
+	val++;
+
+	return 0;
+}
+#endif
+
 /*
  *  stress_vm_rowhammer()
  *
@@ -1970,6 +2147,293 @@ static size_t TARGET_CLONES stress_vm_rowhammer(
 	return bit_errors;
 }
 
+/*
+ *  stress_vm_popcount()
+ *	population count - count number of 1 bits in val (K & R method)
+ */
+static inline unsigned int OPTIMIZE3 stress_vm_popcount(register uint8_t val)
+{
+	register unsigned int count;
+
+	for (count = 0; val; count++)
+		val &= val - 1;
+
+	return count;
+}
+
+/*
+ *  stress_vm_mscan()
+ *	for each byte, walk through each bit set to 0, check, set to 1, check
+ */
+static size_t TARGET_CLONES stress_vm_mscan(
+	void *buf,
+	void *buf_end,
+	const size_t sz,
+	const stress_args_t *args,
+	const uint64_t max_ops)
+{
+	size_t bit_errors = 0;
+	volatile uint8_t *ptr = (volatile uint8_t *)buf, *end;
+	uint64_t c = get_counter(args);
+
+	(void)sz;
+
+	for (ptr = (volatile uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr++, c++) {
+		*ptr |= 0x01;
+		*ptr |= 0x02;
+		*ptr |= 0x04;
+		*ptr |= 0x08;
+		*ptr |= 0x10;
+		*ptr |= 0x20;
+		*ptr |= 0x40;
+		*ptr |= 0x80;
+
+		if (UNLIKELY(!keep_stressing_flag() || (max_ops && (c >= max_ops))))
+			break;
+	}
+	end = (volatile uint8_t *)ptr;
+
+	add_counter(args, c);
+
+	for (ptr = (volatile uint8_t *)buf; ptr < end; ptr++) {
+		bit_errors += 8 - stress_vm_popcount(*ptr);
+	}
+
+	for (ptr = (volatile uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr++, c++) {
+		*ptr &= 0xfe;
+		*ptr &= 0xfd;
+		*ptr &= 0xfb;
+		*ptr &= 0xf7;
+		*ptr &= 0xef;
+		*ptr &= 0xdf;
+		*ptr &= 0xbf;
+		*ptr &= 0x7f;
+
+		if (UNLIKELY(!keep_stressing_flag() || (max_ops && (c >= max_ops))))
+			goto abort;
+	}
+	end = (volatile uint8_t *)ptr;
+
+	add_counter(args, c);
+
+	for (ptr = (volatile uint8_t *)buf; ptr < end; ptr++) {
+		bit_errors += stress_vm_popcount(*ptr);
+	}
+
+abort:
+	stress_vm_check("mscan", bit_errors);
+	set_counter(args, c);
+
+	return bit_errors;
+}
+
+/*
+ *  stress_vm_cache_stripe()
+ *	work through memory in cache chunks and write data
+ *	in forward/reverse byte wide stripes
+ */
+static size_t TARGET_CLONES stress_vm_cache_stripe(
+	void *buf,
+	void *buf_end,
+	const size_t sz,
+	const stress_args_t *args,
+	const uint64_t max_ops)
+{
+	volatile uint8_t *ptr;
+	size_t bit_errors = 0;
+	uint64_t c = get_counter(args);
+
+	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += 64) {
+		ptr[0x00] = 0xa0;
+		ptr[0x3f] = 0xcf;
+		ptr[0x01] = 0xa1;
+		ptr[0x3e] = 0xce;
+		ptr[0x02] = 0xa2;
+		ptr[0x3d] = 0xcd;
+		ptr[0x03] = 0xa3;
+		ptr[0x3c] = 0xcc;
+		ptr[0x04] = 0xa4;
+		ptr[0x3b] = 0xcb;
+		ptr[0x05] = 0xa5;
+		ptr[0x3a] = 0xca;
+		ptr[0x06] = 0xa6;
+		ptr[0x39] = 0xc9;
+		ptr[0x07] = 0xa7;
+		ptr[0x38] = 0xc8;
+		ptr[0x08] = 0xa8;
+		ptr[0x37] = 0xc7;
+		ptr[0x09] = 0xa9;
+		ptr[0x36] = 0xc6;
+		ptr[0x0a] = 0xaa;
+		ptr[0x35] = 0xc5;
+		ptr[0x0b] = 0xab;
+		ptr[0x34] = 0xc4;
+		ptr[0x0c] = 0xac;
+		ptr[0x33] = 0xc3;
+		ptr[0x0d] = 0xad;
+		ptr[0x32] = 0xc2;
+		ptr[0x0e] = 0xae;
+		ptr[0x31] = 0xc1;
+		ptr[0x0f] = 0xaf;
+		ptr[0x30] = 0xc0;
+		ptr[0x10] = 0x50;
+		ptr[0x2f] = 0x3f;
+		ptr[0x11] = 0x51;
+		ptr[0x2e] = 0x3e;
+		ptr[0x12] = 0x52;
+		ptr[0x2d] = 0x3d;
+		ptr[0x13] = 0x53;
+		ptr[0x2c] = 0x3c;
+		ptr[0x14] = 0x54;
+		ptr[0x2b] = 0x3b;
+		ptr[0x15] = 0x55;
+		ptr[0x2a] = 0x3a;
+		ptr[0x16] = 0x56;
+		ptr[0x29] = 0x39;
+		ptr[0x17] = 0x57;
+		ptr[0x28] = 0x38;
+		ptr[0x18] = 0x58;
+		ptr[0x27] = 0x37;
+		ptr[0x19] = 0x59;
+		ptr[0x25] = 0x35;
+		ptr[0x1a] = 0x5a;
+		ptr[0x26] = 0x36;
+		ptr[0x1b] = 0x5b;
+		ptr[0x24] = 0x34;
+		ptr[0x1c] = 0x5c;
+		ptr[0x23] = 0x33;
+		ptr[0x1d] = 0x5d;
+		ptr[0x22] = 0x32;
+		ptr[0x1e] = 0x5e;
+		ptr[0x21] = 0x31;
+		ptr[0x1f] = 0x5f;
+		ptr[0x20] = 0x30;
+
+		c++;
+		if (UNLIKELY(max_ops && c >= max_ops))
+			goto abort;
+		if (UNLIKELY(!keep_stressing_flag()))
+			goto abort;
+	}
+
+	for (ptr = (uint8_t *)buf; ptr < (uint8_t *)buf_end; ptr += 64) {
+		bit_errors += (ptr[0x00] != 0xa0);
+		bit_errors += (ptr[0x3f] != 0xcf);
+		bit_errors += (ptr[0x01] != 0xa1);
+		bit_errors += (ptr[0x3e] != 0xce);
+		bit_errors += (ptr[0x02] != 0xa2);
+		bit_errors += (ptr[0x3d] != 0xcd);
+		bit_errors += (ptr[0x03] != 0xa3);
+		bit_errors += (ptr[0x3c] != 0xcc);
+		bit_errors += (ptr[0x04] != 0xa4);
+		bit_errors += (ptr[0x3b] != 0xcb);
+		bit_errors += (ptr[0x05] != 0xa5);
+		bit_errors += (ptr[0x3a] != 0xca);
+		bit_errors += (ptr[0x06] != 0xa6);
+		bit_errors += (ptr[0x39] != 0xc9);
+		bit_errors += (ptr[0x07] != 0xa7);
+		bit_errors += (ptr[0x38] != 0xc8);
+		bit_errors += (ptr[0x08] != 0xa8);
+		bit_errors += (ptr[0x37] != 0xc7);
+		bit_errors += (ptr[0x09] != 0xa9);
+		bit_errors += (ptr[0x36] != 0xc6);
+		bit_errors += (ptr[0x0a] != 0xaa);
+		bit_errors += (ptr[0x35] != 0xc5);
+		bit_errors += (ptr[0x0b] != 0xab);
+		bit_errors += (ptr[0x34] != 0xc4);
+		bit_errors += (ptr[0x0c] != 0xac);
+		bit_errors += (ptr[0x33] != 0xc3);
+		bit_errors += (ptr[0x0d] != 0xad);
+		bit_errors += (ptr[0x32] != 0xc2);
+		bit_errors += (ptr[0x0e] != 0xae);
+		bit_errors += (ptr[0x31] != 0xc1);
+		bit_errors += (ptr[0x0f] != 0xaf);
+		bit_errors += (ptr[0x30] != 0xc0);
+		bit_errors += (ptr[0x10] != 0x50);
+		bit_errors += (ptr[0x2f] != 0x3f);
+		bit_errors += (ptr[0x11] != 0x51);
+		bit_errors += (ptr[0x2e] != 0x3e);
+		bit_errors += (ptr[0x12] != 0x52);
+		bit_errors += (ptr[0x2d] != 0x3d);
+		bit_errors += (ptr[0x13] != 0x53);
+		bit_errors += (ptr[0x2c] != 0x3c);
+		bit_errors += (ptr[0x14] != 0x54);
+		bit_errors += (ptr[0x2b] != 0x3b);
+		bit_errors += (ptr[0x15] != 0x55);
+		bit_errors += (ptr[0x2a] != 0x3a);
+		bit_errors += (ptr[0x16] != 0x56);
+		bit_errors += (ptr[0x29] != 0x39);
+		bit_errors += (ptr[0x17] != 0x57);
+		bit_errors += (ptr[0x28] != 0x38);
+		bit_errors += (ptr[0x18] != 0x58);
+		bit_errors += (ptr[0x27] != 0x37);
+		bit_errors += (ptr[0x19] != 0x59);
+		bit_errors += (ptr[0x25] != 0x35);
+		bit_errors += (ptr[0x1a] != 0x5a);
+		bit_errors += (ptr[0x26] != 0x36);
+		bit_errors += (ptr[0x1b] != 0x5b);
+		bit_errors += (ptr[0x24] != 0x34);
+		bit_errors += (ptr[0x1c] != 0x5c);
+		bit_errors += (ptr[0x23] != 0x33);
+		bit_errors += (ptr[0x1d] != 0x5d);
+		bit_errors += (ptr[0x22] != 0x32);
+		bit_errors += (ptr[0x1e] != 0x5e);
+		bit_errors += (ptr[0x21] != 0x31);
+		bit_errors += (ptr[0x1f] != 0x5f);
+		bit_errors += (ptr[0x20] != 0x30);
+	}
+
+	(void)stress_mincore_touch_pages(buf, sz);
+	inject_random_bit_errors(buf, sz);
+	stress_vm_check("cache-stripe", bit_errors);
+abort:
+	set_counter(args, c);
+
+	return bit_errors;
+}
+
+/*
+ *  stress_vm_cache_lines()
+ *	work through memory in cache line steps touching one byte per
+ * 	cache line
+ */
+static size_t TARGET_CLONES stress_vm_cache_lines(
+	void *buf,
+	void *buf_end,
+	const size_t sz,
+	const stress_args_t *args,
+	const uint64_t max_ops)
+{
+	volatile uint8_t *ptr;
+	size_t bit_errors = 0;
+	uint64_t c = get_counter(args);
+	uint8_t i;
+	static size_t offset = 0;
+
+	for (i = 0, ptr = (uint8_t *)buf + offset; ptr < (uint8_t *)buf_end; ptr += 64) {
+		*ptr = i++;
+	}
+	c++;
+	if (UNLIKELY(max_ops && c >= max_ops))
+		goto abort;
+	if (UNLIKELY(!keep_stressing_flag()))
+		goto abort;
+
+	for (i = 0, ptr = (uint8_t *)buf + offset; ptr < (uint8_t *)buf_end; ptr += 64) {
+		bit_errors += (*ptr != i++);
+	}
+	(void)stress_mincore_touch_pages(buf, sz);
+	inject_random_bit_errors(buf, sz);
+	stress_vm_check("cache-stripe", bit_errors);
+abort:
+	set_counter(args, c);
+
+	offset = (offset + 1) & 0x3f;
+
+	return bit_errors;
+}
+
+
 /*
  *  stress_vm_all()
  *	work through all vm stressors sequentially
@@ -1993,32 +2457,42 @@ static size_t stress_vm_all(
 }
 
 static const stress_vm_method_info_t vm_methods[] = {
-	{ "all",	stress_vm_all },
-	{ "flip",	stress_vm_flip },
-	{ "galpat-0",	stress_vm_galpat_zero },
-	{ "galpat-1",	stress_vm_galpat_one },
-	{ "gray",	stress_vm_gray },
-	{ "rowhammer",	stress_vm_rowhammer },
-	{ "incdec",	stress_vm_incdec },
-	{ "inc-nybble",	stress_vm_inc_nybble },
-	{ "rand-set",	stress_vm_rand_set },
-	{ "rand-sum",	stress_vm_rand_sum },
-	{ "read64",	stress_vm_read64 },
-	{ "ror",	stress_vm_ror },
-	{ "swap",	stress_vm_swap },
-	{ "move-inv",	stress_vm_moving_inversion },
-	{ "modulo-x",	stress_vm_modulo_x },
-	{ "prime-0",	stress_vm_prime_zero },
-	{ "prime-1",	stress_vm_prime_one },
-	{ "prime-gray-0",stress_vm_prime_gray_zero },
-	{ "prime-gray-1",stress_vm_prime_gray_one },
-	{ "prime-incdec",stress_vm_prime_incdec },
-	{ "walk-0d",	stress_vm_walking_zero_data },
-	{ "walk-1d",	stress_vm_walking_one_data },
-	{ "walk-0a",	stress_vm_walking_zero_addr },
-	{ "walk-1a",	stress_vm_walking_one_addr },
-	{ "write64",	stress_vm_write64 },
-	{ "zero-one",	stress_vm_zero_one },
+	{ "all",		stress_vm_all },
+	{ "cache-lines",	stress_vm_cache_lines },
+	{ "cache-stripe",	stress_vm_cache_stripe },
+	{ "flip",		stress_vm_flip },
+	{ "galpat-0",		stress_vm_galpat_zero },
+	{ "galpat-1",		stress_vm_galpat_one },
+	{ "gray",		stress_vm_gray },
+	{ "grayflip",		stress_vm_grayflip },
+	{ "rowhammer",		stress_vm_rowhammer },
+	{ "incdec",		stress_vm_incdec },
+	{ "inc-nybble",		stress_vm_inc_nybble },
+	{ "rand-set",		stress_vm_rand_set },
+	{ "rand-sum",		stress_vm_rand_sum },
+	{ "read64",		stress_vm_read64 },
+	{ "ror",		stress_vm_ror },
+	{ "swap",		stress_vm_swap },
+	{ "move-inv",		stress_vm_moving_inversion },
+	{ "modulo-x",		stress_vm_modulo_x },
+	{ "mscan",		stress_vm_mscan },
+	{ "prime-0",		stress_vm_prime_zero },
+	{ "prime-1",		stress_vm_prime_one },
+	{ "prime-gray-0",	stress_vm_prime_gray_zero },
+	{ "prime-gray-1",	stress_vm_prime_gray_one },
+	{ "prime-incdec",	stress_vm_prime_incdec },
+	{ "walk-0d",		stress_vm_walking_zero_data },
+	{ "walk-1d",		stress_vm_walking_one_data },
+	{ "walk-0a",		stress_vm_walking_zero_addr },
+	{ "walk-1a",		stress_vm_walking_one_addr },
+	{ "write64",		stress_vm_write64 },
+#if defined(HAVE_NT_STORE64)
+	{ "write64nt",		stress_vm_write64nt },
+#endif
+#if defined(HAVE_VECMATH)
+	{ "write1024v",		stress_vm_write1024v },
+#endif
+	{ "zero-one",		stress_vm_zero_one },
 	{ NULL,		NULL  }
 };
 
@@ -2210,5 +2684,6 @@ stressor_info_t stress_vm_info = {
 	.set_default = stress_vm_set_default,
 	.class = CLASS_VM | CLASS_MEMORY | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
