@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_LINUX_WATCHDOG_H)
+#include <linux/watchdog.h>
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"watchdog N",	  "start N workers that exercise /dev/watchdog" },
 	{ NULL,	"watchdog-ops N", "stop after N bogo watchdog operations" },
@@ -67,7 +66,7 @@ static const int sigs[] = {
 #endif
 };
 
-static const char *dev_watchdog = "/dev/watchdog";
+static const char dev_watchdog[] = "/dev/watchdog";
 static int fd;
 
 static void stress_watchdog_magic_close(void)
@@ -122,12 +121,12 @@ static int stress_watchdog(const stress_args_t *args)
 	if (access(dev_watchdog, R_OK | W_OK) < 0) {
 		if (errno == ENOENT) {
 			if (args->instance == 0)
-				pr_inf("%s: %s does not exist, skipping test\n",
+				pr_inf_skip("%s: %s does not exist, skipping test\n",
 					args->name, dev_watchdog);
 			return EXIT_SUCCESS;
 		} else {
 			if (args->instance == 0)
-				pr_inf("%s: cannot access %s, errno=%d (%s), skipping test\n",
+				pr_inf_skip("%s: cannot access %s, errno=%d (%s), skipping test\n",
 					args->name, dev_watchdog, errno, strerror(errno));
 			return EXIT_SUCCESS;
 		}
@@ -164,6 +163,8 @@ static int stress_watchdog(const stress_args_t *args)
 #if defined(WDIOC_KEEPALIVE)
 		ret = ioctl(fd, WDIOC_KEEPALIVE, 0);
 		(void)ret;
+#else
+		UNEXPECTED
 #endif
 
 #if defined(WDIOC_GETTIMEOUT)
@@ -173,6 +174,8 @@ static int stress_watchdog(const stress_args_t *args)
 			ret = ioctl(fd, WDIOC_GETTIMEOUT, &timeout);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(WDIOC_GETPRETIMEOUT)
@@ -182,6 +185,8 @@ static int stress_watchdog(const stress_args_t *args)
 			ret = ioctl(fd, WDIOC_GETPRETIMEOUT, &timeout);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(WDIOC_GETPRETIMEOUT)
@@ -191,6 +196,8 @@ static int stress_watchdog(const stress_args_t *args)
 			ret = ioctl(fd, WDIOC_GETTIMELEFT, &timeout);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(WDIOC_GETSUPPORT)
@@ -200,6 +207,8 @@ static int stress_watchdog(const stress_args_t *args)
 			ret = ioctl(fd, WDIOC_GETSUPPORT, &ident);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(WDIOC_GETSTATUS)
@@ -209,6 +218,8 @@ static int stress_watchdog(const stress_args_t *args)
 			ret = ioctl(fd, WDIOC_GETSTATUS, &flags);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(WDIOC_GETBOOTSTATUS)
@@ -218,6 +229,8 @@ static int stress_watchdog(const stress_args_t *args)
 			ret = ioctl(fd, WDIOC_GETBOOTSTATUS, &flags);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 
 #if defined(WDIOC_GETTEMP)
@@ -227,6 +240,8 @@ static int stress_watchdog(const stress_args_t *args)
 			ret = ioctl(fd, WDIOC_GETBOOTSTATUS, &temperature);
 			(void)ret;
 		}
+#else
+		UNEXPECTED
 #endif
 		stress_watchdog_magic_close();
 		ret = close(fd);
