@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2015 Christian Ehrhardt
+ * Copyright (C) 2015 Christian Ehrhardt.
  * Copyright (C) 2015-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +17,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-arch.h"
+
+#if defined(HAVE_BSD_WCHAR)
+#include <bsd/wchar.h>
+#endif
+
+#if defined(HAVE_WCHAR)
+#include <wchar.h>
+#endif
 
 #define STR1LEN 256
 #define STR2LEN 128
@@ -39,7 +43,7 @@ static const stress_help_t help[] = {
  *  the wide string stress test has different classes of stressors
  */
 typedef void (*stress_wcs_func)(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -50,7 +54,7 @@ typedef void (*stress_wcs_func)(
 typedef struct {
 	const char		*name;	/* human readable form of stressor */
 	const stress_wcs_func	func;	/* the wcs method function */
-	const void		*libc_func;
+	void			*libc_func;
 } stress_wcs_method_info_t;
 
 static const stress_wcs_method_info_t wcs_methods[];
@@ -108,7 +112,7 @@ static inline void wcschk(
  *	stress on wcscasecmp
  */
 static void stress_wcscasecmp(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -116,8 +120,10 @@ static void stress_wcscasecmp(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_wcscasecmp_t)(const wchar_t *s1, const wchar_t *s2);
+
 	register size_t i;
-	int (*test_wcscasecmp)(const wchar_t *s1, const wchar_t *s2) = libc_func;
+	test_wcscasecmp_t test_wcscasecmp = (test_wcscasecmp_t)libc_func;
 
 	(void)len2;
 
@@ -144,7 +150,7 @@ static void stress_wcscasecmp(
  *	stress on wcsncasecmp
  */
 static void stress_wcsncasecmp(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -152,8 +158,10 @@ static void stress_wcsncasecmp(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_wcsncasecmp_t)(const wchar_t *s1, const wchar_t *s2, size_t n);
+
 	register size_t i;
-	int (*test_wcsncasecmp)(const wchar_t *s1, const wchar_t *s2, size_t n) = libc_func;
+	test_wcsncasecmp_t test_wcsncasecmp = (test_wcsncasecmp_t)libc_func;
 
 	(void)len2;
 
@@ -183,7 +191,7 @@ static void stress_wcsncasecmp(
  *	stress on wcslcpy
  */
 static void stress_wcslcpy(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -209,7 +217,7 @@ static void stress_wcslcpy(
  *	stress on wcscpy
  */
 static void stress_wcscpy(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -217,8 +225,10 @@ static void stress_wcscpy(
 	const size_t len2,
 	bool *failed)
 {
+	typedef wchar_t * (*test_wcscpy_t)(wchar_t *dest, const wchar_t *src);
+
 	register size_t i;
-	wchar_t * (*test_wcscpy)(wchar_t *dest, const wchar_t *src) = libc_func;
+	test_wcscpy_t test_wcscpy = (test_wcscpy_t)libc_func;
 	wchar_t buf[len1 + len2 + 1];
 
 	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
@@ -237,7 +247,7 @@ static void stress_wcscpy(
  *	stress on wcslcat
  */
 static void stress_wcslcat(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -272,7 +282,7 @@ static void stress_wcslcat(
  *	stress on wcscat
  */
 static void stress_wcscat(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -280,8 +290,10 @@ static void stress_wcscat(
 	const size_t len2,
 	bool *failed)
 {
+	typedef wchar_t * (*test_wcscat_t)(wchar_t *dest, const wchar_t *src);
+
 	register size_t i;
-	wchar_t * (*test_wcscat)(wchar_t *dest, const wchar_t *src) = libc_func;
+	test_wcscat_t test_wcscat = (test_wcscat_t)libc_func;
 	wchar_t buf[len1 + len2 + 1];
 
 	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
@@ -305,7 +317,7 @@ static void stress_wcscat(
  *	stress on wcsncat
  */
 static void stress_wcsncat(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -313,8 +325,10 @@ static void stress_wcsncat(
 	const size_t len2,
 	bool *failed)
 {
+	typedef wchar_t * (*test_wcsncat_t)(wchar_t *dest, const wchar_t *src, size_t n);
+
 	register size_t i;
-	wchar_t * (*test_wcsncat)(wchar_t *dest, const wchar_t *src, size_t n) = libc_func;
+	test_wcsncat_t test_wcsncat = (test_wcsncat_t)libc_func;
 	wchar_t buf[len1 + len2 + 1];
 
 	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
@@ -338,7 +352,7 @@ static void stress_wcsncat(
  *	stress on wcschr
  */
 static void stress_wcschr(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -346,8 +360,10 @@ static void stress_wcschr(
 	const size_t len2,
 	bool *failed)
 {
+	typedef wchar_t * (*test_wcschr_t)(const wchar_t *wcs, wchar_t wc);
+
 	register size_t i;
-	wchar_t * (*test_wcschr)(const wchar_t *wcs, wchar_t wc) = libc_func;
+	test_wcschr_t test_wcschr = (test_wcschr_t)libc_func;
 
 	(void)len2;
 
@@ -367,7 +383,7 @@ static void stress_wcschr(
  *	stress on wcsrchr
  */
 static void stress_wcsrchr(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -375,8 +391,10 @@ static void stress_wcsrchr(
 	const size_t len2,
 	bool *failed)
 {
+	typedef wchar_t * (*test_wcsrchr_t)(const wchar_t *wcs, wchar_t wc);
+
 	register size_t i;
-	wchar_t * (*test_wcsrchr)(const wchar_t *wcs, wchar_t wc) = libc_func;
+	test_wcsrchr_t test_wcsrchr = (test_wcsrchr_t)libc_func;
 
 	(void)len2;
 
@@ -391,13 +409,13 @@ static void stress_wcsrchr(
 #endif
 
 #if defined(HAVE_WCSCMP) &&	\
-    !defined(STRESS_ARCH_M68K)	
+    !defined(STRESS_ARCH_M68K)
 /*
  *  stress_wcscmp()
  *	stress on wcscmp
  */
 static void stress_wcscmp(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -405,8 +423,10 @@ static void stress_wcscmp(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int * (*test_wcscmp_t)(const wchar_t *s1, const wchar_t *s2);
+
 	register size_t i;
-	int * (*test_wcscmp)(const wchar_t *s1, const wchar_t *s2) = libc_func;
+	test_wcscmp_t test_wcscmp = (test_wcscmp_t)libc_func;
 
 	(void)len2;
 
@@ -433,7 +453,7 @@ static void stress_wcscmp(
  *	stress on wcsncmp
  */
 static void stress_wcsncmp(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -441,8 +461,10 @@ static void stress_wcsncmp(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_wcsncmp_t)(const wchar_t *s1, const wchar_t *s2, size_t n);
+
 	register size_t i;
-	int (*test_wcsncmp)(const wchar_t *s1, const wchar_t *s2, size_t n) = libc_func;
+	test_wcsncmp_t test_wcsncmp = (test_wcsncmp_t)libc_func;
 
 	for (i = 1; keep_stressing_flag() && (i < len1); i++) {
 		WCSCHK(name, 0 == test_wcsncmp(str1, str1, len1), failed);
@@ -467,7 +489,7 @@ static void stress_wcsncmp(
  *	stress on wcslen
  */
 static void stress_wcslen(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -475,8 +497,10 @@ static void stress_wcslen(
 	const size_t len2,
 	bool *failed)
 {
+	typedef size_t (*test_wcslen_t)(const wchar_t *s);
 	register size_t i;
-	size_t (*test_wcslen)(const wchar_t *s) = libc_func;
+
+	test_wcslen_t test_wcslen = (test_wcslen_t)libc_func;
 
 	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
 		WCSCHK(name, len1 - 1 == test_wcslen(str1), failed);
@@ -496,7 +520,7 @@ static void stress_wcslen(
  *	stress on wcscoll
  */
 static void stress_wcscoll(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -504,8 +528,10 @@ static void stress_wcscoll(
 	const size_t len2,
 	bool *failed)
 {
+	typedef int (*test_wcscoll_t)(const wchar_t *ws1, const wchar_t *ws2);
+
 	register size_t i;
-	int (*test_wcscoll)(const wchar_t *ws1, const wchar_t *ws2) = libc_func;
+	test_wcscoll_t test_wcscoll = (test_wcscoll_t)libc_func;
 
 	(void)len2;
 
@@ -532,7 +558,7 @@ static void stress_wcscoll(
  *	stress on wcsxfrm
  */
 static void stress_wcsxfrm(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -540,8 +566,10 @@ static void stress_wcsxfrm(
 	const size_t len2,
 	bool *failed)
 {
+	typedef size_t (*test_wcsxfrm_t)(wchar_t* destination, const wchar_t* source, size_t num);
+
 	register size_t i;
-	size_t (*test_wcsxfrm)(wchar_t* destination, const wchar_t* source, size_t num) = libc_func;
+	test_wcsxfrm_t test_wcsxfrm = (test_wcsxfrm_t)libc_func;
 	wchar_t buf[len1 + len2];
 
 	for (i = 0; keep_stressing_flag() && (i < len1 - 1); i++) {
@@ -564,7 +592,7 @@ static void stress_wcsxfrm(
  *	iterate over all wcs stressors
  */
 static void stress_wcs_all(
-	const void *libc_func,
+	void *libc_func,
 	const char *name,
 	wchar_t *str1,
 	const size_t len1,
@@ -602,7 +630,7 @@ static const stress_wcs_method_info_t wcs_methods[] = {
 	{ "wcschr",		stress_wcschr,		wcschr },
 #endif
 #if defined(HAVE_WCSCMP) &&	\
-    !defined(STRESS_ARCH_M68K)	
+    !defined(STRESS_ARCH_M68K)
 	{ "wcscmp",		stress_wcscmp,		wcscmp },
 #endif
 #if defined(HAVE_WCSLCPY) &&	\
@@ -669,7 +697,7 @@ static int stress_wcs(const stress_args_t *args)
 {
 	stress_wcs_method_info_t const *wcs_method = &wcs_methods[0];
 	stress_wcs_func func;
-	const void *libc_func;
+	void *libc_func;
 	bool failed = false;
 	wchar_t ALIGN64 str1[STR1LEN], ALIGN64 str2[STR2LEN];
 	register wchar_t *ptr1, *ptr2;
@@ -730,5 +758,6 @@ stressor_info_t stress_wcs_info = {
 	.set_default = stress_wcs_set_default,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
