@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +16,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-cpu.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"x86syscall N",		"start N workers exercising functions using syscall" },
@@ -45,11 +41,9 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 	{ 0,			NULL }
 };
 
-#if defined(__linux__) &&	\
-    (defined(__x86_64__) || defined(__x86_64)) && \
-    defined(HAVE_CPUID_H) &&    \
-    defined(HAVE_CPUID) &&      \
-    NEED_GNUC(4,6,0)
+#if defined(__linux__) &&		\
+    !defined(__PCC__) &&		\
+    (defined(__x86_64__) || defined(__x86_64))
 
 typedef int (*stress_wfunc_t)(void);
 
@@ -68,18 +62,15 @@ typedef struct stress_x86syscall {
  */
 static int stress_x86syscall_supported(const char *name)
 {
-	uint32_t eax, ebx, ecx, edx;
-
 	/* Intel CPU? */
 	if (!stress_cpu_is_x86()) {
-		pr_inf("%s stressor will be skipped, "
+		pr_inf_skip("%s stressor will be skipped, "
 			"not a recognised Intel CPU\n", name);
 		return -1;
 	}
 	/* ..and supports syscall? */
-	__cpuid(0x80000001, eax, ebx, ecx, edx);
-	if (!(edx & (1ULL << 11))) {
-		pr_inf("%s stressor will be skipped, CPU "
+	if (!stress_cpu_x86_has_syscall()) {
+		pr_inf_skip("%s stressor will be skipped, CPU "
 			"does not support the syscall instruction\n", name);
 		return -1;
 	}
@@ -89,7 +80,7 @@ static int stress_x86syscall_supported(const char *name)
     defined(__NR_time)
 	return 0;
 #else
-	pr_inf("%s: stressor will be skipped, no definitions for __NR_getcpu, __NR_gettimeofday or __NR_time\n", name);
+	pr_inf_skip("%s: stressor will be skipped, no definitions for __NR_getcpu, __NR_gettimeofday or __NR_time\n", name);
 	return -1;
 #endif
 }
@@ -248,7 +239,7 @@ static char *x86syscall_list_str(void)
 	char *str = NULL;
 	size_t i, len = 0;
 
-	for (i = 0; i < SIZEOF_ARRAY(x86syscalls); i++) {
+	for (i = 0; x86syscalls[i].name; i++) {
 		if (x86syscalls[i].exercise) {
 			char *tmp;
 
@@ -262,7 +253,6 @@ static char *x86syscall_list_str(void)
 				*tmp = '\0';
 			} else {
 				(void)strcat(tmp, " ");
-				str = tmp;
 			}
 			(void)strcat(tmp, x86syscalls[i].name);
 			str = tmp;
@@ -285,7 +275,7 @@ static int x86syscall_check_x86syscall_func(void)
 	if (!stress_get_setting("x86syscall-func", &name))
 		return 0;
 
-	for (i = 0; i < SIZEOF_ARRAY(x86syscalls); i++) {
+	for (i = 0; x86syscalls[i].name; i++) {
 		const bool match = !strcmp(x86syscalls[i].name, name);
 
 		exercise |= match;
@@ -294,7 +284,7 @@ static int x86syscall_check_x86syscall_func(void)
 
 	if (!exercise) {
 		(void)fprintf(stderr, "invalid x86syscall-func '%s', must be one of:", name);
-		for (i = 0; i < SIZEOF_ARRAY(x86syscalls); i++)
+		for (i = 0; x86syscalls[i].name; i++)
 			(void)fprintf(stderr, " %s", x86syscalls[i].name);
 		(void)fprintf(stderr, "\n");
 		return -1;
@@ -308,7 +298,6 @@ static int x86syscall_check_x86syscall_func(void)
  */
 static int stress_x86syscall(const stress_args_t *args)
 {
-	char *str;
 	double t1, t2, t3, dt, overhead_ns;
 	uint64_t counter;
 
@@ -316,6 +305,8 @@ static int stress_x86syscall(const stress_args_t *args)
 		return EXIT_FAILURE;
 
 	if (args->instance == 0) {
+		char *str;
+
 		str = x86syscall_list_str();
 		if (str) {
 			pr_inf("%s: exercising syscall on: %s\n",
@@ -330,7 +321,7 @@ static int stress_x86syscall(const stress_args_t *args)
 	do {
 		size_t i;
 
-		for (i = 0; i < SIZEOF_ARRAY(x86syscalls); i++) {
+		for (i = 0; x86syscalls[i].name; i++) {
 			if (x86syscalls[i].exercise) {
 				x86syscalls[i].func();
 				inc_counter(args);
