@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_XATTR_H)
+#include <sys/xattr.h>
+#undef HAVE_ATTR_XATTR_H
+#elif defined(HAVE_ATTR_XATTR_H)
+#include <attr/xattr.h>
+#endif
+/*  Sanity check */
+#if defined(HAVE_SYS_XATTR_H) &&        \
+    defined(HAVE_ATTR_XATTR_H)
+#error cannot have both HAVE_SYS_XATTR_H and HAVE_ATTR_XATTR_H
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"xattr N",	"start N workers stressing file extended attributes" },
 	{ NULL,	"xattr-ops N",	"stop after N bogo xattr operations" },
@@ -81,7 +88,7 @@ static int stress_xattr(const stress_args_t *args)
 			args->name, filename, errno, strerror(errno));
 		goto out;
 	}
-	snprintf(bad_filename, sizeof(bad_filename), "%s_bad", filename);
+	(void)snprintf(bad_filename, sizeof(bad_filename), "%s_bad", filename);
 
 	hugevalue = calloc(1, hugevalue_sz);
 	if (hugevalue)
@@ -424,7 +431,7 @@ static int stress_xattr(const stress_args_t *args)
 		(void)sret;
 
 		/*
-		 *  Exercise invaid path, ENOENT
+		 *  Exercise invalid path, ENOENT
 		 */
 		sret = shim_listxattr(bad_filename, NULL, 0);
 		(void)sret;
@@ -508,7 +515,7 @@ out_close:
 out:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	free(hugevalue);
-	(void)unlink(filename);
+	(void)shim_unlink(filename);
 	(void)stress_temp_dir_rm_args(args);
 out_free:
 #if defined(XATTR_SIZE_MAX)
@@ -520,6 +527,7 @@ out_free:
 stressor_info_t stress_xattr_info = {
 	.stressor = stress_xattr,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
