@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -157,7 +152,7 @@ static int stress_zero(const stress_args_t *args)
 			 *  check if we can mmap /dev/zero
 			 */
 			ptr = mmap(NULL, page_size, PROT_READ, mmap_flags[i].flag,
-				fd, page_size * stress_mwc16());
+				fd, (off_t)(page_size * stress_mwc16()));
 			if (ptr == MAP_FAILED) {
 				if ((errno == ENOMEM) || (errno == EAGAIN))
 					continue;
@@ -197,5 +192,6 @@ static int stress_zero(const stress_args_t *args)
 stressor_info_t stress_zero_info = {
 	.stressor = stress_zero,
 	.class = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
