@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
@@ -1194,18 +1189,9 @@ static int stress_zlib_inflate(
 				stream_inf.avail_out = DATA_SIZE;
 				stream_inf.next_out = out;
 
-				pr_dbg_v("INF: %6d %6d\n", stream_inf.avail_in,
-					stream_inf.avail_out);
 				ret = inflate(&stream_inf, Z_NO_FLUSH);
-				pr_dbg_v("INF: %6d %6d written=%6lld rc=%2d\n",
-					stream_inf.avail_in,
-					stream_inf.avail_out,
-					DATA_SIZE - stream_inf.avail_out, ret);
-
 				switch (ret) {
 				case Z_BUF_ERROR:
-					pr_dbg_v("%s: zlib inflate error: %s\n",
-						args->name, stress_zlib_err(ret));
 					break;
 				case Z_NEED_DICT:
 				case Z_DATA_ERROR:
@@ -1233,12 +1219,6 @@ static int stress_zlib_inflate(
 	} while (sz > 0);
 
 finish:
-	if (g_opt_flags & OPT_FLAGS_VERIFY) {
-		pr_dbg_v("%s: inflate xsum value %" PRIu64
-			", xsum.xchars %" PRIu64 "\n",
-			args->name, xsum.xsum, xsum.xchars);
-	}
-
 	if (write(xsum_fd, &xsum, sizeof(xsum)) < 0) {
 		pr_fail("%s: zlib inflate pipe write failed for xsum, errno=%d (%s)\n",
 			args->name, err, strerror(err));
@@ -1349,14 +1329,7 @@ static int stress_zlib_deflate(
 				stream_def.avail_out = DATA_SIZE;
 				stream_def.next_out = out;
 
-				pr_dbg_v("DEF: %6d %6d\n", stream_def.avail_in,
-					stream_def.avail_out);
 				ret = deflate(&stream_def, flush);
-				pr_dbg_v("DEF: %6d %6d written=%6lld rc=%2d\n",
-					stream_def.avail_in,
-					stream_def.avail_out,
-					DATA_SIZE - stream_def.avail_out, ret);
-
 				if (ret == Z_STREAM_ERROR) {
 					pr_fail("%s: zlib deflate error: %s\n",
 						args->name, stress_zlib_err(ret));
@@ -1426,12 +1399,6 @@ finish:
 		bytes_in ? 100.0 * (double)bytes_out / (double)bytes_in : 0,
 		(t2 - t1 > 0.0) ? (bytes_in / (t2 - t1)) / MB : 0.0);
 
-	if (g_opt_flags & OPT_FLAGS_VERIFY) {
-		pr_dbg_v("%s: deflate xsum value %" PRIu64
-			", xsum.xchars %" PRIu64 "\n",
-			args->name, xsum.xsum, xsum.xchars);
-	}
-
 	ret = EXIT_SUCCESS;
 xsum_error:
 	if (write(xsum_fd, &xsum, sizeof(xsum)) < 0 ) {
@@ -1614,6 +1581,7 @@ stressor_info_t stress_zlib_info = {
 	.set_default = stress_zlib_set_default,
 	.class = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
