@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#define MIN_ZOMBIES		(1)
+#define MAX_ZOMBIES		(1000000)
+#define DEFAULT_ZOMBIES		(8192)
+
 typedef struct stress_zombie {
 	struct stress_zombie *next;
 	pid_t	pid;
@@ -45,6 +44,51 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
+/*
+ *  stress_pid_not_a_zombie()
+ *	return true if we are 100% sure the process is not a zombie
+ */
+static bool stress_pid_not_a_zombie(const pid_t pid)
+{
+#if defined(__linux__)
+	char path[PATH_MAX];
+	char buf[4096], *ptr = buf;
+	int fd;
+	ssize_t n;
+
+	(void)snprintf(path, sizeof(path), "/proc/%" PRIdMAX "/stat", (intmax_t)pid);
+	fd = open(path, O_RDONLY);
+	if (fd < 0)
+		return false;	/* Unknown */
+	n = read(fd, buf, sizeof(buf));
+	(void)close(fd);
+	if (n < 0)
+		return false;	/* Unknown */
+	while (*ptr) {
+		if (*ptr == ')')
+			break;
+		ptr++;
+	}
+	if (!*ptr)
+		return false;	/* Unknown */
+	ptr++;
+	while (*ptr) {
+		if (*ptr != ' ')
+			break;
+		ptr++;
+	}
+	if (!*ptr)
+		return false;	/* Unknown */
+
+	/* Process should not be in runnable state */
+	return (*ptr == 'R');
+#else
+	(void)pid;
+
+	return false; 	/* No idea */
+#endif
+}
+
 /*
  *  stress_zombie_new()
  *	allocate a new zombie, add to end of list
@@ -80,12 +124,25 @@ static stress_zombie_t *stress_zombie_new(void)
  *	reap a zombie and remove a zombie from head of list, put it onto
  *	the free zombie list
  */
-static void stress_zombie_head_remove(void)
+static void stress_zombie_head_remove(const stress_args_t *args, const bool check)
 {
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
+
 	if (zombies.head) {
 		int status;
 		stress_zombie_t *head;
 
+		if (verify && check) {
+			const pid_t pid = zombies.head->pid;
+
+			if (pid > 1) {
+				(void)kill(pid, SIGKILL);
+				if (stress_pid_not_a_zombie(zombies.head->pid))
+					pr_fail("%s: pid %" PRIdMAX " is not in the expected zombie state\n",
+						args->name, (intmax_t)pid);
+			}
+		}
+
 		(void)shim_waitpid(zombies.head->pid, &status, 0);
 
 		head = zombies.head;
@@ -162,7 +219,7 @@ static int stress_zombie(const stress_args_t *args)
 
 			zombie = stress_zombie_new();
 			if (!zombie) {
-				stress_zombie_head_remove();
+				stress_zombie_head_remove(args, false);
 				continue;
 			}
 
@@ -177,7 +234,7 @@ static int stress_zombie(const stress_args_t *args)
 			}
 			if (zombie->pid == -1) {
 				/* Reached max forks? .. then reap */
-				stress_zombie_head_remove();
+				stress_zombie_head_remove(args, false);
 				continue;
 			}
 			(void)setpgid(zombie->pid, g_pgrp);
@@ -186,7 +243,7 @@ static int stress_zombie(const stress_args_t *args)
 				max_zombies = zombies.length;
 			inc_counter(args);
 		} else {
-			stress_zombie_head_remove();
+			stress_zombie_head_remove(args, true);
 		}
 	} while (keep_stressing(args));
 
@@ -197,7 +254,7 @@ static int stress_zombie(const stress_args_t *args)
 
 	/* And reap */
 	while (zombies.head) {
-		stress_zombie_head_remove();
+		stress_zombie_head_remove(args, false);
 	}
 	/* And free */
 	stress_zombie_free();
@@ -214,5 +271,6 @@ stressor_info_t stress_zombie_info = {
 	.stressor = stress_zombie,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
