@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #define _XOPEN_SOURCE
 #define _GNU_SOURCE
@@ -31,8 +26,8 @@
 int main(void)
 {
 	char *ptr;
-	const char *key = "keystring";
-	const char *salt = "saltstring";
+	static const char key[] = "keystring";
+	static const char salt[] = "saltstring";
 	static struct crypt_data data;
 
 	ptr = crypt_r(key, salt, &data);
