@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2012-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,18 +16,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
  */
-#include "stress-ng.h"
 
 #if !defined(__linux__)
 #error requires linux to build
 #endif
 
+#include <fcntl.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <mntent.h>
 #include <sys/select.h>
 #include <sys/fanotify.h>
