@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #define _GNU_SOURCE
 
@@ -44,6 +39,8 @@ int main(void)
 		struct kmod_module *mod = kmod_module_get_module(l);
 		const char *module_name = kmod_module_get_name(mod);
 
+		(void)module_name;
+
 		ret = kmod_module_get_initstate(mod);
 		(void)ret;
 		ret = kmod_module_get_refcnt(mod);
