@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 
 #define _GNU_SOURCE
@@ -29,19 +24,7 @@
 #include <unistd.h>
 #include <features.h>
 
-#include "../stress-version.h"
-#include <sys/syscall.h>
-
 int main(void)
 {
-	int ret;
-
-#if NEED_GLIBC(2,27,0)
-	ret = pkey_free(0);
-	(void)ret;
-#endif
-	ret = syscall(__NR_pkey_free, 0);
-	(void)ret;
-
-	return 0;
+	return pkey_free(0);
 }
