@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +16,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 
 #include <features.h>
@@ -34,14 +29,15 @@
     defined(__GLIBC__) && \
     NEED_GNUC(5,5,0)
 
-#if (defined(__x86_64__) || defined(__x86_64))
-#define TARGET_CLONES	__attribute__((target_clones("sse","sse2","ssse3", "sse4.1", "sse4a", "avx","avx2","default")))
-#elif defined(__PPC64__)
-#define TARGET_CLONES	 __attribute__((target_clones("cpu=power9,default")))
+#if defined(__x86_64__) || \
+    defined(__x86_64) || \
+    defined(__PPC64__)
 #else
-#error target clones not supported
+#error arch not supported
 #endif
 
+#define TARGET_CLONES	__attribute__((target_clones(TARGET_CLONE)))
+
 static int TARGET_CLONES have_target_clones(void)
 {
 	return 0;
